/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8170_CMPSTPO_NECESSIDADE_DE_COMPRAS.sql

Descrio:
Clculo das quantidades de produtos a serem comprados e gravao na tabela
TRABNECECOMP.
Tambm registra o LOG do clculo.

P_CDFILIAL    - Lista de filiais.
P_TCDFILIAL   - Todas as filiais.
P_CDGRUPFILI  - Agrupamento de filiais.
P_DTINICIAL   - Data inicial.
P_DTFINAL     - Data final.
P_CDPRODINI   - Cdigo de produto inicial.
P_CDPRODFIN   - Cdigo de produto final.
P_PEACRESCIMO - Percentual de acrscimo.
P_CDOPERADOR  - Cdigo do operador que solicitou o clculo.
P_DTHRCALCNC  - Data/hora do clculo.
P_IDQTDEZERO  - Indentificador que indica se os produtos com quantidade zero
                sero mantidos na tabela de trabalho.
P_NRNECECOMP  - Nmero sequencial a ser utilizado para gravao dos dados.

Criao:
23/02/1999 Yoris L. de Souza
*/

CREATE OR REPLACE
PROCEDURE SF_NECESSIDADE_DE_COMPRAS (
    P_CDFILIAL                IN  VARCHAR2,
    P_TCDFILIAL               IN  VARCHAR2,
    P_CDGRUPFILI              IN  GRUPFILI.CDGRUPFILI%TYPE,
    P_DTINICIAL               IN  DATE,
    P_DTFINAL                 IN  DATE,
    P_CDPRODINI               IN  PRODUTO.CDARVPROD%TYPE,
    P_CDPRODFIN               IN  PRODUTO.CDARVPROD%TYPE,
    P_PEACRESCIMO             IN  NUMBER,
    P_CDOPERADOR              IN  OPERADOR.CDOPERADOR%TYPE,
    P_DTHRCALCNC              IN  DATE,
    P_IDQTDEZERO              IN  VARCHAR2,
    P_NRNECECOMP              IN  TRABNECECOMP.NRNECECOMP%TYPE,
    P_EXTRA                   IN  VARCHAR2 DEFAULT 'N',
    P_NRORG                   IN  TRABNECECOMP.NRORG%TYPE DEFAULT 1)
IS

    TYPE CURTYPE              IS REF CURSOR;
    CUR_FILIAL                CURTYPE;
    C_PREV_DATA               CURTYPE;
    C_PREV                    CURTYPE;
    C_PRODUTO                 CURTYPE;
    V_SQLFILIAL               VARCHAR2(2000);
    V_SQLCPREV                VARCHAR2(6000);
    V_SQLF_GFILIAL            VARCHAR2(1000);
    V_SQLW_GFILIAL            VARCHAR2(1000);
    V_SQLFILTER               VARCHAR2(2000);
    V_FILIAL_CORRENTE         VARCHAR2(4);
    V_FILIAL_UNICA            VARCHAR2(4);
    V_IDSISDOS                VARCHAR2(1);
    
    V_RETIRDATA               NUMBER;
    V_ENTRADADATA             NUMBER;
    V_NRDIASEG                NUMBER;
    V_NRDIASEGANT             NUMBER;
    V_APENAS_CONSUMO          NUMBER;
    V_QTDESTOQUENEGCALC       NUMBER;
    V_QTESTOQUENECCMP         NUMBER;
    V_QTESTOQUEVND            NUMBER;
    V_QTDIAESTOQUE            NUMBER;
    V_VALRESTO                NUMBER;
    V_VALRBRUT                NUMBER;

    V_FILINECE                INT;
    V_PRODFILI                INT;

    V_CDPRODINI               VARCHAR2(40);
    V_CDPRODFIN               VARCHAR2(40);
    V_CODPRODUTO              VARCHAR2(10);
    V_CODARVPRODUTO           VARCHAR2(20);
    V_CODPRODUTOPREF          VARCHAR2(10);
    V_CDPERIENTR              VARCHAR2(2);
    V_NRLIMISEGCAD            LIMISEGPROD.NRLIMISEGU%TYPE;
    V_DTPROCESS               DATE;
    V_DTENTREGA               DATE;
    V_DTANTPROXENT            DATE;
    V_DTMAXENTREGA            DATE;
    V_DTMAXENTREGA_ADD1       DATE;
    V_DTPRIMEIRAENTREGA       DATE;
    V_DTPRIMEIRAUTILIZAPROD   DATE;
    V_IDESTOQUE               CHAR(1);
    V_IDEVOLESTONC            CHAR(1);
    V_IDUTILDTPROC            CHAR(1);
    V_IDENTRADAFORAPERIODO    VARCHAR2(1);
    V_IDESTOQUENEGATIVO       CHAR(1);
    V_IDMARCNECCOMPRA         CHAR(1);
    V_UTILIZAPRODPREF         CHAR(1);
    V_EHPRIMEIRAVEZPROD       CHAR(1);
    V_PRIMEIRAENTFORAPERIODO  CHAR(1);
    V_IDUTILLOTE              CHAR(1);
    V_IDCONSAFNECCMP          CHAR(1);
    V_PRODUTOPERFILPROD       VARCHAR2(20);        
    V_QUANTIDADE              FLOAT;
    V_QTCONSUMOPERIODO        FLOAT;
    V_QTCONSUMOPERIODOFILNECE FLOAT;
    V_QTCONSUMOANTECIPADO     FLOAT;
    V_QTCONSUMOANTECIPADOFILNECE FLOAT;
    V_QTASERENTREGUE          FLOAT;
    V_QTASERENTREGUEFILNECE   FLOAT;
    V_FATORCONVERSAO          FLOAT;
    V_QTJASOLICITADA          FLOAT;
    V_QTJASOLICPEXOL          FLOAT;
    V_QTCONSUMOATEPENTREGA    FLOAT;
    V_QTCONSUMOATEPENTREGAFILNECE FLOAT;
    V_QTESTOQUESEGURANCA      FLOAT;
    V_QTDESTOQUEATUAL         FLOAT;
    V_QTDESTOQUEUTILIZADO     FLOAT;
    V_QTDTEORICANECCOMPRA     FLOAT;
    V_QTDORIGINALNECCOMPRA    FLOAT;
    V_QTDPRACOMPRAR           FLOAT;
    V_VRARREDONDACMP          FLOAT;
    V_VRFATORCONVERSAO        FLOAT;
    V_QTDPRODUTOCONVCOMPRA    FLOAT;
    V_QTDESTOQUENEGATIVOIMPR  FLOAT;
    V_QTDAFANTPRIMEIRAENT     FLOAT;

    V_QTESTAFPANT             FLOAT;
    V_QTESTAFPDEP             FLOAT;
    V_QTESTAFPEND             FLOAT;
    V_QTESTAFPEXOL            FLOAT;
    V_QTESTOQATUAL            FLOAT;
    V_EXTRA                   CHAR(1);
    V_IDMAPDISTALMOX          CHAR(1);


    C_UNDERSCORE              CHAR(1):= '_';
    W_A                       CHAR(1):= 'A';
    W_N                       CHAR(1):= 'N';
    W_P                       CHAR(1):= 'P';
    W_S                       CHAR(1):= 'S';
    W_0                       CHAR(1):= '0';
    W_1                       CHAR(1):= '1';
    W_2                       CHAR(1):= '2';
    W_3                       CHAR(1):= '3';
    W_4                       CHAR(1):= '4';
    W_5                       CHAR(1):= '5';
    W_6                       CHAR(1):= '6';

BEGIN
    IF P_EXTRA IS NULL THEN
       V_EXTRA := 'N';
    ELSE
       V_EXTRA := P_EXTRA;
    END IF;

    BEGIN
      SELECT NVL(IDSISDOS, W_1), NVL(IDUTILDTPROC, W_N)
      INTO V_IDSISDOS, V_IDUTILDTPROC
      FROM PARAMGERAL
      WHERE PARAMGERAL.NRORG = P_NRORG;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
        V_IDSISDOS:= W_1;
        V_IDUTILDTPROC:= W_N;
      END;
    END;

    IF P_CDPRODINI > P_CDPRODFIN THEN
       V_CDPRODINI := P_CDPRODINI || 'ZZZZZZZZZ';
       V_CDPRODFIN := P_CDPRODFIN;
    ELSE
       V_CDPRODFIN := P_CDPRODFIN || 'ZZZZZZZZZ';
       V_CDPRODINI := P_CDPRODINI;
    END IF;

    /* MONTAGEM DA QUERY PARA BUSCAR FILIAIS */
    IF (((P_TCDFILIAL <> 'T') OR (P_TCDFILIAL IS NULL)) AND (P_CDFILIAL IS NOT NULL)) THEN
      IF (LENGTH(P_CDFILIAL) > 6) THEN
        V_SQLFILTER:= ' WHERE INSTR(''' || P_CDFILIAL || ''', CDFILIAL) > 0 ';
      ELSE
        IF INSTR(P_CDFILIAL, '_') > 0 THEN
           V_FILIAL_UNICA:= SUBSTR(P_CDFILIAL, 2, LENGTH(P_CDFILIAL)-2);
        ELSE
           V_FILIAL_UNICA:= P_CDFILIAL;
        END IF;
        V_SQLFILTER:= ' WHERE  CDFILIAL = ''' || V_FILIAL_UNICA || '''';
      END IF;
    END IF;

    IF P_CDGRUPFILI IS NOT NULL THEN
      V_SQLF_GFILIAL:= '(SELECT CDFILIAL '     ||
                       '  FROM FILGRUPFILI '   ||
                       '  WHERE  (CDGRUPFILI = ''' || P_CDGRUPFILI || ''')'||' AND (NRORG = '||P_NRORG||')) G, ';
      V_SQLW_GFILIAL:= ' G.CDFILIAL = F.CDFILIAL AND ';
    END IF;

    V_SQLFILIAL:= 'SELECT F.CDFILIAL AS COD_FILIAL_CORRENTE, NVL(PCMP.IDUTPRODPREF, ''' || W_N || '''),' ||
                  'NVL(PCMP.IDSCONESTNEC, ''' || W_N || '''), ' ||
                  'NVL(PCMP.IDMAPDISTALMOX, ''' || W_N || '''), ' ||
                  'NVL(P.IDEVOLESTONC, ''' || W_N || '''), ' ||
                  'DECODE('''||V_IDUTILDTPROC||''',  ''' || W_S || ''', NVL(PF.DTPROCESSA, TRUNC(SYSDATE)), TRUNC(SYSDATE)),' ||
                  'NVL(PF.IDUTILLOTE,'''|| W_N ||'''),'||
                  'NVL(P.IDCONSAFNECCMP,'''|| W_N ||''')'||
                  'FROM (SELECT CDFILIAL, NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP, FILIAL.NRORG ' ||
                  '      FROM FILIAL ' || V_SQLFILTER   || ' ) F, '  || V_SQLF_GFILIAL  ||
                  '      PARCMPSUP PCMP, PARASUPRI P, PARAMFILIAL PF '  ||
                  'WHERE ' || V_SQLW_GFILIAL            ||
                  '      P.CDFILIAL     = F.CDFILCENTCMP ' ||
                  '  AND F.CDFILIAL     = PCMP.CDFILIAL '  ||
                  '  AND PF.CDFILIAL(+) = F.CDFILIAL '     ||
                  '  AND F.NRORG    = '||P_NRORG||' '||
                  '  AND PCMP.NRORG = '||P_NRORG||' '||
                  '  AND P.NRORG    = '||P_NRORG||' '||
                  '  AND PF.NRORG   = '||P_NRORG||' ';
    /* FIM -- MONTAGEM DA QUERY PARA BUSCAR FILIAIS */

    V_QTDESTOQUENEGCALC:= 0;

    OPEN CUR_FILIAL FOR V_SQLFILIAL;
    LOOP
      FETCH CUR_FILIAL INTO V_FILIAL_CORRENTE, V_UTILIZAPRODPREF, V_IDESTOQUE, V_IDMAPDISTALMOX, V_IDEVOLESTONC, V_DTPROCESS, V_IDUTILLOTE, V_IDCONSAFNECCMP;
      EXIT WHEN CUR_FILIAL%NOTFOUND;
          V_SQLCPREV:= NULL;
          SELECT COUNT(*)
          INTO V_FILINECE
          FROM FILINECE
          WHERE CDFILIAL = V_FILIAL_CORRENTE
            AND NRORG = P_NRORG;

          -- PROCESSO EXCLUSIVO GR
          IF V_IDSISDOS = W_3 THEN
             INSERT INTO NECESSIDADE_TMP
                (NRNECECOMP,
                 CDFILIAL,
                 NRLANCTONF,
                 CDPRODUTO,
                 CDPRODESTO,
                 VRFATOCONV,
                 IDENTRADAAF,
                 NRORG)
             SELECT
                NRNECECOMP,
                CDFILIAL,
                NRLANCTONF,
                CDPRODUTO,
                CDPRODESTO,
                VRFATOCONV,
                MIN(IDENTRADAAF),
                P_NRORG
             FROM
                (SELECT /*+ ORDERED */
                 DISTINCT
                   P_NRNECECOMP AS NRNECECOMP,
                   NFRELACAF.CDFILIAL,
                   NFRELACAF.NRLANCTONF,
                   NFRELACAF.CDPRODUTO,
                   PRODUTO.CDPRODESTO,
                   PRODUTO.VRFATOCONV,
                   DECODE(TRUNC(V_DTPROCESS) - ITEMAUTF.DTPROGENTAF, ABS(TRUNC(V_DTPROCESS) - ITEMAUTF.DTPROGENTAF), W_P, W_A) IDENTRADAAF
                 FROM NFRELACAF, ITEMAUTF, PRODUTO
                 WHERE NFRELACAF.CDFILDEST  = V_FILIAL_CORRENTE
                  AND ITEMAUTF.CDFILDEST   = NFRELACAF.CDFILDEST
                  AND ITEMAUTF.CDFILLANC   = NFRELACAF.CDFILLANC
                  AND ITEMAUTF.NRAUTOFORN  = NFRELACAF.NRAUTOFORN
                  AND ITEMAUTF.CDPRODUTO   = NFRELACAF.CDPRODUTO
                  AND ITEMAUTF.CDMARCA     = NFRELACAF.CDMARCA
                  AND ITEMAUTF.DTPROGENTAF = NFRELACAF.DTPROGENTAF
                  AND PRODUTO.CDPRODUTO    = NFRELACAF.CDPRODUTO
                  AND NFRELACAF.NRORG      = P_NRORG
                  AND ITEMAUTF.NRORG       = P_NRORG
                  AND PRODUTO.NRORG        = P_NRORG)
             GROUP BY
                NRNECECOMP,
                CDFILIAL,
                NRLANCTONF,
                CDPRODUTO,
                CDPRODESTO,
                VRFATOCONV;
          END IF;

          IF V_FILINECE > 0 THEN
            V_SQLCPREV:=  'SELECT P.CDPRODESTO AS CDPRODUTO, P.CDARVPROD AS CDARVPRODUTO, P.VRFATOCONV ' ||
                           'FROM SF_V_PREVISAO_COMPRA X, PRODUTO P ' ||
                           'WHERE X.CDFILIAL    = ''' || V_FILIAL_CORRENTE || ''' ' ||
                           '  AND X.DTPREVCONS >= ''' || V_DTPROCESS       || ''' ' ||
                           '  AND P.CDPRODUTO   = X.CDPRODUTO ' ||
                           '  AND X.NRORG       = '||P_NRORG||' '||
                           '  AND P.NRORG       = '||P_NRORG||' '||
                           '  AND P.CDARVPROD BETWEEN ''' || V_CDPRODINI || ''' AND '''  || V_CDPRODFIN || ''' ';

            V_SQLCPREV:= V_SQLCPREV || ' UNION ' ||
                                       'SELECT ' ||
                                       '   ESTOSEGU.CDPRODESTO AS CDPRODUTO, ' ||
                                       '   PRODUTO.CDARVPROD, '||
                                       '   PRODUTO.VRFATOCONV '||
                                       'FROM '                 ||
                                       '   PRODUTO, '          ||
                                       '   ESTOSEGU '          ||
                                       'WHERE '                ||
                                       '   ESTOSEGU.CDFILIAL = ''' || V_FILIAL_CORRENTE  || ''' ' ||
                                       '   AND PRODUTO.CDPRODUTO = ESTOSEGU.CDPRODESTO ' ||
                                       '   AND ESTOSEGU.NRORG = '||P_NRORG||' '||
                                       '   AND PRODUTO.NRORG  = '||P_NRORG||' '|| 
                                       '   AND PRODUTO.CDARVPROD BETWEEN''' || V_CDPRODINI || ''' AND '''  || V_CDPRODFIN || ''' ';

            V_SQLCPREV:= V_SQLCPREV || ' UNION ' ||
                                       'SELECT P.CDPRODESTO AS CDPRODUTO, P.CDARVPROD AS CDARVPRODUTO, P.VRFATOCONV ' ||
                                       'FROM SF_V_PREVISAO_COMPRA_FILIAL X, PRODUTO P ' ||
                                       'WHERE X.CDFILIAL    = ''' || V_FILIAL_CORRENTE || ''' ' ||
                                       '  AND X.DTPREVCONS >= ''' || V_DTPROCESS       || ''' ' ||
                                       '  AND P.CDPRODUTO   = X.CDPRODUTO ' ||
                                       '  AND X.NRORG       = '||P_NRORG||' '||
                                       '  AND P.NRORG       = '||P_NRORG||' '||
                                       '  AND P.CDARVPROD BETWEEN ''' || V_CDPRODINI || ''' AND '''  || V_CDPRODFIN || ''' ';
          ELSE
            V_SQLCPREV:= 'SELECT ' ||
                         '   ESTOSEGU.CDPRODESTO AS CDPRODUTO, ' ||
                         '   PRODUTO.CDARVPROD, '                ||
                         '   PRODUTO.VRFATOCONV '                 ||
                         'FROM '                                 ||
                         '   PRODUTO, '                          ||
                         '   ESTOSEGU '                          ||
                         'WHERE '                                ||
                         '   ESTOSEGU.CDFILIAL = ''' || V_FILIAL_CORRENTE  || ''' ' ||
                         '   AND PRODUTO.CDPRODUTO = ESTOSEGU.CDPRODESTO ' ||
                         '   AND ESTOSEGU.NRORG = '||P_NRORG||' '||
                         '   AND PRODUTO.NRORG  = '||P_NRORG||' '||
                         '   AND PRODUTO.CDARVPROD BETWEEN ''' || V_CDPRODINI || ''' AND '''  || V_CDPRODFIN || ''' ';

            V_SQLCPREV:= V_SQLCPREV || ' UNION ' ||
                          'SELECT P.CDPRODESTO AS CDPRODUTO, P.CDARVPROD AS CDARVPRODUTO, P.VRFATOCONV ' ||
                          'FROM SF_V_PREVISAO_COMPRA X, PRODUTO P ' ||
                          'WHERE X.CDFILIAL    = ''' || V_FILIAL_CORRENTE || ''' ' ||
                          '  AND X.DTPREVCONS >= ''' || V_DTPROCESS       || ''' ' ||
                          '  AND P.CDPRODUTO   = X.CDPRODUTO ' ||
                          '  AND X.NRORG       = '||P_NRORG||' '||
                          '  AND P.NRORG       = '||P_NRORG||' '||
                          '  AND P.CDARVPROD BETWEEN ''' || V_CDPRODINI || ''' AND '''  || V_CDPRODFIN || ''' ';
          END IF;

          OPEN C_PREV FOR V_SQLCPREV;
          LOOP
            FETCH C_PREV INTO V_CODPRODUTO, V_CODARVPRODUTO, V_FATORCONVERSAO;
            EXIT WHEN C_PREV%NOTFOUND;
              SELECT
                 COUNT(*)
              INTO
                 V_PRODFILI
              FROM
                 PRODFILI
              WHERE
                 V_FILIAL_CORRENTE    = PRODFILI.CDFILIAL   AND
                 V_CODPRODUTO         = PRODFILI.CDPRODUTO  AND
                 P_NRORG              = PRODFILI.NRORG      AND
                 NVL(PRODFILI.IDPRODATIV,'A') = 'A';
              IF V_PRODFILI > 0 THEN
                 V_PRODUTOPERFILPROD := NULL;
                 SELECT
                    MAX(B.CDARVPROD)
                 INTO V_PRODUTOPERFILPROD
                 FROM
                    PERFILPROD A,
                    PRODUTO B
                 WHERE
                    A.CDPRODUTO = B.CDPRODUTO       AND
                    A.CDFILIAL  = V_FILIAL_CORRENTE AND
                    A.NRORG     = P_NRORG           AND
                    B.NRORG     = P_NRORG           AND
                    INSTR(V_CODARVPRODUTO, B.CDARVPROD) = 1;

                 -- PROCESSO EXCLUSIVO GR
                 IF V_IDSISDOS = W_3 THEN
                   V_ENTRADADATA:= 0;
                   V_RETIRDATA:= 0;
                 ELSE
                    /* 2 - VERIFICA SE HOUVE ENTRADA PARA CONSIDERAR COMO ESTOQUE ENTREGUE */
                    SELECT NVL(SUM(DECODE(IDTIPOMOVI, 2, 1, 0)), 0) IDTIPOMOVI_2,
                           NVL(SUM(DECODE(IDTIPOMOVI, 3, 1, 0)), 0) IDTIPOMOVI_3
                    INTO V_ENTRADADATA, V_RETIRDATA
                    FROM ITLANCTOEST
                    WHERE CDFILIMOVI = V_FILIAL_CORRENTE
                      AND DTLANCMOVI = V_DTPROCESS
                      AND CDPRODMOVI = V_CODPRODUTO
                      AND NRORG      = P_NRORG
                      AND IDTIPOMOVI IN (W_2, W_3);
                 END IF;
                 V_EHPRIMEIRAVEZPROD:= W_S;
                 V_PRIMEIRAENTFORAPERIODO:= W_S;
                 V_QTDESTOQUENEGCALC:= 0;
                 V_QTESTOQUENECCMP:= 0;
                 V_QTDESTOQUENEGATIVOIMPR:= 0;
                 V_DTPRIMEIRAENTREGA:= NULL;

                 OPEN C_PREV_DATA FOR
                    SELECT CALECOMP.DTENTREGA, CALECOMP.DTANTPROXENT, CALECOMP.CDPERIENTR,
                           W_N AS IDENTFORPERI
                    FROM PERFILPROD, CALECOMP, PRODUTO P
                    WHERE PERFILPROD.CDPRODUTO = P.CDPRODUTO
                      AND PERFILPROD.CDFILIAL  = V_FILIAL_CORRENTE
                      AND CALECOMP.CDFILIAL    = PERFILPROD.CDFILIAL
                      AND CALECOMP.CDPERIENTR  = PERFILPROD.CDPERIENTR
                      AND CALECOMP.NRORG       = P_NRORG
                      AND PERFILPROD.NRORG     = P_NRORG
                      AND P.NRORG              = P_NRORG
                      AND P.CDARVPROD = V_PRODUTOPERFILPROD
                      AND CALECOMP.DTENTREGA   BETWEEN V_DTPROCESS AND P_DTFINAL
                    UNION ALL
                    SELECT DISTINCT ITEMAUTF.DTPROGENTAF AS DTENTREGA, TO_DATE(NULL) AS DTANTPROXENT,
                                    LPAD(' ', 2, ' ') AS CDPERIENTR,  W_S  AS IDENTFORPERI
                    FROM PRODUTO P, AUTOFORNE, ITEMAUTF
                    WHERE ITEMAUTF.CDFILDEST     = V_FILIAL_CORRENTE
                      AND V_IDSISDOS = W_3
                      AND AUTOFORNE.IDAFFORALIM  = W_N
                      AND AUTOFORNE.CDFILDEST    = ITEMAUTF.CDFILDEST
                      AND AUTOFORNE.CDFILLANC    = ITEMAUTF.CDFILLANC
                      AND AUTOFORNE.NRAUTOFORN   = ITEMAUTF.NRAUTOFORN
                      AND ITEMAUTF.CDPRODUTO     = P.CDPRODUTO
                      AND P.CDPRODESTO           = V_CODPRODUTO
                      AND P.NRORG                = P_NRORG
                      AND AUTOFORNE.NRORG        = P_NRORG
                      AND ITEMAUTF.NRORG         = P_NRORG
                      AND AUTOFORNE.IDSITUATUAAF IS NULL
                      AND (ITEMAUTF.IDBAIXITMAF  IS NULL OR ITEMAUTF.IDBAIXITMAF = W_1)
                      AND ITEMAUTF.DTPROGENTAF   BETWEEN V_DTPROCESS AND P_DTFINAL
                      AND NVL(AUTOFORNE.IDAFEXPIRADA, W_N) = W_N
                      AND ITEMAUTF.DTPROGENTAF NOT IN (SELECT CALECOMP.DTENTREGA
                                                       FROM PERFILPROD, CALECOMP, PRODUTO
                                                       WHERE PERFILPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                                                         AND PERFILPROD.CDFILIAL  = ITEMAUTF.CDFILDEST
                                                         AND CALECOMP.CDFILIAL    = ITEMAUTF.CDFILDEST
                                                         AND CALECOMP.CDPERIENTR  = PERFILPROD.CDPERIENTR
                                                         AND CALECOMP.NRORG       = P_NRORG
                                                         AND PERFILPROD.NRORG     = P_NRORG
                                                         AND PRODUTO.NRORG        = P_NRORG
                                                         AND CALECOMP.DTENTREGA   BETWEEN V_DTPROCESS AND P_DTFINAL
                                                         AND PRODUTO.CDARVPROD    = V_PRODUTOPERFILPROD)
                    ORDER BY 1;
                 LOOP
                   FETCH C_PREV_DATA INTO V_DTENTREGA, V_DTANTPROXENT, V_CDPERIENTR, V_IDENTRADAFORAPERIODO;
                   EXIT WHEN C_PREV_DATA%NOTFOUND;
                     -- FAZ CHECAGEM NAS FUNES DE LIMITE DE SEGURANA
                     SF_LIMISEGU_CONS_DNUT(V_FILIAL_CORRENTE, V_CODARVPRODUTO, V_DTENTREGA, V_CDPERIENTR, V_NRDIASEG, V_NRLIMISEGCAD, P_NRORG);
                     SF_LIMISEGU(V_FILIAL_CORRENTE, V_CODARVPRODUTO, V_DTANTPROXENT, V_CDPERIENTR, V_NRDIASEGANT, V_NRLIMISEGCAD, P_NRORG);
                     V_QTCONSUMOANTECIPADO:= 0;
                     V_QTCONSUMOANTECIPADOFILNECE:= 0;
                     V_QTCONSUMOPERIODO:= 0;
                     V_QTCONSUMOPERIODOFILNECE:= 0;
                     V_QTASERENTREGUE:= 0;
                     V_QTASERENTREGUEFILNECE:= 0;
                     IF (V_IDENTRADAFORAPERIODO = W_N) THEN
                       -- BUSCA O CONSUMO NO PERIODO.
                       -- V_QTCONSPERIOD
                       BEGIN
                         SELECT NVL(SUM(V_PREV.QTPREVCONS * V_FATORCONVERSAO), 0) QUANTIDADE
                         INTO V_QTCONSUMOPERIODO
                         FROM SF_V_PREVISAO_COMPRA V_PREV
                         WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                           AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                           AND V_PREV.NRORG      = P_NRORG
                           AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S)
                           AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA AND V_DTANTPROXENT;
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_QTCONSUMOPERIODO:= 0;
                       END;
                       BEGIN
                         IF V_FILINECE > 0 THEN
                           SELECT NVL(SUM(V_PREV.QTPREVCONS * P.VRFATOCONV), 0) QUANTIDADE
                           INTO V_QTCONSUMOPERIODOFILNECE
                           FROM SF_V_PREVISAO_COMPRA_FILIAL V_PREV, PRODUTO P
                           WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                             AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                             AND P.CDPRODUTO       = V_PREV.CDPRODUTO
                             AND V_PREV.NRORG      = P_NRORG
                             AND P.NRORG           = P_NRORG
                             AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S)
                             AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA AND V_DTANTPROXENT;
                         ELSE
                            V_QTCONSUMOPERIODOFILNECE:= 0;
                         END IF;
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_QTCONSUMOPERIODOFILNECE:= 0;
                       END;
                       V_QTCONSUMOPERIODO:= V_QTCONSUMOPERIODO + V_QTCONSUMOPERIODOFILNECE;

                       IF V_NRDIASEG > 0 THEN
                          -- BUSCA O CONSUMO ANTECIPADO.
                          -- V_QTCONSANTECI
                          BEGIN
                            SELECT NVL(SUM(V_PREV.QTPREVCONS * V_FATORCONVERSAO), 0) QUANTIDADE
                            INTO V_QTCONSUMOANTECIPADO
                            FROM SF_V_PREVISAO_COMPRA V_PREV
                            WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                              AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                              AND V_PREV.NRORG      = P_NRORG
                              AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA AND (V_DTENTREGA + V_NRDIASEG - 1);
                          EXCEPTION
                            WHEN NO_DATA_FOUND THEN
                              V_QTCONSUMOANTECIPADO:= 0;
                          END;

                          BEGIN
                            IF V_FILINECE > 0 THEN
                              SELECT NVL(SUM(V_PREV.QTPREVCONS * P.VRFATOCONV), 0) QUANTIDADE
                              INTO V_QTCONSUMOANTECIPADOFILNECE
                              FROM SF_V_PREVISAO_COMPRA_FILIAL V_PREV, PRODUTO P
                              WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                                AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                                AND P.CDPRODUTO       = V_PREV.CDPRODUTO
                                AND V_PREV.NRORG      = P_NRORG
                                AND P.NRORG           = P_NRORG
                                AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA AND (V_DTENTREGA + V_NRDIASEG - 1);
                            ELSE
                              V_QTCONSUMOANTECIPADOFILNECE:= 0;
                            END IF;
                          EXCEPTION
                            WHEN NO_DATA_FOUND THEN
                              V_QTCONSUMOANTECIPADOFILNECE:= 0;
                          END;
                          V_QTCONSUMOANTECIPADO:= V_QTCONSUMOANTECIPADO + V_QTCONSUMOANTECIPADOFILNECE;
                       END IF;

                       -- BUSCA A QUANTIDADE A SER ENTREGUE.
                       -- V_QTASERENTREG
                       BEGIN
                         SELECT NVL(SUM(V_PREV.QTPREVCONS * V_FATORCONVERSAO), 0) QUANTIDADE
                         INTO V_QTASERENTREGUE
                         FROM SF_V_PREVISAO_COMPRA V_PREV
                         WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                           AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                           AND V_PREV.NRORG      = P_NRORG
                           AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S)
                           AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA + V_NRDIASEG AND (V_DTANTPROXENT + V_NRDIASEGANT);
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_QTASERENTREGUE:= 0;
                       END;

                       BEGIN
                         IF V_FILINECE > 0 THEN
                           SELECT NVL(SUM(V_PREV.QTPREVCONS * P.VRFATOCONV), 0) QUANTIDADE
                           INTO V_QTASERENTREGUEFILNECE
                           FROM SF_V_PREVISAO_COMPRA_FILIAL V_PREV, PRODUTO P
                           WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                             AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                             AND P.CDPRODUTO       = V_PREV.CDPRODUTO
                             AND V_PREV.NRORG      = P_NRORG
                             AND P.NRORG           = P_NRORG
                             AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S)
                             AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA + V_NRDIASEG AND (V_DTANTPROXENT + V_NRDIASEGANT);
                         ELSE
                            V_QTASERENTREGUEFILNECE:= 0;
                         END IF;
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_QTASERENTREGUEFILNECE:= 0;
                       END;
                       V_QTASERENTREGUE:= V_QTASERENTREGUE + V_QTASERENTREGUEFILNECE;

                       IF V_QTCONSUMOPERIODO > 0 THEN
                         V_QTCONSUMOPERIODO:= V_QTCONSUMOPERIODO * (1 + (P_PEACRESCIMO / 100));
                       END IF;

                       IF V_QTCONSUMOANTECIPADO > 0 THEN
                         V_QTCONSUMOANTECIPADO:= V_QTCONSUMOANTECIPADO * (1 + (P_PEACRESCIMO / 100));
                       END IF;

                       IF V_QTASERENTREGUE > 0 THEN
                         V_QTASERENTREGUE:= V_QTASERENTREGUE * (1 + (P_PEACRESCIMO / 100));
                       END IF;

                     END IF;

                     SELECT MAX(DTENTREGA)
                     INTO V_DTMAXENTREGA
                     FROM CALECOMP
                     WHERE CDFILIAL   = V_FILIAL_CORRENTE
                       AND CDPERIENTR = V_CDPERIENTR
                       AND DTENTREGA  < V_DTENTREGA
                       AND NRORG      = P_NRORG;

                     V_DTMAXENTREGA_ADD1 := V_DTMAXENTREGA;
                     IF V_IDSISDOS = W_3 THEN
                       IF V_DTPRIMEIRAENTREGA IS NULL THEN
                          V_DTMAXENTREGA:= V_DTENTREGA - 1;
                       END IF;
                     END IF;

                     -- PROCESSO EXCLUSIVO GR OU SE O PARAMETRO IDCONSAFNECCMP ESTIVER MARCADO
                     IF (V_IDSISDOS = W_3) OR (V_IDCONSAFNECCMP = W_S) THEN

                       V_DTMAXENTREGA_ADD1 := V_DTMAXENTREGA;

                       IF (V_IDENTRADAFORAPERIODO = W_N) THEN
                         BEGIN
                           V_DTMAXENTREGA_ADD1 := V_DTMAXENTREGA_ADD1 + 1;
                           /* "AF EM ABERTO" - APARTIR DA PRIMEIRA ENTREGA CALCULADA */
                           -- V_QTJASOLICITA
                           SELECT NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                           INTO V_QTJASOLICITADA
                           FROM AUTOFORNE, ITEMAUTF, PRODUTO
                           WHERE ITEMAUTF.CDFILDEST  = AUTOFORNE.CDFILDEST
                             AND ITEMAUTF.CDFILLANC  = AUTOFORNE.CDFILLANC
                             AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN
                             AND ITEMAUTF.CDPRODUTO  = PRODUTO.CDPRODUTO
                             AND ITEMAUTF.CDFILDEST  = V_FILIAL_CORRENTE
                             AND PRODUTO.CDPRODESTO  = V_CODPRODUTO
                             AND AUTOFORNE.NRORG     = P_NRORG
                             AND ITEMAUTF.NRORG      = P_NRORG
                             AND PRODUTO.NRORG       = P_NRORG
                             AND (ITEMAUTF.IDBAIXITMAF IS NULL OR ITEMAUTF.IDBAIXITMAF = W_1)
                             AND QTPROGENTAF - NVL(QTEFETENTRAF, 0) > 0
                             AND NVL(AUTOFORNE.IDBAIXAAF, W_0) NOT IN (W_4, W_5, W_6)
                             AND AUTOFORNE.IDAFFORALIM = W_N
                             AND AUTOFORNE.IDSITUATUAAF IS NULL
                             AND ITEMAUTF.DTPROGENTAF BETWEEN V_DTMAXENTREGA_ADD1 AND V_DTENTREGA
                             AND NVL(AUTOFORNE.IDAFEXPIRADA, W_N) = W_N
                             AND ITEMAUTF.DTPROGENTAF IN (SELECT CALECOMP.DTENTREGA
                                                          FROM PERFILPROD, CALECOMP, PRODUTO
                                                          WHERE PERFILPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                                                            AND PERFILPROD.CDFILIAL  = V_FILIAL_CORRENTE
                                                            AND CALECOMP.CDFILIAL    = PERFILPROD.CDFILIAL
                                                            AND CALECOMP.CDPERIENTR  = PERFILPROD.CDPERIENTR
                                                            AND PRODUTO.CDARVPROD    = V_PRODUTOPERFILPROD
                                                            AND PERFILPROD.NRORG     = P_NRORG
                                                            AND CALECOMP.NRORG       = P_NRORG
                                                            AND PRODUTO.NRORG        = P_NRORG
                                                            AND CALECOMP.DTENTREGA BETWEEN V_DTPROCESS AND ADD_MONTHS(P_DTFINAL,1) );

                           SELECT NVL(SUM((QTPROGENPEOL - NVL(QTEFETENPEOL,0)) * PRODUTO.VRFATOCONV), 0)
                           INTO V_QTJASOLICPEXOL
                           FROM PEDIDOEXOL, ITEMPEDEXOL, PRODUTO
                           WHERE ITEMPEDEXOL.CDFILDEST = PEDIDOEXOL.CDFILDEST
                             AND ITEMPEDEXOL.CDFILLANC = PEDIDOEXOL.CDFILLANC
                             AND ITEMPEDEXOL.NRPEDEXOL = PEDIDOEXOL.NRPEDEXOL
                             AND ITEMPEDEXOL.CDPRODUTO = PRODUTO.CDPRODUTO
                             AND ITEMPEDEXOL.CDFILDEST = V_FILIAL_CORRENTE
                             AND PEDIDOEXOL.NRORG      = P_NRORG
                             AND ITEMPEDEXOL.NRORG     = P_NRORG
                             AND PRODUTO.NRORG         = P_NRORG
                             AND PRODUTO.CDPRODESTO = V_CODPRODUTO
                             AND (ITEMPEDEXOL.IDBAIXENPEOL IS NULL OR ITEMPEDEXOL.IDBAIXENPEOL = W_1)
                             AND QTPROGENPEOL - NVL(QTEFETENPEOL, 0) > 0
                             AND NVL(PEDIDOEXOL.IDBAIXEXOL, W_0) NOT IN (W_4, W_5)
                             AND ITEMPEDEXOL.DTPROGENPEOL BETWEEN V_DTMAXENTREGA_ADD1 AND V_DTENTREGA
                             AND ITEMPEDEXOL.DTPROGENPEOL IN (SELECT CALECOMP.DTENTREGA
                                                              FROM PERFILPROD, CALECOMP, PRODUTO
                                                              WHERE PERFILPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                                                                AND PERFILPROD.CDFILIAL  = V_FILIAL_CORRENTE
                                                                AND CALECOMP.CDFILIAL    = PERFILPROD.CDFILIAL
                                                                AND CALECOMP.CDPERIENTR  = PERFILPROD.CDPERIENTR
                                                                AND PRODUTO.CDARVPROD    = V_PRODUTOPERFILPROD
                                                                AND PERFILPROD.NRORG     = P_NRORG
                                                                AND CALECOMP.NRORG       = P_NRORG
                                                                AND PRODUTO.NRORG        = P_NRORG
                                                                AND CALECOMP.DTENTREGA BETWEEN V_DTPROCESS AND ADD_MONTHS(P_DTFINAL,1) );

                             V_QTJASOLICITADA := V_QTJASOLICITADA + V_QTJASOLICPEXOL;
                         EXCEPTION
                           WHEN NO_DATA_FOUND THEN
                             V_QTJASOLICITADA:= 0;
                         END;
                       ELSE
                          /* "AF EM ABERTO" - APARTIR DA PRIMEIRA ENTREGA CALCULADA */
                           -- V_QTJASOLICITA
                          BEGIN
                            SELECT NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                            INTO V_QTJASOLICITADA
                            FROM AUTOFORNE, ITEMAUTF, PRODUTO
                            WHERE ITEMAUTF.CDFILDEST    = V_FILIAL_CORRENTE
                              AND PRODUTO.CDPRODESTO    = V_CODPRODUTO
                              AND ITEMAUTF.DTPROGENTAF  = V_DTENTREGA
                              AND ITEMAUTF.CDFILDEST    = AUTOFORNE.CDFILDEST
                              AND ITEMAUTF.CDFILLANC    = AUTOFORNE.CDFILLANC
                              AND ITEMAUTF.NRAUTOFORN   = AUTOFORNE.NRAUTOFORN
                              AND ITEMAUTF.CDPRODUTO    = PRODUTO.CDPRODUTO
                              AND AUTOFORNE.NRORG       = P_NRORG
                              AND ITEMAUTF.NRORG        = P_NRORG
                              AND PRODUTO.NRORG         = P_NRORG
                              AND AUTOFORNE.IDAFFORALIM = W_N
                              AND (ITEMAUTF.IDBAIXITMAF IS NULL OR ITEMAUTF.IDBAIXITMAF = W_1)
                              AND QTPROGENTAF - NVL(QTEFETENTRAF, 0) > 0
                              AND NVL(AUTOFORNE.IDBAIXAAF, W_0) NOT IN (W_4, W_5, W_6)
                              AND NVL(AUTOFORNE.IDAFEXPIRADA, W_N) = W_N
                              AND AUTOFORNE.IDSITUATUAAF IS NULL;
                          EXCEPTION
                            WHEN NO_DATA_FOUND THEN
                              V_QTJASOLICITADA:= 0;
                          END;
                       END IF;
                        /* FIM - CALCULO PELA AF - UTILIZADO SOMENTE NA GR */
                     ELSE
                       /* CALCULO POR SOLICITAES. UTILIZADO PELO RESTANTE DOS CLIENTES. */
                       -- V_QTJASOLICITA
                       BEGIN
                         V_DTMAXENTREGA_ADD1 := V_DTMAXENTREGA_ADD1 + 1;
                         SELECT NVL(SUM(QTSOLIPROD * PRODUTO.VRFATOCONV), 0)
                         INTO V_QTJASOLICITADA
                         FROM SOLICITA, ITEMSOLI, PRODUTO
                         WHERE PRODUTO.CDPRODESTO   = V_CODPRODUTO
                           AND ITEMSOLI.CDFILSOLI   = V_FILIAL_CORRENTE
                           AND ITEMSOLI.DTPROGENTR >= V_DTPROCESS
                           AND ITEMSOLI.CDPRODUTO   = PRODUTO.CDPRODUTO
                           AND ITEMSOLI.CDFILSOLI   = SOLICITA.CDFILSOLI
                           AND ITEMSOLI.CDFILLAN    = SOLICITA.CDFILLANCA
                           AND ITEMSOLI.NRSOLICMP   = SOLICITA.NRSOLICMP
                           AND SOLICITA.NRORG       = P_NRORG
                           AND ITEMSOLI.NRORG       = P_NRORG
                           AND PRODUTO.NRORG        = P_NRORG
                           AND NVL(SOLICITA.IDSOLEXTRA, W_N) = W_N
                           AND ITEMSOLI.DTPROGENTR BETWEEN V_DTMAXENTREGA_ADD1 AND V_DTENTREGA;
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_QTJASOLICITADA:= 0;
                       END;
                     END IF;

                     IF V_QTJASOLICITADA IS NULL THEN
                       V_QTJASOLICITADA:= 0;
                     END IF;

                     IF V_FILINECE > 0 THEN
                        BEGIN
                          SELECT MIN(DTPREVCONS)
                          INTO V_DTPRIMEIRAUTILIZAPROD
                          FROM
                             (SELECT V_PREV.DTPREVCONS
                              FROM SF_V_PREVISAO_COMPRA_FILIAL V_PREV
                              WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                                AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                                AND V_PREV.NRORG      = P_NRORG
                                AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA + V_NRDIASEG AND (V_DTANTPROXENT + V_NRDIASEGANT)
                              UNION ALL
                              SELECT V_PREV.DTPREVCONS
                              FROM SF_V_PREVISAO_COMPRA V_PREV
                              WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                                AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                                AND V_PREV.NRORG      = P_NRORG
                                AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA + V_NRDIASEG AND (V_DTANTPROXENT + V_NRDIASEGANT)
                              );
                        EXCEPTION
                          WHEN NO_DATA_FOUND THEN
                            V_DTPRIMEIRAUTILIZAPROD:= NULL;
                        END;
                     ELSE
                        BEGIN
                          SELECT MIN(V_PREV.DTPREVCONS)
                          INTO V_DTPRIMEIRAUTILIZAPROD
                          FROM SF_V_PREVISAO_COMPRA V_PREV
                          WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                            AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                            AND V_PREV.NRORG      = P_NRORG
                            AND V_PREV.DTPREVCONS BETWEEN V_DTENTREGA + V_NRDIASEG AND (V_DTANTPROXENT + V_NRDIASEGANT);
                        EXCEPTION
                          WHEN NO_DATA_FOUND THEN
                            V_DTPRIMEIRAUTILIZAPROD:= NULL;
                        END;
                     END IF;
                     IF V_EHPRIMEIRAVEZPROD = 'S' THEN
                       IF V_PRIMEIRAENTFORAPERIODO = 'S' THEN
                         V_QTESTAFPEND:= 0;
                         V_QTESTAFPANT:= 0;
                         V_QTESTAFPDEP:= 0;
                         V_QTDAFANTPRIMEIRAENT:= 0;
                       END IF;

                       IF V_DTPRIMEIRAENTREGA IS NULL THEN
                          V_QTDAFANTPRIMEIRAENT := V_QTDAFANTPRIMEIRAENT + V_QTESTOQUENECCMP;
                       END IF;

                       V_QTCONSUMOATEPENTREGA:= 0;
                       V_QTCONSUMOATEPENTREGAFILNECE:= 0;
                       V_QTESTOQUENECCMP:= 0;
                       V_QTDPRODUTOCONVCOMPRA:= 0;
                       V_QTDTEORICANECCOMPRA:= 0;
                       V_QTDESTOQUEUTILIZADO:= 0;
                       V_QTDESTOQUEATUAL:= 0;
                       V_QTDORIGINALNECCOMPRA:= 0;
                       V_QTDPRACOMPRAR:= 0;
                       V_QTESTOQUESEGURANCA:= 0;

                       -- QUANTIDADE CONSUMO ATE PRIMEIRA ENTREGA
                       -- V_QTCONATEPREN

                       IF (V_IDENTRADAFORAPERIODO = W_N) THEN
                          BEGIN
                            SELECT NVL(SUM(V_PREV.QTPREVCONS * V_FATORCONVERSAO), 0) QUANTIDADE
                            INTO V_QTCONSUMOATEPENTREGA
                            FROM SF_V_PREVISAO_COMPRA V_PREV
                            WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                              AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                              AND V_PREV.NRORG      = P_NRORG
                              AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S)
                              AND V_PREV.DTPREVCONS BETWEEN V_DTPROCESS AND (V_DTENTREGA - 1);
                          EXCEPTION
                            WHEN NO_DATA_FOUND THEN
                              V_QTCONSUMOATEPENTREGA:= 0;
                          END;

                          BEGIN
                            IF V_FILINECE > 0 THEN
                              SELECT NVL(SUM(V_PREV.QTPREVCONS * P.VRFATOCONV), 0) QUANTIDADE
                              INTO V_QTCONSUMOATEPENTREGAFILNECE
                              FROM SF_V_PREVISAO_COMPRA_FILIAL V_PREV, PRODUTO P
                              WHERE V_PREV.CDFILIAL   = V_FILIAL_CORRENTE
                                AND V_PREV.CDPRODUTO  = V_CODPRODUTO
                                AND P.CDPRODUTO       = V_PREV.CDPRODUTO
                                AND V_PREV.NRORG      = P_NRORG
                                AND P.NRORG           = P_NRORG
                                AND (V_PREV.DTPREVCONS <> V_DTPROCESS OR V_RETIRDATA = 0 OR V_IDESTOQUE <> W_S )
                                AND V_PREV.DTPREVCONS BETWEEN V_DTPROCESS AND (V_DTENTREGA - 1);
                            ELSE
                               V_QTCONSUMOATEPENTREGAFILNECE:= 0;
                            END IF;
                          EXCEPTION
                            WHEN NO_DATA_FOUND THEN
                              V_QTCONSUMOATEPENTREGAFILNECE:= 0;
                          END;
                          V_QTCONSUMOATEPENTREGA:= V_QTCONSUMOATEPENTREGA + V_QTCONSUMOATEPENTREGAFILNECE;
                          V_DTPRIMEIRAENTREGA:= V_DTENTREGA;
                       END IF;
                       IF (V_IDESTOQUE = W_S) AND (V_DTPRIMEIRAENTREGA IS NOT NULL) THEN
                          -- SE CONSIDERA O ESTOQUE
                          -- PROCESSO EXCLUSIVO GR
                          SELECT DECODE(V_IDSISDOS, W_3, 0, 1)
                          INTO V_QTDIAESTOQUE
                          FROM DUAL;

                          --V_QTESTONECCMP

                          SELECT
                             SF_CALC_ESTOQ_CONS(V_FILIAL_CORRENTE, V_CODPRODUTO, V_DTPROCESS + V_QTDIAESTOQUE, P_NRORG)
                          INTO
                             V_QTESTOQUENECCMP
                          FROM
                             DUAL;
                       END IF;

                       V_QTDESTOQUEATUAL:= V_QTESTOQUENECCMP;

                       -- V_QTESTOSEGU
                       SELECT NVL(MAX(QTESTOSEGU), 0)
                       INTO V_QTESTOQUESEGURANCA
                       FROM ESTOSEGU
                       WHERE CDFILIAL   = V_FILIAL_CORRENTE
                         AND CDPRODESTO = V_CODPRODUTO
                         AND NRORG      = P_NRORG;

                       -- PROCESSO EXCLUSIVO GR
                       IF (V_IDSISDOS = W_3) AND (V_PRIMEIRAENTFORAPERIODO = W_S) THEN
                          /* "AF PENDENTE" - ANTERIOR  PRIMEIRA ENTREGA CALCULADA, CONSIDERADO NO ESTOQUE INICIAL PARA ABATER O CONSUMO AT A PRIMEIRA ENTREGA+ANTECIPADO */
                          SELECT NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                          INTO V_QTESTAFPEND
                          FROM AUTOFORNE, ITEMAUTF, PRODUTO
                          WHERE ITEMAUTF.CDFILDEST     = V_FILIAL_CORRENTE
                            AND PRODUTO.CDPRODESTO     = V_CODPRODUTO
                            AND AUTOFORNE.IDAFFORALIM  = W_N
                            AND ITEMAUTF.CDFILDEST     = AUTOFORNE.CDFILDEST
                            AND ITEMAUTF.CDFILLANC     = AUTOFORNE.CDFILLANC
                            AND ITEMAUTF.NRAUTOFORN    = AUTOFORNE.NRAUTOFORN
                            AND ITEMAUTF.CDPRODUTO     = PRODUTO.CDPRODUTO
                            AND AUTOFORNE.NRORG        = P_NRORG
                            AND ITEMAUTF.NRORG         = P_NRORG
                            AND PRODUTO.NRORG          = P_NRORG
                            AND AUTOFORNE.IDSITUATUAAF IS NULL
                            AND (ITEMAUTF.IDBAIXITMAF IS NULL OR ITEMAUTF.IDBAIXITMAF = W_1)
                            AND ITEMAUTF.QTPROGENTAF - NVL(ITEMAUTF.QTEFETENTRAF, 0) > 0
                            AND NVL(AUTOFORNE.IDBAIXAAF, W_0) NOT IN (W_4, W_5, W_6)
                            AND NVL(AUTOFORNE.IDAFEXPIRADA, W_N) = W_N
                            AND ITEMAUTF.DTPROGENTAF BETWEEN V_DTPROCESS - 30 AND V_DTENTREGA - 1;

                          /* ENTRADA DE AF PENDENTE */
                          SELECT /*+ ordered INDEX(ITEMPRODUTO ITEMPRODUTO_01) */
                             NVL(SUM(ITEMPRODUTO.QTITEMPROD * A.VRFATOCONV), 0)
                          INTO V_QTESTAFPANT
                          FROM  NECESSIDADE_TMP A, NOTAFISCAL, ITEMPRODUTO
                          WHERE ITEMPRODUTO.CDFILIAL   = NOTAFISCAL.CDFILIAL
                            AND ITEMPRODUTO.NRLANCTONF = NOTAFISCAL.NRLANCTONF
                            AND NOTAFISCAL.CDFILIAL    = A.CDFILIAL
                            AND NOTAFISCAL.NRLANCTONF  = A.NRLANCTONF
                            AND ITEMPRODUTO.CDPRODUTO  = A.CDPRODUTO
                            AND A.NRNECECOMP           = P_NRNECECOMP
                            AND A.CDPRODESTO           = V_CODPRODUTO
                            AND NOTAFISCAL.DTENTRSAID  = V_DTPROCESS
                            AND A.NRORG                = P_NRORG
                            AND NOTAFISCAL.NRORG       = P_NRORG
                            AND ITEMPRODUTO.NRORG      = P_NRORG
                            AND A.IDENTRADAAF          = W_P;

                          /* BAIXA DE AF PENDENTE */
                          SELECT V_QTESTAFPANT + NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                          INTO V_QTESTAFPANT
                          FROM PRODUTO, ITEMAUTF
                          WHERE ITEMAUTF.CDPRODUTO   = PRODUTO.CDPRODUTO
                            AND PRODUTO.CDPRODESTO   = V_CODPRODUTO
                            AND ITEMAUTF.CDFILDEST   = V_FILIAL_CORRENTE
                            AND ITEMAUTF.DTBAIXITMAF = V_DTPROCESS
                            AND ITEMAUTF.IDBAIXITMAF = W_3
                            AND PRODUTO.NRORG        = P_NRORG
                            AND ITEMAUTF.NRORG       = P_NRORG
                            AND ITEMAUTF.DTPROGENTAF < V_DTPROCESS;

                          /* ENTRADA DE AF ANTECIPADA */
                          SELECT /*+ ordered INDEX(ITEMPRODUTO ITEMPRODUTO_01) */
                             NVL(SUM(ITEMPRODUTO.QTITEMPROD * A.VRFATOCONV), 0)
                          INTO V_QTESTAFPDEP
                          FROM NECESSIDADE_TMP A, NOTAFISCAL, ITEMPRODUTO
                          WHERE ITEMPRODUTO.CDFILIAL   = NOTAFISCAL.CDFILIAL
                            AND ITEMPRODUTO.NRLANCTONF = NOTAFISCAL.NRLANCTONF
                            AND NOTAFISCAL.CDFILIAL    = A.CDFILIAL
                            AND NOTAFISCAL.NRLANCTONF  = A.NRLANCTONF
                            AND ITEMPRODUTO.CDPRODUTO  = A.CDPRODUTO
                            AND A.NRNECECOMP           = P_NRNECECOMP
                            AND A.CDPRODESTO           = V_CODPRODUTO
                            AND NOTAFISCAL.DTENTRSAID  = V_DTPROCESS
                            AND A.NRORG                = P_NRORG
                            AND NOTAFISCAL.NRORG       = P_NRORG
                            AND ITEMPRODUTO.NRORG      = P_NRORG
                            AND A.IDENTRADAAF          = W_A;

                          /* BAIXA DE AF ANTECIPADA */
                          SELECT V_QTESTAFPDEP + NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                          INTO V_QTESTAFPDEP
                          FROM PRODUTO, ITEMAUTF
                          WHERE ITEMAUTF.CDPRODUTO   = PRODUTO.CDPRODUTO
                            AND PRODUTO.CDPRODESTO   = V_CODPRODUTO
                            AND ITEMAUTF.CDFILDEST   = V_FILIAL_CORRENTE
                            AND ITEMAUTF.DTBAIXITMAF = V_DTPROCESS
                            AND PRODUTO.NRORG        = P_NRORG
                            AND ITEMAUTF.NRORG       = P_NRORG
                            AND ITEMAUTF.IDBAIXITMAF = W_3
                            AND ITEMAUTF.DTPROGENTAF > V_DTPROCESS;

                          V_PRIMEIRAENTFORAPERIODO := W_N;
                       ELSE
                          IF (V_IDCONSAFNECCMP = W_S) AND (V_PRIMEIRAENTFORAPERIODO = W_S) THEN
                             /* "AF PENDENTE" - ANTERIOR  PRIMEIRA ENTREGA CALCULADA, CONSIDERADO NO ESTOQUE INICIAL PARA ABATER O CONSUMO AT A PRIMEIRA ENTREGA+ANTECIPADO */
                             SELECT NVL(SUM((QTPROGENTAF - NVL(QTEFETENTRAF,0)) * PRODUTO.VRFATOCONV), 0)
                             INTO V_QTESTAFPEND
                             FROM AUTOFORNE, ITEMAUTF, PRODUTO
                             WHERE ITEMAUTF.CDFILDEST     = V_FILIAL_CORRENTE
                               AND PRODUTO.CDPRODESTO     = V_CODPRODUTO
                               AND AUTOFORNE.IDAFFORALIM  = W_N
                               AND ITEMAUTF.CDFILDEST     = AUTOFORNE.CDFILDEST
                               AND ITEMAUTF.CDFILLANC     = AUTOFORNE.CDFILLANC
                               AND ITEMAUTF.NRAUTOFORN    = AUTOFORNE.NRAUTOFORN
                               AND ITEMAUTF.CDPRODUTO     = PRODUTO.CDPRODUTO
                               AND AUTOFORNE.NRORG        = P_NRORG
                               AND ITEMAUTF.NRORG         = P_NRORG
                               AND PRODUTO.NRORG          = P_NRORG
                               AND AUTOFORNE.IDSITUATUAAF IS NULL
                               AND (ITEMAUTF.IDBAIXITMAF IS NULL OR ITEMAUTF.IDBAIXITMAF = W_1)
                               AND ITEMAUTF.QTPROGENTAF - NVL(ITEMAUTF.QTEFETENTRAF, 0) > 0
                               AND NVL(AUTOFORNE.IDBAIXAAF, W_0) NOT IN (W_4, W_5, W_6)
                               AND NVL(AUTOFORNE.IDAFEXPIRADA, W_N) = W_N
                               AND ITEMAUTF.DTPROGENTAF BETWEEN V_DTPROCESS - 30 AND V_DTENTREGA - 1;

                             SELECT NVL(SUM((QTPROGENPEOL - NVL(QTEFETENPEOL,0)) * PRODUTO.VRFATOCONV), 0)
                             INTO V_QTESTAFPEXOL
                             FROM PEDIDOEXOL, ITEMPEDEXOL, PRODUTO
                             WHERE ITEMPEDEXOL.CDFILDEST = V_FILIAL_CORRENTE
                               AND PRODUTO.CDPRODESTO = V_CODPRODUTO
                               AND ITEMPEDEXOL.CDFILDEST = PEDIDOEXOL.CDFILDEST
                               AND ITEMPEDEXOL.CDFILLANC = PEDIDOEXOL.CDFILLANC
                               AND ITEMPEDEXOL.NRPEDEXOL = PEDIDOEXOL.NRPEDEXOL
                               AND ITEMPEDEXOL.CDPRODUTO = PRODUTO.CDPRODUTO
                               AND PEDIDOEXOL.NRORG      = P_NRORG
                               AND ITEMPEDEXOL.NRORG     = P_NRORG
                               AND PRODUTO.NRORG         = P_NRORG
                               AND (ITEMPEDEXOL.IDBAIXENPEOL  IS NULL OR ITEMPEDEXOL.IDBAIXENPEOL  = W_1)
                               AND ITEMPEDEXOL.QTPROGENPEOL - NVL(ITEMPEDEXOL.QTEFETENPEOL, 0) > 0
                               AND NVL(PEDIDOEXOL.IDBAIXEXOL, W_0) NOT IN (W_4, W_5)
                               AND ITEMPEDEXOL.DTPROGENPEOL BETWEEN V_DTPROCESS - 30 AND V_DTENTREGA - 1;

                             V_QTESTAFPEND := V_QTESTAFPEND + V_QTESTAFPEXOL;

                             V_PRIMEIRAENTFORAPERIODO := W_N;
                          END IF;
                       END IF;

                       IF (V_DTPRIMEIRAENTREGA IS NOT NULL) THEN
                         V_QTESTOQUENECCMP:= (V_QTDAFANTPRIMEIRAENT + V_QTESTOQUENECCMP + V_QTESTAFPEND + V_QTESTAFPANT + V_QTESTAFPDEP) - V_QTCONSUMOATEPENTREGA;
                         V_EHPRIMEIRAVEZPROD:= W_N;
                       END IF;
                     END IF; -- IF PRIMEIRA VEZ DO PRODUTO.
                     -- V_QTESTNEGATIVIMPR
                     V_QTDESTOQUENEGATIVOIMPR:= V_QTESTOQUENECCMP;

                     IF V_QTESTOQUENECCMP < 0 THEN
                       V_QTESTOQUENECCMP:= 0;
                     END IF;

                     -- V_IDESTNEGATIVO
                     V_IDESTOQUENEGATIVO:= W_N;

                     -- PROCESSO EXCLUSIVO GR
                     IF (V_IDSISDOS = W_3) THEN
                       -- V_QTESTNEGATIVCALC
                       V_QTDESTOQUENEGCALC:= V_QTESTOQUENECCMP;
                       IF (V_QTESTOQUENECCMP - V_QTCONSUMOANTECIPADO) < 0 THEN
                         V_QTESTOQUENECCMP:= 0;
                         V_IDESTOQUENEGATIVO:= W_S;
                       END IF;
                     END IF;

                     IF ((V_DTENTREGA <> V_DTPROCESS) OR (V_ENTRADADATA = 0) OR (V_IDESTOQUE <> W_S)) THEN
                       V_QTESTOQUENECCMP:= V_QTESTOQUENECCMP + V_QTJASOLICITADA;
                     END IF;

                     -- V_QTESTOUTILIZ  >>>>  V_QTDESTOQUEUTILIZADO
                     IF (V_DTENTREGA < P_DTINICIAL) OR (V_IDENTRADAFORAPERIODO = W_S) THEN
                       IF V_IDESTOQUENEGATIVO = W_N THEN -- ADRIANO NA GR
                         IF (V_QTESTOQUENECCMP - V_QTCONSUMOANTECIPADO) >= V_QTASERENTREGUE THEN
                            V_QTDESTOQUEUTILIZADO:= V_QTASERENTREGUE;
                         ELSE
                            V_QTDESTOQUEUTILIZADO:= V_QTESTOQUENECCMP - V_QTCONSUMOANTECIPADO;
                         END IF;
                       ELSE
                         IF (V_QTESTOQUENECCMP) >= V_QTASERENTREGUE THEN
                            V_QTDESTOQUEUTILIZADO:= V_QTASERENTREGUE;
                         ELSE
                            V_QTDESTOQUEUTILIZADO:= V_QTESTOQUENECCMP;
                         END IF;
                       END IF;
                     ELSE
                       IF V_IDESTOQUENEGATIVO = W_N THEN --ADRIANO NA GR
                         IF (V_QTESTOQUENECCMP - V_QTCONSUMOANTECIPADO - V_QTESTOQUESEGURANCA) >= V_QTASERENTREGUE THEN
                            V_QTDESTOQUEUTILIZADO:= V_QTASERENTREGUE;
                         ELSE
                            V_QTDESTOQUEUTILIZADO:= V_QTESTOQUENECCMP - V_QTCONSUMOANTECIPADO - V_QTESTOQUESEGURANCA;
                         END IF;
                       ELSE
                         IF (V_QTESTOQUENECCMP - V_QTESTOQUESEGURANCA) >= V_QTASERENTREGUE THEN
                            V_QTDESTOQUEUTILIZADO:= V_QTASERENTREGUE;
                         ELSE
                            V_QTDESTOQUEUTILIZADO:= V_QTESTOQUENECCMP - V_QTESTOQUESEGURANCA;
                         END IF;
                       END IF;
                     END IF;

                     IF V_QTDESTOQUEUTILIZADO < 0 THEN
                       V_QTDESTOQUEUTILIZADO:= 0;
                     END IF;

                     -- V_QTTEORNECCMP
                     V_QTDTEORICANECCOMPRA:= V_QTASERENTREGUE - V_QTDESTOQUEUTILIZADO;

                     -- V_QTORIGNECCMP
                     IF V_QTDTEORICANECCOMPRA < 0 THEN
                       V_QTDORIGINALNECCOMPRA:= 0;
                     ELSE
                       V_QTDORIGINALNECCOMPRA:= V_QTDTEORICANECCOMPRA;
                     END IF;

                     IF (V_IDENTRADAFORAPERIODO = W_N) AND (V_DTENTREGA >= P_DTINICIAL) THEN
                        IF V_QTESTOQUENECCMP < V_QTESTOQUESEGURANCA THEN
                           V_QTDORIGINALNECCOMPRA := V_QTDORIGINALNECCOMPRA + (V_QTESTOQUESEGURANCA - V_QTESTOQUENECCMP);
                        END IF;
                     END IF;

                     V_APENAS_CONSUMO := 0;
                     SELECT COUNT(*)
                     INTO V_APENAS_CONSUMO
                     FROM PRODUTO, PRODNECECOMP
                     WHERE PRODNECECOMP.CDFILIAL = V_FILIAL_CORRENTE
                       AND PRODUTO.CDPRODUTO     = PRODNECECOMP.CDPRODUTO
                       AND PRODUTO.NRORG         = P_NRORG
                       AND PRODNECECOMP.NRORG    = P_NRORG
                       AND V_CODARVPRODUTO       LIKE PRODUTO.CDARVPROD || '%';

                     -- V_QTPRACOMPRAR
                     IF V_APENAS_CONSUMO = 0 THEN
                       SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(V_QTDORIGINALNECCOMPRA, '999999999999999.99999'), -2, 2)),
                                     0, 0, 0.001) + TRUNC(V_QTDORIGINALNECCOMPRA, 3)
                       INTO V_QTDPRACOMPRAR
                       FROM DUAL;
                     ELSE
                       SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(V_QTASERENTREGUE, '999999999999999.99999'), -2, 2)),
                                     0, 0, 0.001) + TRUNC(V_QTASERENTREGUE, 3)
                       INTO V_QTDPRACOMPRAR
                       FROM DUAL;
                     END IF;

                     V_CODPRODUTOPREF:= NULL;
                     IF V_UTILIZAPRODPREF = W_S THEN
                       BEGIN
                         SELECT PRODPREFMED.CDPRODPREF
                         INTO V_CODPRODUTOPREF
                         FROM PRODPREFMED
                         WHERE PRODPREFMED.CDFILIAL  = V_FILIAL_CORRENTE
                           AND PRODPREFMED.CDPRODUTO = V_CODPRODUTO
                           AND PRODPREFMED.NRORG     = P_NRORG;
                       EXCEPTION
                         WHEN NO_DATA_FOUND THEN
                           V_CODPRODUTOPREF:= NULL;
                       END;
                       IF V_CODPRODUTOPREF IS NOT NULL THEN
                          V_PRODFILI := 0;
                          SELECT
                             COUNT(*)
                          INTO
                             V_PRODFILI
                          FROM
                             PRODFILI
                          WHERE
                             (V_CODPRODUTOPREF  = PRODFILI.CDPRODUTO) AND
                             (V_FILIAL_CORRENTE = PRODFILI.CDFILIAL)  AND
                             (PRODFILI.NRORG    = P_NRORG)            AND
                             (NVL(PRODFILI.IDPRODATIV,'A') = 'A');

                          IF (NOT(V_PRODFILI > 0)) THEN
                             INSERT INTO
                                TRABNECEINAT (NRNECCMPINAT, CDFILNECINAT, CDPRONECINAT, NRORG)
                             SELECT
                                P_NRNECECOMP,
                                V_FILIAL_CORRENTE,
                                V_CODPRODUTOPREF,
                                P_NRORG
                             FROM
                                DUAL
                             WHERE
                                NOT EXISTS(SELECT 1
                                           FROM TRABNECEINAT
                                           WHERE NRNECCMPINAT = P_NRNECECOMP        AND
                                                 CDFILNECINAT = V_FILIAL_CORRENTE   AND
                                                 NRORG        = P_NRORG             AND
                                                 CDPRONECINAT = V_CODPRODUTOPREF
                                           );
                             V_CODPRODUTOPREF:= NULL;
                          END IF;
                       END IF;
                     END IF;

                     -- V_VRARREDCMP, V_VRFATOCONV
                     SELECT VRARREDCMP, VRFATOCONV
                     INTO V_VRARREDONDACMP, V_VRFATORCONVERSAO
                     FROM PRODUTO
                     WHERE CDPRODUTO = NVL(V_CODPRODUTOPREF, V_CODPRODUTO) -- ESTA VARIVEL ESTAR SETADA COM O CODIGO DO PRODUTO OU PRODUTO PREFERENCIAL.
                       AND NRORG     = P_NRORG;

                     /* AS EXPRESSES ABAIXO FORAM FEITAS PARA ARREDONDAR UM NMERO PARA CIMA DEIXANDO 3 CASAS DECIMAIS.
                        EX:  0.00301 >> 0.004
                             0.00300 >> 0.003 */
                     IF (V_CODPRODUTOPREF IS NULL) OR (V_UTILIZAPRODPREF <> W_S) THEN
                         IF V_APENAS_CONSUMO = 0 THEN
                           IF MOD(V_QTDORIGINALNECCOMPRA, V_VRARREDONDACMP) <> 0 AND V_VRARREDONDACMP > 0 THEN
                             SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(CEIL(V_QTDORIGINALNECCOMPRA / V_VRARREDONDACMP) * V_VRARREDONDACMP, '999999999999999.99999'), -2, 2)),
                                           0, 0, 0.001) + TRUNC(CEIL(V_QTDORIGINALNECCOMPRA / V_VRARREDONDACMP) * V_VRARREDONDACMP, 3)
                             INTO V_QTDPRACOMPRAR
                             FROM DUAL;
                           END IF;
                         ELSE
                           IF MOD (V_QTASERENTREGUE, V_VRARREDONDACMP) <> 0 AND V_VRARREDONDACMP > 0 THEN
                             SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(CEIL(V_QTASERENTREGUE / V_VRARREDONDACMP) * V_VRARREDONDACMP, '999999999999999.99999'), -2, 2)),
                                           0, 0, 0.001) + TRUNC(CEIL(V_QTASERENTREGUE / V_VRARREDONDACMP) * V_VRARREDONDACMP, 3)
                             INTO V_QTDPRACOMPRAR
                             FROM DUAL;
                           END IF;
                         END IF;

                         SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(V_QTDPRACOMPRAR, '999999999999999.99999'), -2, 2)), 0, 0, 0.001) + TRUNC(V_QTDPRACOMPRAR, 3)
                         INTO V_QTDPRODUTOCONVCOMPRA
                         FROM DUAL;
                     ELSE
                         IF V_APENAS_CONSUMO = 0 THEN
                           SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR((V_QTDORIGINALNECCOMPRA / V_VRFATORCONVERSAO), '999999999999999.99999'), -2, 2)),
                                         0, 0, 0.001) + TRUNC((V_QTDORIGINALNECCOMPRA / V_VRFATORCONVERSAO), 3)
                           INTO V_QTDPRODUTOCONVCOMPRA
                           FROM DUAL;
                         ELSE
                           SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR((V_QTASERENTREGUE / V_VRFATORCONVERSAO), '999999999999999.99999'), -2, 2)),
                                         0, 0, 0.001) + TRUNC((V_QTASERENTREGUE / V_VRFATORCONVERSAO), 3)
                           INTO V_QTDPRODUTOCONVCOMPRA
                           FROM DUAL;
                         END IF;

                         IF MOD(V_QTDPRODUTOCONVCOMPRA, V_VRARREDONDACMP) <> 0 AND V_VRARREDONDACMP > 0 THEN
                           SELECT DECODE(TO_NUMBER(SUBSTR(TO_CHAR(CEIL(V_QTDPRODUTOCONVCOMPRA / V_VRARREDONDACMP) * V_VRARREDONDACMP, '999999999999999.99999'), -2, 2)),
                                         0, 0, 0.001) + TRUNC(CEIL(V_QTDPRODUTOCONVCOMPRA / V_VRARREDONDACMP) * V_VRARREDONDACMP, 3)
                           INTO V_QTDPRODUTOCONVCOMPRA
                           FROM DUAL;
                         END IF;
                     END IF;

                     IF V_CODPRODUTOPREF IS NULL THEN
                        V_CODPRODUTOPREF:= V_CODPRODUTO;
                     END IF;

                     -- V_IDMARCNECCMP
                     IF (V_DTENTREGA < P_DTINICIAL) OR (V_IDENTRADAFORAPERIODO = W_S) THEN
                       V_IDMARCNECCOMPRA:= W_N;
                     ELSE
                       SELECT MAX(DECODE(NVL(V_QTDPRODUTOCONVCOMPRA, 0), 0, W_N, W_S))
                       INTO V_IDMARCNECCOMPRA
                       FROM DUAL;
                     END IF;

                     IF TRUNC(V_QTDPRODUTOCONVCOMPRA, 3) = 0 THEN
                        V_QTDPRODUTOCONVCOMPRA:= 0;
                     END IF;

                     IF TRUNC(V_QTDPRACOMPRAR, 3) = 0 THEN
                        V_QTDPRACOMPRAR:= 0;
                     END IF;

                     IF (((V_QTDPRACOMPRAR IS NOT NULL) AND (V_QTDPRACOMPRAR > 0)) OR (P_IDQTDEZERO = W_S)) THEN
                         /* INSERE DADOS NA TABELA DE TRABALHO */
                         INSERT INTO TRABNECECOMP (CDFILNECECMP, NRNECECOMP, CDPRODNECCMP, DTENTRNECCMP,
                            DTANENNECCMP, QTCONATEPREN, QTESTOQATUAL, DTPRIUTIPROD, QTASERENTREG,
                            QTCONSPERIOD, QTCONSANTECI, QTESTONECCMP, QTESTOUTILIZ, QTJASOLICITA,
                            QTTEORNECCMP, QTORIGNECCMP, QTPRACOMPRAR, QTPRCNVCOMPR, QTESTAFPEND,
                            QTESTAFPANT,  QTESTAFPDEP,  IDMARCNECCMP, IDENTFORPERI, DTPRIMENTREG,
                            QTESTNEGATIV, NRDIASEGPROD, QTESTOSEGCAD, NRLIMISEGCAD, NRORG)
                         VALUES
                           (V_FILIAL_CORRENTE,       P_NRNECECOMP,  V_CODPRODUTOPREF,  -- FOI TROCADO PORQUE O CODIGO DO PRODUTO PREFERENCIAL ABRANGE AS DUAS VARIAVEIS.
                            V_DTENTREGA,             V_DTANTPROXENT,           V_QTCONSUMOATEPENTREGA, V_QTDESTOQUEATUAL,
                            V_DTPRIMEIRAUTILIZAPROD, V_QTASERENTREGUE,         V_QTCONSUMOPERIODO,     V_QTCONSUMOANTECIPADO,
                            V_QTESTOQUENECCMP,       V_QTDESTOQUEUTILIZADO,    V_QTJASOLICITADA,       V_QTDTEORICANECCOMPRA,
                            V_QTDORIGINALNECCOMPRA,  V_QTDPRACOMPRAR,          V_QTDPRODUTOCONVCOMPRA, V_QTESTAFPEND,
                            V_QTESTAFPANT,           V_QTESTAFPDEP,            V_IDMARCNECCOMPRA,      V_IDENTRADAFORAPERIODO,
                            V_DTPRIMEIRAENTREGA,     V_QTDESTOQUENEGATIVOIMPR, V_NRDIASEG,             V_QTESTOQUESEGURANCA,    V_NRLIMISEGCAD, P_NRORG);
                     END IF;

                     IF (V_IDEVOLESTONC = W_S) AND (V_IDMARCNECCOMPRA = W_N) THEN
                        IF V_IDESTOQUENEGATIVO = W_N THEN
                           V_QTESTOQUENECCMP:= V_QTESTOQUENECCMP - V_QTCONSUMOPERIODO;
                        ELSE
                           V_QTESTOQUENECCMP:= (V_QTESTOQUENECCMP + V_QTDESTOQUENEGCALC) - V_QTCONSUMOPERIODO;
                        END IF;
                     ELSE
                        IF V_IDESTOQUENEGATIVO = W_N THEN
                           V_QTESTOQUENECCMP:= (V_QTESTOQUENECCMP + V_QTDPRACOMPRAR) - V_QTCONSUMOPERIODO;
                        ELSE
                           V_QTESTOQUENECCMP:= (V_QTESTOQUENECCMP + V_QTDESTOQUENEGCALC + V_QTDPRACOMPRAR) - V_QTCONSUMOPERIODO;
                        END IF;
                     END IF;
                     IF (V_IDEVOLESTONC = W_S) THEN
                        IF V_QTESTOQUENECCMP < 0 THEN
                           V_QTESTOQUENECCMP:= 0;
                        END IF;
                     END IF;
                 END LOOP;
                 CLOSE C_PREV_DATA;
              ELSE
                 INSERT INTO
                    TRABNECEINAT (NRNECCMPINAT, CDFILNECINAT, CDPRONECINAT, NRORG)
                 SELECT
                    P_NRNECECOMP,
                    V_FILIAL_CORRENTE,
                    V_CODPRODUTO,
                    P_NRORG
                 FROM
                    DUAL
                 WHERE
                    NOT EXISTS(SELECT 1
                               FROM TRABNECEINAT
                               WHERE NRNECCMPINAT = P_NRNECECOMP        AND
                                     CDFILNECINAT = V_FILIAL_CORRENTE   AND
                                     CDPRONECINAT = V_CODPRODUTO        AND
                                     NRORG        = P_NRORG);
              END IF;

          END LOOP;
          CLOSE C_PREV;

          /* INSERE OS DADOS DO CLCULO NA TABELA DE LOG */
          INSERT INTO LOGNECECOMP (CDFILIAL, CDOPERADOR, DTHRCALCNC, DTINICIALNC, DTFINALNC,
                        CDPRODINICNC, CDPRODFINANC, IDUTILESTONC, PEACRESCIMNC, NRNECECMPLOG, IDGERSOLEXTNCMP, NRORG)
          VALUES (V_FILIAL_CORRENTE, P_CDOPERADOR, P_DTHRCALCNC, P_DTINICIAL, P_DTFINAL,
                  P_CDPRODINI, P_CDPRODFIN, V_IDESTOQUE, P_PEACRESCIMO, P_NRNECECOMP, V_EXTRA, P_NRORG);

          /* INSERE OS DADOS REFERENTES AO MAPA DE DISTRIBUIO POR ALMOXARIFADO DO VND */
          IF V_IDMAPDISTALMOX = 'S' THEN
             FOR C_PREVISAOVND IN
                (SELECT
                   PC_PDV.CDFILIAL,
                   PDV.CDLOJA,
                   P.CDPRODESTO AS CDPRODUTO,
                   LOJA.CDALMOXARIFE,
                   LOJA.CDLOCALESTOQ,
                   SUM(PC_PDV.QTPRODCONPDV*P.VRFATOCONV) AS QTPREVCONS
                 FROM PREVISAOPDV PDV, PREVCONSPDV PC_PDV, PRODUTO P, LOJA
                 WHERE PDV.NRSEQPREVPDV = PC_PDV.NRSEQPREVPDV
                  AND PC_PDV.CDPRODUTO = P.CDPRODUTO
                  AND PC_PDV.CDFILIAL = V_FILIAL_CORRENTE
                  AND PDV.CDFILIAL = V_FILIAL_CORRENTE
                  AND PDV.NRORG    = P_NRORG
                  AND PC_PDV.NRORG = P_NRORG
                  AND P.NRORG      = P_NRORG
                  AND LOJA.NRORG   = P_NRORG
                  AND TRUNC(PDV.DTPREVCONPDV) BETWEEN V_DTPROCESS AND P_DTFINAL
                  AND PC_PDV.CDPRODUTO = P.CDPRODUTO
                  AND P.CDARVPROD BETWEEN V_CDPRODINI AND V_CDPRODFIN
                  AND LOJA.CDFILIAL = V_FILIAL_CORRENTE
                  AND LOJA.CDLOJA = PDV.CDLOJA
                 GROUP BY
                   PC_PDV.CDFILIAL,
                   PDV.CDLOJA,
                   P.CDPRODESTO,
                   LOJA.CDALMOXARIFE,
                   LOJA.CDLOCALESTOQ)
             LOOP
                V_QTESTOQUEVND := 0;
                SELECT
                   SF_CALC_ESTOQ_CONS(V_FILIAL_CORRENTE, V_CODPRODUTO, V_DTPROCESS + V_QTDIAESTOQUE, P_NRORG)
                INTO
                   V_QTESTOQUENECCMP
                FROM
                   DUAL;
                INSERT INTO MAPDISTALMOX (CDFILIAL, NRNECECMPMAP, CDLOJA,
                                          CDPRODUTO, DTINIMAPDIST, DTFINMAPDIST,
                                          QTPREVCONLOJ, QTESTOQUELOJ, NRORG)
                VALUES(V_FILIAL_CORRENTE, P_NRNECECOMP, C_PREVISAOVND.CDLOJA,
                       C_PREVISAOVND.CDPRODUTO, P_DTINICIAL, P_DTFINAL,
                       C_PREVISAOVND.QTPREVCONS, V_QTESTOQUEVND, P_NRORG);
             END LOOP;
          END IF;

          IF V_IDSISDOS = W_3 THEN
             DELETE NECESSIDADE_TMP
             WHERE NRNECECOMP = P_NRNECECOMP
               AND NRORG = P_NRORG;
          END IF;
          COMMIT;
    END LOOP; -- LOOP DA FILIAL
    CLOSE CUR_FILIAL;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SF_NECESSIDADE_DE_COMPRAS','5.07.008'); END;
/

