// FILE: ./assets/js/utilities/PCPUtil.js
/* global Util angular Configuration moment */
var PCPUtil = function(templateManager, MessageService, ZHPromise, InflightorGeneral, ScreenService) {
    const parent = this;
    var trataNumerico = function(value, decimalSeparator) {
        if (typeof value == "number") {
            return value.toString().split(".").join(decimalSeparator);
        }
        return (value) ? value.toString() : value + '';
    };

    /*
     * Usar no FieldOnKeydown
     *
     */

    this.lengthCurrency = function(args) {
        var value;
        var fieldValue;
        var field = args.owner.field;
        var allowedKeys = ['Tab', 'ArrowLeft', 'ArrowRight', 'ArrowUp', 'ArrowDown', 'End', 'Home', 'Shift', 'Control', 'Delete', 'Backspace', 'Alt'];

        let decimalSeparator = ',';
        let thousandSeparator = '.';

        if (field.mask && field.mask.params && field.mask.params.decimal) {
            decimalSeparator = field.mask.params.decimal;
        }

        if (field.inlineEditable && (field.widget.mode === "list" || field.widget.mode === "inline_edit")) {
            fieldValue = trataNumerico(field.widget.currentRow[field.name + '_inline'], decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        } else {
            fieldValue = trataNumerico(field.value(), decimalSeparator);
            fieldValue = fieldValue.toString();
            if (event.target.selectionStart !== event.target.selectionEnd) {
                fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
            }
            value = Util.isEmptyOrBlank(fieldValue) ? "" : fieldValue;
        }

        fieldValue = fieldValue.toString();
        if (event.target.selectionStart !== event.target.selectionEnd) {
            fieldValue = fieldValue.substring(0, event.target.selectionStart) || fieldValue.substring(event.target.selectionEnd);
        }

        let posInsertChar = event.target.selectionStart;

        var realLength = value.split('-').join('').split('.').join('').split(',')[0].length;
        var last = Math.max(...[value.lastIndexOf(','), value.lastIndexOf('.')]);
        var separador;
        var decimalLength = value.split('-').join('').split('.').join('').split(',')[1] ? value.split('-').join('').split('.').join('').split(',')[1].length : 0;
        var precisionLength = 0;

        if (field.mask && field.mask.params && field.mask.params.precision) {
            precisionLength = field.mask.params.precision;
        }

        if (last > -1) {
            separador = value.substring(0, last).length >= field.maxLengthCurrency - precisionLength ? ',' : value[last];
            value = value.substr(0, last).concat(separador + value.substr(last + 1, value.length));
            value = value.split('.').join('');
            last = value.lastIndexOf(',');
            realLength = value.substr(0, last).length;
            decimalLength = value.substr(last + 1, value.length).length;
        }

        let isDecimalSeparator = (args.e.key === decimalSeparator);
        let allowDecimalSeparator = ((precisionLength > 0) && (!value.includes(decimalSeparator)));
        let posDecimalseparator = fieldValue.indexOf(decimalSeparator);
        if (isDecimalSeparator && !allowDecimalSeparator) {
            args.e.preventDefault();
        }

        if (field.mask && field.mask.params && field.mask.params.thousands) {
            thousandSeparator = field.mask.params.thousands;
        }
        let isThousandSeparator = (args.e.key === thousandSeparator);

        if (isThousandSeparator) {
            args.e.preventDefault();
        }

        let allowNegative = true;
        if (field.mask && field.mask.params) {
            allowNegative = field.mask.params.allowNegative;
        }
        allowNegative = (allowNegative && !value.includes('-'));
        let isNegativeSimbol = (args.e.key === '-');

        if (isNegativeSimbol && (!allowNegative || (event.target.selectionStart > 0))) {
            args.e.preventDefault();
        }
        if (!allowedKeys.includes(args.e.key) && !isNegativeSimbol && !isDecimalSeparator && !isThousandSeparator) {
            if (realLength >= field.maxLengthCurrency) {
                if ((posDecimalseparator < 0) || (posInsertChar <= posDecimalseparator))
                    args.e.preventDefault();
            }

            if ((posDecimalseparator >= 0) && (posInsertChar > posDecimalseparator) && (precisionLength == decimalLength)) {
                args.e.preventDefault();
            }
        }
    };

    this.blockDependentField = function(field, nameFields, modifyValidations = false, treatFirstNode = true) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        const widget = field.widget;
        let firstNode = nameFields[0];
        const fieldValue = field.value();
        let fieldFirstNode = widget.getField(firstNode);
        nameFields.forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                cField.readOnly = true;
                if (modifyValidations) changeValidations(cField);
            }
        });
        if (fieldFirstNode && treatFirstNode) {
            fieldFirstNode.readOnly = !fieldValue;
            if (modifyValidations) changeValidations(fieldFirstNode);
        }
        templateManager.updateTemplate();
    };

    function changeValidations(field) {
        field.validations = field.readOnly ? {} : {
            required: true
        };
    }

    this.updateFilterFields = function(fieldsUpdate, nameFilters, rowToSet, reloadField = false, widget = null) {
        nameFilters = typeof nameFilters === 'string' ? [nameFilters] : nameFilters;
        fieldsUpdate = Array.isArray(fieldsUpdate) ? fieldsUpdate : [fieldsUpdate];
        reloadField = typeof reloadField == 'string' ? [reloadField] : reloadField;
        fieldsUpdate.forEach(cField => {
            cField = typeof cField == 'string' ? widget.getField(cField) : cField;
            if (cField && cField.dataSourceFilter) {
                let oldDataSourceFilter = angular.copy(cField.dataSourceFilter);
                cField.dataSourceFilter.forEach(filter => {
                    if (nameFilters.indexOf(filter.name) > -1) {
                        filter.value = rowToSet[filter.name];
                    }
                });
                if (reloadField == true || (Array.isArray(reloadField) && reloadField.indexOf(cField.name) > -1)) this.reloadField(cField, oldDataSourceFilter);
            }
        });
    };

    this.reloadField = function(field, oldDataSourceFilter = null) {
        let reloaded = false;
        if (field) {
            if (field.dataSourceFilter && oldDataSourceFilter) {
                if (!angular.equals(field.dataSourceFilter, oldDataSourceFilter)) {
                    field.reload();
                    reloaded = true;
                }
            } else {
                field.reload();
                reloaded = true;
            }
        }
        return reloaded;
    };

    this.getValuesInRow = function(widget, fieldAndPropValue, emptyProps = [], emptyValue = '') { // {FIELDNAME: NAME_GET_ROW}
        let result = {};
        emptyProps = Array.isArray(emptyProps) ? emptyProps : [emptyProps];
        Object.keys(fieldAndPropValue).forEach(nameField => {
            let cField = widget.getField(nameField);
            if (cField) {
                let valueFields = Array.isArray(fieldAndPropValue[nameField]) ? fieldAndPropValue[nameField] : [fieldAndPropValue[nameField]];
                let fieldValue = cField.value();
                valueFields.forEach(valueField => {
                    result[valueField] = fieldValue ? widget.currentRow[valueField] : '';
                });
            }
        });
        emptyProps.forEach(prop => {
            result[prop] = emptyValue;
        });
        return result;
    };

    this.clearDataSourceFilter = function(widget, fields, fieldsToReload = [], flagIgnoreValueInFilter = 'T') {
        widget = typeof widget === 'string' ? templateManager.container.getWidget(widget) : widget;
        fieldsToReload = typeof fieldsToReload === 'string' ? [fieldsToReload] : fieldsToReload;
        fields = Array.isArray(fields) ? fields : [fields];
        fields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field && field.dataSourceFilter && field.dataSourceFilter.length) {
                field.dataSourceFilter.forEach(filter => {
                    if (flagIgnoreValueInFilter !== false) {
                        filter.value = filter.value === flagIgnoreValueInFilter ? filter.value : '';
                    } else {
                        filter.value = "";
                    }
                });
                if (fieldsToReload.indexOf(field.name) > -1) field.reload();
            }
        });
    };

    this.clearSelectMultiple = function(field) {
        let widget = field.widget;
        let row = widget.currentRow;
        if (field.valueField && field.descriptionField) {
            let {
                valueField,
                descriptionField
            } = field;
            row[valueField] = [];
            row[descriptionField] = [];
        }
    };

    function changeFieldCheckbox(widget, field, nameFieldCheckbox) {
        let fieldCheckbox = widget.getField(nameFieldCheckbox);
        let row = widget.currentRow;
        if (fieldCheckbox) {
            row[nameFieldCheckbox] = field.readOnly ? fieldCheckbox.valueToTrue : fieldCheckbox.valueToFalse;
            fieldCheckbox.checked = field.readOnly;
            changeValidations(field);
        }
    }

    this.defaultClearField = function(field) {
        let widget = field.widget;
        let row = widget.currentRow;
        let valueToSet = "";
        if (field.valueField && field.descriptionField) {
            let {
                valueField,
                descriptionField
            } = field;
            row[valueField] = valueToSet;
            row[descriptionField] = valueToSet;
        } else if (field.outData) {
            Object.values(field.outData).forEach(itemRow => {
                row[itemRow] = valueToSet;
            });
        } else if (field.valueFields) {
            field.valueFields.forEach(itemRow => {
                row[itemRow] = valueToSet;
            });
        }
    };

    this.clearFields = function(widget, nameFields = [], fieldNameAndCheckbox = null) {
        let functionByTemplate = Object.freeze({
            "select-multiple.html": this.clearSelectMultiple,
            default: this.defaultClearField
        });
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        nameFields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field) {
                let functionClearField = Object.keys(functionByTemplate).find(template => {
                    return field.template.indexOf(template) > -1;
                });
                functionClearField = functionByTemplate[functionClearField] || functionByTemplate.default;
                functionClearField(field);
                if (fieldNameAndCheckbox) {
                    if (typeof fieldNameAndCheckbox == 'string') {
                        changeFieldCheckbox(widget, field, fieldNameAndCheckbox);
                    } else if (typeof fieldNameAndCheckbox == 'object' && !Array.isArray(fieldNameAndCheckbox)) {
                        if (fieldNameAndCheckbox[field.name])
                            changeFieldCheckbox(widget, field, fieldNameAndCheckbox[field.name]);
                    }
                }
            }
        });
        templateManager.updateTemplate();
    };

    function getDatesInRow(widget, fieldStartDate, fieldEndDate) {
        let row = widget.currentRow;
        return {
            startDate: row[fieldStartDate.name],
            endDate: row[fieldEndDate.name]
        };
    }

    function isValidRange(startDate, endDate) {
        return !moment(endDate, 'DD/MM/YYYY').isBefore(moment(startDate, 'DD/MM/YYYY'));
    }

    this.onChangeDate = function(args, fieldStartDate, fieldEndDate, showMessage = true) {
        const currentField = args.owner.field;
        const widget = currentField.widget;
        fieldStartDate = typeof fieldStartDate == 'string' ? widget.getField(fieldStartDate) : fieldStartDate;
        fieldEndDate = typeof fieldEndDate == 'string' ? widget.getField(fieldEndDate) : fieldEndDate;
        if (fieldStartDate && fieldEndDate) {
            if (!fieldStartDate.value() || !fieldEndDate.value()) return;
            let {
                startDate,
                endDate
            } = getDatesInRow(widget, fieldStartDate, fieldEndDate);
            let isValidRangeDate = isValidRange(startDate, endDate);
            if (!isValidRangeDate) {
                fieldEndDate.clearValue(fieldEndDate);
                if (showMessage) MessageService.showMessageByCode("MSG_PCP_DATAINICIAL_MENOR_DATAFINAL");
                return false;
            }
        }
    };

    this.getReportFormats = function() {
        return [
            {
                "name": "pdf",
                "value": "PDF"
            },
            {
                "name": "xlsx",
                "value": "XLSX"
            }
        ];
    };

    this.widgetSave = function(widget, afterValid) {
        var deferred = ZHPromise.defer();
        widget.currentRow = widget.currentRow || {};

        if (widget.isValid()) {
            if (afterValid && angular.isObject(afterValid)) {
                if (angular.isFunction(afterValid.callback))
                    afterValid.callback.apply(null, afterValid.params);
            }

            if (!widget.currentRow.__is_new) {
                widget.currentRow.__is_new = false;
            }

            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                InflightorGeneral.applyParamsNewCodeToRow(widget);
            }

            var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
            if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
                InflightorGeneral.setDuplicationValidateParamsToCurrentRow(widget);
            }

            widget.currentRow.changed = true;

            widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                delete widget.backupRow;

                if (InflightorGeneral.formWidgetSwitchTemplate(widget) && widget.parent) {
                    widget.view();

                    var widgets = widget.parent.widgets;
                    InflightorGeneral.changeWidgetAcessibility(widgets.concat(widget.parent), true);
                } else {
                    InflightorGeneral.changeWidgetAcessibility(widget.widgets, true);
                }

                if (!widget.currentRow.__is_new) {
                    widget.computeDescriptionOrder();
                    widget.view();
                    deferred.resolve(response);
                }

                widget.reload();

                deferred.resolve(response);
            }.bind(this), function(error) {
                parent.treatAndShowError(error);

                var errorMessage = (error.data) ? 'Timeout' : error.data.error;
                deferred.reject(errorMessage);
            });
        } else {
            deferred.reject("Preencha todos os campos corretamente!");
        }

        return deferred;
    };

    this.widgetGoBackToParent = function(widget) {
        ScreenService.closeSwipe().then(() => {
            widget.parent.list();
            widget.parent.activate();
            widget.reload();
            widget.parent.reload();
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("PCPUtil", PCPUtil);
});

// FILE: ./assets/js/services/MalhaDeVooService.js
var MalhaDeVooService = function(ScreenService, InflightorGeneral, restEngine) {

    this.loadDataVoos = function(widget, filtro) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereadata_equipcia_vootrecho', "FilterData", filtro).then(function(callback) {
            widget.dataSource.data = callback.dataset.malhaaereadata_equipcia_vootrecho;
            return widget.dataSource.data;
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };

    this.loadVoos = function(widget, filtro) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereavoo_equipcia_vootrecho', "FilterData", filtro).then(function(callback) {
            widget.dataSource.data = callback.dataset.malhaaereavoo_equipcia_vootrecho;
            return widget.dataSource.data;
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };

    this.deletaData = function(row) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereadata/remove', "Row", row).then(function(success) {
            return ScreenService.showMessage(success.messages[0].message);
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };

    this.deletaClasse = function(row) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereaclasse/remove', "Row", row).then(function(success) {
            return ScreenService.showMessage(success.messages[0].message);
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };

    this.deletaDataVoo = function(row) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereadata/remove', "Row", row).then(function(success) {
            return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereavoo/remove', "Row", row).then(function(success) {
                return ScreenService.showMessage(success.messages[0].message);
            }, function(error) {
                return ScreenService.showMessage(error.data.error);
            });
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };

    this.loadClassesVoo = function(widget, filtro) {
        return InflightorGeneral.callRestEngine('pcp#/crud/malhaaereaclasse_classecia', "FilterData", filtro).then(function(callback) {
            widget.dataSource.data = callback.dataset.malhaaereaclasse_classecia;
            return widget.dataSource.data;
        }, function(error) {
            return ScreenService.showMessage(error.data.error);
        });
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('MalhaDeVooService', MalhaDeVooService);
});

// FILE: ./assets/js/services/pcp00007_fluxoprod_receita_service.js
var pcp00007_fluxoprod_receita_service = function(restEngine) {
    this.getNrtempoprep = function(cdfilial, cdreceita) {
        return restEngine.post("pcp#/crud/filireceita_variacaorec", {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "IS NULL OR CDFILIAL =",
                    "value": cdfilial
        },
                {
                    "name": "CDRECEITA",
                    "operator": "=",
                    "value": cdreceita
        },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
        }
      ]
        });
    };
    this.save = function(row) {
        return restEngine.post("pcp#/fluxprodrec/save", {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('pcp00007_fluxoprod_receita_service', pcp00007_fluxoprod_receita_service);
});

// FILE: ./assets/js/services/pcpReportService.js
var pcpReportService = function(General, ScreenService) {

    this.openReport = function(widget, reportFormat, row = null) {
        if (widget.isValid()) {
            if (!row) {
                row = widget.currentRow;
            }
            row.REPORTFORMAT = reportFormat;
            General.openReport('pcp#/relatorioProducaoReceita', row.REPORTNAME, row).then(function(callback) {
                ScreenService.closePopup();
            }, General.callbackError);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcpReportService', pcpReportService);
});

// FILE: ./assets/js/services/PrevisaoPassageirosProducaoService.js
var PrevisaoPassageirosProducaoService = function(restEngine) {

    this.callProcedure = function(row) {
        return restEngine.request("pcp#/gera_prevpax_prod", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 9000000);
    };

    this.callProcedurePax = function(data) {
        return restEngine.request("pcp#/gera_prevpaxcla_prod", {
            "requestType": "DataSet",
            "dataset": data
        }, null, null, null, null, 9000000);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('PrevisaoPassageirosProducaoService', PrevisaoPassageirosProducaoService);
});

// FILE: ./assets/js/controllers/AtendimentoRequisicao.js
var AtendimentoRequisicao = function(ScreenService, InflightorGeneral, templateManager, ZhFilterPreferences, MessageService) {

    var self = this;

    var NRRETIRADAINF;
    var CDFILIAL;
    var SETOR;
    var TURNO;

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00034_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    MessageService.showMessageByCode(mensagem);
                }
            }
        }
    };

    this.getProdutosFilter = function() {
        return NRRETIRADAINF;
    };

    this.setCDFILIAL = function(widget) {
        CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL || getLocalVar('INFLIGHTOR_CDFILIAL');
    };

    this.saveAtentimentoRequisicao = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.IDORIGEM = "MANUAL";

            if (widget.currentRow.HRPREFERENCIAL) {
                widget.currentRow.HRPREFERENCIAL = widget.currentRow.HRPREFERENCIAL.split(':');
                widget.currentRow.HRPREFERENCIAL = widget.currentRow.HRPREFERENCIAL[0] + widget.currentRow.HRPREFERENCIAL[1];
            } else {
                widget.currentRow.HRPREFERENCIAL = null;
            }

            widget.paramsNewCode.__new_code_pk = '';
            widget.paramsNewCode.__new_code_pk = 'RETIRADAINF' + widget.currentRow.CDFILIAL;
            InflightorGeneral.saveCrud(widget).then(function(success) {
                self.toggleTabVisibility(widget.widgets);
            });
        } else {
            MessageService.showMessageByCode("PCP00034_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.saveNewProduto = function(widget) {
        widget.currentRow.NRRETIRADAINF = NRRETIRADAINF;
        widget.currentRow.CDFILIAL = CDFILIAL;
        if (parseFloat(widget.currentRow.QTPRODUTOINF) >= 0) {
            InflightorGeneral.saveCrud(widget);
        } else {
            MessageService.showMessageByCode("PCP00034_QT_DEVE_SER_MAIOR_Q_ZERO");
        }
    };

    this.toggleTabVisibility = function(widget) {
        widget.map(function(wid) {
            if (wid.isVisible === true) {
                wid.isVisible = false;
            } else {
                wid.isVisible = true;
            }
        });
        templateManager.updateTemplate();
    };

    this.addAtendimentoRequisicao = function(widget) {
        widget.edit();
        widget.activate();
        widget.newRow();

        widget.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widget.getField('NMFILIAL').setValue(getLocalVar('INFLIGHTOR_NMFILIAL'));
        widget.getField('DTRETIRADAINF').setValue(widget.appliedFilter.FILTER_ROW.DTRETIRADAINF);

        ScreenService.openSwipe(widget);
    };

    this.createNewProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            widget.edit();
            widget.activate();
            widget.newRow();
            ScreenService.openSwipe(widget);
        } else {
            MessageService.showMessageByCode("PCP00034_ADD_PROD_ATEND_APROV");
        }
    };

    this.editProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            InflightorGeneral.edit(widget);
            ScreenService.openSwipe(widget);
        } else {
            MessageService.showMessageByCode("PCP00034_EDIT_PROD_ATEND_APROV");
        }
    };

    this.deleteProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            InflightorGeneral.deleteCrud(widget);
        } else {
            MessageService.showMessageByCode("PCP00034_DEL_PROD_ATEND_APROV");
        }
    };

    this.openPopupAdicionar = function(widget) {
        var permiteInclusaoProdutoAtendimento = 'N';

        if (!isAtendimentoPendente(widget.container.getWidget('widgetAtendimentoRequisicao'))) {
            MessageService.showMessageByCode("PCP00034_ADD_PROD_ATEND_APROV");
        } else {
            if (widget.container.getWidget('widgetAtendimentoRequisicao').currentRow.IDORIGEM != 'MANUAL' && permiteInclusaoProdutoAtendimento == 'N') {
                MessageService.showMessageByCode("PCP00034_N_PERMITE_ADD_PROD_ATEND_REQ");
            } else {
                widget.paramsNewCode.__new_code_pk = '';
                widget.paramsNewCode.__new_code_pk = 'RETIRADAINF' + widget.currentRow.CDFILIAL;
                InflightorGeneral.openLinkPopup(widget);
            }
        }
    };

    // FILTRO

    this.startFilter = function(widget) {
        var date = new Date();
        date = moment(date).format("DD/MM/YYYY");
        widget = templateManager.container.getWidget('widgetAtendimentoRequisicao');
        widget.activate();

        InflightorGeneral.openFilterOnEnter(widget, ["CDFILIAL", "NMFILIAL", "DTRETIRADAINF", "IDSITUACAO", "TSETOR", "TTURNO"]);

        widget.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widget.getField('NMFILIAL').setValue(getLocalVar('INFLIGHTOR_NMFILIAL'));
        widget.getField('DTRETIRADAINF').setValue(date);
        this.setSetorFilter(getLocalVar('CDFILIAL'));
        templateManager.updateTemplate();
    };

    this.corrigeRowFiltro = function(args) {

        var widget = args.owner;
        var rowFiltro = angular.copy(args.e.preference.FILTER_ROW);
        var filtro = [];
        var rowAjustada = {
            CDFILIAL: rowFiltro.CDFILIAL,
            DTRETIRADAINF: rowFiltro.DTRETIRADAINF,
            CDSETOR: rowFiltro.TSETOR === 'S' ? 'T' : '_' + rowFiltro.NMSETOR.join('_') + '_',
            NRTURNO: rowFiltro.TTURNO === 'S' ? 'T' : '_' + rowFiltro.NRTURNO.join('_') + '_',
        };

        if (rowFiltro.IDSITUACAO) {
            rowAjustada.IDSITUACAO = rowFiltro.IDSITUACAO;
        }

        var colunas = Object.keys(rowAjustada);

        for (var coluna in colunas) {
            filtro.push({
                name: colunas[coluna],
                operator: '=',
                value: rowAjustada[colunas[coluna]]
            });
        }

        filtro.push({
            operator: 'ORDER_BY',
            name: [
                {
                    "name": "NRRETIRADAINF",
                    "direction": "ASC"
                    }
                ]
        });

        widget.dataSourceFilter = filtro;
        ScreenService.closePopup();

    };

    this.fixFilterInfo = function(widget) {
        widget._filterInfo = widget._filterInfo.filter(fieldInfo => fieldInfo.label !== 'Setor' && fieldInfo.label !== 'Todos');
    };

    this.setSetorFilter = function(value) {
        SETOR = value;
    };

    this.getSetorFilter = function() {
        return SETOR;
    };

    this.setTurnoFilter = function(value) {
        var add_ToTurnoFilter = '';
        value.forEach(function(filter) {
            add_ToTurnoFilter = filter + '_' + add_ToTurnoFilter;
        });
        TURNO = add_ToTurnoFilter;
    };

    this.getTurnoFilter = function() {
        var widget = templateManager.container.getActiveWidget();
        var wid = widget.container.getWidget('widgetAtendimentoRequisicao').appliedFilter.FILTER_ROW;
        if (wid.NMSETOR.length) {
            return TURNO;
        } else {
            return "T";
        }
    };

    this.getAddTurnoFilter = function() {
        var wid = templateManager.container.getWidget('widgetAtendimentoRequisicao');
        return wid.currentRow.CDSETOR || undefined;
    };

    this.setDefaultValue = function(widget, field) {
        widget.dataSourceFilter = widget.dataSourceFilter.filter(function(newFilter) {
            if (newFilter.name === field && newFilter.value !== 'T') {
                newFilter.value = 'T';
            }
        });
    };

    this.clearValue = function(field) {
        field.clearValue();
    };

    this.clearFields = function(fields, position) {
        fields.map(function(field) {
            if (field.sequenceInList > position && field.dataSource) {
                for (var key in field.outData) {
                    fields.find(function(value) {
                        if (value.name === field.outData[key]) {
                            value.clearValue();
                            fields[field.sequenceInList].setValue('S');
                            InflightorGeneral.toggleReadOnlySelect(fields[field.sequenceInList], fields[(field.sequenceInList - 1)]);
                            fields[field.sequenceInList].checked = true;
                            templateManager.updateTemplate();
                        }
                    }.bind(this));
                }
            }
        });
    };

    // FIM FILTRO

    this.handleError = function(widget, callback) {
        var message = 'Timeout';
        if (callback.data && callback.data.error) {
            trataMensagem(callback.data.error);
        }
        if (callback.data && callback.data.dataset && callback.data.dataset.errorList && callback.data.dataset.errorList.length) {
            MessageService.getMessageByCode("PCP00034_ATENDIMENTO_INV_OPER").then(function(msg) {
                for (let i = 0; i < callback.data.dataset.errorList.length; i++) {
                    let code_params = callback.data.dataset.errorList[i].split("#_#");
                    callback.data.dataset.errorList[i] = ScreenService.i18n(msg, {
                        "0": code_params[1]
                    });
                }
                InflightorGeneral.showErrorMessages(widget, callback.data.dataset.errorList);
            });
        } else {
            MessageService.showMessageByCode(message, "error");
        }
    };

    this.approveAtendimentoRequisicao = function(widget) {

        var atendimento_requisicoes = widget.getCheckedRows();

        InflightorGeneral.callRestEngine('pcp#/pcp00034_atendimento_requisicao/approve_atendimento_requisicao', 'DataSet', atendimento_requisicoes).then(function(callback) {
            widget.reload().then(function() {
                ScreenService.closePopup();
                widget.unSelectAll();
                MessageService.showMessageByCode("PCP00034_PED_APROV");
            });
        }).catch(function(error) {
            self.handleError(widget, error);
        });
    };

    this.cancelAtendimentoRequisicao = function(widget) {

        var atendimento_requisicoes = widget.getCheckedRows();

        InflightorGeneral.callRestEngine('pcp#/pcp00034_atendimento_requisicao/cancel_atendimento_requisicao', 'DataSet', atendimento_requisicoes).then(function(callback) {
            widget.reload().then(function() {
                ScreenService.closePopup();
                widget.unSelectAll();
                MessageService.showMessageByCode("PCP00034_PED_CANCEL_APROV");
            });
        }).catch(function(error) {
            self.handleError(widget, error);
        });
    };

    this.deleteAtendimentoRequisicao = function(widget) {
        MessageService.confirmMessageByCode("PCP00034_CONF_EXCL_ATEND").then(function(success) {
            InflightorGeneral.callRestEngine('pcp#/pcp00034_atendimento_requisicao/remove_atendimento_requisicao', 'Row', widget.currentRow).then(function(callback) {
                ScreenService.closeSwipe().then(function() {
                    widget.activate();
                    widget.reload();
                });
            }).catch(function(error) {
                self.handleError(widget, error);
            });
        });
    };

    var isAtendimentoPendente = function(widgetAtendimento) {
        return (widgetAtendimento.currentRow.IDSITUACAO == "PENDENTE");
    };

    this.saveLineEdit = function(widgetProduto) {
        let valueInvalid = false;
        if (isAtendimentoPendente(widgetProduto.getParent())) {
            widgetProduto.fields.forEach(function(field) {
                if (field.template == "field/number.html") {
                    widgetProduto.dataSource.data.forEach(function(row) {
                        InflightorGeneral.setFloatAttrOnCurrentRow(row, field.name);
                        if (InflightorGeneral.valueToFloat(row.QTPRODUTOINF) < 0) {
                            valueInvalid = true;
                        }
                    });
                }
            });

            if (valueInvalid) {
                MessageService.showMessageByCode("PCP00034_QTDE_INV");
            } else {
                InflightorGeneral.saveCrud(widgetProduto).then(function() {
                    widgetProduto.commitInlineEdit();
                    widgetProduto.list();
                    widgetProduto.activate();
                    InflightorGeneral.widgetReload(widgetProduto);
                });
            }
        } else {
            MessageService.showMessageByCode("PCP00034_EDIT_PROD_ATEND_APROV");
            widgetProduto.rollbackInlineEdit();
            widgetProduto.list();
        }
    };

    this.cancelInlineEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    this.clearDatasource = function(widget) {
        widget.dataSource.data = [];
    };

    this.filtraProdutos = function(widget) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.container.getWidget('widgetAtendimentoRequisicao').currentRow.CDFILIAL
            },
            {
                "name": "NRRETIRADAINF",
                "operator": "=",
                "value": widget.container.getWidget('widgetAtendimentoRequisicao').currentRow.NRRETIRADAINF
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.deletaProdutos = function(widgetProduto) {
        try {
            if (isAtendimentoPendente(widgetProduto.container.getWidget('widgetAtendimentoRequisicao'))) {
                var checkedRows = widgetProduto.getCheckedRows();
                checkedRows.forEach(function(row) {
                    if (row.QTPRODUTOCALC) {
                        MessageService.showMessageByCode("PCP00034_N_PERMITE_EXCL_PROD_ORI_ATE");
                        return;
                    }
                });
                InflightorGeneral.deleteCrudCheckedRows(widgetProduto);
            } else {
                MessageService.showMessageByCode("PCP00034_DEL_PROD_ATEND_APROV");
            }
        } catch (e) {
            var errorMessage = ScreenService.i18n(e);
            MessageService.showMessageByCode(errorMessage, 'error');
        }
    };

    this.validValue = function(field) {
        if (InflightorGeneral.valueToFloat(field.value()) < 0) {
            MessageService.showMessageByCode("PCP00034_QTDE_INV");
            field.setValue("");
        } else {
            field.setValue(InflightorGeneral.valueToFloat(field.value()));
        }
    };

    this.cancelOptionsReport = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget('widgetAtendimentoRequisicao').activate();
    };

    this.openOptionsReport = function(widget) {
        let popupOptions = widget.container.getWidget('popup_relatorios');
        ScreenService.openPopup(popupOptions).then(function() {
            popupOptions.activate();
        });
    };

    this.openWindowReport = function(widget) {
        InflightorGeneral.openReport("pcp#/relatorioProducaoReceita", "PCP00333", getRowToReport(widget));
    };

    function getRowToReport(widget) {
        widgetAtendimento = widget.container.getWidget('widgetAtendimentoRequisicao');
        return {
            CDFILIAL: widgetAtendimento.currentRow.CDFILIAL,
            T_SETOR: "T",
            T_TURNO: "T",
            T_RETIRADA: widgetAtendimento.currentRow.NRRETIRADAINF,
            T_PRODUTO: "T",
            DTREFINI: moment(widgetAtendimento.currentRow.DTRETIRADAINF, 'DD/MM/YYYY').format("DD/MM/YYYY"),
            DTREFFIM: moment(widgetAtendimento.currentRow.DTRETIRADAINF, 'DD/MM/YYYY').format("DD/MM/YYYY"),
            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
            REPORTNAME: "PCP00333",
            REPORTFORMAT: widget.currentRow.REPORTFORMAT
        };
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('AtendimentoRequisicao', AtendimentoRequisicao);
});

// FILE: ./assets/js/controllers/CategoriaChecklistPorSetorController.js
var CategoriaChecklistPorSetorController = function(ScreenService, InflightorGeneral, restEngine, MessageService) {

    //this_setorAfterMoveRow
    this.setorAfterMoveRow = function(widgetSetor) {
        var tabbedWidget = widgetSetor.container.getWidget('tabbedWidget');
        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetSetor.appliedFilter.FILTER_ROW.CDFILIAL
      },
            {
                "name": "CDSETOR",
                "operator": "=",
                "value": widgetSetor.currentRow.CDSETOR
      },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widgetSetor.currentRow.NRORG
      }
    ];

        tabbedWidget.widgets.forEach(function(widgetFilho) {
            widgetFilho.dataSourceFilter = dataSourceFilter;
            widgetFilho.reload();
        });
        ScreenService.openSwipe(tabbedWidget);

        if (widgetSetor.active) {
            tabbedWidget.widgets[0].activate();
        }
    };

    //this_categoriaAfterMoveRow
    this.categoriaAfterMoveRow = function(widgetCategoria) {
        var tabbedWidgetSubcat = widgetCategoria.container.getWidget('tabbedWidgetSubcat');
        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetCategoria.currentRow.CDFILIAL
      },
            {
                "name": "CDSETOR",
                "operator": "=",
                "value": widgetCategoria.currentRow.CDSETOR
      },
            {
                "name": "NRCATEGORIAPROD",
                "operator": "=",
                "value": widgetCategoria.currentRow.NRCATEGORIAPROD
      },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widgetCategoria.currentRow.NRORG
      }
    ];

        tabbedWidgetSubcat.widgets.forEach(function(widgetFilho) {
            widgetFilho.dataSourceFilter = dataSourceFilter;
            widgetFilho.reload();
        });
        ScreenService.openSwipe(tabbedWidgetSubcat);

        if (widgetCategoria.active) {
            tabbedWidgetSubcat.widgets[0].activate();
        }
    };

    //this_vinculaSetorFilial
    this.vinculaSetorFilial = function(popupSetor) {
        var datasetToSave = [];
        popupSetor.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            row.IDHABROLLBKDS = 'S';
            row.IDHABSOMKDS = 'S';
            datasetToSave.push(row);
        });
        if (datasetToSave.length > 0) {
            InflightorGeneral.callRestEngine("pcp#/crud/setorfilial/save", "DataSet", datasetToSave).then(
                function(sucesso) {
                    ScreenService.closePopup();
                    popupSetor.unSelectAll();
                    popupSetor.container.getWidget('widgetSetor').reload();
                    popupSetor.container.getWidget('widgetSetor').activate();
                },
                function(erro) {
                    if (erro.data && erro.data.error) {
                        ScreenService.showMessage(erro.data.error);
                    } else {
                        MessageService.showMessageByCode("SAVE_ERROR");
                    }
                });
        }
    };

    //this_abrePopupSetor
    this.abrePopupSetor = function(popupSetor) {

        var widgetSetor = popupSetor.container.getWidget("widgetSetor");
        var widgetSetorFilterRow = widgetSetor.appliedFilter.FILTER_ROW;

        if (Boolean(widgetSetorFilterRow) && widgetSetorFilterRow.CDFILIAL) {
            popupSetor.dataSourceFilter[0].value = widgetSetorFilterRow.CDFILIAL;
            popupSetor.activate();
            popupSetor.reload();
        }

    };

    //this_vinculaCategoria  
    this.vinculaCategoria = function(popupCategoria, nivel) {
        var dataSourceToSave = nivel == 0 ? 'catprodsetor' : 'subcatpsetor';
        var datasetToSave = [];
        popupCategoria.getCheckedRows().forEach(function(row) {
            row.__is_new = true;
            datasetToSave.push(row);
        });
        if (datasetToSave.length > 0) {
            InflightorGeneral.callRestEngine("pcp#/crud/" + dataSourceToSave + "/save", "DataSet", datasetToSave).then(
                function(sucesso) {
                    ScreenService.closePopup();
                    popupCategoria.unSelectAll();
                    popupCategoria.parent.reload();
                    popupCategoria.parent.activate();
                },
                function(erro) {
                    if (erro.data && erro.data.error) {
                        ScreenService.showMessage(erro.data.error);
                    } else {
                        MessageService.showMessageByCode("SAVE_ERROR");
                    }
                });
        }
    };

    //this_vinculaTurnoSetor
    this.vinculaTurnoSetor = function(popupTurno) {
        var datasetToSave = [];
        popupTurno.getCheckedRows().forEach(function(row) {
            row.__is_new = true;

            if (!row.HRINITURNO) {
                row.HRINITURNO = '0000';
            }
            if (!row.HRFIMTURNO) {
                row.HRFIMTURNO = '0000';
            }

            datasetToSave.push(row);
        });
        if (datasetToSave.length > 0) {
            InflightorGeneral.callRestEngine("pcp#/crud/turnosetorfili/save", "DataSet", datasetToSave).then(
                function(sucesso) {
                    ScreenService.closePopup();
                    popupTurno.unSelectAll();
                    popupTurno.parent.reload();
                    popupTurno.parent.activate();
                },
                function(erro) {
                    if (erro.data && erro.data.error) {
                        ScreenService.showMessage(erro.data.error);
                    } else {
                        MessageService.showMessageByCode("SAVE_ERROR");
                    }
                });
        }
    };

    //this_chamaPopupCategoria
    this.chamaPopupCategoria = function(widgetCategoria, widgetToOpen) {
        var popupToOpen = widgetCategoria.container.getWidget(widgetToOpen);
        ScreenService.openPopup(popupToOpen).then(function() {
            popupToOpen.activate();
            popupToOpen.reload();
        });
    };

    //this_abrePopupCategoriaOuTurno
    this.abrePopupCategoriaOuTurno = function(popup) {
        var widgetSetor = popup.container.getWidget('widgetSetor');
        popup.dataSourceFilter[0].value = widgetSetor.currentRow.CDFILIAL;
        popup.dataSourceFilter[1].value = widgetSetor.currentRow.CDSETOR;
        popup.activate();
        popup.reload();
    };

    //this_abrePopupSubcategoria
    this.abrePopupSubcategoria = function(popup) {
        var widgetCategoria = popup.container.getWidget('widgetCategoria');
        popup.dataSourceFilter[0].value = widgetCategoria.currentRow.CDFILIAL;
        popup.dataSourceFilter[1].value = widgetCategoria.currentRow.CDSETOR;
        popup.dataSourceFilter[2].value = widgetCategoria.currentRow.NRCATEGORIAPROD;
        popup.activate();
        popup.reload();
    };

    //this_salvarEditTurno
    this.salvarEditTurno = function(widgetTurno) {
        widgetTurno.dataSource.data.forEach(function(row) {
            converteFieldTime(row, ["HRINITURNO", "HRFIMTURNO"], "hhmm");

            if (row.HRINITURNO[0] > 2) {
                row.HRINITURNO[0] = 2;
                row.HRINITURNO[1] = 3;
            }
            if (row.HRINITURNO[0] == 2 && row.HRINITURNO[1] > 3) {
                row.HRINITURNO[1] = 3;
            }
            if (row.HRINITURNO[0] > 5) {
                row.HRINITURNO[0] = 5;
                row.HRINITURNO[0] = 9;
            }

            if (row.HRFIMTURNO[0] > 2) {
                row.HRFIMTURNO[0] = 2;
                row.HRFIMTURNO[1] = 3;
            }
            if (row.HRFIMTURNO[0] == 2 && row.HRFIMTURNO[1] > 3) {
                row.HRFIMTURNO[1] = 3;
            }
            if (row.HRFIMTURNO[0] > 5) {
                row.HRFIMTURNO[0] = 5;
                row.HRFIMTURNO[0] = 9;
            }
        });

        InflightorGeneral.saveCrud(widgetTurno).then(function(sucesso) {
            widgetTurno.reload();
            widgetTurno.list();
            widgetTurno.commitInlineEdit();
            widgetTurno.activate();
        });
    };

    //var_converteFieldTime
    var converteFieldTime = function(row, fields, formato) {
        fields.forEach(function(field) {
            var strHora = row[field];
            if (strHora.length != formato.length) {
                switch (formato) {
                    case 'hh:mm':
                        row[field] = strHora[0] + strHora[1] + ':' + strHora[2] + strHora[3];
                        break;
                    case 'hhmm':
                        row[field] = strHora[0] + strHora[1] + strHora[3] + strHora[4];
                        break;
                }
            }
        });
    };

    //this_quebraCategoria
    this.quebraCategoria = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Quebra por tipo de produção?'), 'question', function(success) {
            var dataset = widget.getCheckedRows();
            dataset.forEach(function(row) {
                row.IDQUEBRAPORTIPOPROD = 'S';
                row.changed = true;
            });
            restEngine.post("pcp#/crud/catprodsetor/save", {
                "requestType": "DataSet",
                "dataset": dataset
            });
            widget.unSelectAll();
        }, function(erro) {
            widget.getCheckedRows().forEach(function(row) {
                var dataset = widget.getCheckedRows();
                dataset.forEach(function(row) {
                    row.IDQUEBRAPORTIPOPROD = 'N';
                    row.changed = true;
                });
                restEngine.post("pcp#/crud/catprodsetor/save", {
                    "requestType": "DataSet",
                    "dataset": dataset
                });
                widget.unSelectAll();
            });
        });
    };

    //this_quebraSubcategoria
    this.quebraSubcategoria = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Quebra por tipo de produção?'), 'question', function(success) {
            var dataset = widget.getCheckedRows();
            dataset.forEach(function(row) {
                row.IDQUEBRAPORTIPOPROD = 'S';
                row.changed = true;
            });
            restEngine.post("pcp#/crud/subcatpsetor/save", {
                "requestType": "DataSet",
                "dataset": dataset
            });
            widget.unSelectAll();
        }, function(erro) {
            widget.getCheckedRows().forEach(function(row) {
                var dataset = widget.getCheckedRows();
                dataset.forEach(function(row) {
                    row.IDQUEBRAPORTIPOPROD = 'N';
                    row.changed = true;
                });
                restEngine.post("pcp#/crud/subcatpsetor/save", {
                    "requestType": "DataSet",
                    "dataset": dataset
                });
                widget.unSelectAll();
            });
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('CategoriaChecklistPorSetorController', CategoriaChecklistPorSetorController);
});

// FILE: ./assets/js/controllers/CustomReportMalhaAerea.js
var CustomReportMalhaAerea = function(ScreenService, InflightorGeneral, MalhaDeVooService) {

    var self = this;

    var FILTROCLEAR = [
        {
            name: "NRORG",
            operator: "=",
            value: "-1"
        }
    ];

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL');
        args.owner.currentRow.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL');
    };

    this.limpaField = function(field) {
        if (field.name == 'NMFILIAL') {
            field.widget.currentRow.NMCIA = "";
        }
    };

    this.validaFiltro = function(widget) {
        var array = widget.__filterPopup.currentRow.FREQUENCIA;
        widget.__filterPopup.currentRow.SEG = 'N';
        widget.__filterPopup.currentRow.TER = 'N';
        widget.__filterPopup.currentRow.QUA = 'N';
        widget.__filterPopup.currentRow.QUI = 'N';
        widget.__filterPopup.currentRow.SEX = 'N';
        widget.__filterPopup.currentRow.SAB = 'N';
        widget.__filterPopup.currentRow.DOM = 'N';

        array.forEach(function(dia) {
            widget.__filterPopup.currentRow[dia] = 'S';
        });

        if (widget.__filterPopup.currentRow.FREQUENCIA) {
            widget.__filterPopup.currentRow.FREQUENCIA = "";
        }
    };

    this.unidadeToCompanhia = function(field) {
        var cdfilial = field.widget.currentRow.CDFILIAL;
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': cdfilial
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        if (!field.widget.currentRow.CDFILIAL) {
            field.dataSourceFilter = FILTROCLEAR;
            field.reload();
            msg = ScreenService.i18n("Escolha uma Filial para definir a Companhia.");
            ScreenService.showMessage(msg);
            return;
        }
        field.widget.getField('NMCIA').dataSourceFilter = filter;
        field.widget.getField('NMCIA').reload();
    };

    this.filtraVoos = function(field) {
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': field.widget.currentRow.CDFILIAL
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': field.widget.currentRow.CDCIA
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        if (!field.widget.currentRow.CDCIA) {
            field.dataSourceFilter = FILTROCLEAR;
            field.reload();
            var msgCia = ScreenService.i18n("Escolha uma Companhia para definir o Voo.");
            ScreenService.showMessage(msgCia);
            return;
        }
        field.widget.getField('NRVOO').dataSourceFilter = filter;
        field.widget.getField('NRVOO').reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('CustomReportMalhaAerea', CustomReportMalhaAerea);
});

// FILE: ./assets/js/controllers/EmissaoChecklist.js
var EmissaoChecklist = function(ScreenService, InflightorGeneral) {

    var self = this;

    this.filterVoo = function(widget) {
        var fieldVoo = widget.getField('NRVOO');
        var filter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
        ];
        if (widget.currentRow.TCIA == 'N')
            filter.push({
                "name": "CDCIA",
                "operator": "IN",
                "value": widget.currentRow.CDCIA
            });
        else {
            widget.currentRow.CDCIA = 'T';
        }
        fieldVoo.dataSourceFilter = filter;
        fieldVoo.reload();
    };
    this.setorOnChange = function(widget) {
        var fieldCategoria = widget.getField('NMCATEGORIAPROD');

        widget.currentRow.NRCATEGORIAPROD = '';
        widget.currentRow.NMCATEGORIAPROD = '';

        if (widget.currentRow.CDSETOR) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldCategoria.readOnly = false;
            fieldCategoria.dataSourceFilter = dataSourceFilter;
            fieldCategoria.reload();
        } else {
            fieldCategoria.readOnly = true;
        }

        self.categoriaOnChange(widget);
    };
    this.categoriaOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('NMSUBCATEGORIAPROD');

        widget.currentRow.NRSUBCATEGORIAPROD = '';
        widget.currentRow.NMSUBCATEGORIAPROD = '';

        if (widget.currentRow.NRCATEGORIAPROD) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldSubcategoria.readOnly = false;
            fieldSubcategoria.dataSourceFilter = dataSourceFilter;
            fieldSubcategoria.reload();
        } else {
            fieldSubcategoria.readOnly = true;
        }
    };
    this.beforeSelectOpenFilterField = function(field) {
        var parameters = {
            "CDFILIAL": field.widget.currentRow.CDFILIAL,
            "NRORG": getLocalVar('INFLIGHTOR_NRORGTRAB')
        };
        InflightorGeneral.onClickDependentField(field, parameters);
    };
    this.emitirRelatorio = function(widget) {

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('EmissaoChecklist', EmissaoChecklist);
});

// FILE: ./assets/js/controllers/FluxoProducaoController.js
var FluxoProducaoController = function(ScreenService, InflightorGeneral, pcp00007_fluxoprod_receita_service, templateManager, MessageService) {

    var self = this;
    var Service = pcp00007_fluxoprod_receita_service;

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00007_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    this.newRow = function(widget) {
        widget.newRow();
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;

        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        widget.getField('NMSETOR').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('NMSETOR').reload();

        widget.getField('NMCIA').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('NMCIA').reload();

        widget.getField('NMCATEGORIAPROD').readOnly = true;
        widget.getField('NMSUBCATEGORIAPROD').readOnly = true;

        widget.getField('DSRECEITA').readOnly = true;
        widget.getField('DSRECEITA').validations = {};

        widget.getField('NMCIA').readOnly = true;
        widget.getField('NMCIA').validations = {};

        widget.getField('NRVOO').readOnly = true;
        widget.getField('NRVOO').validations = {};

        widget.getField('DSTURNOANTES').readOnly = true;
        widget.getField('DSTURNODEPOIS').readOnly = true;

        widget.getField('DSTIPONEGOCIOINF').readOnly = true;
        widget.getField('DSTIPONEGOCIOINF').validations = {};

        widget.getField('DSTIPOPROD').readOnly = true;
        widget.getField('DSTIPOPROD').validations = {};

        widget.getField('PAI_NMSETOR').readOnly = true;
        widget.getField('PAI_NMSETOR').validations = {};
        widget.getField('PAI_NMSETOR').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('PAI_NMSETOR').reload();

        widget.getField('PAI_NMCATEGORIAPROD').readOnly = true;
        widget.getField('PAI_NMCATEGORIAPROD').validations = {};

        widget.getField('PAI_NMSUBCATEGORIAPROD').readOnly = true;
        widget.getField('PAI_NMSUBCATEGORIAPROD').validations = {};

        widget.getField('PAI_DSRECEITA').readOnly = true;
        widget.getField('PAI_DSRECEITA').validations = {};
        widget.getField('PAI_DSRECEITA').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('PAI_DSRECEITA').reload();

        widget.getField('ROOT_NMSETOR').readOnly = true;
        widget.getField('ROOT_NMSETOR').validations = {};
        widget.getField('ROOT_NMSETOR').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('ROOT_NMSETOR').reload();

        widget.getField('ROOT_NMCATEGORIAPROD').readOnly = true;
        widget.getField('ROOT_NMCATEGORIAPROD').validations = {};

        widget.getField('ROOT_NMSUBCATEGORIAPROD').readOnly = true;
        widget.getField('ROOT_NMSUBCATEGORIAPROD').validations = {};

        widget.getField('ROOT_DSRECEITA').readOnly = true;
        widget.getField('ROOT_DSRECEITA').validations = {};
        widget.getField('ROOT_DSRECEITA').dataSourceFilter = angular.copy(dataSourceFilter);
        widget.getField('ROOT_DSRECEITA').reload();
    };

    this.edit = function(widget) {
        InflightorGeneral.edit(widget);
        widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;

        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDSETOR",
                "operator": "=",
                "value": widget.currentRow.CDSETOR
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        var dsturnoantes = widget.getField('DSTURNOANTES');
        var dsturnodepois = widget.getField('DSTURNODEPOIS');

        dsturnoantes.readOnly = false;
        dsturnoantes.dataSourceFilter = angular.copy(dataSourceFilter);
        dsturnoantes.reload();

        dsturnodepois.readOnly = false;
        dsturnodepois.dataSourceFilter = angular.copy(dataSourceFilter);
        dsturnodepois.reload();
    };

    this.save = function(widget) {
        if (widget.isValid()) {
            if (widget.currentRow.__is_new) {
                _saveNew(widget);
            } else {
                InflightorGeneral.saveCrud(widget);
            }
        }
    };

    var _saveNew = function(widget) {
        _paramsNewCode(widget);
        Service.save(widget.currentRow).then(function(sucesso) {
            widget.reload().then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.currentRow.__is_new = false;
                    widget.getField('DSRECEITA').validations = {};
                    widget.getField('NMCIA').validations = {};
                    widget.getField('NRVOO').validations = {};
                });
            });
        }, InflightorGeneral.callbackError);
    };

    this.setorOnChange = function(widget) {
        var fieldCategoria = widget.getField('NMCATEGORIAPROD');
        var fieldTurnoAntes = widget.getField('DSTURNOANTES');
        var fieldTurnoDepois = widget.getField('DSTURNODEPOIS');

        widget.currentRow.NRCATEGORIAPROD = '';
        widget.currentRow.NMCATEGORIAPROD = '';
        widget.currentRow.CDTURNOANTES = '';
        widget.currentRow.DSTURNOANTES = '';
        widget.currentRow.CDTURNODEPOIS = '';
        widget.currentRow.DSTURNODEPOIS = '';

        if (widget.currentRow.CDSETOR) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldCategoria.readOnly = false;
            fieldCategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldCategoria.reload();

            fieldTurnoAntes.readOnly = false;
            fieldTurnoAntes.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldTurnoAntes.reload();

            fieldTurnoDepois.readOnly = false;
            fieldTurnoDepois.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldTurnoDepois.reload();
        } else {
            fieldCategoria.readOnly = true;
            fieldTurnoAntes.readOnly = true;
            fieldTurnoDepois.readOnly = true;
        }

        self.categoriaOnChange(widget);
    };

    this.categoriaOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('NMSUBCATEGORIAPROD');
        var fieldReceita = widget.getField('DSRECEITA');

        widget.currentRow.NRSUBCATEGORIAPROD = '';
        widget.currentRow.NMSUBCATEGORIAPROD = '';
        widget.currentRow.CDRECEITA = '';
        widget.currentRow.DSRECEITA = '';

        if (widget.currentRow.NRCATEGORIAPROD) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldSubcategoria.readOnly = false;
            fieldSubcategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldSubcategoria.reload();

            fieldReceita.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldReceita.reload();

            if (widget.currentRow.IDABRANREC == '1') {
                fieldReceita.readOnly = false;
            }
        } else {
            fieldSubcategoria.readOnly = true;
            fieldReceita.readOnly = true;
        }

        self.subcategoriaOnChange(widget);
    };

    this.subcategoriaOnChange = function(widget) {
        var fieldReceita = widget.getField('DSRECEITA');

        widget.currentRow.CDRECEITA = '';
        widget.currentRow.DSRECEITA = '';
        var dataSourceFilter;

        if (widget.currentRow.NRSUBCATEGORIAPROD) {
            dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRCATEGORIAPROD
                },
                {
                    "name": "NRSUBCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRSUBCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];
        } else {
            dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];
        }

        fieldReceita.dataSourceFilter = angular.copy(dataSourceFilter);
        fieldReceita.reload();
    };

    this.companhiaOnChange = function(widget) {
        var fieldVoo = widget.getField('NRVOO');
        widget.currentRow.NRVOO = '';

        if (widget.currentRow.CDCIA) {
            fieldVoo.dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": widget.currentRow.CDCIA
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];
            fieldVoo.reload();

            if (widget.currentRow.IDABRANVOO == '3') {
                fieldVoo.readOnly = false;
            }
        } else {
            fieldVoo.readOnly = true;
        }
    };

    this.abranTipoOnChange = function(widget) {
        var fieldTipoProd = widget.getField('DSTIPOPROD');
        if (widget.currentRow.IDABRANTPP == '1') {
            fieldTipoProd.readOnly = false;
            fieldTipoProd.validations = {
                "required": true
            };
        } else {
            fieldTipoProd.readOnly = true;
            fieldTipoProd.validations = {};
            widget.currentRow.NRTIPOPROD = '';
            widget.currentRow.DSTIPOPROD = '';
        }
    };

    this.abranTipoNegOnChange = function(widget) {
        var fieldTipoNegocio = widget.getField('DSTIPONEGOCIOINF');
        if (widget.currentRow.IDABRANTPN == '1') {
            fieldTipoNegocio.readOnly = false;
            fieldTipoNegocio.validations = {
                "required": true
            };
        } else {
            fieldTipoNegocio.readOnly = true;
            fieldTipoNegocio.validations = {};
            widget.currentRow.NRTIPONEGOCIOINF = '';
            widget.currentRow.DSTIPONEGOCIOINF = '';
        }
    };

    this.abranRecOnChange = function(widget) {
        var fieldReceita = widget.getField('DSRECEITA');
        if (widget.currentRow.IDABRANREC == '1') {
            fieldReceita.readOnly = !widget.currentRow.NRCATEGORIAPROD;
            fieldReceita.validations = {
                "required": true
            };
        } else {
            fieldReceita.readOnly = true;
            fieldReceita.validations = {};
            widget.currentRow.CDRECEITA = '';
            widget.currentRow.DSRECEITA = '';
        }
    };

    this.abranCiaOnChange = function(widget) {
        var fieldCia = widget.getField('NMCIA');
        if (widget.currentRow.IDABRANCIA == '1') {
            fieldCia.readOnly = false;
            fieldCia.validations = {
                "required": true
            };
        } else {
            if (widget.currentRow.IDABRANVOO == '3') {
                MessageService.showMessageByCode("PCP00007_VOO_CANT_BE_SPECIFIC");
                widget.currentRow.IDABRANCIA = '1';
                return;
            }
            fieldCia.readOnly = true;
            fieldCia.validations = {};
            widget.currentRow.CDCIA = '';
            widget.currentRow.NMCIA = '';
        }
    };

    this.abranVooOnChange = function(widget) {
        var fieldVoo = widget.getField('NRVOO');
        if (widget.currentRow.IDABRANVOO == '3') {
            if (widget.currentRow.IDABRANCIA != '1') {
                widget.currentRow.IDABRANCIA = '1';
                this.abranCiaOnChange(widget);
                widget.isValid();
            }
            fieldVoo.readOnly = !widget.currentRow.CDCIA;
            fieldVoo.validations = {
                "required": true
            };
        } else {
            fieldVoo.readOnly = true;
            fieldVoo.validations = {};
            widget.currentRow.NRVOO = '';
        }
    };

    this.abranSetorPaiOnChange = function(widget) {
        var fieldSetor = widget.getField('PAI_NMSETOR');
        if (widget.currentRow.IDSTRPARENTDEPENDENT == '1') {
            fieldSetor.readOnly = false;
            fieldSetor.validations = {
                "required": true
            };
        } else {
            if (widget.currentRow.IDCATPARENTDEPENDENT == '1') {
                MessageService.showMessageByCode("PCP00007_CATEGORIA_CANT_BE_YES");
                widget.currentRow.IDSTRPARENTDEPENDENT = '1';
                return;
            }
            fieldSetor.readOnly = true;
            fieldSetor.validations = {};
            widget.currentRow.PAI_CDSETOR = '';
            widget.currentRow.PAI_NMSETOR = '';
            self.setorPaiOnChange(widget);
        }
    };

    this.setorPaiOnChange = function(widget) {
        var fieldCategoria = widget.getField('PAI_NMCATEGORIAPROD');

        widget.currentRow.PAI_NRCATEGORIAPROD = '';
        widget.currentRow.PAI_NMCATEGORIAPROD = '';

        if (widget.currentRow.PAI_CDSETOR && widget.currentRow.IDCATPARENTDEPENDENT == '1') {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.PAI_CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldCategoria.readOnly = false;
            fieldCategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldCategoria.reload();
        } else {
            fieldCategoria.readOnly = true;
        }

        self.categoriaPaiOnChange(widget);
    };

    this.abranCategoriaPaiOnChange = function(widget) {
        var fieldCategoria = widget.getField('PAI_NMCATEGORIAPROD');
        if (widget.currentRow.IDCATPARENTDEPENDENT == '1') {
            if (widget.currentRow.IDSTRPARENTDEPENDENT != '1') {
                widget.currentRow.IDSTRPARENTDEPENDENT = '1';
                this.abranSetorPaiOnChange(widget);
                widget.isValid();
            }
            fieldCategoria.readOnly = !widget.currentRow.PAI_CDSETOR;
            fieldCategoria.validations = {
                "required": true
            };
        } else {
            if (widget.currentRow.IDSUBCATPARENTDEPENDENT == '1') {
                MessageService.showMessageByCode("PCP00007_SUBCATEGORIA_CANT_BE_YES");
                widget.currentRow.IDCATPARENTDEPENDENT = '1';
                return;
            }
            fieldCategoria.readOnly = true;
            fieldCategoria.validations = {};
            widget.currentRow.PAI_NRCATEGORIAPROD = '';
            widget.currentRow.PAI_NMCATEGORIAPROD = '';
            self.categoriaPaiOnChange(widget);
        }
    };

    this.categoriaPaiOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('PAI_NMSUBCATEGORIAPROD');

        widget.currentRow.PAI_NRSUBCATEGORIAPROD = '';
        widget.currentRow.PAI_NMSUBCATEGORIAPROD = '';

        if (widget.currentRow.PAI_NRCATEGORIAPROD && widget.currentRow.IDSUBCATPARENTDEPENDENT == '1') {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.PAI_CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.PAI_NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldSubcategoria.readOnly = false;
            fieldSubcategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldSubcategoria.reload();
        } else {
            fieldSubcategoria.readOnly = true;
        }
    };

    this.abranSubCatPaiOnChange = function(widget) {
        var fieldSubategoria = widget.getField('PAI_NMSUBCATEGORIAPROD');
        if (widget.currentRow.IDSUBCATPARENTDEPENDENT == '1') {
            if (widget.currentRow.IDCATPARENTDEPENDENT != '1') {
                widget.currentRow.IDCATPARENTDEPENDENT = '1';
                this.abranCategoriaPaiOnChange(widget);
            }
            fieldSubategoria.readOnly = !widget.currentRow.PAI_NRCATEGORIAPROD;
            fieldSubategoria.validations = {
                "required": true
            };
        } else {
            fieldSubategoria.readOnly = true;
            fieldSubategoria.validations = {};
            widget.currentRow.PAI_NRSUBCATEGORIAPROD = '';
            widget.currentRow.PAI_NMSUBCATEGORIAPROD = '';
        }
    };

    this.parentDependentOnChange = function(widget) {
        var fieldReceita = widget.getField('PAI_DSRECEITA');
        if (widget.currentRow.IDPARENTDEPENDENT == '1') {
            fieldReceita.readOnly = false;
            fieldReceita.validations = {
                "required": true
            };
        } else {
            fieldReceita.readOnly = true;
            fieldReceita.validations = {};
            widget.currentRow.PAI_CDRECEITA = '';
            widget.currentRow.PAI_DSRECEITA = '';
        }
    };

    this.abranSetorRootOnChange = function(widget) {
        var fieldSetor = widget.getField('ROOT_NMSETOR');
        if (widget.currentRow.IDSTRROOTDEPENDENT == '1') {
            fieldSetor.readOnly = false;
            fieldSetor.validations = {
                "required": true
            };
        } else {
            if (widget.currentRow.IDCATROOTDEPENDENT == '1') {
                MessageService.showMessageByCode("PCP00007_CATEGORIA_CANT_BE_YES");
                widget.currentRow.IDSTRROOTDEPENDENT = '1';
                return;
            }
            fieldSetor.readOnly = true;
            fieldSetor.validations = {};
            widget.currentRow.ROOT_CDSETOR = '';
            widget.currentRow.ROOT_NMSETOR = '';
            self.setorRootOnChange(widget);
        }
    };

    this.setorRootOnChange = function(widget) {
        var fieldCategoria = widget.getField('ROOT_NMCATEGORIAPROD');

        widget.currentRow.ROOT_NRCATEGORIAPROD = '';
        widget.currentRow.ROOT_NMCATEGORIAPROD = '';

        if (widget.currentRow.ROOT_CDSETOR && widget.currentRow.IDCATROOTDEPENDENT == '1') {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.ROOT_CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldCategoria.readOnly = false;
            fieldCategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldCategoria.reload();
        } else {
            fieldCategoria.readOnly = true;
        }

        self.categoriaRootOnChange(widget);
    };

    this.abranCategoriaRootOnChange = function(widget) {
        var fieldCategoria = widget.getField('ROOT_NMCATEGORIAPROD');
        if (widget.currentRow.IDCATROOTDEPENDENT == '1') {
            if (widget.currentRow.IDSTRROOTDEPENDENT != '1') {
                widget.currentRow.IDSTRROOTDEPENDENT = '1';
                this.abranSetorRootOnChange(widget);
                widget.isValid();

            }
            fieldCategoria.readOnly = !widget.currentRow.ROOT_CDSETOR;
            fieldCategoria.validations = {
                "required": true
            };
        } else {
            if (widget.currentRow.IDSUBCATROOTDEPENDENT == '1') {
                MessageService.showMessageByCode("PCP00007_SUBCATEGORIA_CANT_BE_YES");
                widget.currentRow.IDCATROOTDEPENDENT = '1';
                return;
            }
            fieldCategoria.readOnly = true;
            fieldCategoria.validations = {};
            widget.currentRow.ROOT_NRCATEGORIAPROD = '';
            widget.currentRow.ROOT_NMCATEGORIAPROD = '';
            self.categoriaRootOnChange(widget);
        }
    };

    this.categoriaRootOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('ROOT_NMSUBCATEGORIAPROD');

        widget.currentRow.ROOT_NRSUBCATEGORIAPROD = '';
        widget.currentRow.ROOT_NMSUBCATEGORIAPROD = '';

        if (widget.currentRow.ROOT_NRCATEGORIAPROD && widget.currentRow.IDSUBCATROOTDEPENDENT == '1') {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.ROOT_CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.ROOT_NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldSubcategoria.readOnly = false;
            fieldSubcategoria.dataSourceFilter = angular.copy(dataSourceFilter);
            fieldSubcategoria.reload();
        } else {
            fieldSubcategoria.readOnly = true;
        }
    };

    this.abranSubCatRootOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('ROOT_NMSUBCATEGORIAPROD');
        if (widget.currentRow.IDSUBCATROOTDEPENDENT == '1') {
            if (widget.currentRow.IDCATROOTDEPENDENT != '1') {
                widget.currentRow.IDCATROOTDEPENDENT = '1';
                this.abranCategoriaRootOnChange(widget);
            }
            fieldSubcategoria.readOnly = !widget.currentRow.ROOT_NRCATEGORIAPROD;
            fieldSubcategoria.validations = {
                "required": true
            };
        } else {
            fieldSubcategoria.readOnly = true;
            fieldSubcategoria.validations = {};
            widget.currentRow.ROOT_NRSUBCATEGORIAPROD = '';
            widget.currentRow.ROOT_NMSUBCATEGORIAPROD = '';
        }
    };

    this.rootDependentOnChange = function(widget) {
        var fieldReceita = widget.getField('ROOT_DSRECEITA');
        if (widget.currentRow.IDROOTDEPENDENT == '1') {
            fieldReceita.readOnly = false;
            fieldReceita.validations = {
                "required": true
            };
        } else {
            fieldReceita.readOnly = true;
            fieldReceita.validations = {};
            widget.currentRow.ROOT_CDRECEITA = '';
            widget.currentRow.ROOT_DSRECEITA = '';
        }
    };

    this.beforeOpenSetor = function(field) {
        field.dataSourceFilter = [
            {
                name: 'CDFILIAL',
                operator: '=',
                value: ''
        },
            {
                name: 'NRORG',
                operator: '=',
                value: ''
        }
        ];
        var parameters = {
            CDFILIAL: field.widget.currentRow.CDFILIAL,
            NRORG: getLocalVar('INFLIGHTOR_NRORGTRAB')
        };
        InflightorGeneral.onClickDependentField(field, parameters);
    };

    this.beforeOpenCategoria = function(field) {
        var parameters = {
            CDFILIAL: field.widget.currentRow.CDFILIAL,
            NRORG: getLocalVar('INFLIGHTOR_NRORGTRAB')
        };
        InflightorGeneral.onClickDependentField(field, parameters);
    };

    this.beforeOpenVoo = function(field) {
        var parameters = {
            CDFILIAL: field.widget.currentRow.CDFILIAL,
            CDCIA: field.widget.currentRow.CDCIA,
            NRORG: getLocalVar('INFLIGHTOR_NRORGTRAB')
        };
        InflightorGeneral.onClickDependentField(field, parameters);
    };

    this.readOnlyVoo = function(field) {
        field.widget.getField('NRVOO').readOnlyOnEdit = field.widget.currentRow.CDCIA === '' || field.widget.currentRow.CDCIA === undefined;
    };

    var _paramsNewCode = function(widget) {
        var paramsNewCode = widget.paramsNewCode;
        if (paramsNewCode && paramsNewCode.__is_new_code) {
            widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
            widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
            widget.currentRow.__table_pk = paramsNewCode.__table_pk;
            widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
            widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('FluxoProducaoController', FluxoProducaoController);
});

// FILE: ./assets/js/controllers/GeracaoPrevisaoPassageiros.js
var GeracaoPrevisaoPassageiros = function(ZHPromise, ScreenService, templateManager, InflightorGeneral, restEngine, MessageService) {
    var self = this;

    function validateDate(widget) {
        if ((widget.currentRow.DATAFIM && !widget.currentRow.DATAINI) || (widget.currentRow.DATAINI && !widget.currentRow.DATAFIM)) {
            MessageService.showMessageByCode("INTERVAL_ERROR", 'error');
            return false;
        } else if (moment(widget.currentRow.DATAFIM, 'DD/MM/YYYY') < moment(widget.currentRow.DATAINI, 'DD/MM/YYYY')) {
            MessageService.showMessageByCode("FINAL_DATE_ERROR", 'error');
            return false;
        }
        return true;
    }
    this.validateDateForFilter = function(args) {
        if (!validateDate(args.owner.__filterPopup)) {
            args.e.preventDefault();
        }
    };
    this.intervalDate = function(field) {
        var widget = field.widget;
        if (field.name == 'DATAINI') {
            if (moment(widget.getField('DATAINI').value(), 'DD/MM/YYYY') > moment(widget.getField('DATAFIM').value(), 'DD/MM/YYYY')) {
                widget.getField('DATAFIM').setValue("");
            }
            widget.getField('DATAFIM').applyDefaultValue();
            widget.getField('DATAFIM').reload().then(function() {
                widget.getField('DATAFIM').minDate = field.value();
            });
        }
    };

    this.openPopupGerarPrevisao = function(widget) {
        var widgetPopup = widget.container.getWidget('gerarPrevisaoPassageiros');
        widgetPopup.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        widgetPopup.currentRow.DATAINI = widget.appliedFilter.FILTER_ROW.DATAINI;
        widgetPopup.currentRow.DATAFIM = widget.appliedFilter.FILTER_ROW.DATAFIM;
        ScreenService.openPopup(widgetPopup);
    };

    this.getFilterWidgetPrevisao = function(widget) {
        var data_inicial = widget.__filterPopup.currentRow.DATAINI;
        var data_final = widget.__filterPopup.currentRow.DATAFIM;
        var interval_dias = moment(data_final, "DD/MM/YYYY").diff(moment(data_inicial, "DD/MM/YYYY"), 'days');
        var data = [];
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.__filterPopup.currentRow.CDFILIAL
        }];
        if (widget.__filterPopup.currentRow.DATAFIM && widget.__filterPopup.currentRow.DATAINI != 'T') {
            filter.push({
                "name": "DTPREVISAOPROD",
                "operator": "BETWEEN",
                "value": [widget.__filterPopup.currentRow.DATAINI, widget.__filterPopup.currentRow.DATAFIM]
            });
        }
        return filter;
    };

    this.setFilterPrevisao = function(widget) {
        widget.dataSourceFilter = self.getFilterWidgetPrevisao(widget);
    };

    this.gerarPrevisaoPax = function(widget) {
        if (widget.isValid()) {
            if (validateDate(widget)) {
                InflightorGeneral.callRestEngine('pcp#/crud/prevpaxprodm', 'FilterData', [
                    {
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": widget.currentRow.CDFILIAL
            },
                    {
                        "name": "DTPREVISAOPROD",
                        "operator": "BETWEEN",
                        "value": [widget.currentRow.DATAINI, widget.currentRow.DATAFIM]
            }
          ]).then(function(callback) {
                    if (callback.dataset.prevpaxprodm.length) {
                        var msg = 'Existem previsões já geradas para o intervalo informado! Deseja gera-las novamente?';
                        var icon = 'question';
                        var type = 'CONFIRMATION';
                        return ScreenService.showCustomDialog(msg, type, icon, [{
                            label: ScreenService.i18n("Gerar previsão sobrescrevendo as datas já geradas"),
                            "default": true,
                            code: function(e, deferred) {
                                //Gera a previsão para o período informado sobreescrevendo os dias com previsões
                                var row = getRowToSave(widget, 'S');
                                callProcedureGeraPax(widget, row);
                            }
                  }, {
                            label: ScreenService.i18n("Gerar previsão mantendo as datas já geradas"),
                            code: function(e, deferred) {
                                //Gera a previsão para o período informado desconsiderando os dias com previsões
                                var row = getRowToSave(widget, 'N');
                                callProcedureGeraPax(widget, row);
                            }
                  }, {
                            label: ScreenService.i18n("Cancelar geração"),
                            code: function(e, deferred) {
                                ScreenService.closePopup();
                            }
                  }]);
                    } else {
                        //Nenhuma previsão encontrada para o intervalo informado, gera a previsão para todo o período
                        var row = getRowToSave(widget, 'S');
                        callProcedureGeraPax(widget, row);
                    }
                }, function(error) {
                    //error
                    InflightorGeneral.callbackError(error);
                });
            }
        } else MessageService.showMessageByCode("FILL_ALL_FIELDS");
    };

    function callProcedure(row) {
        return restEngine.request("pcp#/gera_prevpax_prod", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 9000000);
    }

    function callProcedureGeraPax(widget, row) {
        callProcedure(row).then(function(callback) {
            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, 'Lista de Erros');
            } else {
                //success
                ScreenService.closePopup();
                widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').reload().then(function() {
                    MessageService.showMessageByCode("GENERATE_SUCCESS", 'SUCCESS');
                    widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').activate();
                });
            }
        }, function(error) {
            //error
            var errorMessage = 'Timeout';
            if (error.data) {
                if (error.error == "APP-00059") {
                    errorMessage = "Não há dados para previsão do período informado";
                } else {
                    errorMessage = error.data.error;
                }
            }
            ScreenService.showMessage(errorMessage);
        });
    }

    function getRowToSave(widget, substitute) {
        return {
            'P_CDFILIAL': widget.currentRow.CDFILIAL,
            'P_CDFILIPROC': getLocalVar('INFLIGHTOR_CDFILIAL'),
            'P_DATAINI': widget.currentRow.DATAINI,
            'P_DATAFIN': widget.currentRow.DATAFIM,
            'P_SUBSTITUI_JA_GERADOS': substitute
        };
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('GeracaoPrevisaoPassageiros', GeracaoPrevisaoPassageiros);
});

// FILE: ./assets/js/controllers/MalhaDeVooController.js
var MalhaDeVooController = function(ScreenService, InflightorGeneral, MalhaDeVooService, templateManager) {

    var self = this;
    var FILTROCLEAR = [{
        name: "NRORG",
        operator: "=",
        value: "-1"
    }];

    this.setFilialLogada = function(args) {
        args.owner.currentRow.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL');
        args.owner.currentRow.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL');
    };

    this.setaRowOnView = function(widget) {
        widget.currentRow = widget.dataSource.data[0];
        widget.reload();
    };

    this.openSwipeVoos = function(mainWidget, tabbed, widget) {
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': mainWidget.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': mainWidget.currentRow.NRMALHAAEREA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': mainWidget.currentRow.CDCIA
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        ScreenService.openSwipe(tabbed).then(function(success) {
            MalhaDeVooService.loadVoos(widget, filter);
            widget.reload();
            widget.activate();
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.afterMoveRowDataVoos = function(widget) {
        if (widget.currentWidget && widget.currentWidget.name == 'malha_aerea_data_tabbed_voos') {
            var filter = [
                {
                    'name': 'CDFILIAL',
                    'operator': '=',
                    'value': widget.currentRow.CDFILIAL
                },
                {
                    'name': 'NRMALHAAEREA',
                    'operator': '=',
                    'value': widget.currentRow.NRMALHAAEREA
                },
                {
                    'name': 'CDCIA',
                    'operator': '=',
                    'value': widget.currentRow.CDCIA
                },
                {
                    'name': 'NRVOO',
                    'operator': '=',
                    'value': widget.currentRow.NRVOO
                },
                {
                    'name': 'NRORG',
                    'operator': '=',
                    'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];
            self.openSwipeDataVoos(widget.currentWidget, filter);
        }
    };
    this.openSwipeDataVoos = function(widget, filter) {
        var filterData = [];
        filterData = filter ? filter : [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': widget.parent.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': widget.parent.currentRow.NRMALHAAEREA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': widget.parent.currentRow.CDCIA
            },
            {
                'name': 'NRVOO',
                'operator': '=',
                'value': widget.parent.currentRow.NRVOO
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        MalhaDeVooService.loadDataVoos(widget, filterData);
        widget.reload();
        widget.activate();
    };

    this.afterMoveRowClasseVoos = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = true;
        }
        if (widget.currentWidget && widget.currentWidget.name == 'malha_aerea_classes_tabbed_classes') {
            var filter = [
                {
                    'name': 'CDFILIAL',
                    'operator': '=',
                    'value': widget.currentRow.CDFILIAL
            },
                {
                    'name': 'NRMALHAAEREA',
                    'operator': '=',
                    'value': widget.currentRow.NRMALHAAEREA
            },
                {
                    'name': 'NRSEQMALHADATA',
                    'operator': '=',
                    'value': widget.currentRow.NRSEQMALHADATA
            },
                {
                    'name': 'CDCIA',
                    'operator': '=',
                    'value': widget.currentRow.CDCIA
            },
                {
                    'name': 'NRVOO',
                    'operator': '=',
                    'value': widget.currentRow.NRVOO
            },
                {
                    'name': 'NRORG',
                    'operator': '=',
                    'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
            ];
            self.openSwipeClasseVoos(widget.currentWidget, filter);
        }
    };
    this.openSwipeClasseVoos = function(widget, filter) {
        var filterData = [];
        filterData = filter ? filter : [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': widget.parent.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': widget.parent.currentRow.NRMALHAAEREA
            },
            {
                'name': 'NRSEQMALHADATA',
                'operator': '=',
                'value': widget.parent.currentRow.NRSEQMALHADATA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': widget.parent.currentRow.CDCIA
            },
            {
                'name': 'NRVOO',
                'operator': '=',
                'value': widget.parent.currentRow.NRVOO
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        MalhaDeVooService.loadClassesVoo(widget, filterData);
        widget.reload();
        widget.activate();
    };

    this.limpaField = function(field) {
        if (field.name == 'NMFILIAL') {
            field.widget.currentRow.NMCIA = "";
        }
    };

    this.adicionar = function(widget) {
        widget.newRow();
        widget.activate();

        ScreenService.openSwipe(widget);
    };

    this.changeTipo = function(field, tipo) {
        if (tipo == "2") {
            field.readOnly = false;
            field.validations.required = true;
        } else {
            field.readOnly = true;
            field.setValue(null);
            field.validations = {};
        }
    };

    this.incluirCompanhia = function(widget, widgetCompanhia) {
        MalhaDeVooService.findMalhaAerea([{
            'name': 'CDFILIAL',
            'operator': '=',
            'value': widgetCompanhia.__filterPopup.currentRow.CDFILIAL
            }]).then(function(callback) {
            widgetCompanhia.currentRow.CDCIA = widget.currentRow.CDCIA;
            widgetCompanhia.currentRow.__is_new = true;
            if (callback && callback.dataset.malhaaerea.length) {
                widgetCompanhia.currentRow.CDFILIAL = callback.dataset.malhaaerea[0].CDFILIAL;
                widgetCompanhia.currentRow.NRMALHAAEREA = callback.dataset.malhaaerea[0].NRMALHAAEREA;
                widgetCompanhia.saveRow().then(function() {
                    widgetCompanhia.currentRow.__is_new = true;
                    widgetCompanhia.currentRow.changed = false;
                    if (widgetCompanhia) {
                        ScreenService.closePopup();
                        widgetCompanhia.reload();
                        widgetCompanhia.activate();
                    }
                });
            } else {
                var rowToSave = {
                    "CDFILIAL": widgetCompanhia.__filterPopup.currentRow.CDFILIAL,
                    "DTHRGERACAO": moment().format("DD/MM/YYYY"),
                    "CDOPERGERACAO": getLocalVar('INFLIGHTOR_CDOPERADOR'),
                    "__is_new": true,
                    "__using_unit": false,
                    "__is_new_code": true,
                    "__new_code_pk": "MALHAAEREA" + widgetCompanhia.__filterPopup.currentRow.CDFILIAL,
                    "__new_code_length": 5,
                    "__table_pk": "NRMALHAAEREA",
                    "__columns_to_return": ["NRMALHAAEREA", "CDFILIAL"]
                };
                MalhaDeVooService.saveMalha(rowToSave).then(function(callback) {
                    widgetCompanhia.currentRow.NRMALHAAEREA = parseInt(callback.dataset.columnsToReturn[0].NRMALHAAEREA);
                    widgetCompanhia.currentRow.CDFILIAL = callback.dataset.columnsToReturn[0].CDFILIAL;
                    MalhaDeVooService.saveCompanhia(widgetCompanhia.currentRow).then(function() {
                        widgetCompanhia.currentRow.__is_new = true;
                        widgetCompanhia.currentRow.changed = false;
                        if (widgetCompanhia) {
                            ScreenService.closePopup();
                            widgetCompanhia.reload();
                            widgetCompanhia.activate();
                        }
                    });
                }, function(error) {
                    InflightorGeneral.callbackError(error);
                });
            }
        });
    };

    this.incluirVoos = function(widget, param) {
        var dadosDoVooWidget = widget.container.getWidget('malha_aerea_voo_tabbed_voos');
        var companhiasWidget = widget.container.getWidget('malha_aerea_cia_tabbed_companhias');

        if (widget.isValid()) {
            if (widget.getField('DESCRIPTION').isVisible == true) {
                validaRowToSave(widget, companhiasWidget);
                widget.currentRow.__new_voo = true;
                InflightorGeneral.callRestEngine('pcp#/malhaaereadata/saveDatasVoo', "Row", widget.currentRow).then(function(success) {
                    ScreenService.goBack();
                    reloadVoos(dadosDoVooWidget);
                    dadosDoVooWidget.reload();
                    dadosDoVooWidget.activate();
                    widget.list();
                }, function(error) {
                    InflightorGeneral.callbackError(error);
                });
            } else {
                validaRowToSave(widget, companhiasWidget);
                widget.currentRow.CDBASEDES = widget.parent.currentRow.CDBASEDES;
                widget.currentRow.__new_voo = false;
                InflightorGeneral.callRestEngine('pcp#/malhaaereadata/saveDatasVoo', "Row", widget.currentRow).then(function(success) {
                    reloadDataVoos(widget);
                    widget.reload().then(function() {
                        widget.view();
                        widget.activate();
                    });
                }, function(error) {
                    InflightorGeneral.callbackError(error);
                });

            }
            widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = true;
        }
    };

    this.salvaClasse = function(widget, status) {
        var dadosDoVooWidget = widget.container.getWidget('malha_aerea_voo_tabbed_voos');
        var companhiasWidget = widget.container.getWidget('malha_aerea_cia_tabbed_companhias');

        widget.currentRow.CDCIA = widget.parent.currentRow.CDCIA;
        widget.currentRow.NRVOO = widget.parent.currentRow.NRVOO;
        widget.currentRow.CDFILIAL = widget.parent.currentRow.CDFILIAL;
        widget.currentRow.CDTRECHO = widget.parent.currentRow.CDTRECHO;
        widget.currentRow.QTPAXPREVINF = widget.currentRow.QTMAXIMAPAX;
        widget.currentRow.QTPAXPREVCALC = widget.currentRow.QTMAXIMAPAX;
        widget.currentRow.NRMALHAAEREA = widget.parent.currentRow.NRMALHAAEREA;
        widget.currentRow.NRSEQMALHADATA = widget.parent.currentRow.NRSEQMALHADATA;
        widget.currentRow.VRPERCOCUPACAO = widget.currentRow.IDTIPOCALCULO != "2" ? null : widget.currentRow.VRPERCOCUPACAO;

        if (widget.isValid()) {
            InflightorGeneral.callRestEngine('pcp#/crud/malhaaereaclasse/save', "Row", widget.currentRow).then(function(success) {
                if (widget.currentRow.__is_new)
                    ScreenService.goBack();
                else widget.view();
                reloadClasse(widget);
                widget.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    var validaRowToSave = function(widget, companhiasWidget) {
        widget.currentRow.__is_new = widget.currentRow.__is_new ? widget.currentRow.__is_new : true;
        widget.currentRow.changed = false;
        widget.currentRow.CDCIA = companhiasWidget.currentRow.CDCIA;
        widget.currentRow.CDFILIAL = companhiasWidget.currentRow.CDFILIAL;
        widget.currentRow.NRMALHAAEREA = companhiasWidget.currentRow.NRMALHAAEREA;
        widget.currentRow.__using_unit = false;
        widget.currentRow.__is_new_code = true;
        widget.currentRow.__new_code_pk = 'MALHAAEREADATA' + widget.currentRow.CDFILIAL + widget.currentRow.CDCIA + widget.currentRow.NRMALHAAEREA;
        widget.currentRow.__new_code_length = 5;
        widget.currentRow.__table_pk = "NRSEQMALHADATA";
        widget.currentRow.HRETD = widget.currentRow.HRETD.replace(':', '');
        widget.currentRow.HRKDT = widget.currentRow.HRKDT.replace(':', '');
        widget.currentRow.HRKRT = widget.currentRow.HRKRT.replace(':', '');
        widget.currentRow.HRQUEBRA = widget.currentRow.HRQUEBRA.replace(':', '');
    };

    this.deleteDatas = function(widget, row) {
        ScreenService.confirmMessage(ScreenService.i18n('Deletar Registro?'), 'question', function(success) {
            if (widget.dataSource.data.length == 1) {
                ScreenService.confirmMessage(ScreenService.i18n('Última Data do Voo! Ao excluir último registro o voo também será deletado. <br> Confirma Operação?'), 'question', function(success) {
                    row.__remove_voo = true;
                    InflightorGeneral.callRestEngine('pcp#/malhaaereadata/removeDatasVoo', "Row", row).then(function(success) {
                        ScreenService.showMessage(success.messages[0].message);
                        ScreenService.closeSwipe(1).then(function() {
                            self.refreshVoos();
                            widget.container.getWidget('malha_aerea_voo_tabbed_voos').activate();
                        });
                    }, function(error) {
                        ScreenService.showMessage(error.data.error);
                    });
                });
            } else {
                row.__remove_voo = false;
                InflightorGeneral.callRestEngine('pcp#/malhaaereadata/removeDatasVoo', "Row", row).then(function(success) {
                    ScreenService.goBack().then(function() {
                        ScreenService.showMessage(success.messages[0].message);
                        self.refreshData();
                        widget.container.getWidget('malha_aerea_data_tabbed_voos').activate();
                    });
                }, function(error) {
                    InflightorGeneral.callbackError(error);
                });
            }
        });
    };

    this.deletaClasse = function(widget, row) {
        ScreenService.confirmMessage(ScreenService.i18n('Deletar Registro?'), 'question', function(success) {
            MalhaDeVooService.deletaClasse(row).then(function() {
                reloadClasse(widget);
                ScreenService.goBack();
                widget.reload().then(function() {
                    widget.activate();
                });
            });
        });
    };

    this.delete = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            widget.removeCheckedRows().then(function() {
                widget.reload();
                widget.activate();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    var reloadVoos = function(widget) {
        var companhiasWidget = widget.container.getWidget('malha_aerea_cia_tabbed_companhias');
        var voosWidget = widget.container.getWidget('malha_aerea_voo_tabbed_voos');
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': companhiasWidget.currentRow.NRMALHAAEREA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDCIA
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        MalhaDeVooService.loadVoos(voosWidget, filter);
    };

    var reloadDataVoos = function(widget) {
        var companhiasWidget = widget.container.getWidget('malha_aerea_cia_tabbed_companhias');
        var voosWidget = widget.container.getWidget('malha_aerea_voo_tabbed_voos');

        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': companhiasWidget.currentRow.NRMALHAAEREA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDCIA
            },
            {
                'name': 'NRVOO',
                'operator': '=',
                'value': voosWidget.currentRow.NRVOO
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        MalhaDeVooService.loadDataVoos(widget, filter);
        widget.reload();
        widget.activate();
    };

    var reloadClasse = function(widget) {
        var companhiasWidget = widget.container.getWidget('malha_aerea_cia_tabbed_companhias');
        var datasWidget = widget.container.getWidget('malha_aerea_data_tabbed_voos');

        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDFILIAL
            },
            {
                'name': 'NRMALHAAEREA',
                'operator': '=',
                'value': companhiasWidget.currentRow.NRMALHAAEREA
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': companhiasWidget.currentRow.CDCIA
            },
            {
                'name': 'NRVOO',
                'operator': '=',
                'value': datasWidget.currentRow.NRVOO
            },
            {
                'name': 'NRSEQMALHADATA',
                'operator': '=',
                'value': datasWidget.currentRow.NRSEQMALHADATA
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        MalhaDeVooService.loadClassesVoo(widget, filter);
    };

    this.backToMainWidget = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.container.getWidget('malha_aerea_cia_tabbed_companhias').activate();
        });
    };

    this.onEnterCiaPopup = function(mainWidget, widget, status) {
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': mainWidget.currentRow.CDFILIAL
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        widget.dataSourceFilter = filter;
        widget.activate();
        widget.reload();
    };

    var validaDias = function(widget) {
        var dataInicial = widget.currentRow.DTINICIAL;
        dataInicial = moment(dataInicial, 'DD/MM/YYYY');
        widget.getField('DTFINAL').minDate = moment(dataInicial, 'DD/MM/YYYY');
    };

    this.openAddSwipe = function(widget, param) {
        if (widget.name == 'malha_aerea_data_tabbed_voos') {
            if (!param) {
                ScreenService.openSwipe(widget).then(function() {
                    widget.newRow();
                    widget.activate();
                    widget.getField('SGEQUIP').isVisible = true;
                    widget.getField('DESCRIPTION').isVisible = true;
                    widget.getField('FREQUENCIA').isVisible = false;
                    widget.getField('DESCRIPTION').readOnly = false;
                    widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = false;
                });
            } else {
                widget.newRow();
                widget.activate();
                widget.getField('SGEQUIP').isVisible = true;
                widget.getField('DESCRIPTION').isVisible = false;
                widget.getField('ORIGEM').isVisible = false;
                widget.getField('CDTRECHO').isVisible = false;
                widget.getField('FREQUENCIA').isVisible = false;
                widget.currentRow.NRVOO = widget.parent.currentRow.NRVOO;
                widget.currentRow.NRSEQMALHADATA = widget.parent.currentRow.NRSEQMALHADATA;
                widget.currentRow.CDTRECHO = widget.parent.currentRow.CDTRECHO;
                widget.currentRow.NRSEQMALHADATA = widget.parent.currentRow.NRSEQMALHADATA;
                widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = false;
            }
        } else if (widget.name == 'malha_aerea_classes_tabbed_classes') {
            widget.getField('SGCLASSE').readOnly = false;
            if (param) {
                widget.newRow();
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = true;
            } else {
                widget.getField('SGCLASSE').readOnly = true;
                var ocupacaoField = widget.getField('VRPERCOCUPACAO');
                ocupacaoField.readOnly = widget.currentRow.IDTIPOCALCULO != '2';
                if (widget.currentRow.IDTIPOCALCULO == '2')
                    ocupacaoField.validations.required = true;
                else
                    ocupacaoField.validations = {};
                InflightorGeneral.edit(widget);
                widget.currentRow.changed = true;
                widget.currentRow.__is_new = false;
            }
        } else {
            ScreenService.openPopup(widget);
        }
    };

    this.backDataVoo = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.parent.activate();
        });
    };

    this.habilitaFieldData = function(widget) {
        widget.currentRow.DTFINAL = "";
        widget.getField('DTFINAL').readOnly = true;
        widget.getField('DTFINAL').reload();

        if (widget.currentRow.DTINICIAL) {
            widget.getField('DTFINAL').readOnly = false;
            validaDias(widget);
        }
    };

    this.editaVoo = function(widget) {
        widget.edit();
        widget.currentRow.__is_new = false;
        widget.getField('FREQUENCIA').isVisible = false;
        widget.getField('DESCRIPTION').isVisible = false;
        widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = false;
    };

    this.cancelaVoo = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.list();
            if (widget.getField('DESCRIPTION').isVisible == true) {
                widget.container.getWidget('malha_aerea_voo_tabbed_voos').activate();
            } else {
                widget.container.getWidget('malha_aerea_data_tabbed_voos').activate();
            }
            widget.container.getWidget('malha_aerea_classes_tabbed_classes').isVisible = true;
        });
    };

    this.unidadeToCompanhia = function(field) {
        var cdfilial = field.widget.currentRow.CDFILIAL;
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': cdfilial
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        if (!field.widget.currentRow.CDFILIAL) {
            field.dataSourceFilter = FILTROCLEAR;
            field.reload();
            msg = ScreenService.i18n("Escolha uma Filial para definir a Companhia.");
            ScreenService.showMessage(msg);
            return;
        }
        field.widget.getField('NMCIA').dataSourceFilter = filter;
        field.widget.getField('NMCIA').reload();
    };

    this.companhiaDependence = function(field) {
        var cdfilial = field.widget.container.getWidget('malha_aerea_cia_tabbed_companhias').currentRow.CDFILIAL;
        var cdcia = field.widget.container.getWidget('malha_aerea_cia_tabbed_companhias').currentRow.CDCIA;
        var filter = [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': cdfilial
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': cdcia
            },
            {
                'name': 'NRORG',
                'operator': '=',
                'value': getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        field.dataSourceFilter = filter;
        field.reload();
    };

    this.setMaxClass = function(field) {
        var widget = field.widget;
        var datasWidget = widget.container.getWidget('malha_aerea_data_tabbed_voos');
        InflightorGeneral.callRestEngine('pcp#/crud/config_pax_voo', "FilterData", createFilterMaxClass(datasWidget, widget.currentRow.CDCLASSE)).then(function(callback) {
            if (callback.dataset && callback.dataset.config_pax_voo.length) {
                widget.getField('QTMAXIMAPAX').setValue(callback.dataset.config_pax_voo[0]);
            }
        });
    };

    function createFilterMaxClass(widget, classeValue) {
        return [
            {
                'name': 'CDFILIAL',
                'operator': '=',
                'value': widget.currentRow.CDFILIAL
            },
            {
                'name': 'CDCIA',
                'operator': '=',
                'value': widget.currentRow.CDCIA
            },
            {
                'name': 'NRVOO',
                'operator': '=',
                'value': widget.currentRow.NRVOO
            },
            {
                'name': 'CDTRECHO',
                'operator': '=',
                'value': widget.currentRow.CDTRECHO
            },
            {
                'name': 'CDCLASSE',
                'operator': '=',
                'value': classeValue
            },
        ];
    }

    this.refreshVoos = function(widget = templateManager.container.getWidget('malha_aerea_voo_tabbed_voos')) {
        reloadVoos(widget);
        widget.reload();
    };
    this.refreshData = function(widget = templateManager.container.getWidget('malha_aerea_data_tabbed_voos')) {
        reloadDataVoos(widget);
        widget.reload();
    };
    this.refreshClasse = function(widget = templateManager.container.getWidget('malha_aerea_classes_tabbed_classes')) {
        reloadClasse(widget);
        widget.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('MalhaDeVooController', MalhaDeVooController);
});

// FILE: ./assets/js/controllers/OrdemClasseProducaoController.js
var OrdemClasseProducaoController = function(ScreenService, InflightorGeneral, restEngine) {

    var _self = this;

    function arrayDistinct(a, param) {
        return a.filter(function(item, pos, array) {
            return array.map(function(mapItem) {
                    return mapItem[param];
                })
                .indexOf(item[param]) === pos;
        });
    }

    this.cancelEdit = function(widget) {
        if (widget.inEditMode()) {
            widget.list();
            widget.dataSource.data.pop();
            widget.currentRow = {};
            widget.activate();
        } else {
            widget.rollbackInlineEdit();
            widget.list();
            widget.activate();
        }
    };

    this.newOrdem = function(widget) {
        widget.activate();
        widget.newRow();

        if (!Util.isEmptyOrBlank(widget.appliedFilter.FILTER_ROW)) {
            widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
            widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        }

        widget.fields.forEach(function(field) {
            if (field.name == 'NRORDEM' || field.name == 'DSCLASSE') {
                field.setValue("");
            }
        });
    };

    this.deleteOrdem = function(widget) {
        let ordens = widget.dataSource.getCheckedRows();
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            _effect(widget, ordens, 'remove');
        });
    };

    function createFilterSave(widget) {
        return [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
        },
            {
                "name": "NRORDEM",
                "operator": "=",
                "value": widget.currentRow.NRORDEM
        }
      ];
    }

    function changedOrdens(ordens) {
        let classes = [];
        classes = InflightorGeneral.sortArrayByProperty(ordens, ['NRORDEM']);
        for (let i = 0; i < classes.length; i++) {
            if (i) {
                let diff = classes[i]['NRORDEM'] - classes[i - 1]['NRORDEM'];
                if (!diff) {
                    classes[i]['NRORDEM']++;
                    classes[i]['__is_new'] = false;
                    classes[i]['changed'] = true;
                }
            }
        }
        return classes;
    }

    function _chagendAndSave(widget, filter) {
        let classesChanged = [];
        classesChanged = widget.dataSource.data.filter(function(row) {
            return filter <= parseInt(row.NRORDEM);
        });
        if (filter) {
            classesChanged.unshift(classesChanged.pop());
        }
        novasOrdens = changedOrdens(classesChanged);
        _effect(widget, novasOrdens, 'save');
    }

    this.saveOrdem = function(widget) {
        let novasOrdens = [];
        if (widget.inEditMode()) {
            //Salva nova Ordem
            widget.currentRow.NRORDEM = parseInt(widget.currentRow.NRORDEM);
            if (widget.currentRow.NRORDEM > 0) {
                let filter = createFilterSave(widget);
                InflightorGeneral.callRestEngine('pcp#/crud/ordclasseprod', 'FilterData', filter).then(function(callback) {
                    if (callback.dataset && callback.dataset.ordclasseprod && callback.dataset.ordclasseprod.length) {
                        let msg = 'A ordem informada já existe. Salvar e redistribuir ordens?';
                        let icon = 'question';
                        ScreenService.confirmMessage(ScreenService.i18n(msg), icon, function(callback) {
                            _chagendAndSave(widget, widget.currentRow.NRORDEM);
                        });
                    } else {
                        _effect(widget, [widget.currentRow], 'save');
                    }
                });
            } else {
                ScreenService.showMessage(ScreenService.i18n("Ordem inválida."), 'error');
            }
        } else {
            //Edição da Ordem na linha
            let ordemInvalid = false;
            widget.dataSource.data.forEach(function(row) {
                if (row.changed) {
                    row.NRORDEM = row.NRORDEM_inline !== row.NRORDEM ? parseInt(row.NRORDEM_inline) : parseInt(row.NRORDEM);
                }
                if (row.NRORDEM <= 0) {
                    ordemInvalid = true;
                }
            });
            let distinctOrdem = arrayDistinct(widget.dataSource.data, 'NRORDEM');
            if (distinctOrdem.length !== widget.dataSource.data.length) {
                ordemInvalid = true;
            }
            if (ordemInvalid) {
                ScreenService.showMessage(ScreenService.i18n("Ordem inválida."), 'error');
            } else {
                _chagendAndSave(widget, -1);
            }
        }
    };

    function _effect(widget, ordens, operation) {
        InflightorGeneral.callRestEngine('pcp#/crud/ordclasseprod/' + operation, 'DataSet', ordens).then(function(callback) {
            if (widget.inEditMode()) {
                widget.reload().then(function() {
                    widget.list();
                    widget.activate();
                });
            } else {
                widget.reload().then(function() {
                    if (operation === 'remove') {
                        widget.unSelectAll();
                    }
                    widget.list();
                    widget.activate();
                });
            }
        }, InflightorGeneral.callbackError);
    }
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('OrdemClasseProducaoController', OrdemClasseProducaoController);
});

// FILE: ./assets/js/controllers/PadraoProducaoController.js
var PadraoProducaoController = function(ScreenService, InflightorGeneral, maskEngine, ZhUtil, templateManager, MessageService) {

    var self = this;

    var mensagensGerais = {};

    this.getMensagensGerais = function() {
        MessageService.getMessagesByCode(["PCP00005_PADRAO"]).then(function(mensagens) {
            mensagensGerais.padrao = mensagens[0];
        });
    };

    function _factoryFilter(widget, filter) {
        let dataSourceFilter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        if (filter && filter.length) {
            filter.forEach(function(obj) {
                dataSourceFilter.push({
                    "name": obj["name"],
                    "operator": "=",
                    "value": obj["value"]
                });
            });
        }
        widget.dataSourceFilter = dataSourceFilter;
    }

    this.afterMoveRow = function(widget) {
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        let activateWidget = widget.container.getActiveWidget();
        if (activateWidget && activateWidget.name === 'variacoesReceita') {
            _factoryFilter(activateWidget, [{
                "name": "CDRECEITA",
                "value": widget.currentRow.CDRECEITA
            }]);
            activateWidget.reload().then(function() {
                activateWidget.activate();
            });
        } else if (activateWidget && widget.mode !== 'edit' && activateWidget.mode !== 'edit' && (activateWidget.name === 'widgetPadrao' || activateWidget.name === 'widgetTipoNegocioGrid')) {
            let widgetTipoNegocio = widget.container.getWidget('widgetTipoNegocioGrid');
            ScreenService.openSwipe(widget.container.getWidget('widgetTipoNegocioTabbed')).then(function() {
                _factoryFilter(widgetTipoNegocio, [{
                    "name": "CDRECEITA",
                    "value": widget.currentRow.CDRECEITA
                    }, {
                    "name": "CDFILIAL",
                    "value": widget.currentRow.CDFILIAL
                    }]);
                widgetTipoNegocio.reload().then(function() {
                    widgetTipoNegocio.activate();
                });
            });
        }
    };

    this.newRow = function(widget) {
        if (!widget.appliedFilter) {
            MessageService.showMessageByCode("PCP00005_FILTRO_DEVE_SER_APLICADO");
            return;
        }
        let popupAddReceita = templateManager.container.getWidget("popupAddReceita");
        popupAddReceita.newRow();
        popupAddReceita.getField('CDFILIAL').setValue(widget.appliedFilter.FILTER_ROW.CDFILIAL);
        popupAddReceita.getField('NMFILIAL').setValue(widget.appliedFilter.FILTER_ROW.NMFILIAL);

        ScreenService.openPopup(popupAddReceita);
        var detalhesReceitaAction = widget.getAction('detalhesReceita');
        detalhesReceitaAction.isVisible = false;
    };

    this.addTipoNegocio = function(widget, newReceita = false) {
        let widgetPadrao = widget.container.getWidget('widgetPadrao');
        widget.newRow();
        widget.newReceita = newReceita;
        widget.currentRow = widgetPadrao.currentRow;
        widget.currentRow.__is_new = true;
        widget.getField('DSTIPONEGOCIOINF').readOnly = false;
        widget.getField('DSRECEITA').readOnly = !newReceita;
        widget.getField('NMCATEGORIAPROD').readOnly = true;
        widget.getField('NMSUBCATEGORIAPROD').readOnly = true;
        widget.getField('DSTIPONEGOCIOINF').setValue("");
        let filterUnitSetor = widget.getField('NMSETOR').dataSourceFilter.find(cFilter => cFilter.name == 'CDFILIAL');
        if (filterUnitSetor) {
            filterUnitSetor.value = widgetPadrao.currentRow.CDFILIAL;
        }
        widget.fields.forEach(function(field) {
            if (field.fieldGroup && field.fieldGroup === "CONFIG")
                field.setValue("");
        });
    };

    this.setFilter = function(field) {
        let widget = field.widget;
        if (widget.mode == 'edit') {
            if (field.name === 'DSTIPONEGOCIOINF')
                _factoryFilter(field, []);
            else
                _factoryFilter(field, [{
                    "name": "CDFILIAL",
                    "value": widget.currentRow.CDFILIAL
                }]);
        }
    };

    this.setFilterNegocioReceita = function(field) {
        let widget = field.widget;
        if (widget.mode == 'edit') {

            _factoryFilter(field, [
                {
                    "name": "CDFILIAL",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDRECEITA",
                    "value": widget.currentRow.CDRECEITA
                }
            ]);
        }
    };

    this.edit = function(widget) {
        widget.getField('NMCATEGORIAPROD').dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDSETOR",
                "operator": "=",
                "value": widget.currentRow.CDSETOR
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widget.currentRow.NRORG
            }
        ];
        widget.getField('NMCATEGORIAPROD').readOnly = false;

        widget.getField('NMSUBCATEGORIAPROD').dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "CDSETOR",
                "operator": "=",
                "value": widget.currentRow.CDSETOR
            },
            {
                "name": "NRCATEGORIAPROD",
                "operator": "=",
                "value": widget.currentRow.NRCATEGORIAPROD
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widget.currentRow.NRORG
            }
        ];
        widget.getField('NMSUBCATEGORIAPROD').readOnly = false;
        if (widget.name != 'widgetTipoNegocioGrid') {
            var detalhesReceitaAction = widget.getAction('detalhesReceita');
            detalhesReceitaAction.isVisible = true;
        } else {
            widget.getField('DSTIPONEGOCIOINF').readOnly = true;
        }
        widget.currentRow.__is_new = false;
        InflightorGeneral.edit(widget);
    };

    this.receitaOnChange = function(widget) {
        if (widget.currentRow.CDSETOR) {
            var filtro = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];
            InflightorGeneral.callRestEngine('pcp#/crud/setorfilial', 'FilterData', filtro).then(
                function(callback) {
                    if (!(callback.dataset && callback.dataset.setorfilial && callback.dataset.setorfilial.length > 0)) {
                        MessageService.showMessageByCode("PCP00005_SETOR_N_VINC_FILIAL", {
                            "0": widget.currentRow.NMSETOR,
                            "1": widget.currentRow.NMFILIAL
                        });
                        widget.currentRow.NMSETOR = '';
                        widget.currentRow.CDSETOR = '';
                    }
                    self.setorOnChange(widget);
                },
                function(erro) {
                    if (erro.data && erro.data.error) {
                        ScreenService.showMessage(erro.data.error);
                    } else {
                        MessageService.showMessageByCode("PCP00005_ERRO_AO_VALIDAR_REC");
                    }
                    widget.currentRow.NMSETOR = '';
                    widget.currentRow.CDSETOR = '';
                    self.setorOnChange(widget);
                });
        }

        var detalhesReceitaAction = widget.getAction('detalhesReceita');
        if (detalhesReceitaAction)
            detalhesReceitaAction.isVisible = !!widget.currentRow.CDRECEITA;
    };

    this.setorOnChange = function(widget) {
        var fieldCategoria = widget.getField('NMCATEGORIAPROD');

        widget.currentRow.NRCATEGORIAPROD = '';
        widget.currentRow.NMCATEGORIAPROD = '';

        if (widget.currentRow.CDSETOR) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldCategoria.readOnly = false;
            fieldCategoria.dataSourceFilter = dataSourceFilter;
        } else {
            fieldCategoria.readOnly = true;
        }

        self.categoriaOnChange(widget);
    };

    this.categoriaOnChange = function(widget) {
        var fieldSubcategoria = widget.getField('NMSUBCATEGORIAPROD');

        widget.currentRow.NRSUBCATEGORIAPROD = '';
        widget.currentRow.NMSUBCATEGORIAPROD = '';

        if (widget.currentRow.NRCATEGORIAPROD) {
            var dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": widget.currentRow.CDSETOR
                },
                {
                    "name": "NRCATEGORIAPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRCATEGORIAPROD
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
                }
            ];

            fieldSubcategoria.readOnly = false;
            fieldSubcategoria.dataSourceFilter = dataSourceFilter;
        } else {
            fieldSubcategoria.readOnly = true;
        }
    };

    this.openPopupDetalhesDaReceita = function(widgetForm) {
        var popupDetalhesReceita = widgetForm.container.getWidget('detalhesReceita');
        popupDetalhesReceita.label = widgetForm.currentRow.DSRECEITA;

        var composicaoReceita = widgetForm.container.getWidget('composicaoReceita');
        var modoPreparoReceita = widgetForm.container.getWidget('modoPreparoReceita');

        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetForm.currentRow.CDFILIAL
            },
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": widgetForm.currentRow.CDRECEITA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": widgetForm.currentRow.NRORG
            }
        ];

        composicaoReceita.dataSourceFilter = filter;
        composicaoReceita.reload();

        modoPreparoReceita.dataSourceFilter = filter;
        modoPreparoReceita.reload().then(function(success) {
            var modPrepData = modoPreparoReceita.dataSource.data;
            modoPreparoReceita.currentRow = (modPrepData && modPrepData[0]) ? modPrepData[0] : {};
        });

        ScreenService.openPopup(popupDetalhesReceita);
        popupDetalhesReceita.activate();
    };

    this.cancelTipoNegocio = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.list();
            widget.reload().then(function() {
                widget.activate();
            });
        });
    };

    this.closePopupDetalhesDaReceita = function(container) {
        var widgetForm = container.getWidget('widgetPadrao');
        ScreenService.closePopup();
        widgetForm.activate();
    };

    this.afterMoveRowTipoNegocio = function(widget) {
        let oldRow = angular.copy(widget.currentRow);
        if (!widget.currentRow.NRTIPONEGOCIOINF)
            Object.assign(widget.currentRow, widget.container.getWidget('widgetPadrao').currentRow, oldRow);
    };

    function _rowToSave(widget) {
        let widgetPadrao = widget.container.getWidget('widgetPadrao');
        let oldRow = angular.copy(widget.currentRow);
        Object.assign(widget.currentRow, widgetPadrao.paramsNewCode, oldRow);
    }

    this.afterLoadDataSource = function(widget) {
        if (widget.dataSource.data) {
            widget.dataSource.data.forEach(function(row) {
                if (row.NRTIPONEGOCIOINF === null)
                    row.DSTIPONEGOCIOINF = mensagensGerais.padrao;
            });
            widget.dataSource.data = InflightorGeneral.sortArrayByProperty(widget.dataSource.data, ["NRTIPONEGOCIOINF", "DSTIPONEGOCIOINF"]);
        }
    };
    this.backListTipoNegocio = function(args, widget) {
        InflightorGeneral.backOrCancel(args).then(function() {
            if (widget.modify)
                widget.reload().then(function() {
                    widget.activate();
                });
        });
    };
    this.cancelNewTipoNegocio = function(widget) {
        if (widget.currentRow.__is_new) {
            widget.list();
            widget.reload();
            ScreenService.closeSwipe();
        } else {
            InflightorGeneral.widgetCancelForm(widget);
        }
    };

    this.cancelNovaReceita = function() {
        ScreenService.closePopup();

        let widgetPadrao = templateManager.container.getWidget('widgetPadrao');
        widgetPadrao.reload().then(function() {
            widgetPadrao.activate();
        });
    };

    function _unSelectAll(widget) {
        widget.fields.forEach(function(field) {
            if (field.selectWidget)
                field.selectWidget.unSelectAll();
        });
    }

    this.saveProducaoReceita = function(widget) {
        InflightorGeneral.saveCrud(widget).then(function() {
            _unSelectAll(widget);
        });
    };
    this.saveTipoNegocio = function(widget) {
        if (!widget.isValid())
            return;
        _rowToSave(widget);
        InflightorGeneral.callRestEngine('pcp#/crud/padraoprodrec/save', 'Row', widget.currentRow).then(function(success) {

            if (widget.currentRow.__is_new) {
                widget.list();
                widget.reload();
                ScreenService.closeSwipe();
            } else {
                widget.view();
            }
            MessageService.showMessageByCode("PCP00005_OPERACAO_SUCES");

        }, function(error) {
            if (error.data && error.data.error.indexOf('ORA-00001') !== -1) {
                MessageService.showMessageByCode("PCP00005_REGISTRO_JA_EXISTE");
            } else {
                InflightorGeneral.callbackError(error);
            }
        });
    };

    this.saveNovaReceita = function(widget) {
        if (!widget.isValid())
            return;
        _rowToSave(widget);
        InflightorGeneral.callRestEngine('pcp#/crud/padraoprodrec/save', 'Row', widget.currentRow).then(function(success) {

            self.cancelNovaReceita();
            MessageService.showMessageByCode("PCP00005_OPERACAO_SUCES");
        }, function(error) {
            if (error.data && error.data.error.indexOf('ORA-00001') !== -1) {
                MessageService.showMessageByCode("PCP00005_REGISTRO_JA_EXISTE");
            } else {
                InflightorGeneral.callbackError(error);
            }
        });
    };

    this.deleteTipoNegocio = function(widget) {
        InflightorGeneral.deleteCrud(widget).then(function(success) {
            widget.reload().then(function() {
                if (widget.dataSource.data && !widget.dataSource.data.length) {
                    ScreenService.closeSwipe(0).then(function() {
                        widget.parent.parent.reload().then(function() {
                            widget.parent.parent.activate();
                        });
                    });
                } else {
                    widget.reload().then(function() {
                        widget.activate();
                    });
                }
            });
        }, InflightorGeneral.callbackError);
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('PadraoProducaoController', PadraoProducaoController);
});

// FILE: ./assets/js/controllers/PadroesGeracaoPrevisao.js
var PadroesGeracaoPrevisao = function(InflightorGeneral, ScreenService, templateManager, restEngine, metaDataFactory, ZHPromise) {
    var self = this;

    this.getCrosstabData = function(crosstabWidget) {
        var filter = crosstabWidget.dataSourceFilter;
        InflightorGeneral.callRestEngine("pcp#/crud/itpadraogerprevpax_crosstab", "FilterData", filter).then(function(response) {
            if (response && response.dataset && response.dataset.itpadraogerprevpax_crosstab) {
                var data = response.dataset.itpadraogerprevpax_crosstab;
                buildCrosstabData(crosstabWidget, data);
            }
        });
    };

    var buildCrosstabData = function(crosstabWidget, data) {
        var distClasse = [];
        var distVoo = {};

        crosstabWidget.fields = [];
        crosstabWidget._fields.forEach(function(field) {
            crosstabWidget.fields.push(
                metaDataFactory.fieldFactory(field, crosstabWidget, false)
            );
        });

        var filterFunc = function(e) {
            return e.CDCLASSE == cdclasse;
        };

        for (var i = 0; i < data.length; i++) {
            var dataItem = data[i];

            var cdclasse = dataItem.CDCLASSE;
            if (distClasse.filter(filterFunc).length == 0) {
                distClasse.push({
                    'CDCLASSE': dataItem.CDCLASSE,
                    'SGCLASSE': dataItem.SGCLASSE,
                    'NRORDEM': dataItem.NRORDEM
                });
            }

            var vooID = dataItem.CDCIA + '|' +
                dataItem.NRVOO + '|' +
                dataItem.CDTRECHO + '|' +
                dataItem.CDEQUIP;

            if (!distVoo[vooID]) {
                distVoo[vooID] = {
                    'CDFILIAL': dataItem.CDFILIAL,
                    'CDCIA': dataItem.CDCIA,
                    'NRVOO': dataItem.NRVOO,
                    'CDTRECHO': dataItem.CDTRECHO,
                    'CDEQUIP': dataItem.CDEQUIP,
                    'CDCLASSE': dataItem.CDCLASSE,
                    'NRPADRAOGERPREVPAX': dataItem.NRPADRAOGERPREVPAX,
                    'NRITPADRAOGERPREVPAX': dataItem.NRITPADRAOGERPREVPAX,
                    'VOOCIA': dataItem.VOOCIA,
                    'TRECHO': dataItem.TRECHO,
                    'SGEQUIP': dataItem.SGEQUIP,
                    'NRORG': dataItem.NRORG,
                    'classeArr': {}
                };
            }
            distVoo[vooID]['classeArr'][cdclasse] = {
                "QTITEM": dataItem.QTITEM,
                "NRITPADRAOGERPREVPAX": dataItem.NRITPADRAOGERPREVPAX
            };
        }

        var newData = [];

        for (var voo_ID in distVoo) {
            var vooObj = distVoo[voo_ID];
            var row = {
                'CDFILIAL': vooObj.CDFILIAL,
                'CDCIA': vooObj.CDCIA,
                'NRVOO': vooObj.NRVOO,
                'CDTRECHO': vooObj.CDTRECHO,
                'CDEQUIP': vooObj.CDEQUIP,
                'NRPADRAOGERPREVPAX': vooObj.NRPADRAOGERPREVPAX,
                'VOOCIA': vooObj.VOOCIA,
                'TRECHO': vooObj.TRECHO,
                'SGEQUIP': vooObj.SGEQUIP,
                'NRORG': vooObj.NRORG
            };
            for (i = 0; i < distClasse.length; i++) {
                var classeObj = distClasse[i];
                var fieldName = 'CLASSE' + classeObj['CDCLASSE'];
                row[fieldName] = '--';
            }

            for (var _cdclasse in vooObj.classeArr) {
                var classeID = 'CLASSE' + _cdclasse;
                var seqID = classeID + '_NRITPADRAOGERPREVPAX';

                row[classeID] = vooObj.classeArr[_cdclasse]['QTITEM'];
                row[seqID] = vooObj.classeArr[_cdclasse]['NRITPADRAOGERPREVPAX'];
            }

            newData.push(row);
        }

        crosstabWidget.dataSource.data = newData;

        distClasse = InflightorGeneral.sortArrayByProperty(distClasse, ['NRORDEM', 'SGCLASSE']);
        factorAndInjectClassesFields(crosstabWidget, distClasse);
    };

    var factorAndInjectClassesFields = function(crosstabWidget, distClasse) {
        var factorField = function(fieldName, fieldLabel, widget) {
            var field = {
                "name": fieldName,
                "label": fieldLabel,
                "template": "field/text-edit.html",
                "class": 4,
                "isVisible": true,
                "align": "right",
                "inlineEditable": "number",
                "showOnList": true,
                "maxlength": 3,
                "events": [
                    {
                        "name": "FieldInlineEdit",
                        "code": "PadroesGeracaoPrevisao.validateFieldInlineEdit(args.owner.field);"
					}
                ],
                "validations": {
                    "required": true
                }
            };
            return metaDataFactory.fieldFactory(field, widget, false);
        };

        for (var i = 0; i < distClasse.length; i++) {
            var classeObj = distClasse[i];

            var fieldName = 'CLASSE' + classeObj['CDCLASSE'];
            var fieldLabel = classeObj['SGCLASSE'];

            var classeField = factorField(fieldName, fieldLabel, crosstabWidget);
            crosstabWidget.fields.push(classeField);
        }
    };

    this.validateFieldInlineEdit = function(field) {
        var widget = field.widget;
        var currentRow = widget.currentRow;

        if (currentRow.changed) {
            var oldValue = currentRow['changed_' + field.name];
            var value = Math.round(InflightorGeneral.valueToFloat(field.value()));

            value = isNaN(value) ? null : value;
            value = (value < 0) ? 0 : value;

            if (value != oldValue) {
                if (oldValue == '--') {
                    var msg = ScreenService.i18n("Impossível editar! A classe selecionada não foi configurada para o voo!");
                    ScreenService.showMessage(msg, 'error');

                    field.setValue(oldValue);
                    delete currentRow['changed_' + field.name];
                    delete currentRow[field.name + '_inline'];
                } else {
                    field.setValue(value);
                    currentRow[field.name + '_inline'] = value;
                }
            } else {
                delete currentRow['changed_' + field.name];
                delete currentRow[field.name + '_inline'];
            }
        }
    };

    this.crosstabSaveInlineEdit = function(crosstabWidget) {
        if (crosstabWidget.dataSource && crosstabWidget.dataSource.data) {
            var data = crosstabWidget.dataSource.data;

            var dataSetToSave = [];
            var dataSetToDelete = [];

            var chngdIdnt = 'changed_CLASSE';

            for (var i = 0; i < data.length; i++) {
                var row = data[i];
                if (row.changed) {

                    for (var property in row) {
                        if (property.indexOf(chngdIdnt) == 0) {
                            var cdclasse = property.slice(chngdIdnt.length);

                            var oldKey = property;
                            var oldValue = row[oldKey];
                            var newKey = 'CLASSE' + cdclasse + '_inline';
                            var newValue = row[newKey];

                            var nritpadraogerprevpax = row['CLASSE' + cdclasse + '_NRITPADRAOGERPREVPAX'];

                            if (oldValue != newValue) {
                                if (newValue == null && nritpadraogerprevpax) {
                                    dataSetToDelete.push({
                                        'CDFILIAL': row.CDFILIAL,
                                        'NRITPADRAOGERPREVPAX': nritpadraogerprevpax,
                                        'NRPADRAOGERPREVPAX': row.NRPADRAOGERPREVPAX,
                                        'NRORG': row.NRORG
                                    });
                                } else if (newValue != null) {
                                    dataSetToSave.push({
                                        'CDFILIAL': row.CDFILIAL,
                                        'NRITPADRAOGERPREVPAX': nritpadraogerprevpax,
                                        'NRPADRAOGERPREVPAX': row.NRPADRAOGERPREVPAX,
                                        'CDCIA': row.CDCIA,
                                        'NRVOO': row.NRVOO,
                                        'CDTRECHO': row.CDTRECHO,
                                        'CDEQUIP': row.CDEQUIP,
                                        'CDCLASSE': cdclasse,
                                        'QTITEM': newValue,
                                        '__is_new': !oldValue,
                                        '__new_code_pk': 'ITPADRAOGERPREVPAX' + row.CDFILIAL,
                                        '__table_pk': 'NRITPADRAOGERPREVPAX',
                                        '__cast_number': true,
                                        '__new_code_length': 5,
                                        '__using_unit': false
                                    });
                                }
                            }
                        }
                    }
                }
            }

            var promisesArr = [
                InflightorGeneral.callRestEngine('pcp#/crud/itpadraogerprevpax/save', 'DataSet', dataSetToSave),
                InflightorGeneral.callRestEngine('pcp#/crud/itpadraogerprevpax/remove', 'DataSet', dataSetToDelete)
            ];
            ZHPromise.all(promisesArr).then(function(success) {
                crosstabWidget.commitInlineEdit();
                crosstabWidget.list();
                crosstabWidget.activate();
                this.getCrosstabData(crosstabWidget);
            }, InflightorGeneral.treatAndShowError);
        }
    };

    /**
     * Define o códgio da filial filtrada no formulário de inserção
     * de um novo padrão de geração.
     * 
     * @param {object} widget widget principal da tela
     * ('pcp00037_padroes_geracao_previsao').
     */
    this.defineFilteredBranchOnPatterFormSave = function(widget) {

        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;

    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PadroesGeracaoPrevisao', PadroesGeracaoPrevisao);
});

// FILE: ./assets/js/controllers/pcp00006_horario_de_quebra_de_voo_controller.js
var pcp00006_horario_de_quebra_de_voo_controller = function(ScreenService, templateManager, InflightorGeneral, ZhFilterPreferences, restEngine, metaDataFactory, MessageService) {

    this.abrePopupFiltro = function(widget) {
        ScreenService.openPopup(widget);
    };

    this.carregaFiltrosPadrao = function(widget) {
        widget.currentRow.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL');
        widget.currentRow.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL');
    };

    this.aplicaFiltro = function(widget) {
        if (widget.isValid()) {
            var formUnidade = widget.container.getWidget('form_unidade');
            var gridCompanhiasHorario = widget.container.getWidget('grid_companhias_horario');
            var widgetStripe = widget.container.getWidget('stripeWidget');

            var filtro = [
                {
                    name: 'CDFILIAL',
                    operator: '=',
                    value: widget.currentRow.CDFILIAL
                }
            ];

            formUnidade.dataSourceFilter = filtro;
            gridCompanhiasHorario.dataSourceFilter = filtro;
            if (typeof widget.currentRow.NMFILIAL != 'undefined' && widget.currentRow.NMFILIAL != "") {
                widgetStripe.currentRow.NMFILIAL = widget.currentRow.NMFILIAL;
                widgetStripe.getField('NMFILIAL').isVisible = true;
            } else {
                widgetStripe.currentRow.NMFILIAL = widget.currentRow.NMFILIAL;
                widgetStripe.getField('NMFILIAL').isVisible = false;
            }

            formUnidade.reload().then(function() {
                gridCompanhiasHorario.reload().then(function() {
                    formUnidade.activate();
                });
                ScreenService.closePopup();
            });
        }
    };

    this.aplicaCurrentRow = function(widget) {
        var widgetData = widget.dataSource.data;
        widget.currentRow = widgetData[0];
    };

    this.definiHoraQuebraUnidade = function(widget) {
        var currentRow = widget.currentRow;

        if (validaHora(currentRow.HRQUEBRAETD)) {
            currentRow.HRQUEBRAETD = currentRow.HRQUEBRAETD.replace(":", "");
            currentRow.changed = true;
            currentRow.__is_new = false;

            restEngine.post("pcp#/crud/paramcomis_operador/save", {
                "requestType": "Row",
                "row": currentRow
            }).then(function(success) {
                InflightorGeneral.widgetCancelForm(widget);
                widget.reload().then(function() {
                    MessageService.showMessageByCode("SAVE_TIMER_SUCCESS");
                });
            });
        }
    };

    this.definiHoraQuebra = function(widget) {
        var horaQuebra = widget.currentRow.HRQUEBRAETD;
        var companhiasHorario = widget.container.getWidget('grid_companhias_horario');
        var companhiasSelecionadas = companhiasHorario.dataSource.getCheckedRows();
        if (validaHora(horaQuebra)) {
            companhiasSelecionadas.forEach(function(companhia) {
                companhia.HRQUEBRAETD = horaQuebra.replace(":", "");
                companhia.changed = true;
                companhia.__is_new = false;
            });

            restEngine.post("pcp#/crud/ciafili_companhia_hora/save", {
                "requestType": "DataSet",
                "dataset": companhiasSelecionadas
            }).then(function(success) {
                companhiasHorario.reload().then(function() {
                    ScreenService.closePopup();
                    widget.dataSource.clearCheckedRows();
                });
            });
        }
    };

    this.excluiHoraQuebra = function(widget) {
        var companhiasSelecionadas = widget.dataSource.getCheckedRows();

        companhiasSelecionadas.forEach(function(companhia) {
            companhia.HRQUEBRAETD = "";
            companhia.changed = true;
            companhia.__is_new = false;
        });

        restEngine.post("pcp#/crud/ciafili_companhia_hora/save", {
            "requestType": "DataSet",
            "dataset": companhiasSelecionadas
        }).then(function(success) {
            widget.reload().then(function() {
                MessageService.showMessageByCode("EXCLUDE_TIMER");
                widget.dataSource.clearCheckedRows();
            });
        });
    };

    var validaHora = function(value) {
        var hora = parseInt(value.slice(0, 2));
        var minutos = parseInt(value.slice(3, 5));
        if (hora < 24 && minutos < 60) {
            return true;
        } else {
            MessageService.showMessageByCode("INVALID_TIMER");
            return false;
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00006_horario_de_quebra_de_voo_controller', pcp00006_horario_de_quebra_de_voo_controller);
});

// FILE: ./assets/js/controllers/pcp00012_previsao_passageiros_customizavel_controller.js
var pcp00012_previsao_passageiros_customizavel_controller = function(ScreenService, InflightorGeneral) {

    this.preparaFiltro = function(widget) {
        var rowFiltro = widget.__filterPopup.currentRow;
        var filtroPossivel = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
			},
            {
                "name": "NRPREVISAOPRO",
                "operator": "=",
                "value": ""
			},
            {
                "name": "IDPREVISAOPROD",
                "operator": "=",
                "value": ""
			},
            {
                "name": "DTPREVISAOPROD",
                "operator": "BETWEEN",
                "value": ""
			},
            {
                "name": "NRTURNODECOLAGEM",
                "operator": "=",
                "value": ""
			},
            {
                "name": "CDCIA",
                "operator": "=",
                "value": ""
			},
            {
                "name": "NRVOO",
                "operator": "=",
                "value": ""
			},
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": ""
			}
		];
        var novoFiltro = [];
        filtroPossivel.forEach(function(condicao) {
            if (rowFiltro.hasOwnProperty(condicao.name) && rowFiltro[condicao.name] != "") {
                condicao.value = rowFiltro[condicao.name];
                novoFiltro.push(condicao);
            }
            if (condicao.name === "DTPREVISAOPROD" && rowFiltro.hasOwnProperty("DTINICIAL") && rowFiltro.hasOwnProperty("DTFINAL")) {
                if (rowFiltro["DTINICIAL"] !== "" && rowFiltro["DTFINAL"] !== "") {
                    condicao.value = [rowFiltro["DTINICIAL"], rowFiltro["DTFINAL"]];
                    novoFiltro.push(condicao);
                }
            }
        });
        novoFiltro.push({
            "name": "CDOPERADOR",
            "operator": "=",
            "value": "OPERADOR_LOGADO"
        });
        widget.dataSourceFilter = novoFiltro;

    };

    var validaDias = function(widget) {
        var dataInicial = widget.currentRow.DTINICIAL;
        dataInicial = moment(dataInicial, 'DD/MM/YYYY');
        widget.getField('DTFINAL').minDate = moment(dataInicial, 'DD/MM/YYYY');
    };

    this.habilitaFieldData = function(widget) {
        widget.currentRow.DTFINAL = "";
        widget.getField('DTFINAL').readOnly = true;
        widget.getField('DTFINAL').reload();

        if (widget.currentRow.DTINICIAL) {
            widget.getField('DTFINAL').readOnly = false;
            widget.getField('DTFINAL').validations = {
                "required": true
            };
            validaDias(widget);
        } else {
            widget.getField('DTFINAL').validations = {};
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('pcp00012_previsao_passageiros_customizavel_controller', pcp00012_previsao_passageiros_customizavel_controller);
});

// FILE: ./assets/js/controllers/pcp00021_relatorios_producao_receitaController.js
var pcp00021_relatorios_producao_receitaController = function(ZHPromise, ScreenService, templateManager, InflightorGeneral, $rootScope, Colors) {

    var _self = this;
    var reportDefault = "PCP00001";
    var _fieldsConfigDefault = {
        "PCP00001": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DTINIVIGENCIA": true,
            "DTFIMVIGENCIA": true,
            "NMSETOR": true,
            "TSETOR": true
        },
        "PCP00002": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DTINIVIGENCIA": true,
            "DTFIMVIGENCIA": true,
            "NMSETOR": true,
            "TSETOR": true
        },
        "PCP00003": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DTMESANO": true,
            "NMSETOR": true,
            "TSETOR": true
        },
        "PCP00004": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DTINIVIGENCIA": true,
            "DTFIMVIGENCIA": true,
            "NMCIA": true,
            "TNMCIA": true,
            "NMSETOR": true,
            "TSETOR": true
        }
    };

    this.onChangeFilial = function(widget) {
        var fieldCompanhia = widget.getField('NMCIA');

        var filtro = [
            {
                name: 'CDFILIAL',
                operator: '=',
                value: widget.currentRow.CDFILIAL
        }
      ];
        fieldCompanhia.clearValue();
        fieldCompanhia.dataSourceFilter = filtro;
        fieldCompanhia.reload();
    };

    this.initialAndChangedStep = function(widget) {
        let reportName = widget.currentRow.REPORTNAME || widget.getField('REPORTNAME').value() || reportDefault;
        if (!widget.getField('REPORTNAME').value()) {
            widget.getField('REPORTNAME').applyDefaultValue();
        }
        widget.fields.forEach(function(field) {
            if (field.name !== 'REPORTNAME') {
                if (field.defaultValue) {
                    field.applyDefaultValue();
                } else {
                    field.setValue("");
                }
                if (field.template.indexOf('select-multiple') !== -1) {
                    field.readOnly = true;
                    field.validations = {};
                    field.setValue('');
                }
                field.isVisible = _fieldsConfigDefault[reportName][field.name] || false;
            }
        });
        templateManager.updateTemplate();
    };

    this.changedDataFim = function(field) {
        let _valueIni = field.widget.getField('DTINIVIGENCIA').value();
        let _valueFim = field.value();
        if (_valueIni && moment(_valueIni, 'DD/MM/YYYY').unix() > moment(_valueFim, 'DD/MM/YYYY').unix()) {
            ScreenService.showMessage(ScreenService.i18n("Período de vigência inválido."), "error");
            field.setValue("");
        }
    };

    this.changedDataInicio = function(field) {
        let fieldFim = field.widget.getField('DTFIMVIGENCIA');
        if (field.value()) {
            if (fieldFim.value() && moment(fieldFim.value(), 'DD/MM/YYYY').unix() < moment(field.value(), 'DD/MM/YYYY').unix()) {
                fieldFim.setValue("");
            }
            fieldFim.readOnly = false;
        } else {
            fieldFim.readOnly = true;
            fieldFim.setValue("");
        }
    };

    this.report = function(widget, format) {
        if (widget.isValid()) {
            if (widget.currentRow.TSETOR == 'S') {
                widget.currentRow.CDSETOR = 'T';
            } else if (widget.currentRow.NMSETOR.length) {
                widget.currentRow.CDSETOR = widget.currentRow.NMSETOR.join('_');
            }

            if (widget.currentRow.TNMCIA == 'S') {
                widget.currentRow.CDCIA = 'T';
            } else if (widget.currentRow.NMCIA.length) {
                widget.currentRow.CDCIA = widget.currentRow.NMCIA.join('_');
            }

            widget.currentRow.REPORTFORMAT = format;
            InflightorGeneral.openReport('pcp#/relatorioProducaoReceita', widget.currentRow.REPORTNAME, widget.currentRow);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00021_relatorios_producao_receitaController', pcp00021_relatorios_producao_receitaController);
});

// FILE: ./assets/js/controllers/pcp00024_planejamento_apresentacao_cheftableController.js
var pcp00024_planejamento_apresentacao_cheftableController = function(ScreenService, InflightorGeneral, ZhFilterPreferences, restEngine, MessageService) {

    var self = this;

    this.aplicaUnidadeSelecionada = function(widget) {
        var filtro = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: widget.currentRow.CDFILIAL
            }
        ];
        widget.getField('NMCIA').clearValue();
        widget.getField('SGCIA').clearValue();
        widget.getField('NMCIA').dataSourceFilter = filtro;
        widget.getField('NMCIA').reload();
    };

    this.aplicaValoresPadrao = function(widget) {
        InflightorGeneral.openFilterOnEnter(widget.container.getWidget('planejamento_apresentacao_cheftable'), ["CDFILIAL", "NMFILIAL", "DTPLANAPREINF", "IDSITUACAO"]);
        // widget.container.getWidget('planejamento_apresentacao_cheftable').appliedFilter.FILTER_ROW.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL'); 
        // widget.container.getWidget('planejamento_apresentacao_cheftable').appliedFilter.FILTER_ROW.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL'); 
    };

    this.FilterAfterApply = function(args) {
        // delete args.owner.dataSourceFilter;
        args.owner.collapsible = false;
        // args.owner.__filterInfoStripe.getField("stripe_filters").isVisible = false;
        // args.owner.__filterInfoStripe.getField("linha1").label += "Unidade: "+ args.owner.appliedFilter.FILTER_ROW.NMFILIAL;
        // args.owner.__filterInfoStripe.getField("linha1").label += " - Companhia: "+ args.owner.appliedFilter.FILTER_ROW.NMCIA;
        // args.owner.__filterInfoStripe.getField("linha2").label += "Data: "+ args.owner.appliedFilter.FILTER_ROW.DTPLANAPREINF;
    };

    this.adicionaPlanejamento = function(widget) {
        widget.newRow();
        widget.getField('NMFILIAL').value(widget.appliedFilter.FILTER_ROW.NMFILIAL);
        widget.getField('NMCIA').value(widget.appliedFilter.FILTER_ROW.NMCIA);
        widget.getField('DTPLANAPREINF').value(widget.appliedFilter.FILTER_ROW.DTPLANAPREINF);
        widget.currentRow.CDCIA = widget.appliedFilter.FILTER_ROW.CDCIA;
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        self.filtraMenu(widget.getField('NMCIA'));
        widget.container.getWidget('subgrupo').isVisible = false;
        widget.container.getWidget('checklist_gerados').isVisible = false;

    };

    this.salvar = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.IDSITUACAO = 'PENDENTE';
            widget.currentRow.DTINCLUSAO = moment().format('DD/MM/YYYY HH:mm:ss');
            widget.currentRow.__using_unit = false;
            widget.currentRow.__is_new_code = true;
            widget.currentRow.__new_code_pk = "PLANAPREINF" + widget.currentRow.CDFILIAL;
            widget.currentRow.__table_pk = "NRPLANAPREINF";
            widget.currentRow.__new_code_column_size = 20;
            widget.currentRow.__new_code_length = 1;
            // widget.currentRow.NRPLANAPREINF = widget.currentRow.__new_code_pk + widget.currentRow.CDFILIAL;
            InflightorGeneral.callRestEngine('pcp#/pcp00024_salva_planejamento', 'Row', widget.currentRow).then(function() {
                ScreenService.closeSwipe();
                widget.container.getWidget('subgrupo').isVisible = true;
                widget.container.getWidget('checklist_gerados').isVisible = true;
                widget.mode = 'list';
                widget.activate();
                widget.reload();
            }, function(error) {
                InflightorGeneral.callbackError(error);
            });
        } else {
            MessageService.showMessageByCode("PCP00034_PREENCHA_CAMPOS_OBRIG");
        }
    };

    this.cancelarVoltar = function(widget) {
        var widgetMain = widget.container.getWidget('planejamento_apresentacao_cheftable');
        ScreenService.closeSwipe().then(function() {
            widgetMain.activate();
            widgetMain.reload();
            widget.container.getWidget('subgrupo').isVisible = true;
            widget.container.getWidget('checklist_gerados').isVisible = true;
        });
    };

    this.aprovarPla = function(widget) {
        if (widget.currentRow.IDSITUACAO == 'PENDENTE') {
            widget.currentRow.PROC = 'A';
            InflightorGeneral.callRestEngine("pcp#/callProcedurePlana", 'Row', widget.currentRow).then(function(callback) {
                if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                    InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                } else {
                    MessageService.showMessageByCode("APPROVED_SUCCESS");
                    ScreenService.closePopup();
                    widget.getField('IDSITUACAO').value('APROVADO');
                    widget.reload();
                }
            }, function(err) {
                if (err.data) {
                    ScreenService.showMessage(err.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        } else {
            MessageService.showMessageByCode("PLANNING_PENDING");
        }
    };

    this.cancelarPla = function(widget) {
        if (widget.currentRow.IDSITUACAO == 'APROVADO') {
            widget.currentRow.PROC = 'C';
            InflightorGeneral.callRestEngine("pcp#/callProcedurePlana", 'Row', widget.currentRow).then(function(callback) {
                if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                    InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros);
                } else {
                    MessageService.showMessageByCode("CANCEL_APPROVED");
                    ScreenService.closePopup();
                    widget.getField('IDSITUACAO').value('PENDENTE');
                    widget.reload();
                }
            }, function(err) {
                if (err.data) {
                    ScreenService.showMessage(err.data.error);
                } else {
                    MessageService.showMessageByCode("TIMEOUT");
                }
            });
        } else {
            MessageService.showMessageByCode("PLANNING_CANCEL");
        }
    };

    this.filtraMenu = function(field) {
        var fieldMenu = field.widget.getField('DSMANUAL');

        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": field.widget.currentRow.CDCIA
            }
        ];
        fieldMenu.dataSourceFilter = filter;
        if (field.value() === "") {
            fieldMenu.readOnly = true;
        } else {
            fieldMenu.readOnly = false;
        }
        fieldMenu.reload();
    };

    this.filtraCompanhia = function(field) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            }
        ];
        field.dataSourceFilter = filter;
        field.reload();
    };

    this.openSubgrupo = function(widget) {
        var widgetPlanejamento = widget.container.getWidget('planejamento_apresentacao_cheftable');

        var filter = [
            {
                "name": "NRPLANAPREINF",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.NRPLANAPREINF
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.CDMANCIA
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.saveEdit = function(widget) {
        var widgetPla = widget.container.getWidget('planejamento_apresentacao_cheftable').currentRow;
        var changedRows = [];
        widget.dataSource.data.forEach(function(row) {
            if (row.QTSUBGRU_inline !== undefined) {
                if (!row.NRITPLANAPREINF && row.QTSUBGRU_inline == 0) {
                    //todo
                } else {
                    row.NRPLANAPREINF = widgetPla.NRPLANAPREINF;
                    changedRows.push(row);
                }
            }
        });

        InflightorGeneral.callRestEngine("pcp#/saveChanges", 'Row', changedRows).then(function(callback) {
            widget.list();
            widget.reload();
        }, function(err) {
            if (err.data) {
                ScreenService.showMessage(err.data.error);
            } else {
                MessageService.showMessageByCode("TIMEOUT");
            }
        });

    };

    this.cancelEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
    };

    this.filtraChecklist = function(widget) {
        var widgetPlanejamento = widget.container.getWidget('planejamento_apresentacao_cheftable');

        var filter = [
            {
                "name": "NRPLANAPREINF",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.NRPLANAPREINF
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widgetPlanejamento.currentRow.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];
        widget.dataSourceFilter = filter;
        widget.reload();
    };

    function refreshGrid(widget) {

        var filter = [
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "DTPLANAPREINF",
                "operator": "=",
                "value": widget.currentRow.DTPLANAPREINF
            }
        ];
        widget.dataSourceFilter = filter;
        widget.reload();
    }

    this.deleteApresentacao = function(widget) {
        ScreenService.confirmMessage('Excluir planejamento?', null, function() {

            var currentRow = widget.currentRow;
            restEngine.post("pcp#/planapreinf_manualciam/delete", {
                "requestType": "Row",
                "row": currentRow
            }).then(function(success) {
                ScreenService.closeSwipe();
                refreshGrid(widget);
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });

        });

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00024_planejamento_apresentacao_cheftableController', pcp00024_planejamento_apresentacao_cheftableController);
});

// FILE: ./assets/js/controllers/pcp00025_solicitacao_checklist_emergencial_controller.js
var pcp00025_solicitacao_checklist_emergencial_controller = function(ScreenService, InflightorGeneral, ZhFilterPreferences, localStorageEngine, templateManager, restEngine, MessageService, PCPUtil) {
    var self = this;

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00025_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };
    /**
     * Define qual rotina de inserção será chamada.
     * 
     * @param {object} widget widget princiapl da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.saveEmergencyCheckListSolicitation = function(widget) {

        var currentRow = widget.currentRow;

        if (Boolean(currentRow.__is_new)) {
            this.saveNewEmergencyCheckListSolicitation(widget);
        } else {
            this.saveEmergencyCheckListSolicitationModification(widget);
        }

    };

    /**
     * Salva uma nova solicitação de checklist
     * emergencial.
     * 
     * @param {object} widget widget princiapl da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.saveNewEmergencyCheckListSolicitation = function(widget) {

        var currentRow = widget.currentRow;
        this.setEmergencyCheckListSolicitationDefaultValues(currentRow);

        InflightorGeneral.widgetSave(widget).then(function(response) {
            if (response && response.dataset && response.dataset.columnsToReturn && Boolean(response.dataset.columnsToReturn[0])) {
                currentRow.NRSOLEMERGPROD = response.dataset.columnsToReturn[0].NRSOLEMERGPROD;
            }

            widget.reload();

        });

    };

    /**
     * Salva a modificação de uma solicitação de checklist
     * emergencial.
     * 
     * @param {object} widget widget princiapl da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.saveEmergencyCheckListSolicitationModification = function(widget) {

        InflightorGeneral.widgetSave(widget).then(function() {
            widget.reload();
        });

    };

    /**
     * Habilia o ambiente de inclusão de 
     * uma solicitação.
     * 
     * @param {object} widget widget princiapl da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.addSolicitation = function(widget) {

        InflightorGeneral.widgetNewRow(widget);

        this.setSolicitationDefaultValue(widget);
        this.setFieldPermission(widget.getField('DTPRODUCAO'), false);
        this.setBranchFieldPermission(widget, false);
    };

    /**
     * Define os valores iniciais de uma nova solicitação.
     * 
     * @param {object} widget widget de solicitações.
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').c
     */
    this.setSolicitationDefaultValue = function(widget) {

        var currentRow = widget.currentRow;

        currentRow.DTPRODUCAO = this.getCurrentDate('DD/MM/YYYY');
        currentRow.NMFILIAL = getLocalVar("INFLIGHTOR_NMFILIAL");
        currentRow.CDFILIAL = getLocalVar("INFLIGHTOR_CDFILIAL");

    };

    /**
     * Habilia o ambiente de edição da 
     * solicitação em caso.
     * 
     * @param {object} widget widget princiapl da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.editSolicitation = function(widget) {

        InflightorGeneral.widgetEdit(widget);
        this.setBranchFieldPermission(widget, true);
        this.setProdutionDateFieldPermissionBySolicitationSituation(widget);

    };

    /**
     * Define a permissão de acesso do campo de filial
     * da widget de solicitações ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     * 
     * @param {object} widget widget de solicitações.
     * 
     * @param {boolean} isBlocked valor lógico definindo se o
     * campo ficará bloqueado ou não.
     */
    this.setBranchFieldPermission = function(widget, isBlocked) {

        var branchField = widget.getField('NMFILIAL');
        this.setFieldPermission(branchField, isBlocked);

    };

    /**
     * Define a permissão de acesso do campo de data
     * de produção no ambiente de edição de uma solicitação.
     * 
     * @param {object} widget widget de solicitações
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.setProdutionDateFieldPermissionBySolicitationSituation = function(widget) {

        var currentRow = widget.currentRow;
        var produtionDateField = widget.getField('DTPRODUCAO');

        if (currentRow.IDSITUACAO === "PENDENTE") {
            this.setFieldPermission(produtionDateField, false);
        } else {
            this.setFieldPermission(produtionDateField, true);
        }

    };

    /**
     * Define o bloqueio de um campo através da 
     * propriedade "readOnly".
     * 
     * @param {object} field field a ser tratada.
     * 
     * @param {boolean} isBlocked valor lógico definindo se o
     * campo ficará bloqueado ou não.
     */
    this.setFieldPermission = function(field, isBlocked) {

        field.readOnly = isBlocked;

    };

    /**
     * Define os valores iniciais para a inserção
     * de uma solicitação de checklist emergencial.
     * 
     * @param {object} currentRow currentRow da widget
     * principal('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.setEmergencyCheckListSolicitationDefaultValues = function(currentRow) {

        this.setColumnToReturnInCurrentRow(currentRow, 'NRSOLEMERGPROD');

    };

    /**
     * Define a coluna a ser retornada
     * na resposta do 'save'.
     * 
     * @param {object} currentRow currentRow que receberá
     * o parâmetro.
     * 
     * @param {string} columnName nome da coluna a ser
     * retornada.
     */
    this.setColumnToReturnInCurrentRow = function(currentRow, columnName) {

        currentRow.__columns_to_return = [columnName];

    };

    /**
     * Executa os métodos solicitados no eventos
     * 'WidgetOnEnterOnce' da widget principal da tela.
     * 
     * @param {object} widget widget principal da tela
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.widgetOnEnterOnce = function(widget) {

        InflightorGeneral.openFilterOnEnter(widget, ["CDFILIAL", "NMFILIAL", "DTPRODUCAO", "IDSITUACAO"]);

    };

    /**
     * Função para retornar a data e hora atual.
     * 
     * @param {string} format formato da data
     * a ser retornada.
     * 
     * @return {string} data e hora atual.
     */
    this.getCurrentDate = function(format) {
        return moment().format(format);
    };

    /**
     * Salva uma receita referenciando uma solicitação.
     * 
     * @param {object} widget widget de receitas
     * (WDGT_RECEITAS).
     */
    this.saveSolicitationReceipe = function(widget) {

        var currentRow = widget.currentRow;

        this.setReceipDefaultValues(widget);
        currentRow.QTRECEITA = this.removeValueCurrencyMask(currentRow.QTRECEITA);

        return PCPUtil.widgetSave(widget).then(function(response) {
            if (response && response.dataset && response.dataset.columnsToReturn) {
                currentRow.NRRECSOLEMERGPROD = response.dataset.columnsToReturn[0].NRRECSOLEMERGPROD;
                return response.dataset.columnsToReturn[0].NRRECSOLEMERGPROD;
            }
            return "";
        });

    };

    /**
     * Define os valores iniciais de uma receita.
     * 
     * @param {object} widget widget de receitas
     * (WDGT_RECEITAS).
     */
    this.setReceipDefaultValues = function(widget) {

        var solicitationWidgetCurrentRow = templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow;
        var currentRow = widget.currentRow;

        currentRow.NRORG = solicitationWidgetCurrentRow.NRORG;
        currentRow.CDFILIAL = solicitationWidgetCurrentRow.CDFILIAL;
        currentRow.NRSOLEMERGPROD = solicitationWidgetCurrentRow.NRSOLEMERGPROD;

        this.setColumnToReturnInCurrentRow(currentRow, 'NRRECSOLEMERGPROD');

    };

    /**
     * Função para remover todos os caracteres
     * não numéricos de um determinado valor,
     * caso o valor recebido seja vazio, uma
     * String vazia é retornada.
     * 
     * @param {String} valueWithCurrencyMask valor que será retornado
     * sem caracteres não numéricos.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem caracteres não
     * numéricos.
     * 
     */
    this.removeValueMask = function(valueWithMask) {

        if (valueWithMask) {

            var valueWithoutSymbols = valueWithMask.replace(/[^0-9]/g, "");
            return valueWithoutSymbols;

        } else {
            return "";
        }
    };

    /**
     * Função para removar mascarás de moedas do valor
     * recebido como parâmetro.
     * 
     * @param {String} valueWithCurrencyMask valor que 
     * será retornado sem máscara de moeda.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem máscara de moeda.
     * 
     */
    this.removeValueCurrencyMask = function(valueWithCurrencyMask) {

        if (valueWithCurrencyMask) {

            var valuesWithoutCurrencyMask = $.currency.unmaskString(valueWithCurrencyMask, {
                "thousands": "."
            });
            return valuesWithoutCurrencyMask;

        } else {
            return "";
        }

    };

    /**
     * Adiciona um destino relacionado a uma receita.
     * 
     * @param {object} widget widget de destino('WDGT_DESTINOS').
     */
    this.addDestiny = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }
        InflightorGeneral.widgetNewRow(widget);
        this.setFieldBlockageByRelatedFieldValue(widget.getField('DSSERVICOCIAFILI'), 'NRVOO');
        this.setFieldBlockageByRelatedFieldValue(widget.getField('DSVOO'), 'DTATENDIMENTO');
        this.setFieldBlockageByRelatedFieldValue(widget.getField('IDCICLO'), 'IDCICLO');
        this.setSolicitationDestinationDefaultValues(widget);
        this.setVooFilterByTakeOffDate(widget);
    };

    /**
     * Habilita o ambiente de edição da widget
     * de destinos.
     * 
     * @param {object} widget widget de destinos
     * ('WDGT_DESTINOS').
     */
    this.editDestination = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }

        InflightorGeneral.widgetEdit(widget);

        this.setFieldBlockageByRelatedFieldValue(widget.getField('DSSERVICOCIAFILI'), 'NRVOO');
        this.setFieldBlockageByRelatedFieldValue(widget.getField('DSVOO'), 'DTATENDIMENTO');
        this.setFieldBlockageByRelatedFieldValue(widget.getField('IDCICLO'), 'IDCICLO');
        self.setVooFilterByTakeOffDate(widget);
        self.filtraCiclo(widget);

    };

    /**
     * Define o filtro do campo de voo de acordo com
     * a data informada pelo usuário.
     * 
     * @param {object} widget widget de destinos
     * ('WDGT_DESTINOS').
     */
    this.setVooFilterByTakeOffDate = function(widget) {

        var solicitationWidgetCurrentRow = templateManager.container.getWidget("WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL").currentRow;
        var currentRow = widget.currentRow;

        var vooField = widget.getField('DSVOO');
        var vooFieldFilter = vooField.dataSourceFilter;

        var takeOffDateFilteIndex = this.getObjectPositionInArrayByName(vooFieldFilter, "DTDECOLAGEM");
        var branchCodeFilteIndex = this.getObjectPositionInArrayByName(vooFieldFilter, "CDFILIAL");

        vooFieldFilter[takeOffDateFilteIndex].value = currentRow.DTATENDIMENTO;
        vooFieldFilter[branchCodeFilteIndex].value = solicitationWidgetCurrentRow.CDFILIAL;
        vooField.reload();

        this.setFieldBlockageByRelatedFieldValue(widget.getField('DSVOO'), 'DTATENDIMENTO');

    };

    /**
     * Define o bloqueio do field recebido como parâmetro
     * de acordo com o valor do campo relacionado(também recebido como
     * parâmetro).
     * 
     * @param {object} field field a ser modificada.
     * 
     * @param {boolean} relatedFieldName campo relacionado.
     */
    this.setFieldBlockageByRelatedFieldValue = function(field, relatedFieldName) {

        var widget = field.widget;
        var currentRow = widget.currentRow;

        if (Boolean(currentRow[relatedFieldName])) {
            field.readOnly = false;
        } else {
            field.readOnly = true;
            currentRow[field.name] = "";
        }

    };

    /*
     * Filtra o campo de ciclo ciclo que seja da companhia
     */
    this.filtraCiclo = function(widget) {
        var currentRow = widget.currentRow;
        let widgetSol = templateManager.container.getWidget("WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL");
        if (currentRow.CDCIA === "") {
            widget.getField('IDCICLO').readOnly = true;
        } else {
            var filter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widgetSol.currentRow.CDFILIAL
                },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": currentRow.CDCIA
                }
            ];
            widget.getField('IDCICLO').dataSourceFilter = filter;
            widget.getField('IDCICLO').readOnly = false;
            widget.getField('IDCICLO').reload();
        }

        self.setItemsFilterBySelectedFlight(widget.getField("DSSERVICOCIAFILI"));
    };

    /**
     * Define o filtro do campo de itens do faturamento
     * pelo voo selecionado.
     * 
     * @param {object} widget widget de destino('WDGT_DESTINOS'). 
     */
    this.setItemsFilterBySelectedFlight = function(field) {

        var widget = field.widget;
        var currentRow = widget.currentRow;
        var solicitationWidgetCurrentRow = templateManager.container.getWidget("WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL").currentRow;
        var fieldFilter = field.dataSourceFilter;

        var cdciaFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, 'CDCIA');
        var nrvooFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, 'NRVOO');
        var cdtrechoFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, 'CDTRECHO');
        var datarefeFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, 'DATAREF');

        fieldFilter[cdciaFilterIndex].value = currentRow.CDCIA;
        fieldFilter[nrvooFilterIndex].value = currentRow.NRVOO;
        fieldFilter[cdtrechoFilterIndex].value = currentRow.CDTRECHO;
        fieldFilter[datarefeFilterIndex].value = solicitationWidgetCurrentRow.DTPRODUCAO;

        field.reload();

    };

    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
     */
    this.getObjectPositionInArrayByName = function(array, objectName) {

        var indexFieldObject;

        array.forEach(function(value, i) {
            if (value.name === objectName) {
                indexFieldObject = i;
            }
        });

        return indexFieldObject;

    };

    /**
     * Define o método de inserção a ser chamado diferenciando
     * uma edição no modo 'inLineEdit' de uma rotina comum em um
     * formulário.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.saveRecipe = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }

        if (widget.inEditMode()) {
            this.saveRecipeDestinationBeforeRecipeSave(widget);
        } else {
            this.saveEditedRecipesInLineEdit(widget, 'QTRECEITA', 'NRRECSOLEMERGPROD');
        }

    };

    /**
     * Define se o sistema realizará o um rateio da quantidade da receita em caso pelos
     * seus respectivos destinos, conferindo o valor definido e o backup criado 
     * anteriormente.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS');
     * 
     * @param {number} oldValue valor antigo da quantidade de receita.
     * 
     * @param {number} newValue valor antigo da quantidade de receita.
     */
    this.checkRecipeAmountValueToDoApportionment = function(row, oldValue, newValue) {

        if (oldValue !== newValue) {
            this.setApportionmentParameter(row, true);
        } else {
            this.setApportionmentParameter(row, false);
        }

    };

    /**
     * Definir se o sistema realizará um novo rateio.
     * 
     * @param {object} row row da widget de receitas
     * ('WDGT_RECEITAS').
     * 
     * @param {boolean} doApportionment valor lógico que define se o sistema
     * realizará a rotina de rateio;
     */
    this.setApportionmentParameter = function(row, doApportionment) {
        row.REALIZARRATEIO = doApportionment;
    };

    /**
     * Define o método de cancelamento de edição a ser chamado 
     * diferenciando uma edição no modo 'inLineEdit' de uma rotina comum em um
     * formulário.
     * 
     * @param {object} widget widget referência.
     */
    this.cancelRecipeWidgetModeEdit = function(widget) {

        if (widget.inEditMode()) {

            InflightorGeneral.widgetCancelForm(widget);
            this.deleteDestinationInsertingParameter();

        } else {
            this.cancelInlineEdit(widget);
        }

    };

    /**
     * Define o método de cancelamento de edição a ser chamado 
     * diferenciando uma edição no modo 'inLineEdit' de uma rotina comum em um
     * formulário.
     * 
     * @param {object} widget widget referência.
     */
    this.cancelRecipeDestinationWidgetModeEdit = function(widget) {

        if (widget.inEditMode()) {

            InflightorGeneral.widgetCancelForm(widget);

            if (Boolean(widget.ISRECIPEFIRSTDESTINATION)) {
                this.cancelDestinationInsertBeforeRecipeSave(widget);
            }

        } else {
            this.cancelInlineEdit(widget);
        }

    };

    /**
     * Cancela a rotina de inserção de uma novo destino antes de
     * inserir uma nova receita e retorna para a widget de receita
     * no mode edição.
     * 
     * @param {object} widget widget de destinos ('WDGT_DESTINOS').
     */
    this.cancelDestinationInsertBeforeRecipeSave = function(widget) {

        var parentWidget = templateManager.container.getWidget("WDGT_RECEITAS");

        parentWidget.activate().then(function() {
            parentWidget.edit();
        });

        widget.enabled = false;

    };

    /**
     * Prepara os registros editados na própria
     * linha do grid.
     * 
     * @param {object} widget widget editada no grid.
     * 
     * @param {string} editedFieldNameInLineEdit nome do campo
     * editado no grid.
     * 
     * @param {string} referenceField campo referência do registro
     * a ser salvo, geralmente a pk da tabela em caso.
     */
    this.prepareRowToSaveInLineEdit = function(widget, editedFieldNameInLineEdit, referenceField) {

        var dataWidget = widget.dataSource.data;

        var editedRows = dataWidget.filter(function(row) {
            return row['changed_' + editedFieldNameInLineEdit] || Object.keys(row).filter(function(key) {
                return key.indexOf('_inline') !== -1;
            }).length;
        });

        var rowsToSave = editedRows.map(function(row) {

            if (row[editedFieldNameInLineEdit + '_inline'] !== row[editedFieldNameInLineEdit]) {

                row['changed_' + editedFieldNameInLineEdit] = row[editedFieldNameInLineEdit];
                row[editedFieldNameInLineEdit] = InflightorGeneral.valueToFloat(row[editedFieldNameInLineEdit + '_inline']);

            }

            return {
                [referenceField]: row[referenceField],
                [editedFieldNameInLineEdit]: this.removeValueCurrencyMask(row[editedFieldNameInLineEdit]),
                ['changed_' + editedFieldNameInLineEdit]: row['changed_' + editedFieldNameInLineEdit],
                "__is_new": false
            };

        }.bind(this));

        return rowsToSave;

    };

    /** 
     * Salva as alterações realizadas diretamente na linha
     * do grid de receitas.
     * 
     * @param {object} widget widget editada no grid.
     * 
     * @param {string} editedFieldNameInLineEdit nome do campo
     * editado no grid.
     * 
     * @param {string} referenceField campo referência do registro
     * a ser salvo, geralmente a pk da tabela em caso.
     */
    this.saveEditedRecipesInLineEdit = function(widget, editedFieldNameInLineEdit, referenceField) {

        var rowsToSave = this.prepareRowToSaveInLineEdit(widget, editedFieldNameInLineEdit, referenceField);

        this.defineRecipesToDoApportionment(rowsToSave);

        InflightorGeneral.callRestEngine('pcp#/recsolemergprod_receitam/save', 'DataSet', rowsToSave).then(function(success) {

            widget.list();
            widget.commitInlineEdit();
            widget.activate();

        }, function(error) {
            if (error.data)
                trataMensagem(error.data.error);
        });

    };

    this.loadDataSourceWeirdWidget = function(widget) {
        var filtro = [{
            "name": "NRSOLEMERGPROD",
            "operator": "=",
            "value": widget.getParent().NRSOLEMERGPROD
        }];

        InflightorGeneral.callRestEngine('pcp#/crud/cklistsolemergprod_cklistprodinf_setor_turno_categoriaprod_subcatpsetor_categoriaprod_tipoprod_tiponegocioinf', 'FilterData', filtro).then(function(success) {

            widget.list();
            widget.commitInlineEdit();
            widget.activate();

        }, function(error) {
            if (error.data)
                trataMensagem(error.data.error);
        });

    };

    /**
     * Define as receitas que chamarão a rotina de rateio.
     * 
     * @param {object} rows receitas editadas no 'inLineEdit'.
     */
    this.defineRecipesToDoApportionment = function(rows) {

        rows.forEach(function(row) {

            this.checkRecipeAmountValueToDoApportionment(row, row.changed_QTRECEITA, InflightorGeneral.valueToFloat(row.QTRECEITA));
            row.QTRECEITABACKUP = row.changed_QTRECEITA;

        }.bind(this));

    };

    this.deleteEmergencyCheckListSolicitation = function(widget) {

        if (widget.currentRow.IDSITUACAO !== 'APROVADA') {
            InflightorGeneral.deleteCrud(widget);
        } else {
            MessageService.showMessageByCode("PCP00025_N_EXCL_SOL_APROV");
        }

    };

    /**
     * Define a visibilidade das ações voltadas para
     * edição e inserção da widget de receitas.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.setEditionActionsVisibility = function(widget, isVisible) {

        var widgetActions = widget.actions;

        var actSaveActionsIndex = this.getObjectPositionInArrayByName(widgetActions, 'ACT_SALVA_RECEITA');
        var actCancelFormActionsIndex = this.getObjectPositionInArrayByName(widgetActions, 'ACT_CANCELA_FORMULARIO');

        widgetActions[actSaveActionsIndex].isVisible = isVisible;
        widgetActions[actCancelFormActionsIndex].isVisible = isVisible;

    };

    /**
     * Cancela as edições feita diretamente do grid.
     * 
     * @param {object} widget widget em edição no grid.
     */
    this.cancelInlineEdit = function(widget) {

        widget.rollbackInlineEdit();
        widget.list();

    };

    /**
     * Define os valores inicias de um destino.
     * 
     * @param {object} widget widget de destino('WDGT_DESTINOS').
     */
    this.setSolicitationDestinationDefaultValues = function(widget) {

        var parentWidget = templateManager.container.getWidget("WDGT_RECEITAS");
        var widgetSol = templateManager.container.getWidget("WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL");
        var parentWidgetCurrentRow = parentWidget.currentRow;
        var currentRow = widget.currentRow;

        currentRow.NRORG = widgetSol.currentRow.NRORG;
        currentRow.CDFILIAL = widgetSol.currentRow.CDFILIAL;
        currentRow.NRRECSOLEMERGPROD = (parentWidgetCurrentRow.NRRECSOLEMERGPROD) ? parentWidgetCurrentRow.NRRECSOLEMERGPROD : "";
        currentRow.SGUNIDADE = parentWidgetCurrentRow.SGUNIDADE;
        currentRow.CDRECEITADEST = parentWidgetCurrentRow.CDRECEITA;
        currentRow.DTATENDIMENTO = self.getCurrentDate("DD/MM/YYYY");

        this.setColumnToReturnInCurrentRow(currentRow, 'NRDESTSOLEMERGPROD');

    };

    /**
     * Define o método de inserção a ser chamado diferenciando
     * uma edição no modo 'inLineEdit' de uma rotina comum em um
     * formulário.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.saveRecipeDestination = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }

        if (widget.inEditMode()) {
            this.checkRelatedFirstDestinationToRecipe(widget);
        } else {
            this.saveEditedDestinationRecipesInLineEdit(widget, 'QTRECEITA', 'NRDESTSOLEMERGPROD');
        }

    };

    /**
     * Salva dados do destino vinculado a receita 
     * selecionada.
     * 
     * @param {object} widget widget de destinos
     * ('WDGT_DESTINOS').
     */
    this.checkRelatedFirstDestinationToRecipe = function(widget) {

        if (widget.isValid()) {

            var currentRow = widget.currentRow;

            if (!Boolean(currentRow.NRRECSOLEMERGPROD) && Boolean(widget.ISRECIPEFIRSTDESTINATION)) {

                this.saveSolicitationReceipe(widget.getParent()).then(function(recipeCode) {

                    currentRow.NRRECSOLEMERGPROD = recipeCode;
                    this.saveRecipeDestinationOnForm(widget);

                }.bind(this));

            } else {
                this.saveRecipeDestinationOnForm(widget);
            }

        }

    };

    /**
     * Salva um destino vinculado a receita 
     * selecionada.
     * 
     * @param {object} widget widget de destinos
     * ('WDGT_DESTINOS').
     */
    this.saveRecipeDestinationOnForm = function(widget) {

        const widgetReceitas = widget.container.getWidget("WDGT_RECEITAS");
        const parentWidgetCurrentRow = widgetReceitas.currentRow;
        const currentRow = widget.currentRow;

        currentRow.QTRECEITA = this.removeValueCurrencyMask(currentRow.QTRECEITA);

        this.checkRecipeAmountValueToDoApportionment(currentRow, currentRow.QTRECEITABACKUP, currentRow.QTRECEITA);
        const paramsNewCode = widget.paramsNewCode;
        if (paramsNewCode && paramsNewCode.__is_new_code) {
            InflightorGeneral.applyParamsNewCodeToRow(widget);
        }

        const fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
        if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
            InflightorGeneral.setDuplicationValidateParamsToCurrentRow(widget);
        }

        widget.saveRow().then((response) => {

            if (Boolean(response && response.dataset && response.dataset.columnsToReturn)) {

                currentRow.NRDESTSOLEMERGPROD = response.dataset.columnsToReturn.NRDESTSOLEMERGPROD;
                parentWidgetCurrentRow.QTRECEITA = response.dataset.columnsToReturn.QTDRECEITASUSADAS;

            }

            this.deleteDestinationInsertingParameter();
            if (widget.currentRow.__is_new) {
                ScreenService.closeSwipe().then(() => {
                    widget.currentRow = {};
                    ScreenService.closePopup();
                    widgetReceitas.list();
                    widgetReceitas.activate();
                    widgetReceitas.reload();
                });
            }

        });

    };

    /**
     * Delete o destino selecionado, relacionado a receita
     * em caso.
     * 
     * @param {object} widget widget de destinos('WDGT_DESTINOS');
     */
    this.deleteRecipeDestination = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }

        if (widget.dataSource.data.length <= 1) {
            MessageService.showMessageByCode("PCP00025_RECE_DEVE_TER_1_DEST");
            return;
        }

        var parentWidgetCurrentRow = templateManager.container.getWidget("WDGT_RECEITAS").currentRow;
        var currentRow = widget.currentRow;

        InflightorGeneral.deleteCrud(widget).then(function(response) {

            InflightorGeneral.callRestEngine("/getDestinationRecipeAmoutByRelatedRecipe", "Row", [currentRow]).then(function(response) {
                if (response && response.dataset && response.dataset.destinationRecipeAmout)
                    parentWidgetCurrentRow.QTRECEITA = response.dataset.destinationRecipeAmout[0];

            });

        });

    };

    /** 
     * Salva as alterações realizadas diretamente na linha
     * do grid de receitas.
     * 
     * @param {object} widget widget('WDGT_DESTINOS') modificada no grid.
     * 
     * @param {string} editedFieldNameInLineEdit nome do campo
     * editado no grid.
     * 
     * @param {string} referenceField campo referência do registro
     * a ser salvo, geralmente a pk da tabela em caso.
     */
    this.saveEditedDestinationRecipesInLineEdit = function(widget, editedFieldNameInLineEdit, referenceField) {

        var parentWidgetCurrentRow = templateManager.container.getWidget("WDGT_RECEITAS").currentRow;
        var rowsToSave = this.prepareRowToSaveInLineEdit(widget, editedFieldNameInLineEdit, referenceField);

        this.relationDestinationsWithParentRecipe(widget, rowsToSave);

        InflightorGeneral.callRestEngine('pcp#/destsolemergprod_receitam_dsvoo/save', 'DataSet', rowsToSave).then(function(success) {

            widget.list();
            widget.commitInlineEdit();
            widget.activate();

            parentWidgetCurrentRow.QTRECEITA = success.dataset.columnsToReturn.QTDRECEITASUSADAS;

        }.bind(this), function(error) {
            if (error.data)
                trataMensagem(error.data.error);
        });

    };

    /**
     * Relaciona destinos a receita pai de acordo com a widget pai.
     * 
     * @param {object} widget widget de destino ('WDGT_DESTINOS').
     * 
     * @param {object} rowsToSave destinos a serem relacionados.
     */
    this.relationDestinationsWithParentRecipe = function(widget, rowsToSave) {

        var parentWidgetCurrentRow = templateManager.container.getWidget("WDGT_RECEITAS").currentRow;

        rowsToSave.forEach(function(row) {
            row.NRRECSOLEMERGPROD = parentWidgetCurrentRow.NRRECSOLEMERGPROD;
        });
    };

    /**
     * Habilita o ambiente de edição de uma receita.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.editSolicitationRecipe = function(widget) {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }

        widget.getField("DSRECEITA").readOnly = true;
        InflightorGeneral.widgetEdit(widget);
        this.createRecipeAmountValueBackup(widget);
    };

    /**
     * Cria um backup do valor atual da quantidade da receita
     * em caso.
     * 
     * @param {object} widget widget de receitas ('WDGT_RECEITAS').
     */
    this.createRecipeAmountValueBackup = function(widget) {

        var currentRow = widget.currentRow;

        currentRow.QTRECEITABACKUP = currentRow.QTRECEITA;

    };

    /**
     * Direciona o sistema para o ambiente de inserção de
     * um destino se o usuário estiver inserido uma nova receita.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.saveRecipeDestinationBeforeRecipeSave = function(widget) {

        if (widget.isValid()) {

            var currentRow = widget.currentRow;

            if (currentRow.__is_new) {

                let widgetDestinoAdd = widget.container.getWidget("WDGT_DESTINOS_ADD_REC");
                widgetDestinoAdd.enabled = true;
                self.novoDestinoInicialReceita();
                ScreenService.openPopup(widgetDestinoAdd);

            } else {
                this.saveRecipeDestinationOnForm(widget);
            }

        }

    };

    this.closeRecipeDestinoContainer = function(widget) {
        const widgetDestinoAdd = widget.container.getWidget("WDGT_DESTINOS_ADD_REC");
        const widgetReceitas = widget.container.getWidget("WDGT_RECEITAS");
        ScreenService.closePopup();
        widgetDestinoAdd.deactivate();
        widgetReceitas.activate();
    };
    /**
     * Deleta o parâmetro que aponta a inserção do primeiro
     * destino de uma nova receita.
     */
    this.deleteDestinationInsertingParameter = function() {

        var widget = templateManager.container.getWidget('WDGT_DESTINOS');

        if (Boolean(widget.ISRECIPEFIRSTDESTINATION)) {
            delete widget.ISRECIPEFIRSTDESTINATION;
        }

    };

    /**
     * Define o filtro do campo de receitas, na widget de receitas,
     * de acordo com a solicitação selecionada.
     * 
     * @param {object} widget widge de receitas('WDGT_RECEITAS').
     */
    this.setRecipeOptionsFilterBySelectedSolicitation = function(widget) {

        var currentRow = widget.currentRow;

        var recipeWidgetIndex = this.getObjectPositionInArrayByName(widget.widgets, "WDGT_RECEITAS");
        var recipeWidget = widget.widgets[recipeWidgetIndex];
        var recipeField = recipeWidget.getField('DSRECEITA');

        var fieldFilter = recipeField.dataSourceFilter;
        var nrsolemergprodFilterIndex = this.getObjectPositionInArrayByName(fieldFilter, 'NRSOLEMERGPROD');

        fieldFilter[nrsolemergprodFilterIndex].value = currentRow.NRSOLEMERGPROD;

    };

    /**
     * Busca o número de receitas usadas pelos destinos de
     * uma determinada receita.
     * 
     * @param {object} widget widget de receitas('WDGT_RECEITAS').
     */
    this.getRecipeTotalAmountByDestination = function(widget) {

        var currentRow = widget.currentRow;

        var filter = [{
            "name": "NRRECSOLEMERGPROD",
            "operator": "=",
            "value": currentRow.NRRECSOLEMERGPROD
        }];

        return localStorageEngine.download("product#/crud/destsolemergprod_qtdtotal_por_receita", filter).then(function(response) {

            if (Boolean(response[0])) {
                widget.QTDRECEITASUSADAS = response[0].QTDTOTAL;
            } else {
                widget.QTDRECEITASUSADAS = 0;
            }

        });

    };

    /**
     * Aprova uma solicitação pendente.
     * 
     * @param {object} widget widget de 
     * solicitações ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.approveSolicitation = function(widget) {

        var currentRow = widget.currentRow;

        if (currentRow.IDSITUACAO === "PENDENTE") {
            this.callProcedure(widget, 'aprovasolemergprod', 'PCP00025_SOL_APROV');
        } else {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            ScreenService.closePopup();
        }

    };

    /**
     * cancela a aprovação de uma solicitação.
     * 
     * @param {object} widget widget de 
     * solicitações ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     */
    this.cancelSolicitationApproved = function(widget) {

        var currentRow = widget.currentRow;

        if (currentRow.IDSITUACAO === "APROVADA") {
            this.callProcedure(widget, 'cancelaprovasolemergprod', 'PCP00025_APROV_CANCEL');
        } else {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_N_APROV");
            ScreenService.closePopup();
        }

    };

    /**
     * Realiza uma requisição para uma procedure.
     * 
     * @param {object} row linha que será passada para a
     * requisição.
     * 
     * @param {string} procedureRouteName nome da rota a ser
     * chamada no backend.
     */
    this.requestProcedure = function(row, procedureRouteName) {

        return restEngine.request("pcp#/" + procedureRouteName, {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 300000);

    };

    /**
     * Função para chamar uma procedure através de uma rota
     * passada como parâmetro.
     * 
     * @param {object} widget widget de solicitações
     * ('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').
     * 
     * @param {string} procedureRouteName nome da rota
     * para chamar a procedure no backend.
     * 
     * @param {string} callBackMessage mensagem a ser exibida para o usuário
     * no retorno do call back.
     */
    this.callProcedure = function(widget, procedureRouteName, callBackMessage) {

        var currentRow = widget.currentRow;
        currentRow.CDOPERADOR = getLocalVar("INFLIGHTOR_CDOPERADOR");

        this.requestProcedure(currentRow, procedureRouteName).then(function(callback) {

            if (callback && callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros);
            } else {
                //success
                MessageService.showMessageByCode(callBackMessage);
                ScreenService.closePopup();
                ScreenService.closeSwipe();

                widget.reload();
            }

        }, function(error) {

            //error
            var errorMessage = 'Timeout';

            if (error.data)
                trataMensagem(error.data.error);

        });

    };

    this.novoDestinoInicialReceita = function() {
        let widget = templateManager.container.getWidget("WDGT_DESTINOS_ADD_REC");
        widget.dataSource.data = [];
        // widget.newRow();
        widget.currentRow['__is_new'] = true;
        widget.currentRow['__createdLocal'] = true;
        widget.currentRow.NRSOLEMERGPROD = "";
        widget.currentRow.NRDESTSOLEMERGPROD = "";
        widget.edit();
        self.setFieldBlockageByRelatedFieldValue(widget.getField('DSSERVICOCIAFILI'), 'NRVOO');
        self.setFieldBlockageByRelatedFieldValue(widget.getField('DSVOO'), 'DTATENDIMENTO');
        self.setFieldBlockageByRelatedFieldValue(widget.getField('IDCICLO'), 'IDCICLO');
        self.setSolicitationDestinationDefaultValues(widget);
        self.setVooFilterByTakeOffDate(widget);
    };

    this.salvaReceitaDestino = function() {
        let widgetDest = templateManager.container.getWidget("WDGT_DESTINOS_ADD_REC");
        if (widgetDest.isValid()) {
            let widgetReceita = templateManager.container.getWidget("WDGT_RECEITAS");
            self.saveSolicitationReceipe(widgetReceita).then(function(recipeCode) {
                widgetDest.currentRow.NRRECSOLEMERGPROD = recipeCode;
                self.saveRecipeDestinationOnForm(widgetDest);
            });
        }
    };

    this.novaReceita = function() {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }
        const widget = templateManager.container.getWidget("WDGT_RECEITAS");
        InflightorGeneral.widgetNewRow(widget);
        widget.getField("DSRECEITA").readOnly = false;
    };

    this.excluirReceita = function() {
        if (templateManager.container.getWidget('WDGT_SOLICITACAO_CHECK_LIST_EMERGENCIAL').currentRow.IDSITUACAO === "APROVADA") {
            MessageService.showMessageByCode("PCP00025_OPER_BLOQ_SOL_APROV");
            return;
        }
        let widgetReceita = templateManager.container.getWidget('WDGT_RECEITAS');
        InflightorGeneral.deleteCrud(widgetReceita);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00025_solicitacao_checklist_emergencial_controller', pcp00025_solicitacao_checklist_emergencial_controller);
});

// FILE: ./assets/js/controllers/pcp00027_planejamento_producao_do_setor_controller.js
var pcp00027_planejamento_producao_do_setor_controller = function(ScreenService, templateManager, InflightorGeneral, ZhFilterPreferences, restEngine, MessageService) {

    var self = this;

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00027_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    this.carregaFiltrosPadrao = function(args, widget) {

        var widgetProducaoSetor = widget.container.getWidget('planejamento_producao_do_setor');
        InflightorGeneral.openFilterOnEnter(widgetProducaoSetor, ["CDFILIAL", "NMFILIAL", "IDSITUACAO", "DTPRODUCAOINF", 'TTURNO', 'TCATEGORIAPROD', 'TSUBCATEGORIAPROD', 'TTIPOPROD', 'TTIPONEGOCIOINF']);
        // args.e.preference.FILTER_ROW.currentRow.CDFILIAL   = getLocalVar('INFLIGHTOR_CDFILIAL');
        // args.e.preference.FILTER_ROW.currentRow.NMFILIAL   = getLocalVar('INFLIGHTOR_NMFILIAL');
        // args.e.preference.FILTER_ROW.currentRow.IDSITUACAO   = "T";

    };

    this.transformaValorStringRow = function(field) {

        var currentRow = field.widget.currentRow;

        currentRow[field.valueField] = currentRow[field.name].join('_');

        this.mudaEstadoField(field);

    };

    this.mudaEstadoField = function(field) {

        var currentRow = field.widget.currentRow;
        var widgetFields = field.widget.fields;
        var fieldsComDataSourceFilter = widgetFields.filter(function(fld) {
            return typeof fld.dataSourceFilter !== "undefined" && fld.length !== 0 && fld.name !== "NMFILIAL" && avaliaUtilizacaoDataSourceFilter(fld, field);
        });

        preencheValoresDataSourceFilter(currentRow, fieldsComDataSourceFilter);
        alteraVisibilidadeFieldDataSourceField(field, fieldsComDataSourceFilter);

    };

    var avaliaUtilizacaoDataSourceFilter = function(field, fieldAvaliado) {
        var resultadoAvaliacao = false;
        if (typeof field.dataSourceFilter !== "undefined" && field.dataSourceFilter.length) {
            field.dataSourceFilter.forEach(function(filter) {
                if (filter.name === fieldAvaliado.valueField)
                    resultadoAvaliacao = true;
            });
        }
        return resultadoAvaliacao;
    };

    var preencheValoresDataSourceFilter = function(currentRow, fieldsComDataSourceFilter) {

        for (var fld in fieldsComDataSourceFilter) {
            var fieldDataSourceFilter = fieldsComDataSourceFilter[fld].dataSourceFilter;

            for (var filtro in fieldDataSourceFilter) {
                fieldDataSourceFilter[filtro].value = typeof currentRow[fieldDataSourceFilter[filtro].name] !== 'undefined' && currentRow[fieldDataSourceFilter[filtro].name] !== "" ? currentRow[fieldDataSourceFilter[filtro].name] : null;
            }
        }

    };

    var alteraVisibilidadeFieldDataSourceField = function(field, fieldsComDataSourceFilter) {
        for (var fld in fieldsComDataSourceFilter) {
            var fieldDataSourceFilter = fieldsComDataSourceFilter[fld].dataSourceFilter;

            for (var filtro in fieldDataSourceFilter) {
                if (fieldDataSourceFilter[filtro].operator != "=")
                    continue;
                if (fieldDataSourceFilter[filtro].value === null || fieldDataSourceFilter[filtro].value === "") {
                    if (typeof fieldsComDataSourceFilter[fld].checkboxField !== 'undefined' && fieldsComDataSourceFilter[fld].checkboxField !== '') {
                        restauraField(fieldsComDataSourceFilter[fld]);
                        field.widget.getField(fieldsComDataSourceFilter[fld].checkboxField).setValue('S');
                    } else {
                        fieldsComDataSourceFilter[fld].setValue('');
                        fieldsComDataSourceFilter[fld].readOnly = true;
                    }
                } else {
                    if (typeof fieldsComDataSourceFilter[fld].checkboxField !== 'undefined' && fieldsComDataSourceFilter[fld].checkboxField !== '') {
                        restauraField(fieldsComDataSourceFilter[fld]);
                        field.widget.getField(fieldsComDataSourceFilter[fld].checkboxField).setValue('S');
                    } else {
                        fieldsComDataSourceFilter[fld].setValue('');
                        fieldsComDataSourceFilter[fld].readOnly = false;
                        fieldsComDataSourceFilter[fld].reload();
                    }
                }
            }
        }
    };

    var restauraField = function(field) {
        field.setValue('');
        field.readOnly = true;
        field.validations = {};
        field.reload();
    };

    this.corrigeRowFiltro = function(args) {
        let widget = args.owner;
        let currentRow = args.e.preference.FILTER_ROW;
        let rowToFilter = {
            CDFILIAL: currentRow.CDFILIAL,
            NMFILIAL: currentRow.NMFILIAL,
            DTPRODUCAOINF: currentRow.DTPRODUCAOINF,
            CDSETOR: currentRow.CDSETOR,
            NMSETOR: currentRow.NMSETOR
        };

        if (currentRow.IDSITUACAO && currentRow.IDSITUACAO !== 'T') {
            rowToFilter.IDSITUACAO = currentRow.IDSITUACAO;
            rowToFilter.DSSITUACAO = currentRow.DSSITUACAO;
        }

        var setFilterForFieldPair = function(selectField, checkboxField, valueToTrue) {
            if (currentRow[checkboxField] === valueToTrue) {
                widget.getField(selectField).operator = '=';
                rowToFilter[selectField] = "T";
            } else {
                widget.getField(selectField).operator = '=';
                rowToFilter[selectField] = currentRow[selectField].join("_") + "_";
            }
        };

        setFilterForFieldPair('DSTURNO', 'TTURNO', 'S');
        setFilterForFieldPair('NMCATEGORIAPROD', 'TCATEGORIAPROD', 'S');
        setFilterForFieldPair('NMSUBCATEGORIAPROD', 'TSUBCATEGORIAPROD', 'S');
        setFilterForFieldPair('DSTIPOPROD', 'TTIPOPROD', 'S');
        setFilterForFieldPair('DSTIPONEGOCIOINF', 'TTIPONEGOCIOINF', 'S');

        args.e.preference.CUSTOM_FILTER_ROW = rowToFilter;
    };

    function _setFilterReceitas(currentWidget, widget) {
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
    }

    function _closeAndReloadWidget(currentWidget) {
        ScreenService.closeSwipe(1).then(function() {
            currentWidget.reload().then(function() {
                currentWidget.activate();
            });
        });
    }

    function _reloadWidget(currentWidget) {
        currentWidget.reload().then(function() {
            currentWidget.activate();
        });
    }

    this.afterMoveRowChecklist = function(widget, currentName) {
        let currentWidget = widget.currentWidget;
        if (event && event.target.className && event.target.className.indexOf('editable-content') === -1) {
            if (currentWidget && !currentWidget.editing) {
                if (currentWidget.name === 'receitas_do_checklist' || currentWidget.name === 'destino_receita_checklist' || currentWidget.name === 'justificativa_receita_checklist') {
                    _setFilterReceitas(currentWidget, widget);
                    if (currentName && currentName === 'planejamento_producao_do_setor') {
                        _closeAndReloadWidget(currentWidget);
                    } else {
                        _reloadWidget(currentWidget);
                    }
                }
            }
        }
    };

    this.salvaAlteracoes = function(widget) {

        var receitas;
        var widgetReceitas = widget.container.getWidget('receitas_do_checklist');
        var widgetDestinos = widget.container.getWidget('destino_receita_checklist');

        if (widget.container.getWidget('planejamento_producao_do_setor').currentRow.IDSITUACAO === "APROVADOMRP") {
            if (widget.name === "receitas_do_checklist") {

                receitas = widget.container.getWidget('receitas_do_checklist').dataSource.data.filter(function(receita) {
                    return receita.changed || Object.keys(receita).filter(function(key) {
                        return key.indexOf('_inline') !== -1;
                    }).length;
                });

                if (verificaQuantidadeAlteradaQuantidadeEstoque(receitas)) {
                    restEngine.post("pcp#/salva_planejamento_producao_do_setor", {
                        "requestType": "DataSet",
                        "dataset": [
                            {
                                receitas: transformaValorFloat(receitas),
                                origem: 'receitas_do_checklist'
                            }
                        ]
                    }).then(function(success) {
                        trataMensagem(success.messages[0].message);
                        widget.list();
                        widgetReceitas.reload();
                        widgetDestinos.reload();
                    }, function(error) {
                        if (error.data)
                            trataMensagem(error.data.error);
                    });
                }

            } else if (widget.name === "destino_receita_checklist") {

                receitas = widget.container.getWidget('destino_receita_checklist').dataSource.data.filter(function(receita) {
                    return receita.changed || Object.keys(receita).filter(function(key) {
                        return key.indexOf('_inline') !== -1;
                    }).length;
                });

                if (verificaQuantidadeAlteradaQuantidadeEstoque(receitas)) {
                    restEngine.post("pcp#/salva_planejamento_producao_do_setor", {
                        "requestType": "DataSet",
                        "dataset": [
                            {
                                receitas: transformaValorFloat(receitas),
                                origem: 'destino_receita_checklist'
                            }
                        ]
                    }).then(function(success) {
                        trataMensagem(success.messages[0].message);
                        widget.list();
                        widgetReceitas.reload();
                        widgetDestinos.reload();
                    }, function(error) {
                        if (error.data)
                            trataMensagem(error.data.error);
                    });
                }

            }
        } else {
            MessageService.showMessageByCode("PCP00027_ALT_PERM_CKLIST_PEND");
        }

    };

    this.cancelaAlteracoes = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
    };

    var transformaValorFloat = function(data) {

        data.forEach(function(row) {
            row.QTRECEITACALC = InflightorGeneral.valueToFloat(row.QTRECEITACALC);
            row.QTRECEITAREJDESP = InflightorGeneral.valueToFloat(row.QTRECEITAREJDESP);
            row.QTRECEITAPROD = InflightorGeneral.valueToFloat(row.QTRECEITAPROD);
            row.QUANTIDADE = InflightorGeneral.valueToFloat(row.QUANTIDADE);
            row.QTRECEITAEST = row.QTRECEITAEST_inline !== row.QTRECEITAEST ? InflightorGeneral.valueToFloat(row.QTRECEITAEST_inline) : InflightorGeneral.valueToFloat(row.QTRECEITAEST);
            row.QTRECEITAINF = row.QTRECEITAINF_inline !== row.QTRECEITAINF ? InflightorGeneral.valueToFloat(row.QTRECEITAINF_inline) : InflightorGeneral.valueToFloat(row.QTRECEITAINF);
        });

        return data;

    };

    var verificaQuantidadeAlteradaQuantidadeEstoque = function(receitas) {

        var receitasComErro = [];
        var mensagem = "";
        receitas.forEach(function(receita) {
            if (InflightorGeneral.valueToFloat(receita.QTRECEITAINF) < InflightorGeneral.valueToFloat(receita.QTRECEITAEST)) {
                if (receita.DSRECEITA) {
                    receitasComErro.push(receita.DSRECEITA);
                } else {
                    receitasComErro.push(receita.DSRECEITADEST);
                }
            }
        });

        if (receitasComErro.length > 0) {
            mensagem = "";
            receitasComErro.forEach(function(receitaComErro) {
                if (receitaComErro != null) {
                    mensagem = mensagem + receitaComErro + "<br><br>";
                }
            });
            MessageService.showMessageByCode("PCP00027_RECE_QTESTOQUE_MAIOR", {
                "0": mensagem
            });

            return false;
        } else {
            return true;
        }

    };

    function effectChangedStatus(widget, checklistsPendentes) {
        restEngine.post("pcp#/aprova_planejamento_producao", {
            "requestType": "DataSet",
            "dataset": checklistsPendentes
        }).then(function(success) {
            MessageService.showMessageByCode("PCP00027_CKLIST_APROV");
            widget.reload();
            widget.unSelectAll();
            ScreenService.closePopup();
        }, function(error) {
            if (error.data)
                trataMensagem(error.data.error);
        });
    }

    this.aprovacaoPlanejamentoProducao = function(widget) {
        var checklistsPendentes = widget.dataSource.getCheckedRows();
        if (verificaPlanejadosAprovacao(checklistsPendentes)) {
            InflightorGeneral.callRestEngine('pcp#/inventarios_zerados', 'DataSet', checklistsPendentes).then(function(callback) {
                if (callback.dataset && callback.dataset.inventarios_zerados && callback.dataset.inventarios_zerados.length) {
                    MessageService.confirmMessageByCode(callback.dataset.inventarios_zerados[0]).then(function(success) {
                        effectChangedStatus(widget, checklistsPendentes);
                    });
                } else {
                    effectChangedStatus(widget, checklistsPendentes);
                }
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    var verificaPlanejadosAprovacao = function(checkListsPendentes) {
        var checkListComErro = [];
        var mensagem = "";
        var todosValidos = true;
        checkListsPendentes.forEach(function(checklist) {
            todosValidos = checklist.IDSITUACAO === 'APROVADOMRP';
            if (!todosValidos) {
                checkListComErro.push(checklist.CDCKLISTPRODINF);
            }
        });

        if (!todosValidos) {
            mensagem = "";
            checkListComErro.forEach(function(checklist) {
                mensagem += checklist + "<br><br>";
            });
            MessageService.showMessageByCode("PCP00027_CKLIST_N_APROV", {
                "0": mensagem
            });
            return todosValidos;
        } else {
            return todosValidos;
        }

    };

    this.cancelaAprovacaoPlanejamentoProducao = function(widget) {
        var checklistsSelecionados = widget.dataSource.getCheckedRows();
        if (verificaPlanejadosCancelaAprovacao(checklistsSelecionados)) {
            restEngine.post("pcp#/cancela_aprova_planejamento_producao", {
                "requestType": "DataSet",
                "dataset": checklistsSelecionados
            }).then(function(success) {
                MessageService.showMessageByCode("PCP00027_APROV_CANCEL");
                widget.reload();
                widget.unSelectAll();
                ScreenService.closePopup();
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    var verificaPlanejadosCancelaAprovacao = function(checklistsSelecionados) {
        var checkListComErro = [];
        var mensagem = "";
        var todosValidos = true;
        checklistsSelecionados.forEach(function(checklist) {
            todosValidos = checklist.NRREQUISICAOINF == null && (checklist.IDSITUACAO == 'APROVADOPROD' || checklist.IDSITUACAO == 'EMITIDO' || checklist.IDSITUACAO == 'CONCLUIDO');
            if (!todosValidos) {
                checkListComErro.push(checklist.CDCKLISTPRODINF);
            }
        });

        if (!todosValidos) {
            mensagem = "";
            checkListComErro.forEach(function(checklist) {
                mensagem += checklist + "<br><br>";
            });
            MessageService.showMessageByCode("PCP00027_CKLIST_REQ_APROV_GER", {
                "0": mensagem
            });
            return todosValidos;
        } else {
            return todosValidos;
        }
    };

    this.gerarRequisicao = function(widget) {
        var checklistsSelecionados = widget.dataSource.getCheckedRows();
        if (verificaPlanejadosAprovadosEmitidos(checklistsSelecionados)) {
            restEngine.post("pcp#/gerar_requisicao", {
                "requestType": "DataSet",
                "dataset": checklistsSelecionados
            }).then(function(success) {
                if (success.dataset && success.dataset.ListaErros && success.dataset.ListaErros.length) {
                    InflightorGeneral.mensagensErroProcedure(success.dataset.ListaErros);
                } else {
                    MessageService.showMessageByCode("PCP00027_REQ_GER");
                    widget.reload();
                    widget.unSelectAll();
                    ScreenService.closePopup();
                }
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    var verificaPlanejadosAprovadosEmitidos = function(checklistsSelecionados) {
        var checkListComErro = [];
        var mensagem = "";
        var todosValidos = true;
        checklistsSelecionados.forEach(function(checklist) {
            todosValidos = (checklist.IDSITUACAO === 'APROVADOPROD' || checklist.IDSITUACAO === 'EMITIDO' || checklist.IDSITUACAO === 'CONCLUIDO') && checklist.NRREQUISICAOINF == null;
            if (!todosValidos) {
                checkListComErro.push(checklist.CDCKLISTPRODINF);
            }
        });

        if (!todosValidos) {
            mensagem = "";
            checkListComErro.forEach(function(checklist) {
                mensagem += checklist + "<br><br>";
            });
            MessageService.showMessageByCode("PCP00027_OPER_REQ_PEND_GER", {
                "0": mensagem
            });
            return todosValidos;
        } else {
            return todosValidos;
        }
    };

    this.cancelaRequisicao = function(widget) {
        var checklistsSelecionados = widget.dataSource.getCheckedRows();
        if (verificaPlanejadosJaRequisitados(checklistsSelecionados)) {
            restEngine.post("pcp#/cancela_requisicao", {
                "requestType": "DataSet",
                "dataset": checklistsSelecionados
            }).then(function(success) {
                if (success.dataset && success.dataset.response && success.dataset.response.length) {
                    InflightorGeneral.showErrorMessages(widget, success.dataset.response);
                } else {
                    MessageService.showMessageByCode("PCP00027_REQ_CANCEL");
                    widget.reload();
                    widget.unSelectAll();
                    ScreenService.closePopup();
                }
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    var verificaPlanejadosJaRequisitados = function(checklistsSelecionados) {
        var checkListComErro = [];
        var mensagem = "";
        var todosValidos = true;
        checklistsSelecionados.forEach(function(checklist) {
            todosValidos = checklist.NRREQUISICAOINF != null;
            if (!todosValidos) {
                checkListComErro.push(checklist.CDCKLISTPRODINF);
            }
        });

        if (!todosValidos) {
            mensagem = "";
            checkListComErro.forEach(function(checklist) {
                mensagem += checklist + "<br><br>";
            });
            MessageService.showMessageByCode("PCP00027_CKLIST_REQ_APROV_GER", {
                "0": mensagem
            });
            return todosValidos;
        } else {
            return todosValidos;
        }
    };

    this.controlaAcaoCalculo = function(widget) {
        var widgetCurrentRow = widget.currentRow;
        var actionCalculo = widget.getAction('CALCULAR');
        if (widget.currentRow.CDRECEITADEST == null || widget.currentRow.CDRECEITADEST == '') {
            actionCalculo.isVisible = false;
        } else {
            actionCalculo.isVisible = true;
        }
    };

    this.calculaReceitaAProduzir = function(widget) {

        var currentRow = widget.currentRow;
        var widgetCalculoReceitaProduzir = widget.container.getWidget('calculo_receita_produzir');
        var currentRowProducaoSetor = widget.container.getWidget('planejamento_producao_do_setor').currentRow;
        var currentRowReceitas = widget.container.getWidget('receitas_do_checklist').currentRow;

        var filtro = [
            {
                name: 'CDFILIAL',
                operator: '=',
                value: currentRow.CDFILIAL
            },
            {
                name: 'CDRECEITA',
                operator: '=',
                value: currentRowReceitas.CDRECEITA
            },
            {
                name: 'QUANTIDADE',
                operator: '=',
                value: currentRow.QUANTIDADE
            },
            {
                name: 'DTPRODUCAO',
                operator: '=',
                value: currentRowProducaoSetor.DTPRODUCAOINF
            }
        ];

        restEngine.post("pcp#/calculo_receita_produzir", {
            "requestType": "FilterData",
            "filter": filtro
        }).then(function(success) {
            widgetCalculoReceitaProduzir.dataSource.data = success.dataset.composicao_calculo_receita_produzir;
            ScreenService.openPopup(widgetCalculoReceitaProduzir);
        });

    };

    this.filtraReceitas = function() {
        let widgetInventario = templateManager.container.getWidget("planejamento_producao_do_setor");
        let widgetReceitas = templateManager.container.getWidget("receitas_do_checklist");
        widgetReceitas.dataSourceFilter.forEach(function(filter) {
            filter.value = widgetInventario.currentRow[filter.name];
        });
        widgetReceitas.reload();
    };

    this.filtraDestinos = function() {
        let widgetReceitas = templateManager.container.getWidget("receitas_do_checklist");
        let widgetDestino = templateManager.container.getWidget("destino_receita_checklist");
        widgetDestino.dataSourceFilter.forEach(function(filter) {
            filter.value = widgetReceitas.currentRow[filter.name];
        });
        widgetDestino.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00027_planejamento_producao_do_setor_controller', pcp00027_planejamento_producao_do_setor_controller);
});

// FILE: ./assets/js/controllers/pcp00028_producao_do_setor_controller.js
var pcp00028_producao_do_setor_controller = function(ScreenService, templateManager, InflightorGeneral, ZhFilterPreferences, restEngine, MessageService) {

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00027_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    this.carregaFiltrosPadrao = function(widget) {

        var widgetProducaoSetor = widget.container.getWidget('producao_do_setor');
        InflightorGeneral.openFilterOnEnter(widgetProducaoSetor, ["NMFILIAL", "CDFILIAL", "TTURNO", "TCATEGORIAPROD", "TSUBCATEGORIAPROD", "TTIPOPROD", "TTIPONEGOCIOINF"]);
    };

    this.transformaValorStringRow = function(field) {

        var currentRow = field.widget.currentRow;

        currentRow[field.valueField] = currentRow[field.name].join('_');

        this.mudaEstadoField(field);

    };

    this.mudaEstadoField = function(field) {

        var currentRow = field.widget.currentRow;
        var widgetFields = field.widget.fields;
        var fieldsComDataSourceFilter = widgetFields.filter(function(fld) {
            return typeof fld.dataSourceFilter !== "undefined" && fld.length !== 0 && fld.name !== "NMFILIAL" && avaliaUtilizacaoDataSourceFilter(fld, field);
        });

        preencheValoresDataSourceFilter(currentRow, fieldsComDataSourceFilter);
        alteraVisibilidadeFieldDataSourceField(field, fieldsComDataSourceFilter);

    };

    var avaliaUtilizacaoDataSourceFilter = function(field, fieldAvaliado) {
        var resultadoAvaliacao = false;
        if (typeof field.dataSourceFilter !== "undefined" && field.dataSourceFilter.length) {
            field.dataSourceFilter.forEach(function(filter) {
                if (filter.name === fieldAvaliado.valueField)
                    resultadoAvaliacao = true;
            });
        }
        return resultadoAvaliacao;
    };

    var preencheValoresDataSourceFilter = function(currentRow, fieldsComDataSourceFilter) {

        for (var fld in fieldsComDataSourceFilter) {
            var fieldDataSourceFilter = fieldsComDataSourceFilter[fld].dataSourceFilter;

            for (var filtro in fieldDataSourceFilter) {
                if (fieldDataSourceFilter[filtro].operator !== 'LIKE_ALL')
                    fieldDataSourceFilter[filtro].value = typeof currentRow[fieldDataSourceFilter[filtro].name] !== 'undefined' && currentRow[fieldDataSourceFilter[filtro].name] !== "" ? currentRow[fieldDataSourceFilter[filtro].name] : null;
            }
        }

    };

    var alteraVisibilidadeFieldDataSourceField = function(field, fieldsComDataSourceFilter) {
        for (var fld in fieldsComDataSourceFilter) {
            var fieldDataSourceFilter = fieldsComDataSourceFilter[fld].dataSourceFilter;

            for (var filtro in fieldDataSourceFilter) {
                if ((fieldDataSourceFilter[filtro].value === null || fieldDataSourceFilter[filtro].value === "") && fieldDataSourceFilter[filtro].operator !== 'LIKE_ALL') {
                    if (typeof fieldsComDataSourceFilter[fld].checkboxField !== 'undefined' && fieldsComDataSourceFilter[fld].checkboxField !== '') {
                        restauraField(fieldsComDataSourceFilter[fld]);
                        field.widget.getField(fieldsComDataSourceFilter[fld].checkboxField).setValue('S');
                    } else {
                        fieldsComDataSourceFilter[fld].setValue('');
                        fieldsComDataSourceFilter[fld].readOnly = true;
                    }
                } else {
                    if (typeof fieldsComDataSourceFilter[fld].checkboxField !== 'undefined' && fieldsComDataSourceFilter[fld].checkboxField !== '') {
                        restauraField(fieldsComDataSourceFilter[fld]);
                        field.widget.getField(fieldsComDataSourceFilter[fld].checkboxField).setValue('S');
                    } else {
                        fieldsComDataSourceFilter[fld].setValue('');
                        fieldsComDataSourceFilter[fld].readOnly = false;
                        fieldsComDataSourceFilter[fld].reload();
                    }
                }
            }
        }
    };

    var restauraField = function(field) {
        field.setValue('');
        field.readOnly = true;
        field.validations = {};
        field.reload();
    };

    this.defineFiltroSetor = function(widget) {

        var filterPopupCurrentRow = widget.currentRow;

        var valorFiltro = filterPopupCurrentRow.CDFILIAL ? filterPopupCurrentRow.CDFILIAL : "";
        var fieldSetor = widget.getField("NMSETOR");
        let filterUnit = fieldSetor.dataSourceFilter.find(filter => filter.name == 'CDFILIAL');
        if (filterUnit) {
            filterUnit.value = valorFiltro;
        } else {
            fieldSetor.dataSourceFilter.push({
                name: "CDFILIAL",
                operator: "=",
                value: valorFiltro
            });
        }

    };

    this.corrigeRowFiltro = function(args) {

        var widget = args.owner;
        var rowFiltro = angular.copy(args.e.preference.FILTER_ROW);
        var filtro = [];
        var rowAjustada = {
            CDFILIAL: rowFiltro.CDFILIAL,
            DTPRODUCAOINF: rowFiltro.DTPRODUCAOINF,
            CDSETOR: rowFiltro.CDSETOR,
            DSTURNO: rowFiltro.TTURNO === 'S' ? 'T' : '_' + rowFiltro.DSTURNO.join('_') + '_',
            NMCATEGORIAPROD: rowFiltro.TCATEGORIAPROD === 'S' ? 'T' : '_' + rowFiltro.NMCATEGORIAPROD.join('_') + '_',
            NMSUBCATEGORIAPROD: rowFiltro.TSUBCATEGORIAPROD === 'S' ? 'T' : '_' + rowFiltro.NMSUBCATEGORIAPROD.join('_') + '_',
            DSTIPOPROD: rowFiltro.TTIPOPROD === 'S' ? 'T' : '_' + rowFiltro.DSTIPOPROD.join('_') + '_',
            DSTIPONEGOCIOINF: rowFiltro.TTIPONEGOCIOINF === 'S' ? 'T' : '_' + rowFiltro.DSTIPONEGOCIOINF.join('_') + '_'
        };

        var colunas = Object.keys(rowAjustada);

        for (var coluna in colunas) {
            filtro.push({
                name: colunas[coluna],
                operator: '=',
                value: rowAjustada[colunas[coluna]]
            });
        }

        filtro.push({
            name: 'IDSITUACAO',
            operator: 'IN',
            value: ['APROVADOPROD', 'EMITIDO', 'CONCLUIDO']
        });

        widget.dataSourceFilter = filtro;

    };

    function _setFilterReceitas(currentWidget, widget) {
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
    }

    function _closeAndReloadWidget(currentWidget) {
        ScreenService.closeSwipe(1).then(function() {
            currentWidget.reload().then(function() {
                currentWidget.activate();
            });
        });
    }

    function _reloadWidget(currentWidget) {
        currentWidget.reload().then(function() {
            currentWidget.activate();
        });
    }

    this.afterMoveRowChecklist = function(widget, currentName) {
        let currentWidget = widget.currentWidget;
        if (event && event.target.className && event.target.className.indexOf('editable-content') === -1) {
            if (currentWidget && !currentWidget.editing) {
                if (currentWidget.name === 'receitas_do_checklist' || currentWidget.name === 'destino_receita_checklist' || currentWidget.name === 'justificativa_receita_checklist') {
                    _setFilterReceitas(currentWidget, widget);
                    if (currentName && currentName === 'producao_do_setor') {
                        _closeAndReloadWidget(currentWidget);
                    } else {
                        _reloadWidget(currentWidget);
                    }
                }
            }
        }
    };

    this.setFiltroReceita = function() {
        let currentWidget = templateManager.container.getWidget("receitas_do_checklist");
        let widget = templateManager.container.getWidget("producao_do_setor");
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
        currentWidget.reload();
    };

    this.setFiltroDestinos = function() {
        let currentWidget = templateManager.container.getWidget("destino_receita_checklist");
        let widget = templateManager.container.getWidget("receitas_do_checklist");
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
        currentWidget.reload();

        currentWidget = templateManager.container.getWidget("justificativa_receita_checklist");

        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
        currentWidget.reload();
    };

    /*
     * todo remove later
     */
    this.test = function(a) {
        console.log(a);
        // debugger;
    };

    function _checkedRowsByOperation(widget, situacao, operation) {
        let validCheckedRows = [];
        let checkedRows = widget.dataSource.getCheckedRows();
        validCheckedRows.push(checkedRows.filter(function(row) {
            return row.IDSITUACAO === situacao;
        }));
        if (checkedRows.length !== validCheckedRows[0].length) {
            validCheckedRows.push(checkedRows.filter(function(row) {
                return row.IDSITUACAO !== situacao;
            }));
        } else {
            validCheckedRows.push({});
        }
        return validCheckedRows;
    }

    this.changedStatusChecklist = function(widget, situacao, value, operation, newData) {
        let dataChanged = _checkedRowsByOperation(widget, situacao, 'changed');
        if (dataChanged.length === 2 && !dataChanged[1].length) {
            if (value == 'ALTERDATA') {
                dataChanged[0].forEach(function(row) {
                    row.DTPRODUCAOINF = newData;
                });
            } else {
                dataChanged[0].forEach(function(row) {
                    row.IDSITUACAO = value;
                });
            }
            InflightorGeneral.callRestEngine('pcp#/crud/cklistprodinf/save', 'DataSet', dataChanged[0]).then(function(callback) {
                //success
                ScreenService.closePopup(true);
                widget.dataSource.unSelectAll();
                widget.reload().then(function() {
                    MessageService.showMessageByCode("PCP00022_OPER_SUCES");
                    widget.activate();
                });
            }, InflightorGeneral.callbackError);
        } else {
            //Existem rows invalidas selecionadas.
            _showCheckedError(widget, dataChanged[1], operation);
        }
    };

    function _showCheckedError(widget, dataError, operation) {
        MessageService.getMessageByCode(["PCP00022_CHECKLIST_PRODSET"]).then(function(mensagens) {
            let messagesError = [];
            dataError.forEach(function(row) {
                let label = {
                    "SITUACAO": widget.getField('IDSITUACAO').dataSource.data.filter(function(options) {
                        return options.IDSITUACAO === row.IDSITUACAO;
                    })[0]
                };
                messagesError.push(ScreenService.i18n(mensagens[0], {
                    "0": row.CDCKLISTPRODINF,
                    "1": label['SITUACAO']['DSSITUACAO']
                }));
            });
            InflightorGeneral.showErrorMessages(widget, messagesError);
        });
    }

    this.salvaAlteracoes = function(widget) {

        var receitas;
        var widgetJustificativa;
        var widgetReceitas = widget.container.getWidget('receitas_do_checklist');
        var widgetDestinos = widget.container.getWidget('destino_receita_checklist');

        if (widget.container.getWidget('producao_do_setor').currentRow.IDSITUACAO === "APROVADOPROD" || widget.container.getWidget('producao_do_setor').currentRow.IDSITUACAO === "EMITIDO") {
            if (widget.name === "receitas_do_checklist") {

                receitas = widget.container.getWidget('receitas_do_checklist').dataSource.data.filter(function(receita) {
                    return receita.changed || Object.keys(receita).filter(function(key) {
                        return key.indexOf('_inline') !== -1;
                    }).length;
                });

                widgetJustificativa = widget.container.getWidget('justificativa');
                widgetJustificativa.__origem = 'receitas_do_checklist';

                if (verificaRejeitoDesperdicio(receitas)) {
                    ScreenService.openPopup(widgetJustificativa).then(function() {
                        widgetJustificativa.reload().then(function() {
                            widgetJustificativa.activate();
                            widgetJustificativa.unSelectAll();
                        });
                    });
                } else {
                    restEngine.post("pcp#/salva_alteracoes_producao_setor", {
                        "requestType": "DataSet",
                        "dataset": [
                            {
                                justificativa: [],
                                receitas: transformaValorFloat(receitas),
                                origem: 'receitas_do_checklist'
                            }
                        ]
                    }).then(function(success) {
                        MessageService.showMessageByCode(success.messages[0].message);
                        widget.list();
                        widgetReceitas.reload();
                        widgetDestinos.reload();
                    }, function(error) {
                        if (error.data)
                            trataMensagem(error.data.error);
                    });
                }

            } else if (widget.name === "destino_receita_checklist") {

                receitas = widget.container.getWidget('destino_receita_checklist').dataSource.data.filter(function(receita) {
                    return receita.changed || Object.keys(receita).filter(function(key) {
                        return key.indexOf('_inline') !== -1;
                    }).length;
                });

                widgetJustificativa = widget.container.getWidget('justificativa');
                widgetJustificativa.__origem = 'destino_receita_checklist';

                if (verificaRejeitoDesperdicio(receitas)) {
                    ScreenService.openPopup(widgetJustificativa);
                } else {
                    restEngine.post("pcp#/salva_alteracoes_producao_setor", {
                        "requestType": "DataSet",
                        "dataset": [
                            {
                                justificativa: [],
                                receitas: transformaValorFloat(receitas),
                                origem: 'destino_receita_checklist'
                            }
                        ]
                    }).then(function(success) {
                        MessageService.showMessageByCode(success.messages[0].message);
                        widget.list();
                        widgetReceitas.reload();
                        widgetDestinos.reload();
                    }, function(error) {
                        if (error.data)
                            trataMensagem(error.data.error);
                    });
                }

            }
        } else {
            MessageService.showMessageByCode("PCP00027_OPER_PERM_APROVPROD_EMIT");
        }
    };

    this.cancelaAlteracoes = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    var verificaRejeitoDesperdicio = function(data) {

        var possuiRejitoDesperdicio = false;

        for (var row in data) {
            if (typeof data[row].changed_QTRECEITAREJDESP !== "undefined" && data[row].changed_QTRECEITAREJDESP !== data[row].QTRECEITAREJDESP)
                possuiRejitoDesperdicio = true;
        }

        return possuiRejitoDesperdicio;

    };

    this.salvaAlteracoesComJustificativa = function(widget) {

        var receitas;
        var justificativa;
        var widgetReceitas = widget.container.getWidget('receitas_do_checklist');
        var widgetDestinos = widget.container.getWidget('destino_receita_checklist');

        if (widget.__origem === "receitas_do_checklist") {

            receitas = widget.container.getWidget('receitas_do_checklist').dataSource.data.filter(function(receita) {
                return receita.changed || Object.keys(receita).filter(function(key) {
                    return key.indexOf('_inline') !== -1;
                }).length;
            });
            justificativa = widget.getCheckedRows();

            restEngine.post("pcp#/salva_alteracoes_producao_setor", {
                "requestType": "DataSet",
                "dataset": [
                    {
                        justificativa: justificativa,
                        receitas: transformaValorFloat(receitas),
                        origem: 'receitas_do_checklist'
                    }
                ]
            }).then(function(success) {
                MessageService.showMessageByCode(success.messages[0].message);
                ScreenService.closePopup();
                widgetReceitas.list();
                widget.container.getWidget('receitas_do_checklist').activate();
                widgetReceitas.reload();
                widgetDestinos.reload();
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        } else if (widget.__origem === "destino_receita_checklist") {
            receitas = widget.container.getWidget('destino_receita_checklist').dataSource.data.filter(function(receita) {
                return receita.changed || Object.keys(receita).filter(function(key) {
                    return key.indexOf('_inline') !== -1;
                }).length;
            });

            justificativa = widget.getCheckedRows();
            restEngine.post("pcp#/salva_alteracoes_producao_setor", {
                "requestType": "DataSet",
                "dataset": [
                    {
                        justificativa: justificativa,
                        receitas: transformaValorFloat(receitas),
                        origem: 'destino_receita_checklist'
                    }
                ]
            }).then(function(success) {
                MessageService.showMessageByCode(success.messages[0].message);
                ScreenService.closePopup();
                widget.container.getWidget('destino_receita_checklist').list();
                widget.container.getWidget('destino_receita_checklist').activate();
                widget.container.getWidget('destino_receita_checklist').reload();
                widgetReceitas.reload();
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    var transformaValorFloat = function(data) {

        data.forEach(function(row) {
            row.QTRECEITACALC = InflightorGeneral.valueToFloat(row.QTRECEITACALC);
            row.QTRECEITAINF = InflightorGeneral.valueToFloat(row.QTRECEITAINF);
            row.QTRECEITAEST = InflightorGeneral.valueToFloat(row.QTRECEITAEST);
            row.QUANTIDADE = InflightorGeneral.valueToFloat(row.QUANTIDADE);
            row.QTRECEITAPROD = row.QTRECEITAPROD_inline !== row.QTRECEITAPROD ? InflightorGeneral.valueToFloat(row.QTRECEITAPROD_inline) : InflightorGeneral.valueToFloat(row.QTRECEITAPROD);
            row.QTRECEITAREJDESP = row.QTRECEITAREJDESP_inline !== row.QTRECEITAREJDESP ? InflightorGeneral.valueToFloat(row.QTRECEITAREJDESP_inline) : InflightorGeneral.valueToFloat(row.QTRECEITAREJDESP);
        });

        return data;

    };

    this.controlaAcaoCalculo = function(widget) {
        var widgetCurrentRow = widget.currentRow;
        var actionCalculo = widget.getAction('CALCULAR');
        if (widget.currentRow.CDRECEITADEST == null || widget.currentRow.CDRECEITADEST == '') {
            actionCalculo.isVisible = false;
        } else {
            actionCalculo.isVisible = true;
        }
    };

    this.unSelect = function(widget) {
        widget.unSelectAll();
    };

    this.calculaReceitaAProduzir = function(widget) {

        var currentRow = widget.currentRow;
        var widgetCalculoReceitaProduzir = widget.container.getWidget('calculo_receita_produzir');
        var currentRowProducaoSetor = widget.container.getWidget('producao_do_setor').currentRow;
        var currentRowReceitas = widget.container.getWidget('receitas_do_checklist').currentRow;

        var filtro = [
            {
                name: 'CDFILIAL',
                operator: '=',
                value: currentRow.CDFILIAL
            },
            {
                name: 'CDRECEITA',
                operator: '=',
                value: currentRowReceitas.CDRECEITA
            },
            {
                name: 'QUANTIDADE',
                operator: '=',
                value: currentRow.QUANTIDADE
            },
            {
                name: 'DTPRODUCAO',
                operator: '=',
                value: currentRowProducaoSetor.DTPRODUCAOINF
            }
        ];

        restEngine.post("pcp#/calculo_receita_produzir", {
            "requestType": "FilterData",
            "filter": filtro
        }).then(function(success) {
            widgetCalculoReceitaProduzir.dataSource.data = success.dataset.composicao_calculo_receita_produzir;
            ScreenService.openPopup(widgetCalculoReceitaProduzir);
        });

    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00028_producao_do_setor_controller', pcp00028_producao_do_setor_controller);
});

// FILE: ./assets/js/controllers/pcp00031_relatorios_da_producao_controller.js
var pcp00031_relatorios_da_producao_controller = function(ScreenService, templateManager, InflightorGeneral, ZhFilterPreferences, restEngine, PCPUtil) {

    var VISIBILIDADE_FIELDS = {
        'PCP00310': {
            'NMFILIAL': true,
            'T_SETOR': true,
            'T_TURNO': true,
            'T_CATEGORIAPROD': true,
            'T_SUBCATEGORIAPROD': true,
            'T_TIPOPROD': true,
            'T_TIPONEGOCIOINF': true,
            'T_RECEITA': true,
            'DTREFINI': true,
            'DTREFFIM': true
        },
        'PCP00311': {
            'NMFILIAL': true,
            'T_SETOR': true,
            'T_TURNO': true,
            'T_CATEGORIAPROD': true,
            'T_SUBCATEGORIAPROD': true,
            'T_TIPOPROD': true,
            'T_TIPONEGOCIOINF': true,
            'T_RECEITA': true,
            'DTREFINI': true,
            'DTREFFIM': true
        },
        'PCP00312': {
            'NMFILIAL': true,
            'T_SETOR': true,
            'T_TURNO': true,
            'T_CATEGORIAPROD': false,
            'T_SUBCATEGORIAPROD': false,
            'T_TIPOPROD': true,
            'T_TIPONEGOCIOINF': false,
            'T_RECEITA': false,
            'DTREFINI': true,
            'DTREFFIM': true
        }
    };

    this.alteraVisibilidadeFields = function(field) {
        var fields = field.widget.fields;
        var nomeRelatorio = field.widget.currentRow.REPORTNAME;
        fields.forEach(function(field) {
            if (VISIBILIDADE_FIELDS[nomeRelatorio].hasOwnProperty(field.name)) {
                field.isVisible = VISIBILIDADE_FIELDS[nomeRelatorio][field.name];
            }
        });
        templateManager.updateTemplate();
    };

    var validaDias = function(widget) {
        var dataInicial = widget.currentRow.DTREFINI;
        dataInicial = moment(dataInicial, 'DD/MM/YYYY');
        widget.getField('DTREFFIM').minDate = moment(dataInicial, 'DD/MM/YYYY');
    };

    this.habilitaFieldData = function(widget) {
        widget.getField('DTREFFIM').readOnly = true;

        if (widget.currentRow.DTREFINI) {
            widget.getField('DTREFFIM').readOnly = false;
            widget.getField('DTREFFIM').validations = {
                "required": true
            };
            validaDias(widget);
        } else {
            widget.getField('DTREFFIM').validations = {};
        }
    };

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        const rowFilter = PCPUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        clearFieldCustomSelectMultiple(['T_SETOR', 'T_RECEITA'], widget);
        PCPUtil.updateFilterFields(['T_SETOR', 'T_RECEITA'], 'CDFILIAL', rowFilter, true, widget);
    };

    function clearFieldCustomSelectMultiple(fields, widget) {
        fields = Array.isArray(fields) ? fields : [fields];
        fields.forEach(field => {
            field = typeof field == 'string' ? widget.getField(field) : field;
            if (field) {
                field.multipleSelectValueStr = null;
                field.multipleSelectCheckedRows = [];
                if (field.dataSource && field.dataSource.clearCheckedRows) {
                    field.dataSource.clearCheckedRows();
                }
                field.widget.currentRow[field.name] = 'T';
            }
        });
    }

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00031_relatorios_da_producao_controller', pcp00031_relatorios_da_producao_controller);
});

// FILE: ./assets/js/controllers/pcp00033_relatorios_requisicaoController.js
var pcp00033_relatorios_requisicaoController = function(PCPUtil, templateManager, pcpReportService) {

    this.onChangeUnit = function(fieldUnit) {
        const widget = fieldUnit.widget;
        let rowFilter = PCPUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        PCPUtil.clearFields(widget, ['CDSETOR', 'NRREQUISICAOINF', 'NRRETIRADAINF']);
        PCPUtil.updateFilterFields(['CDSETOR', 'NRREQUISICAOINF', 'NRRETIRADAINF'], 'CDFILIAL', rowFilter, true, widget);
    };

    this.setFieldsVisibility = function(widget) {
        let nameFieldsRequisicao = ['NRREQUISICAOINF', 'T_REQUISICAO'];
        let nameFieldsRetirada = ['NRRETIRADAINF', 'T_RETIRADA'];
        if (widget.currentRow.REPORTNAME == 'PCP00330' || widget.currentRow.REPORTNAME == 'PCP00332') {
            setVisibility(widget, nameFieldsRequisicao, true);
            setVisibility(widget, nameFieldsRetirada, false);
        } else if (widget.currentRow.REPORTNAME == 'PCP00331' || widget.currentRow.REPORTNAME == 'PCP00333') {
            setVisibility(widget, nameFieldsRequisicao, false);
            setVisibility(widget, nameFieldsRetirada, true);
        }
        templateManager.updateTemplate();
    };

    function setVisibility(widget, nameFields, isVisible = true) {
        nameFields = Array.isArray(nameFields) ? nameFields : [nameFields];
        nameFields.forEach(nameField => {
            let field = widget.getField(nameField);
            if (field) field.isVisible = isVisible;
        });
    }

    var validaDias = function(widget) {
        var dataInicial = widget.currentRow.DTREFINI;
        dataInicial = moment(dataInicial, 'DD/MM/YYYY');
        widget.getField('DTREFFIM').minDate = moment(dataInicial, 'DD/MM/YYYY');
    };

    this.habilitaFieldData = function(widget) {
        const fieldEndDate = widget.getField('DTREFFIM');
        fieldEndDate.readOnly = true;

        if (widget.currentRow.DTREFINI) {
            fieldEndDate.readOnly = false;
            fieldEndDate.validations = {
                "required": true
            };
            validaDias(widget);
        } else {
            fieldEndDate.validations = {};
        }
    };

    this.generateReport = function(widget, format) {
        if (!widget.isValid()) return;
        let row = angular.copy(widget.currentRow);
        let config = {
            T_PRODUTO: 'CDPRODUTO',
            T_REQUISICAO: 'NRREQUISICAOINF',
            T_RETIRADA: 'NRRETIRADAINF',
            T_SETOR: 'CDSETOR',
            T_TURNO: 'NRTURNO'
        };
        Object.keys(config).forEach(checkboxFieldName => {
            let fieldCheckbox = widget.getField(checkboxFieldName);
            if (fieldCheckbox && fieldCheckbox.isVisible) {
                let propInRow = config[checkboxFieldName];
                row[checkboxFieldName] = row[checkboxFieldName] == 'T' ? 'T' : row[propInRow].join('_');
            }
        });
        pcpReportService.openReport(widget, format, row);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00033_relatorios_requisicaoController', pcp00033_relatorios_requisicaoController);
});

// FILE: ./assets/js/controllers/pcp00035_relatorio_de_previsao_de_passageirosController.js
var pcp00035_relatorio_de_previsao_de_passageirosController = function(ScreenService, InflightorGeneral, PCPUtil) {

    this.initialAndChangedStep = function(widget) {
        widget.container.getWidget('widgetPrevisaoPassageiros').activate();
        widget.container.getWidget('widgetPrevisaoPassageiros').newRow();
    };

    var FIELDS_VISIBILITY = {
        'PCP00350': {
            'CDFILIAL': false,
            'NMFILIAL': true,
            'DTREFINI': true,
            'DTREFFIM': true,
            'T_CIA': true
        },
        'PCP00313': {
            'CDFILIAL': false,
            'NMFILIAL': true,
            'DTREFINI': true,
            'DTREFFIM': true,
            'T_CIA': false
        }
    };

    this.onChangeReport = function(field) {
        var fields = field.widget.fields;
        var reportName = field.widget.currentRow.REPORTNAME;
        fields.forEach(function(field) {
            if (FIELDS_VISIBILITY[reportName].hasOwnProperty(field.name)) {
                field.isVisible = FIELDS_VISIBILITY[reportName][field.name];
            }
        });
    };

    this.beforeSelectOpenCia = function(fieldCia) {
        const widget = fieldCia.widget;
        let rowFilter = PCPUtil.getValuesInRow(widget, {
            NMFILIAL: 'CDFILIAL'
        });
        PCPUtil.updateFilterFields(fieldCia, 'CDFILIAL', rowFilter, true);
    };

    this.report = function(widget, format) {
        if (widget.isValid()) {
            widget.currentRow.REPORTFORMAT = format;
            InflightorGeneral.openReport('pcp#/relatorioPrevisaoPassageiros', widget.currentRow.REPORTNAME, widget.currentRow);
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00035_relatorio_de_previsao_de_passageirosController', pcp00035_relatorio_de_previsao_de_passageirosController);
});

// FILE: ./assets/js/controllers/pcp00038_parametros_de_producao_controller.js
var pcp00038_parametros_de_producao_controller = function(InflightorGeneral, ScreenService, templateManager, restEngine, ZhFilterPreferences, MessageService) {

    var self = this;

    /* 
     * @deprecated 
     */
    this.aplicaValoresPadrao = function(widget) {
        var widgetParametrosProducao = widget.container.getWidget('parametros_de_producao');
        InflightorGeneral.openFilterOnEnter(widgetParametrosProducao, ['CDFILIAL', 'NMFILIAL']);
    };

    this.aplicaValoresRow = function(widget) {
        widget.currentRow = widget.dataSource.data[0];
    };

    this.mudaParaEdicao = function(widget) {
        if (widget.currentRow.CDFILIAL) {
            widget.backupRow = angular.copy(widget.currentRow);
            widget.template = "widget/form.html";
            widget.edit();
        } else {
            MessageService.showMessageByCode("UNIT_NOT_INFORMED");
        }
    };

    this.cancelaEdicao = function(widget, cancelaEdicao = true) {
        if (cancelaEdicao) {
            widget.currentRow = widget.backupRow;
        }
        widget.template = "widget/form-view.html";
        widget.list();
    };

    this.salvaEdicao = function(widget) {
        var colunasFloat = ["VRPER1CALCMEDIAPAX", "VRPER2CALCMEDIAPAX", "VRPER3CALCMEDIAPAX"];
        for (var i = 0; i <= colunasFloat.length; i++) {
            widget.currentRow[colunasFloat[i]] = InflightorGeneral.valueToFloat(widget.currentRow[colunasFloat[i]]);
        }
        restEngine.post('pcp#/crud/paramcomis/save', {
            "requestType": "Row",
            "row": widget.currentRow
        }).then(function(success) {
            widget.reload().then(function() {
                self.cancelaEdicao(widget, false);
            });
        });
    };

    /* filter stuff  */

    this.openFilter = function() {
        var filterWidget = templateManager.container.getWidget('filterPopup');
        var formWidget = templateManager.container.getWidget('parametros_de_producao');
        var filterRow = formWidget.filterRow || {};
        if (Util.isEmptyObject(filterRow)) {
            ScreenService.openPopup(filterWidget);
            filterWidget.activate();
            filterWidget.newRow();
        } else {
            ScreenService.openPopup(filterWidget);
            filterWidget.activate();
            // filterWidget.currentRow = angular.copy(filterRow);
        }
    };

    this.applyFilter = function(filterWidget) {
        if (!filterWidget) {
            filterWidget = templateManager.container.getWidget('filterPopup');
        }

        if (filterWidget.isValid()) {
            var paramsWidget = filterWidget.container.getWidget('parametros_de_producao');
            var row = filterWidget.currentRow;
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": row.CDFILIAL
                    }
                ];
            paramsWidget.filterRow = angular.copy(row);
            paramsWidget.dataSourceFilter = filter;
            paramsWidget.reload().then((data) => {
                if (paramsWidget.dataSource &&
                    paramsWidget.dataSource.data &&
                    paramsWidget.dataSource.data.length > 0) {
                    paramsWidget.currentRow = paramsWidget.dataSource.data[0];
                }
            }).finally(() => {
                paramsWidget.activate();
            });
            ScreenService.closePopup();
        }

    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('pcp00038_parametros_de_producao_controller', pcp00038_parametros_de_producao_controller);
});

// FILE: ./assets/js/controllers/pcp00039_quantidade_refeicoes_dia_controller.js
var pcp00039_quantidade_refeicoes_dia_controller = function(InflightorGeneral, ScreenService, templateManager, restEngine) {

    this.aplicaDefaultValue = function(widget) {
        widget.newRow();
        widget.currentRow.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL');
        widget.currentRow.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL');
    };

    this.aplicaFiltroField = function(field) {
        var widget = field.widget;

        widget.fields.forEach(function(fld) {
            if (typeof fld.dataSourceFilter !== 'undefined' && fld.dataSourceFilter.length) {
                fld.dataSourceFilter.forEach(function(filtro) {
                    filtro.value = filtro.name == field.valueField ? widget.currentRow[field.valueField] : filtro.value;
                });
            }
        });
    };

    this.habilitaFieldData = function(field) {
        var row = field.widget.currentRow;
        if (field.widget.currentRow.DTINICIAL) {
            habilitaFields(row, field, ['DTFINAL'], true);
        } else {
            habilitaFields(row, field, ['DTFINAL'], false);
        }
    };

    var habilitaFields = function(row, field, fieldsDependentes, habilita) {

        var filho;

        if (habilita) {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = false;
            }
        } else {
            for (filho in fieldsDependentes) {
                field.widget.getField(fieldsDependentes[filho]).readOnly = true;
                field.widget.getField(fieldsDependentes[filho]).value('');
            }
        }
    };

    this.relatorio = function(widget, formato) {
        if (widget.isValid()) {
            var row = ajustaRowParaRelatorio(widget.currentRow, formato);
            if (moment(row.DTFINAL, 'DD/MM/YYYY').isBefore(moment(row.DTINICIAL, 'DD/MM/YYYY'))) {
                MessageService.showMessageByCode("PCP00041_PERIODO_INV");
            } else {
                InflightorGeneral.openReport('ate#/quantidadeRefeicoesDiaController', row.REPORTNAME, row).then(function(callback) {
                    ScreenService.closePopup();
                    widget.activate();
                    widget.reload();
                }, function(error) {
                    var errorMessage = 'Timeout';
                    if (error.data)
                        errorMessage = error.data.error;
                    ScreenService.showMessage(errorMessage);
                });
            }
        }
    };

    var ajustaRowParaRelatorio = function(row, formato) {

        var newRow = angular.copy(row);

        newRow.REPORTFORMAT = formato;

        if (row.TNMCIA == "T") {
            newRow.CDCIA = row.TNMCIA;
        } else {
            newRow.CDCIA = "_" + row.NMCIA.join('_') + "_";
        }

        return newRow;

    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('pcp00039_quantidade_refeicoes_dia_controller', pcp00039_quantidade_refeicoes_dia_controller);
});

// FILE: ./assets/js/controllers/pcp00040_relatorios_estimativas_refeicoesController.js
var pcp00040_relatorios_estimativas_refeicoesController = function(ZHPromise, ScreenService, InflightorGeneral, MessageService, templateManager) {

    var _self = this;
    var reportDefault = "PCP00400";
    var _fieldsConfigDefault = {
        "PCP00400": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "NRVOO": true,
            "TVOO": true,
            "DSNACIONALIDADE": true,
            "DATAINI": true,
            "DATAFIN": true
        },
        "PCP00401": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "NRVOO": true,
            "TVOO": true,
            "DATAINI": true,
            "DATAFIN": true
        },
        "PCP00402": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DSTIPONEGOCIOINF": true,
            "TNEGOCIOINF": true,
            "DATAINI": true,
            "DATAFIN": true
        },
        "PCP00403": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DTMESANO": true
        },
        "PCP00404": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DSNACIONALIDADE": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DSSERVICOCIAFILI": true,
            "TSERVICO": true,
            "DATAINI": true,
            "DATAFIN": true
        },
        "PCP00405": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "DSNACIONALIDADE": true,
            "NMCIA": true,
            "TNMCIA": true,
            "DSSERVICOCIAFILI": true,
            "TSERVICO": true,
            "DTMESANO": true
        },
        "PCP00406": {
            "REPORTNAME": true,
            "NMFILIAL": true,
            "NMCIA": true,
            "TNMCIA": true,
            "NRVOO": true,
            "TVOO": true,
            "DSNACIONALIDADE": true,
            "DATAINI": true,
            "DATAFIN": true
        }
    };

    this.initialAndChangedStep = function(widget) {
        let reportName = widget.currentRow.REPORTNAME || widget.getField('REPORTNAME').value() || reportDefault;
        if (!widget.getField('REPORTNAME').value()) {
            widget.getField('REPORTNAME').applyDefaultValue();
            widget.getField('CDFILIAL').applyDefaultValue();
            widget.getField('NMFILIAL').applyDefaultValue();
        }
        if (widget.fields && Array.isArray(widget.fields)) {
            widget.fields.forEach(function(field) {
                if (field.name !== 'REPORTNAME') {
                    /*if (field.defaultValue) {
                        field.applyDefaultValue();
                    } else {
                        field.setValue("");
                    }*/
                    /*if (field.template && field.template.indexOf('select-multiple') !== -1) {
                        field.readOnly = true;
                        field.validations = {};
                        field.setValue('');
                    }*/
                    if (!field.value()) {
                        field.setValue('');
                    }
                    field.isVisible = !!_fieldsConfigDefault[reportName][field.name];
                }
            });
            templateManager.updateTemplate();
        }
    };

    this.filterSelectBeforeOpen = function(field) {
        field.widget.fields.forEach(function(selectField) {
            if (selectField.template && selectField.template.indexOf('select-multiple') !== -1) {
                field.widget.currentRow[selectField.valueField] = field.widget.currentRow[selectField.name] && Array.isArray(field.widget.currentRow[selectField.name]) ? field.widget.currentRow[selectField.name].join('_') : "T";
            }
        });
        InflightorGeneral.onClickDependentField(field, field.widget.currentRow);
    };

    this.fieldOnChangeFilial = function(field, camposDependentes) {
        var currentRow = field.widget.currentRow;
        var filtro = {
            "name": "CDFILIAL",
            "operator": "=",
            "value": field.widget.currentRow.CDFILIAL
        };
        for (let campo in camposDependentes) {
            field.widget.getField(camposDependentes[campo]).dataSourceFilter = field.widget.getField(camposDependentes[campo]).dataSourceFilter.filter(function(condicao) {
                return condicao.name != "CDFILIAL";
            });
            field.widget.getField(camposDependentes[campo]).value();
            currentRow[camposDependentes[campo]] = null;
            field.widget.getField(camposDependentes[campo]).dataSourceFilter.push(filtro);
            field.widget.getField(camposDependentes[campo]).reload();
        }
    };

    this.changedDataFim = function(field) {
        let _valueIni = field.widget.getField('DATAINI').value();
        let _valueFim = field.value();
        if (_valueIni && moment(_valueIni, 'DD/MM/YYYY').unix() > moment(_valueFim, 'DD/MM/YYYY').unix()) {
            MessageService.showMessageByCode("PCP00040_VIG_INV");
            field.setValue("");
            return false;
        }
        return _self.validaPeriodo(field, _valueIni, _valueFim);
    };

    this.validaPeriodo = function(field, valueIni, valueFim) {
        if (field.widget.currentRow && field.widget.currentRow.REPORTNAME == 'PCP00404') {
            if (valueFim && valueIni) {
                if (Math.abs(moment(valueIni, 'DD/MM/YYYY').diff(moment(valueFim, 'DD/MM/YYYY'), 'days')) >= 31) {
                    MessageService.showMessageByCode("PCP00040_VIG_INV_31");
                    field.setValue("");
                    return false;
                }
            }
        }
        return true;
    };

    this.changedDataInicio = function(field) {
        let fieldFim = field.widget.getField('DATAFIN');
        if (field.value()) {
            if (fieldFim.value() && moment(fieldFim.value(), 'DD/MM/YYYY').unix() < moment(field.value(), 'DD/MM/YYYY').unix()) {
                fieldFim.setValue("");
            }
            fieldFim.readOnly = false;
        } else {
            fieldFim.readOnly = true;
            fieldFim.setValue("");
        }
        _self.validaPeriodo(field, field.value(), fieldFim.value());
    };

    this.report = function(widget, format) {
        if (widget.isValid()) {
            if (widget.getField('DATAFIN').isVisible) {
                if (!_self.changedDataFim(widget.getField('DATAFIN')))
                    return;
            }
            suitRowReport(widget, format);
            InflightorGeneral.openReport('pcp#/relatorioEstimativaRefeicoes', widget.currentRow.REPORTNAME, widget.currentRow);
        }
    };

    function suitRowReport(widget, format) {
        widget.fields.filter(function(field) {
            return field.template && field.template.indexOf('select-multiple') !== -1;
        }).forEach(function(fieldFilter) {
            if (fieldFilter.fieldCheckAll && widget.currentRow[fieldFilter.fieldCheckAll] == 'S') {
                widget.currentRow[fieldFilter.valueField] = 'T';
            } else if (widget.currentRow[fieldFilter.name] && Array.isArray(widget.currentRow[fieldFilter.name])) {
                widget.currentRow[fieldFilter.valueField] = widget.currentRow[fieldFilter.name].join('_');
            }
        });
        widget.currentRow.REPORTFORMAT = format;
        widget.currentRow.NACIONALIDADE = widget.currentRow.DSNACIONALIDADE;
    }

    this.apagaVooServicos = function() {
        let widget = templateManager.container.getWidget("pcp00040_relatorios_estimativas_refeicoes");
        let fieldServ = widget.getField("DSSERVICOCIAFILI");
        let fieldTServ = widget.getField("TSERVICO");
        let fieldVoo = widget.getField("NRVOO");
        let fieldTVoo = widget.getField("TVOO");
        fieldTVoo.setValue("S");
        InflightorGeneral.toggleReadOnlySelect(fieldTVoo, fieldVoo);
        fieldTServ.setValue("S");
        InflightorGeneral.toggleReadOnlySelect(fieldTServ, fieldServ);
        templateManager.updateTemplate();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00040_relatorios_estimativas_refeicoesController', pcp00040_relatorios_estimativas_refeicoesController);
});

// FILE: ./assets/js/controllers/pcp00044_custo_planejado_realizadoController.js
/* global Configuration, getLocalVar, moment */
var pcp00044_custo_planejado_realizadoController = function(PCPUtil, MessageService, templateManager, General) {

    const WDGT_MAIN = 'widget_relatorio';

    function getWidgetMain() {
        return templateManager.container.getWidget(WDGT_MAIN);
    }

    this.setDefaultValues = function() {
        const widget = getWidgetMain();
        const fieldReport = widget.getField('report');
        const fieldTipoCusto = widget.getField('CDTIPOCUSTO');
        fieldTipoCusto.applyDefaultValue();
        fieldReport.applyDefaultValue();
        widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
        widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
        widget.currentRow.FILIAL = widget.currentRow.CDFILIAL + ' | ' + widget.currentRow.NMFILIAL;
        const startDate = moment().startOf('month').format('DD/MM/YYYY');
        const endDate = moment().endOf('month').format('DD/MM/YYYY');
        widget.currentRow.PERIODO = {
            start: startDate,
            end: endDate,
            formattedStart: startDate,
            formattedEnd: endDate
        };
    };

    this.beforeSelectOpenCia = function(fieldCia) {
        const widget = fieldCia.widget;
        const rowToFilter = PCPUtil.getValuesInRow(widget, {
            FILIAL: 'CDFILIAL'
        });
        PCPUtil.updateFilterFields(fieldCia, 'CDFILIAL', rowToFilter, true);
    };

    this.beforeSelectOpenVoo = function(fieldVoo) {
        const widget = fieldVoo.widget;
        const row = widget.currentRow;
        const rowToFilter = PCPUtil.getValuesInRow(widget, {
            FILIAL: 'CDFILIAL',
            CDCIA: 'CDCIA'
        });
        if (row.T_CIA === 'T') {
            rowToFilter.CDCIA = 'T';
        }
        if (Array.isArray(rowToFilter.CDCIA) && rowToFilter.CDCIA.length) {
            rowToFilter.CDCIA = rowToFilter.CDCIA.join('_');
        }
        PCPUtil.updateFilterFields(fieldVoo, ['CDFILIAL', 'CDCIA'], rowToFilter, true);
    };

    this.clearField = function(nameFields) {
        const widget = getWidgetMain();
        nameFields = typeof nameFields === 'string' ? [nameFields] : nameFields;
        if (Array.isArray(nameFields)) {
            nameFields.forEach(fieldName => {
                let field = widget.getField(fieldName);
                if (field) field.clearValue();
            });
        }
        templateManager.updateTemplate();
    };

    this.openPopupReportFormat = function() {
        const widget = getWidgetMain();
        const row = widget.currentRow;
        if (!widget.isValid()) return;
        General.openReportPopup(widget, generateReport, [row], PCPUtil.getReportFormats());
    };

    function generateReport(params) {
        if (params.PERIODO) {
            params.DTINICIAL = params.PERIODO.start;
            params.DTFINAL = params.PERIODO.end;
        }
        if (params.CDTIPOCUSTO) {
            const widget = getWidgetMain();
            const custo = widget.getField('CDTIPOCUSTO').dataSource.data.find(row => row.CDTIPOCUSTO === params.CDTIPOCUSTO);
            params.NMTIPOCUSTO = custo.NMTIPOCUSTO;
        }
        General.openReport('pcp#/relatorio_custo_planejado_realizado', params.report, params);
    }

};

Configuration(function(ContextRegister) {
    ContextRegister.register('pcp00044_custo_planejado_realizadoController', pcp00044_custo_planejado_realizadoController);
});

// FILE: ./assets/js/controllers/PlanejamentoProducaoController.js
var PlanejamentoProducaoController = function(ScreenService, InflightorGeneral, restEngine, templateManager, MessageService, metaDataFactory) {

    var self = this;

    var mensagensGerais = {};

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00022_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    this.filterSelectBeforeOpen = function(field) {
        field.widget.fields.forEach(function(selectField) {
            if (selectField.template && selectField.template.indexOf('select-multiple') !== -1) {
                field.widget.currentRow[selectField.valueField] = field.widget.currentRow[selectField.name] ? field.widget.currentRow[selectField.name].join('_') : "T";
            }
        });
        InflightorGeneral.onClickDependentField(field, field.widget.currentRow);
    };

    this.filterSelectOnChange = function(field) {
        field.widget.fields.forEach(function(selectField) {
            if (selectField.dataSourceFilter && selectField.dataSourceFilter.length) {
                selectField.dataSourceFilter.forEach(function(filter) {
                    if (filter.name === field.name || filter.name === field.valueField) {
                        selectField.applyDefaultValue();
                        if (selectField.template && selectField.template.indexOf('select-multiple')) {
                            if (selectField.toggleSelectDependecys) {
                                let fieldCheckBox = field.widget.getField(selectField.toggleSelectDependecys);
                                fieldCheckBox.applyDefaultValue();
                                fieldCheckBox.checked = true;
                                InflightorGeneral.toggleReadOnlySelect(fieldCheckBox, selectField);
                            }
                        }
                    }
                });
            }
        });
    };

    this.beforeFilterApply = function(args) {
        let widget = args.owner;
        let CUSTOM_FILTER_ROW = angular.copy(args.e.preference.FILTER_ROW);
        widget.fields.forEach(function(field) {
            if (field.toggleSelectDependecys) {
                if (!CUSTOM_FILTER_ROW[field.name]) {
                    CUSTOM_FILTER_ROW[field.name] = 'T';
                } else {
                    if (Array.isArray(CUSTOM_FILTER_ROW[field.name]) && CUSTOM_FILTER_ROW[field.name].length) {
                        CUSTOM_FILTER_ROW[field.name] = CUSTOM_FILTER_ROW[field.name].join('_');
                    }
                }
                delete CUSTOM_FILTER_ROW[field.toggleSelectDependecys];
            }
        });
        args.e.preference.CUSTOM_FILTER_ROW = CUSTOM_FILTER_ROW;
    };

    this.afterFilterApply = function(args) {
        let widget = args.owner;
        let currentRow = widget.appliedFilter.FILTER_ROW;
        widget.stripeToShowGridFilterInfo = "FILTER";
        delete widget.__filterInfoStripe;
        InflightorGeneral.updateStripe(widget).then(function() {
            var lenFieldsStripe = -1;
            if (widget.__filterInfoStripe.fields && widget.__filterInfoStripe.fields.length) {
                lenFieldsStripe = widget.__filterInfoStripe.fields.length - 1;
            }
            if (lenFieldsStripe !== -1) {
                clearStripeInfo(widget, widget.__filterInfoStripe);
                _buildLabelComplementar(widget, widget.__filterInfoStripe, currentRow);
            }
        });
    };

    function clearStripeInfo(widget, widgetStripe) {
        var propertiesClear = [
            mensagensGerais.todosT,
            mensagensGerais.todosN,
            mensagensGerais.gerarcompS,
            mensagensGerais.gerarcompN
        ];
        widgetStripe.fields.forEach(function(field) {
            propertiesClear.forEach(function(replace) {
                field.label = field.label.replace(replace, "");
            });
            widget.fields.forEach(function(fieldsGrid) {
                field.label = field.label.replace(fieldsGrid.label + ':  •', "");
            });
            if (field.label.replace(/ /g, "") == "") {
                field.isVisible = false;
                field.class = 0;
            }
        });
    }

    function _buildLabelComplementar(widget, widgetFilter, currentRow) {
        var lenFieldsStripe = 0;
        if (widgetFilter.fields && widgetFilter.fields.length) {
            lenFieldsStripe = widgetFilter.fields.filter(function(field) {
                return field.isVisible;
            }).length - 1;
        }
        clearStripeInfo(widget, widgetFilter);
        let filter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
                    },
            {
                "name": "DTGERARCOMPL",
                "operator": "=",
                "value": currentRow.DTPRODUCAOINF
                    }];
        InflightorGeneral.callRestEngine('pcp#/crud/prevpaxgerarcompl', 'FilterData', filter).then(function(callback) {
            if (callback.dataset && callback.dataset.prevpaxgerarcompl && callback.dataset.prevpaxgerarcompl.length) {
                widgetFilter.fields[lenFieldsStripe].label += mensagensGerais.gerarcompS;
                widget.currentRow.GERARCOMPLEMENTAR = mensagensGerais.sim;
            } else {
                widgetFilter.fields[lenFieldsStripe].label += mensagensGerais.gerarcompN;
                widget.currentRow.GERARCOMPLEMENTAR = mensagensGerais.nao;
            }
            delete widget.stripeToShowGridFilterInfo;
        });
    }

    function _keyIsValid(key, widget) {
        return widget.fields.filter(function(field) {
            return field.name === key;
        }).length;
    }

    function _buildFilterByCurrentRow(widget, currentRow) {
        widget.dataSourceFilter = [];
        Object.keys(currentRow).forEach(function(key) {
            if (currentRow[key] && _keyIsValid(key, widget)) {
                widget.dataSourceFilter.push({
                    "name": '' + key,
                    "operator": "=",
                    "value": currentRow[key]
                });
            }
        });
    }

    function _setFilterReceitas(currentWidget, widget) {
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
    }

    function _closeAndReloadWidget(currentWidget) {
        ScreenService.closeSwipe(1).then(function() {
            currentWidget.reload().then(function() {
                if (currentWidget.redraw) {
                    currentWidget.redraw(true);
                }
                currentWidget.activate();
            });
        });
    }

    function _reloadWidget(currentWidget) {
        currentWidget.reload().then(function() {
            if (currentWidget.redraw) {
                currentWidget.redraw(true);
            }
            currentWidget.activate();
        });
    }

    this.loadDataSource = function(widget = templateManager.container.getWidget('widgetPlanejamentoProducao')) {
        widget.reload().then(function() {
            widget.activate();
            _buildLabelComplementar(widget, widget.__filterInfoStripe, widget.appliedFilter.FILTER_ROW);
        });
    };

    this.afterMoveRowChecklist = function(widget, currentName) {
        let currentWidget = widget;
        if (event && event.target.className && event.target.className.indexOf('editable-content') === -1) {
            if (currentWidget && !currentWidget.editing) {
                if (currentWidget.name === 'widgetReceitas' || currentWidget.name === 'widgetDestinos') {
                    _setFilterReceitas(currentWidget, widget);
                    if (currentName && currentName === 'widgetPlanejamentoProducao') {
                        _closeAndReloadWidget(currentWidget);
                    } else {
                        _reloadWidget(currentWidget);
                    }
                }
            }
        }
    };

    this.goBackView = function(widget, widgetName) {
        ScreenService.closeSwipe().then(function() {
            let backWidget = widget.name === widgetName ? widget : widget.container.getWidget(widgetName);
            backWidget.list();
            backWidget.activate();
        });
    };

    this.goBackList = function(widget, backWidget) {
        ScreenService.closeSwipe().then(function() {
            backWidget.activate();
        });
    };

    function _showCheckedError(widget, dataError, operation) {
        let messagesError = [];
        dataError.forEach(function(row) {
            let label = [{
                "SITUACAO": widget.getField('IDSITUACAO').dataSource.data.filter(function(options) {
                    return options.IDSITUACAO === row.IDSITUACAO;
                })[0],
                "TIPO": widget.getField('IDTIPOPLAN').dataSource.data.filter(function(options) {
                    return options.IDTIPOPLAN === row.IDTIPOPLAN;
                })[0]
            }];
            messagesError.push(ScreenService.i18n(mensagensGerais.checklist, {
                "0": row.CDCKLISTPRODINF,
                "1": label[0]['TIPO']['SITUACAO'],
                "2": label[0]['SITUACAO']['SITUACAO']
            }));

        });
        InflightorGeneral.showErrorMessages(widget, messagesError);
    }

    function _checkedRowsByOperation(widget, situacao, operation) {
        let validCheckedRows = [];
        let checkedRows = widget.dataSource.getCheckedRows();
        validCheckedRows.push(checkedRows.filter(function(row) {
            return row.IDSITUACAO === situacao;
        }));
        if (operation === 'DELETE') {
            validCheckedRows[0] = validCheckedRows[0].filter(function(row) {
                return row.IDTIPOPLAN !== 'EMERGENCIAL' && row.IDTIPOPLAN !== 'APRESENTACAO';
            });
            if (checkedRows.length !== validCheckedRows[0].length) {
                validCheckedRows.push(checkedRows.filter(function(row) {
                    return row.IDSITUACAO !== situacao || (row.IDTIPOPLAN === 'EMERGENCIAL' || row.IDTIPOPLAN === 'APRESENTACAO');
                }));
            } else {
                validCheckedRows.push({});
            }
        } else {
            if (checkedRows.length !== validCheckedRows[0].length) {
                validCheckedRows.push(checkedRows.filter(function(row) {
                    return row.IDSITUACAO !== situacao;
                }));
            } else {
                validCheckedRows.push({});
            }
        }
        return validCheckedRows;
    }

    this.changedStatusChecklist = function(widget, situacao, value, operation, newData) {
        let dataChanged = _checkedRowsByOperation(widget, situacao, 'changed');
        if (dataChanged.length === 2 && !dataChanged[1].length) {
            if (value == 'ALTERDATA') {
                dataChanged[0].forEach(function(row) {
                    row.DTPRODUCAOINF = newData;
                });
            } else {
                dataChanged[0].forEach(function(row) {
                    row.IDSITUACAO = value;
                });
            }
            InflightorGeneral.callRestEngine('pcp#/crud/cklistprodinf/save', 'DataSet', dataChanged[0]).then(function(callback) {
                //success
                ScreenService.closePopup(true);
                widget.dataSource.unSelectAll();
                widget.reload().then(function() {
                    MessageService.showMessageByCode("PCP00022_OPER_SUCES");
                    widget.activate();
                });
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        } else {
            //Existem rows invalidas selecionadas.
            _showCheckedError(widget, dataChanged[1], operation);
        }
    };

    var dataProducaoWidget = {
        "id": 0,
        "name": "dataProducaoWidget",
        "label": "Alterar Data da Produção",
        "template": "widget/form.html",
        "container": {},
        "isVisible": true,
        "popupNoBlock": false,
        "fields": [],
        "actions": [],
        "events": []
    };

    var dataProducaoFields = [
        {
            "name": "DTPRODUCAOINF",
            "label": "Nova Data",
            "class": 12,
            "template": "field/calendar.html",
            "isVisible": true,
            "readOnly": false,
            "showOnView": true,
            "showOnForm": true,
            "showOnList": true,
            "fields": [],
            "events": [],
            "dateField": "DTCUSTOAP",
            "titleField": "DTCUSTOAP",
            "dateFormat": "DD/MM/YYYY",
            "defaultValue": "@moment().format('DD/MM/YYYY')@",
            "dataSource": {},
            "validations": {
                "required": true
            }
        }
    ];

    var dataProducaoActions = [{
        "name": "cancelar",
        "label": "Cancelar",
        "idVisible": true,
        "hideIcon": true,
        "showAsAction": "back",
        "events": [{
            "name": "ActionEvent",
            "code": "ScreenService.closePopup();"
        }]
    }, {
        "name": "alterar",
        "label": "Alterar Data",
        "idVisible": true,
        "hideIcon": true,
        "showAsAction": "never",
        "events": [{
            "name": "ActionEvent",
            "code": "PlanejamentoProducaoController.changedDataChecklist(args.owner.widget);"
        }]
    }];

    this.openPopupAlteraData = function(widget) {
        dataProducaoWidget.container = widget.container;
        dataProducaoWidget.fields = dataProducaoFields;
        dataProducaoWidget.actions = dataProducaoActions;

        var dataProducaoWidgetInitialized = metaDataFactory.widgetFactory(dataProducaoWidget);
        dataProducaoWidgetInitialized.getField('DTPRODUCAOINF').setValue("");
        ScreenService.openPopup(dataProducaoWidgetInitialized).then(function() {
            dataProducaoWidgetInitialized.activate();
        });
    };

    this.openPopupGerarChecklist = function(widget) {
        let widgetPopup = widget.container.getWidget('widgetGerarChecklist');
        widgetPopup.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        widgetPopup.currentRow.DTPRODUCAO = widget.appliedFilter.FILTER_ROW.DTPRODUCAOINF;
        widgetPopup.currentRow.NMSETOR = "";
        widgetPopup.getField('NMSETOR').setValue("");
        widgetPopup.getField('NMSETOR').readOnly = true;
        widgetPopup.getField('NMSETOR').validations = {};
        widgetPopup.getField("TSETOR").applyDefaultValue();
        ScreenService.openPopup(widgetPopup);
    };

    this.changedDataChecklist = function(widget) {
        let widgetPlanejamentoProducao = widget.container.getWidget('widgetPlanejamentoProducao');
        if (moment(moment().format('DD/MM/YYYY'), 'DD/MM/YYYY').unix() <= moment(widget.currentRow.DTPRODUCAOINF, 'DD/MM/YYYY').unix()) {
            self.changedStatusChecklist(widgetPlanejamentoProducao, 'PENDENTE', 'ALTERDATA', 'Alterar Data de Produção', widget.currentRow.DTPRODUCAOINF);
        } else {
            MessageService.showMessageByCode("PCP00022_DATA_MENOR_ATUAL");
        }
    };

    this.cancelInlineEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    this.saveEditReceita = function(widget) {

    };

    this.saveInlineEdit = function(widget, widgetOrigin, route) {
        let statusChecklist = widget.container.getWidget('widgetPlanejamentoProducao').currentRow.IDSITUACAO;
        if (statusChecklist == 'PENDENTE') {
            let dataTosave = _createDataSetToSave(widget);
            if (dataTosave.length) {
                InflightorGeneral.callRestEngine('pcp#/' + route, 'DataSet', dataTosave).then(function(success) {
                    widget.container.getWidget(widgetOrigin).reload().then(function() {
                        widget.list();
                        widget.commitInlineEdit();
                        widget.activate();
                        if (widgetOrigin === 'widgetDestinos') {
                            widget.container.getWidget('widgetReceitas').reload();
                        }
                    });
                }, function(error) {
                    let errorMessage = 'Timeout';
                    if (error.data)
                        trataMensagem(error.data.error);
                });
            }
        } else {
            MessageService.showMessageByCode("PCP00022_CHECKLIST_PEND");
            self.cancelInlineEdit(widget);
        }
    };

    this.gerarChecklist = function(widget) {
        if (widget.isValid()) {
            let row = _getRowProcedure(widget);
            callProcedureGeraChecklist(widget, row);
        }
    };

    //To do InflightorGeneral.js
    this.callAuthUser = function(callback, params) {
        let widget_autenticar = templateManager.container.getWidget('widget_autenticar');
        widget_autenticar.callback = callback;
        widget_autenticar.params = params;
        widget_autenticar.newRow();
        ScreenService.openPopup(widget_autenticar);
    };
    //To do InflightorGeneral.js
    this.closePopupAuth = function(widget) {
        ScreenService.closePopup(true).then(function() {
            widget.container.getWidget('widgetPlanejamentoProducao').activate();
        });
    };
    //To do InflightorGeneral.js
    this.authUser = function(widget) {
        if (widget.isValid()) {
            let user = widget.currentRow.user;
            let senha = widget.currentRow.senha;
            InflightorGeneral.callRestEngine('pcp#/Autenticar', 'FilterData', [{
                name: "email",
                operator: "=",
                value: user
          }, {
                name: "senha",
                operator: "=",
                value: senha
          }]).then(function(callback) {
                ScreenService.closePopup();
                if (angular.isFunction(widget.callback)) {
                    widget.callback.apply(null, widget.params);
                }
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    this.deleteValidate = function(widget) {
        MessageService.confirmMessageByCode('PCP00022_CONF_EXCL_CHECKLIST').then(function(success) {
            //validate User
            self.callAuthUser(self.deleteChecklist, [widget.container.getWidget('widgetPlanejamentoProducao')]);
        });
    };

    this.deleteChecklist = function(widget) {
        let dataChanged = _checkedRowsByOperation(widget, 'PENDENTE', 'DELETE');
        if (dataChanged.length === 2 && !dataChanged[1].length) {
            InflightorGeneral.callRestEngine('pcp#/deleteChecklist', 'DataSet', dataChanged[0]).then(function(callback) {
                //success
                ScreenService.closePopup(true);
                widget.dataSource.unSelectAll();
                widget.reload().then(function() {
                    MessageService.showMessageByCode("PCP00022_EXCL_CHECKLIST_SUCES");
                    widget.activate();
                });
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        } else {
            //Existem rows invalidas selecionadas.
            _showCheckedError(widget, dataChanged[1], 'Excluir');
        }
    };

    function callProcedure(row) {
        return restEngine.request("pcp#/gera_checklis_prod", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 300000);
    }

    function callProcedureGeraChecklist(widget, row) {
        callProcedure(row).then(function(callback) {
            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros);
            } else {
                //success
                ScreenService.closePopup();
                let widgetPlanejamento = widget.container.getWidget('widgetPlanejamentoProducao');
                widgetPlanejamento.reload().then(function() {
                    MessageService.showMessageByCode("PCP00022_CHECKLIST_GER_SUCES");
                    widgetPlanejamento.activate();
                    _buildLabelComplementar(widgetPlanejamento, widgetPlanejamento.__filterInfoStripe, widgetPlanejamento.currentRow);
                });
            }
        }, function(error) {
            //error
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            }
        });
    }

    function _getRowProcedure(widget) {
        let setor = "T";
        if (widget.currentRow.TSETOR === "T") {
            setor = "T";
        } else if (widget.currentRow.NMSETOR && widget.currentRow.NMSETOR.length) {
            setor = widget.currentRow.NMSETOR.join('_');
        }
        return {
            'P_CDFILIAL': widget.currentRow.CDFILIAL,
            'P_DTPRODUCAO': widget.currentRow.DTPRODUCAO,
            'P_CDSETOR': setor,
            'P_CDOPERADOR': "",
            "P_NRORG": ""
        };
    }

    function _createDataSetToSave(widget) {
        let data_edit = widget.dataSource.data.filter(function(row) {
            return row.changed || Object.keys(row).filter(function(key) {
                return key.indexOf('_inline') !== -1;
            }).length;
        });
        let destinos = [];
        data_edit.forEach(function(row) {
            row.QTRECEITAINF = row.QTRECEITAINF_inline !== row.QTRECEITAINF ? InflightorGeneral.valueToFloat(row.QTRECEITAINF_inline) : InflightorGeneral.valueToFloat(row.QTRECEITAINF);
            destinos.push(row);
        });
        return destinos;
    }

    this.getMensagens = function() {
        MessageService.getMessagesByCode(['PCP00022_TODOS_T', 'PCP00022_TODOS_N', 'PCP00022_GERAR_COMP_S', 'PCP00022_GERAR_COMP_N',
                                          'PCP00022_SIM', 'PCP00022_NAO', 'PCP00022_CHECKLIST']).then(function(mensagens) {
            mensagensGerais.todosT = mensagens[0];
            mensagensGerais.todosN = mensagens[1];
            mensagensGerais.gerarcompS = mensagens[2];
            mensagensGerais.gerarcompN = mensagens[3];
            mensagensGerais.sim = mensagens[4];
            mensagensGerais.nao = mensagens[5];
            mensagensGerais.checklist = mensagens[6];
        });
    };

    this.filtraReceita = function() {
        let widgetReceita = templateManager.container.getWidget('widgetReceitas');
        let widgetPlanejamentoProducao = templateManager.container.getWidget("widgetPlanejamentoProducao");
        widgetReceita.dataSourceFilter.forEach(function(filter) {
            filter.value = widgetPlanejamentoProducao.currentRow[filter.name];
        });
        widgetReceita.reload();
    };

    this.filtraDestinos = function() {
        let widgetDestino = templateManager.container.getWidget('widgetDestinos');
        let widgetReceitas = templateManager.container.getWidget("widgetReceitas");
        widgetDestino.dataSourceFilter.forEach(function(filter) {
            filter.value = widgetReceitas.currentRow[filter.name];
        });
        widgetDestino.reload();
    };
};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('PlanejamentoProducaoController', PlanejamentoProducaoController);
});

// FILE: ./assets/js/controllers/PrevisaoConsumoController.js
var PrevisaoConsumoController = function(ScreenService, InflightorGeneral, restEngine, templateManager, MessageService) {

    var self = this;

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00041_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    function _keyIsValid(key, widget) {
        return widget.fields.filter(function(field) {
            return field.name === key;
        }).length;
    }

    function _buildFilterByCurrentRow(widget, currentRow) {
        widget.dataSourceFilter = [];
        Object.keys(currentRow).forEach(function(key) {
            if (currentRow[key] && _keyIsValid(key, widget)) {
                widget.dataSourceFilter.push({
                    "name": '' + key,
                    "operator": "=",
                    "value": currentRow[key]
                });
            }
        });
    }

    function _setFilter(currentWidget, widget) {
        let level = ScreenService.getSwipeLevel(widget);
        if (ScreenService.isSwipeOpen(level + 2))
            ScreenService.closeSwipe(level + 2);
        currentWidget.dataSourceFilter.forEach(function(filter) {
            filter.value = widget.currentRow[filter.name];
        });
        currentWidget.reload();
    }

    function _closeAndReloadWidget(currentWidget, lvl) {
        if (lvl) {
            ScreenService.closeSwipe(lvl).then(function() {
                currentWidget.reload().then(function() {
                    currentWidget.activate();
                });
            });
        } else {
            _reloadWidget(currentWidget);
        }
    }

    function _reloadWidget(currentWidget) {
        currentWidget.reload().then(function() {
            currentWidget.activate();
        });
    }

    this.loadDataSource = function(widget = templateManager.container.getWidget('widgetPrevisaoConsumo')) {
        widget.reload().then(function() {
            widget.activate();
        });
    };

    this.afterMoveRowPrevisaoConsumo = function() {
        let widget = templateManager.container.getWidget("widgetPrevisaoConsumo");
        let currentWidget = templateManager.container.getWidget("widgetMateriaPrima");
        _setFilter(currentWidget, widget);
    };

    this.afterMoveRowMateriaPrima = function() {
        let widget = templateManager.container.getWidget("widgetMateriaPrima");
        let currentWidget = templateManager.container.getWidget("widgetProducaoPrevista");
        _setFilter(currentWidget, widget);
        currentWidget = templateManager.container.getWidget("widgetSubgrupos");
        _setFilter(currentWidget, widget);
    };

    this.afterMoveRowProdPrev = function() {
        let widget = templateManager.container.getWidget("widgetProducaoPrevista");
        let currentWidget = templateManager.container.getWidget("widgetReceitas");
        _setFilter(currentWidget, widget);
    };

    this.afterMoveRowReceita = function() {
        let widget = templateManager.container.getWidget("widgetReceitas");
        let currentWidget = templateManager.container.getWidget("widgetDestinos");
        _setFilter(currentWidget, widget);
    };

    this.afterMoveRowUtilSub = function() {
        let widget = templateManager.container.getWidget("widgetSubgrupos");
        let currentWidget = templateManager.container.getWidget("widgetDestinosSubgrupos");
        _setFilter(currentWidget, widget);
    };

    this.goBackView = function(widget, widgetName) {
        ScreenService.closeSwipe().then(function() {
            let backWidget = widget.name === widgetName ? widget : widget.container.getWidget(widgetName);
            backWidget.list();
            backWidget.activate();
        });
    };

    this.goBackList = function(widget, backWidget) {
        ScreenService.closeSwipe().then(function() {
            backWidget.activate();
        });
    };

    this.openPopupGerarPrevisao = function(widget) {
        let widgetPopup = widget.container.getWidget('widgetGerarPrevisao');
        widgetPopup.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        widgetPopup.currentRow.DATAINI = widget.appliedFilter.FILTER_ROW.DATAINI;
        widgetPopup.currentRow.DATAFIN = widget.appliedFilter.FILTER_ROW.DATAFIN;
        ScreenService.openPopup(widgetPopup);
    };

    this.gerarPrevisaoConsumo = function(widget) {
        if (widget.isValid()) {
            let row = _getRowProcedure(widget);
            callProcedureGeraPrevisao(widget, row);
        }
    };

    //To do InflightorGeneral.js
    this.callAuthUser = function(callback, params) {
        let widget_autenticar = templateManager.container.getWidget('widget_autenticar');
        widget_autenticar.callback = callback;
        widget_autenticar.params = params;
        widget_autenticar.newRow();
        ScreenService.openPopup(widget_autenticar);
    };
    //To do InflightorGeneral.js
    this.closePopupAuth = function(widget) {
        ScreenService.closePopup(true).then(function() {
            widget.container.getWidget('widgetPrevisaoConsumo').activate();
        });
    };
    //To do InflightorGeneral.js
    this.authUser = function(widget) {
        if (widget.isValid()) {
            let user = widget.currentRow.user;
            let senha = widget.currentRow.senha;
            InflightorGeneral.callRestEngine('pcp#/Autenticar', 'FilterData', [{
                name: "email",
                operator: "=",
                value: user
          }, {
                name: "senha",
                operator: "=",
                value: senha
          }]).then(function(callback) {
                ScreenService.closePopup();
                if (angular.isFunction(widget.callback)) {
                    widget.callback.apply(null, widget.params);
                }
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

    this.deleteValidate = function(widget) {
        MessageService.confirmMessageByCode("PCP00041_CONF_EXCL_CONSUMO").then(function(success) {
            //validate User
            self.callAuthUser(self.deletePrevisaoConsumo, [widget.container.getWidget('widgetPrevisaoConsumo')]);
        });
    };

    this.deletePrevisaoConsumo = function(widget) {
        InflightorGeneral.callRestEngine('pcp#/delete_previsao_consumo', 'Row', widget.currentRow).then(function(callback) {
            //success
            ScreenService.closePopup(true);
            ScreenService.closeSwipe().then(function() {
                widget.dataSource.unSelectAll();
                widget.reload().then(function() {
                    MessageService.showMessageByCode("PCP00041_PREV_EXCL");
                    widget.activate();
                });
            });
        }, function(error) {
            if (error.data)
                trataMensagem(error.data.error);
        });
    };

    function callProcedure(row) {
        return restEngine.request("pcp#/gera_previsao_consumo", {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 7200000);
    }

    function callProcedureGeraPrevisao(widget, row) {
        callProcedure(row).then(function(callback) {
            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.mensagensErroProcedure(callback.dataset.ListaErros);
            } else if (callback.dataset && callback.dataset.downloadFile) {
                //Lista de erros gerou o download do arquivo com as inconsistências encontradas no processo.
                MessageService.showMessageByCode("PCP00041_ERRO_DOWNLOAD_ARQ").then(function() {
                    _exportFileError(callback.dataset.downloadFile);
                });
            } else {
                //success
                ScreenService.closePopup();
                let widgetPlanejamento = widget.container.getWidget('widgetPrevisaoConsumo');
                widgetPlanejamento.reload().then(function() {
                    MessageService.showMessageByCode("PCP00041_PREV_GERADA");
                    widgetPlanejamento.activate();

                });
            }
        }, function(error) {
            //error
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            }
        });
    }

    function _exportFileError(callback) {
        var serviceUrlModulo = window.serviceUrls.filter(function(row) {
            return row.package == "pcp";
        })[0].baseUrl;
        var date = moment().format("DD_MM_YYYY");
        var fileName = "LISTA_ERROS_" + callback['CDFILIAL'] + "_" + callback['NRLISTAERROS'] + "_" + date;
        window.open(serviceUrlModulo + "/downloadFileListError?" + "&cdfilial=" + callback['CDFILIAL'] + "&nrlistaerros=" + callback['NRLISTAERROS'] + "&filename=" + fileName);
    }

    function _getRowProcedure(widget) {
        return {
            'P_CDFILIAL': widget.currentRow.CDFILIAL,
            'P_DATAINI': widget.currentRow.DATAINI,
            'P_DATAFIN': widget.currentRow.DATAFIN,
            'P_CDOPERADOR': "",
            "P_NRORG": ""
        };
    }

    this.changedDataFim = function(field) {
        let _valueIni = field.widget.getField('DATAINI').value();
        let _valueFim = field.value();
        if (_valueIni && moment(_valueIni, 'DD/MM/YYYY').unix() > moment(_valueFim, 'DD/MM/YYYY').unix()) {
            MessageService.showMessageByCode("PCP00041_PERIODO_INV");
            field.setValue("");
        }
    };

    this.changedDataInicio = function(field) {
        let fieldFim = field.widget.getField('DATAFIN');
        if (field.value()) {
            if (fieldFim.value() && moment(fieldFim.value(), 'DD/MM/YYYY').unix() < moment(field.value(), 'DD/MM/YYYY').unix()) {
                fieldFim.setValue("");
            }
            fieldFim.readOnly = false;
        } else {
            fieldFim.readOnly = true;
            fieldFim.setValue("");
        }
    };

    this.openPopupReportPrevisao = function(widget) {
        var popupReport = widget.container.getWidget('widgetRelatorioPrevisao');
        popupReport.currentRow = angular.copy(widget.currentRow);
        popupReport.fields.forEach(function(field) {
            if (field.defaultValue) {
                field.applyDefaultValue();
            } else {
                field.setValue("");
            }
        });
        popupReport.currentRow.CDFILIAL = widget.currentRow.CDFILIAL;
        popupReport.currentRow.NMFILIAL = widget.currentRow.NMFILIAL;
        ScreenService.openPopup(popupReport).then(function() {
            popupReport.activate();
        });
    };

    this.callReport = function(widget, reportFormat) {
        if (widget.isValid()) {
            widget.currentRow.REPORTFORMAT = reportFormat;
            InflightorGeneral.openReport('pcp#/relatorioPrevisaoConsumo', widget.currentRow.REPORTNAME, widget.currentRow).then(function(callback) {
                ScreenService.closePopup(true);
                widget.container.getWidget('widgetPrevisaoConsumo').activate();
            }, function(error) {
                if (error.data)
                    trataMensagem(error.data.error);
            });
        }
    };

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('PrevisaoConsumoController', PrevisaoConsumoController);
});

// FILE: ./assets/js/controllers/PrevisaoPassageirosProducao.js
var PrevisaoPassageirosProducao = function(ZHPromise, PrevisaoPassageirosProducaoService, ScreenService, templateManager, InflightorGeneral, $rootScope, Colors, metaDataFactory, MessageService) {
    var self = this;
    var widget_filter;
    var widget_filter_stripe;
    var full_classes = [];
    var saveClick;
    var filterNameConnectByRow = {
        "DTPRODUCAOCALC": "DTPREVISAOPROD"
    };

    var trataMensagem = function(mensagem) {
        if (mensagem && mensagem.length) {
            if (!InflightorGeneral.mensagemErroProcedure(mensagem)) {
                if (mensagem.indexOf("PCP00013_") >= 0) {
                    MessageService.showMessageByCode(mensagem);
                } else {
                    ScreenService.showMessage(mensagem);
                }
            }
        }
    };

    var mensagensGerais = {};

    function validateInlineEditable(widget) {
        return widget && !widget.editing && event && event.target.className && (event.target.className.indexOf('inline-editable') === -1 && event.target.className.indexOf('editable-content') === -1);
    }

    function inArray(elem, array) {
        return -1 != (Util.isArray(array) ? array.indexOf(elem) : -1);
    }

    function inObject(index, row) {
        return -1 != (Util.isObject(row) ? Object.keys(row).indexOf(index) : -1);
    }

    function arrayDistinct(a, param) {
        return a.filter(function(item, pos, array) {
            return array.map(function(mapItem) {
                    return mapItem[param];
                })
                .indexOf(item[param]) === pos;
        });
    }

    function rgbToHex(color) {
        if (color.charAt(0) === "#") {
            return color;
        }
        var nums = /(.*?)rgb\((\d+),\s*(\d+),\s*(\d+)\)/i.exec(color),
            r = parseInt(nums[2], 10).toString(16),
            g = parseInt(nums[3], 10).toString(16),
            b = parseInt(nums[4], 10).toString(16);
        return "#" + (
            (r.length == 1 ? "0" + r : r) +
            (g.length == 1 ? "0" + g : g) +
            (b.length == 1 ? "0" + b : b)
        );
    }

    function isFloat(n) {
        return Number(n) === n && n % 1 !== 0;
    }

    function isInt(value) {
        let x;
        if (isNaN(value) || isFloat(value)) {
            return false;
        }
        x = parseFloat(value);
        return (x | 0) === x;
    }

    function setValueInSelectMultiple(field, values) {
        field.value(field.dataSource.data.filter(function(row) {
            return inArray(row[field.valueField], values);
        }).map(function(row) {
            return row[field.valueField];
        }));
    }

    function fieldFactory(fields, widget) {
        var factorysFields = [];
        fields.forEach(function(field) {
            var field_Factory = metaDataFactory.fieldFactory(field, widget, false);
            factorysFields.push(field_Factory);
        });
        return factorysFields;
    }

    function arrayItemMax(arr) {
        return arr.reduce(function(result, val) {
            return Math.max(result, !isNaN(val) ? parseInt(val) : val.length);
        }, 0);
    }

    function createCompletedRow(row) {
        return full_classes.forEach(function(classe) {
            if (!inObject(classe.CDCLASSE + ' - ' + classe.SGCLASSE, row))
                row[classe.CDCLASSE + ' - ' + classe.SGCLASSE] = "--";
        });
    }

    function sortInArray(array, propertiesArray = []) {
        propertiesArray = Util.isArray(propertiesArray) ? propertiesArray : [propertiesArray];
        return array.sort(function(a, b) {
            var index = 0;
            for (var i = 0; i < propertiesArray.length; i++) {
                var property = propertiesArray[i];
                if (a[property] != b[property]) {
                    return +(a[property] > b[property]) || -1;
                }
            }
            return 0;
        });
    }

    this.expandSwipe = function() {
        $('.zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level="1"]').addClass('pcp-crosstab');
    };

    this.retrieveSwipe = function() {
        $('.zh-swipe-inner.zh-content.ng-scope.open[data-swipe-level="1"]').removeClass('pcp-crosstab');
    };

    this.blockFieldClass = function(element, row) {
        if ((!row && !element) || (row && !inlineEditable(row))) {
            setTimeout(function() {
                var div = document.getElementsByClassName('editable-content ng-scope');
                // if(!saveClick)
                //     saveClick = div[0].click;
                for (var key = 0; key < div.length; key++) {
                    div[key].parentElement.getElementsByTagName('span')[0].textContent = div[key].parentElement.getElementsByTagName('span')[0].textContent.replace(/\s/g, "");
                    if (div[key].parentElement.getElementsByTagName('span')[0].textContent === "") {
                        // $(div[key].parentElement).off('click');
                        // div[key].parentElement.removeEventListener("click", saveClick);
                        div[key].parentElement.getElementsByTagName('span')[0].textContent = "--";
                        // div[key].parentElement.style.color = "#B51B3D";
                        // div[key].style.backgroundColor = "#EDEDF2";
                    } else {
                        div[key].style.backgroundColor = "";
                        if (div[key].parentElement.getElementsByTagName('span')[0].textContent == 0) {
                            div[key].parentElement.style.color = "#B51B3D";
                        } else {
                            div[key].parentElement.style.color = "";
                        }
                    }
                }
            }, 500);
        }
    };

    function inlineEditable(row) {
        return Object.keys(row).filter(function(key) {
            return -1 != key.indexOf('_inline');
        }).length;
    }

    this.intervalDate = function(field) {
        var widget = field.widget;
        if (field.name == 'DATAINI') {
            if (moment(widget.getField('DATAINI').value(), 'DD/MM/YYYY') > moment(widget.getField('DATAFIM').value(), 'DD/MM/YYYY'))
                widget.getField('DATAFIM').setValue("");
            widget.getField('DATAFIM').applyDefaultValue();
            widget.getField('DATAFIM').reload().then(function() {});
        } else if (field.name == 'DATAFIM') {
            if (field.value()) {
                if (moment(widget.getField('DATAFIM').value(), 'DD/MM/YYYY') < moment(widget.getField('DATAINI').value(), 'DD/MM/YYYY')) {
                    widget.getField('DATAFIM').setValue("");
                }
            }
        }
    };

    this.sliceInFieldsHours = function(widget) {
        alterDataSourceSlice(widget.dataSource.data, 'find');
    };

    function alterDataSourceSlice(dataSource, method) {
        if (method == 'find') {
            dataSource.forEach(function(row) {
                row.HRKDT = row.HRKDT.slice(0, 2) + ":" + row.HRKDT.slice(2, 4);
                row.HRKRT = row.HRKRT.slice(0, 2) + ":" + row.HRKRT.slice(2, 4);
                row.HRETD = row.HRETD.slice(0, 2) + ":" + row.HRETD.slice(2, 4);
                row.HRQUEBRA = row.HRQUEBRA.slice(0, 2) + ":" + row.HRQUEBRA.slice(2, 4);
            });
        } else if (method == 'save') {
            dataSource.forEach(function(row) {
                row.HRKDT = row.HRKDT.slice(0, 2) + row.HRKDT.slice(3, 5);
                row.HRKRT = row.HRKRT.slice(0, 2) + row.HRKRT.slice(3, 5);
                row.HRETD = row.HRETD.slice(0, 2) + row.HRETD.slice(3, 5);
                row.HRQUEBRA = row.HRQUEBRA.slice(0, 2) + row.HRQUEBRA.slice(3, 5);
            });
        }
    }
    this.getFilterCrosstab = function(widget) {
        return [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        }, {
            "name": "NRPREVISAOPROD",
            "operator": "=",
            "value": widget.currentRow.NRPREVISAOPROD
        }, {

            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
        }];
    };

    this.getFilterClasses = function(widget) {
        return [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        }, {
            "name": "NRPREVISAOPROD",
            "operator": "=",
            "value": widget.currentRow.NRPREVISAOPROD
        }, {
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
        }];
    };

    /*
        25/04/2018 -- Correção do filter da tela feita na
        query do datasource pcp00013_prevpaxprodm_operador.
        this.getFilterWidgetPrevisao
        e this.setFilterPrevisao não são utilizadas mais para sobrescrever o filter da tela. 
    */
    this.getFilterWidgetPrevisao = function(widget) {
        var data_inicial = widget.appliedFilter.FILTER_ROW.DATAINI;
        var data_final = widget.appliedFilter.FILTER_ROW.DATAFIM;
        var interval_dias = moment(data_final, "DD/MM/YYYY").diff(moment(data_inicial, "DD/MM/YYYY"), 'days');
        var data = [];
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.appliedFilter.FILTER_ROW.CDFILIAL
        }, {
            "name": "DTPREVISAOPROD",
            "operator": "BETWEEN",
            "value": [widget.appliedFilter.FILTER_ROW.DATAINI, widget.appliedFilter.FILTER_ROW.DATAFIM]
        }];
        if (widget.appliedFilter.FILTER_ROW.IDPREVISAOPROD && widget.appliedFilter.FILTER_ROW.IDPREVISAOPROD != 'T') {
            filter.push({
                "name": "IDPREVISAOPROD",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.IDPREVISAOPROD
            });
        }
        if (widget.appliedFilter.FILTER_ROW.IDGERACOMPL && widget.appliedFilter.FILTER_ROW.IDGERACOMPL != 'T') {
            filter.push({
                "name": "IDGERACOMPL",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.IDGERACOMPL
            });
        }
        if (widget.appliedFilter.FILTER_ROW.REVISAR && widget.appliedFilter.FILTER_ROW.REVISAR != 'T') {
            filter.push({
                "name": "REVISAR",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.REVISAR
            });
        }
        return filter;
    };

    this.setFilterPrevisao = function(args, widget = args.owner) {
        widget_filter_stripe = args.e.FILTER_ROW;
        widget.dataSourceFilter = self.getFilterWidgetPrevisao(widget);
        widget.reload();
    };

    function findFullClasses(filter) {
        var deferred = ZHPromise.defer();
        InflightorGeneral.callRestEngine('pcp#/crud/prevpaxclaprodm_classe_distinct', "FilterData", filter).then(function(callback) {
            deferred.resolve(callback.dataset.prevpaxclaprodm_classe_distinct);
        }, function(error) {
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
            deferred.reject(error);
        });
        return deferred.promise;
    }

    function findClassesCrosstabSave(filter) {
        var deferred = ZHPromise.defer();
        InflightorGeneral.callRestEngine('pcp#/crud/prevpaxclaprodm_classe_save_crosstab', "FilterData", filter).then(function(callback) {
            deferred.resolve(callback.dataset.prevpaxclaprodm_classe_save_crosstab);
        }, function(error) {
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
            deferred.reject(error);
        });
        return deferred.promise;
    }

    function diffDataSourceFilter(filter, widgetFilter) {
        let diff = false;
        for (let i = 0; i < filter.length; i++) {
            if (filter[i].value !== widgetFilter[i].value) {
                diff = true;
            }
        }
        return diff;
    }

    function _getField(classe, order) {
        return {
            "name": classe.CDCLASSE + ' - ' + classe.SGCLASSE,
            "label": classe.SGCLASSE,
            "template": "field/text-edit.html",
            "class": 3,
            "order": order,
            "isVisible": true,
            "readOnly": false,
            "inlineEditable": "number",
            "showOnView": true,
            "showOnForm": true,
            "showOnList": true,
            "maxlength": 4,
            "align": "right",
            "events": [
                {
                    "name": "FieldOnChange",
                    "code": "PrevisaoPassageirosProducao.validateToInteger(args.owner.field);"
            }
          ],
            "styles": [
                {
                    "condition": "true",
                    "stylingFunction": "@PrevisaoPassageirosProducao.blockFieldClass(element, row)@"
            }
          ],
            "filterProperties": {
                "class": 0,
                "isVisible": false
            }
        };
    }

    this.validateToInteger = function(field) {
        if (field.value()) {
            if (!isInt(field.value())) {
                field.setValue("0");
            }
        }
    };

    this.openSwipePrevisao = function(widget, reload) {
        if (reload) {
            widget = templateManager.container.getWidget('pcp00013_previsao_passageiros_producao_grid');
        }
        var swipeOpen = ScreenService.isSwipeOpen(1);
        var filter = self.getFilterClasses(widget);
        var widgetCrosstab = widget.container.getWidget('previsao_passageiros_crosstab');
        var widgetTabbedCrosstab = widget.container.getWidget('previsao_passageiros_tabbed');
        var filterCrosstab = self.getFilterCrosstab(widget);
        if (diffDataSourceFilter(filterCrosstab, widgetCrosstab.dataSourceFilter) || !swipeOpen) {
            reload = false;
        } else {
            reload = true;
        }
        if (!reload) {
            findFullClasses(filter).then(function(callback) {
                var fields = self.getFieldsIniti(widgetCrosstab);
                var order = fields[fields.length - 1]['order'];
                full_classes = callback;
                if (full_classes.length) {
                    //Distinct for create fields 
                    var distinct_class = arrayDistinct(full_classes, 'CDCLASSE');
                    distinct_class.forEach(function(classe) {
                        order++;
                        var currentField = metaDataFactory.fieldFactory(_getField(classe, order), widgetCrosstab, false);
                        fields.push(currentField);
                    });
                }
                var fieldsAll = fields.concat(self.getFieldsLasted(widget, order));
                widgetCrosstab.fields = fieldsAll;
                widgetCrosstab.fields.forEach(function(field) {
                    field.sequenceInList = field.order;
                });
                self.setDataSourcePrevisao(filterCrosstab).then(function(callback) {
                    widgetCrosstab.dataSource.data = sortInArray(callback, ['SGCIA', 'NRVOO']);
                    widgetCrosstab.dataSourceFilter = filterCrosstab;
                }).then(function() {
                    ScreenService.openSwipe(widgetTabbedCrosstab).then(function() {
                        setTimeout(function() {
                            self.blockFieldClass();
                            self.expandSwipe();
                            templateManager.updateTemplate();
                            widgetCrosstab.activate();
                        }, 700);
                    });
                    // }
                });
            });
        } else {
            widgetCrosstab.fields.forEach(function(field) {
                field.sequenceInList = field.order;
            });
            self.setDataSourcePrevisao(filter).then(function(callback) {
                widgetCrosstab.dataSource.data = sortInArray(callback, ['SGCIA', 'NRVOO']);
                widgetCrosstab.activate();
                self.blockFieldClass();
            });
        }
    };

    this.cancelEditPrevisao = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
    };

    function joinColumn(data, column) {
        return data.map(function(row) {
            return row[column];
        }).join('_');
    }

    function createDistinctFilterClass(data, widget) {
        var cdcia = joinColumn(data, 'CDCIA');
        var nrvoo = joinColumn(data, 'NRVOO');
        var cdtrecho = joinColumn(data, 'CDTRECHO');
        return [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
        }, {
                "name": "NRPREVISAOPROD",
                "operator": "=",
                "value": widget.currentRow.NRPREVISAOPROD
        },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": cdcia
        }, {
                "name": "NRVOO",
                "operator": "=",
                "value": nrvoo
        },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": cdtrecho
        }];
    }

    function createDataSetToSave(widget) {
        var deferred = ZHPromise.defer();
        var data_edit = widget.dataSource.data.filter(function(row) {
            return row.changed;
        });
        var filter = createDistinctFilterClass(data_edit, widget);
        findClassesCrosstabSave(filter).then(function(callback) {
            var classes_edit = [];
            data_edit.forEach(function(row) {
                var classesVoo = callback.filter(function(classe) {
                    return classe.CDCIA === row.CDCIA && classe.NRVOO === row.NRVOO && classe.CDTRECHO === row.CDTRECHO;
                });
                classesVoo.forEach(function(classe) {
                    if (row.hasOwnProperty(classe.CDCLASSE + ' - ' + classe.SGCLASSE + '_inline')) {
                        var row_classe = {
                            CDFILIAL: row.CDFILIAL,
                            NRPREVISAOPROD: widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.NRPREVISAOPROD,
                            DTPREVISAOPROD: widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD.slice(0, 10),
                            CDCIA: row.CDCIA,
                            VOOCIA: row.VOOCIA,
                            TRECHO: row.TRECHO,
                            NRVOO: row.NRVOO,
                            CDTRECHO: row.CDTRECHO
                        };
                        row_classe["QTPAXPREVCALC"] = classe.QTPAXPREVCALC;
                        row_classe["QTPAXPREVINF"] = row[classe.CDCLASSE + ' - ' + classe.SGCLASSE + '_inline'];
                        row_classe["CDCLASSE"] = classe.CDCLASSE;
                        row_classe["SGCLASSE"] = classe.SGCLASSE;
                        row_classe["__is_new"] = false;
                        if (typeof(row_classe["QTPAXPREVINF"]) !== "undefined" && isInt(row_classe["QTPAXPREVINF"])) {
                            classes_edit.push(row_classe);
                        }
                    }
                });
            });
            deferred.resolve(classes_edit);
        }, function(error) {
            deferred.reject(error);
        });
        return deferred.promise;
    }

    function joinChangedClasses(data) {
        let classes = [];
        data.forEach(function(row) {
            classes.push(row['CDCLASSE']);
        });
        return classes;
    }

    this.saveEditPrevisao = function(args) {
        let widget = args.owner;
        if (self.beforeInlineEdit(args)) {
            let statusCia = widget.currentRow.IDPREVISAOPROD;
            createDataSetToSave(widget).then(function(dataTosave) {
                if (statusCia == 'P') {
                    dataTosave.push({
                        "VALIDAR_CHECKLIST": false
                    });
                    _savePrevisao(widget, dataTosave);
                } else {
                    dataTosave.push({
                        "VALIDAR_CHECKLIST": true,
                        "CLASSES_MODIFICADAS": joinChangedClasses(dataTosave)
                    });
                    _savePrevisao(widget, dataTosave);
                }
            });
        }
    };

    function _savePrevisao(widget, dataTosave) {
        InflightorGeneral.callRestEngine('pcp#/prevpaxclaprodm/saveEditClass', 'DataSet', dataTosave).then(function(success) {
            // reload
            // self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), true);
            widget.list();
            widget.commitInlineEdit();
            widget.activate();
            if (success.dataset && success.dataset.checklistsafetados && success.dataset.checklistsafetados.length) {
                MessageService.getMessagesByCode(['PCP00013_NECE_CKL_COMPL', 'PCP00013_NECE_CKL_COMPL_DATA']).then(function(mensagens) {
                    for (let i = 0; i < success.dataset.checklistsafetados.length; i++) {
                        let code_params = success.dataset.checklistsafetados[i].split("#_#");
                        if (code_params[0].indexOf("PCP00013_NECE_CKL_COMPL_DATA") >= 0) {
                            success.dataset.checklistsafetados[i] = ScreenService.i18n(mensagens[1], {
                                "0": code_params[1]
                            });
                        } else if (code_params[0].indexOf("PCP00013_NECE_CKL_COMPL") >= 0) {
                            success.dataset.checklistsafetados[i] = ScreenService.i18n(mensagens[0]);
                        }
                    }
                    InflightorGeneral.showErrorMessages(widget, success.dataset.checklistsafetados);
                });
            } else {
                ScreenService.closePopup();
            }
        }, function(error) {
            let errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
        });
    }

    this.getWidgetPrevisao = function(fields) {
        var widget_previsao = {
            "name": "previsao_passageiros_crosstab",
            "label": "Previsão de Pax",
            "isVisible": true,
            "itemsPerPage": 30,
            "loadMore": false,
            "selectDetailOnMoveRow": false,
            "template": "widget/grid.html",
            "formTemplate": "widget/grid.html",
            "templateToForm": "widget/grid.html",
            "templateToView": "widget/grid.html",
            "dataSource": {},
            "styles": [],
            "dataSourceFilter": [],
            "events": [
                {
                    "name": "WidgetOnSaveInlineEdit",
                    "code": "PrevisaoPassageirosProducao.saveEditPrevisao(args);"
                },
                {
                    "name": "WidgetOnCancelInlineEdit",
                    "code": "PrevisaoPassageirosProducao.cancelEditPrevisao(args.owner)"
                }
            ],
            "actions": [
                {
                    "name": "ZH_BACK",
                    "label": "Voltar",
                    "isVisible": true,
                    "showAsAction": "back|back_view",
                    "hideIcon": "true",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "PrevisaoPassageirosProducao.goBack();"
                  }
                ]
              }
            ],
            "fields": fields ? fields : [],
            "widgets": []
        };
        return widget_previsao;
    };

    this.getFieldsIniti = function(widget) {
        var fields = [
            {
                "name": "CDFILIAL",
                "isVisible": false,
                "class": 0,
                "order": 1,
                "label": "Unidade",
                "defaultValue": "@getLocalVar('INFLIGHTOR_CDFILIAL')@",
                "template": "field/text-edit.html",
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
            },
            {
                "name": "CDCIA",
                "isVisible": false,
                "class": 0,
                "order": 2,
                "label": "Unidade",
                "template": "field/text-edit.html",
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
            },
            {
                "name": "NRVOO",
                "label": "Voo",
                "template": "field/text-edit.html",
                "class": 0,
                "order": 3,
                "descriptionOrder": false,
                "isVisible": false,
                "readOnly": false,
                "showOnView": false,
                "showOnForm": false,
                "showOnList": false,
                "maxlength": 30,
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
            },
            {
                "name": "VOOCIA",
                "label": "Cia - Voo",
                "template": "field/text-edit.html",
                "class": 3,
                "order": 4,
                "descriptionOrder": true,
                "isVisible": true,
                "readOnly": false,
                "showOnView": true,
                "showOnForm": true,
                "align": "left",
                "showOnList": true,
                "maxlength": 30,
                "styles": [],
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
            },
            {
                "name": "TRECHO",
                "label": "Trecho",
                "template": "field/text-edit.html",
                "class": 3,
                "order": 5,
                "descriptionOrder": true,
                "isVisible": true,
                "readOnly": false,
                "align": "left",
                "showOnView": true,
                "showOnForm": true,
                "showOnList": true,
                "maxlength": 30,
                "styles": [],
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
            }
        ];
        return fieldFactory(fields, widget);
    };

    this.getFieldsLasted = function(widget, order) {
        var fieldsLasted = [
            {
                "label": "Tipo de Cálculo",
                "name": "IDPADRAOCALC",
                "class": 12,
                "events": [],
                "isVisible": true,
                "readOnly": false,
                "order": ++order,
                "showOnList": true,
                "showOnView": true,
                "align": "left",
                "showOnForm": true,
                "hideEmptyValue": true,
                "defaultValue": "2",
                "template": "field/select-native.html",
                "valueField": "IDPADRAOCALC",
                "descriptionField": "TIPOCALCLABEL",
                "dataSource": {
                    "data": [
                        {
                            "TIPOCALCLABEL": "Máxima",
                            "IDPADRAOCALC": "2"
                        },
                        {
                            "TIPOCALCLABEL": "Média - His. de Faturamento",
                            "IDPADRAOCALC": "3"
                        },
                        {
                            "TIPOCALCLABEL": "Padrão de Geração",
                            "IDPADRAOCALC": "4"
                        },
                        {
                            "TIPOCALCLABEL": "Pedido",
                            "IDPADRAOCALC": "5"
                        }
                    ]
                },
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
          },
            {
                "label": "Situação",
                "name": "IDPREVISAOPROD",
                "class": 6,
                "events": [],
                "order": ++order,
                "isVisible": true,
                "readOnly": false,
                "showOnList": true,
                "align": "left",
                "showOnView": true,
                "showOnForm": true,
                "template": "field/select-native.html",
                "valueField": "IDPREVISAOPROD",
                "descriptionField": "SITUACAO",
                "dataSource": {
                    "data": [
                        {
                            "SITUACAO": "Pendente",
                            "IDPREVISAOPROD": "P"
                },
                        {
                            "SITUACAO": "Aprovada",
                            "IDPREVISAOPROD": "A"
                },
                        {
                            "SITUACAO": "Check List Gerado",
                            "IDPREVISAOPROD": "G"
                },
                        {
                            "SITUACAO": "Cancelada",
                            "IDPREVISAOPROD": "C"
                }
              ]
                },
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
        },
            {
                "label": "Revisar",
                "name": "REVISAR",
                "class": 6,
                "events": [],
                "order": ++order,
                "isVisible": true,
                "readOnly": false,
                "align": "left",
                "showOnList": true,
                "showOnView": true,
                "showOnForm": true,
                "template": "field/select-native.html",
                "valueField": "REVISAR",
                "descriptionField": "SITUACAO",
                "dataSource": {
                    "data": [
                        {
                            "SITUACAO": "Ativar",
                            "REVISAR": "ATIVAR"
                },
                        {
                            "SITUACAO": "Inativar",
                            "REVISAR": "INATIVAR"
                },
                        {
                            "SITUACAO": "Não",
                            "REVISAR": "VALIDO"
                }
              ]
                },
                "filterProperties": {
                    "class": 0,
                    "isVisible": false
                }
        }
      ];
        return fieldFactory(fieldsLasted, widget);
    };

    this.setDataSourcePrevisao = function(filter) {
        var deferred = ZHPromise.defer();
        var data = [];
        var row = {};
        InflightorGeneral.callRestEngine('pcp#/crud/prevpaxvooprodm_prevpaxclaprodm_classe', "FilterData", filter).then(function(callback) {
            var voos = [];
            var previsoes = sortInArray(callback.dataset.prevpaxvooprodm_prevpaxclaprodm_classe, ['CDCIA', 'NRVOO', 'CDTRECHO']);
            if (previsoes.length) {
                var voo_init = {
                    "CDCIA": previsoes[0].CDCIA,
                    "NRVOO": previsoes[0].NRVOO,
                    "CDTRECHO": previsoes[0].CDTRECHO
                };
                var voo = [];
                for (var key = 0; key < previsoes.length; key++) {
                    if (previsoes[key].CDCIA == voo_init['CDCIA'] && previsoes[key].NRVOO == voo_init['NRVOO'] && previsoes[key].CDTRECHO == voo_init['CDTRECHO']) {
                        voo.push({
                            'CDFILIAL': previsoes[key].CDFILIAL,
                            'CDCIA': previsoes[key].CDCIA,
                            'NRPREVISAOPROD': previsoes[key].NRPREVISAOPROD,
                            'IDPREVISAOPROD': previsoes[key].IDPREVISAOPROD,
                            'IDPADRAOCALC': previsoes[key].IDPADRAOCALC,
                            'CDEQUIP': previsoes[key].CDEQUIP,
                            'NRVOO': previsoes[key].NRVOO,
                            'NRORG': previsoes[key].NRORG,
                            'SGCIA': previsoes[key].SGCIA,
                            'VOOCIA': previsoes[key].VOOCIA,
                            'CDTRECHO': previsoes[key].CDTRECHO,
                            'CDCLASSE': previsoes[key].CDCLASSE,
                            'SGCLASSE': previsoes[key].SGCLASSE,
                            'TRECHO': previsoes[key].TRECHO,
                            'REVISAR': previsoes[key].REVISAR,
                            'QTPAXPREVINF': previsoes[key].QTPAXPREVINF
                        });
                    } else {
                        voos.push(voo);
                        voo = [];
                        voo_init['CDCIA'] = previsoes[key].CDCIA;
                        voo_init['NRVOO'] = previsoes[key].NRVOO;
                        voo_init['CDTRECHO'] = previsoes[key].CDTRECHO;
                        voo.push({
                            'CDFILIAL': previsoes[key].CDFILIAL,
                            'CDCIA': previsoes[key].CDCIA,
                            'NRPREVISAOPROD': previsoes[key].NRPREVISAOPROD,
                            'IDPREVISAOPROD': previsoes[key].IDPREVISAOPROD,
                            'IDPADRAOCALC': previsoes[key].IDPADRAOCALC,
                            'CDEQUIP': previsoes[key].CDEQUIP,
                            'NRVOO': previsoes[key].NRVOO,
                            'NRORG': previsoes[key].NRORG,
                            'SGCIA': previsoes[key].SGCIA,
                            'VOOCIA': previsoes[key].VOOCIA,
                            'CDTRECHO': previsoes[key].CDTRECHO,
                            'CDCLASSE': previsoes[key].CDCLASSE,
                            'SGCLASSE': previsoes[key].SGCLASSE,
                            'TRECHO': previsoes[key].TRECHO,
                            'REVISAR': previsoes[key].REVISAR,
                            'QTPAXPREVINF': previsoes[key].QTPAXPREVINF
                        });
                    }
                }
                voos.push(voo);
                var validateRows = false;
                var maxlength = arrayItemMax(voos);
                voos.forEach(function(voo) {
                    var row = {};
                    row.TRECHO = voo[0].TRECHO;
                    row.CDFILIAL = voo[0].CDFILIAL;
                    row.CDCIA = voo[0].CDCIA;
                    row.CDEQUIP = voo[0].CDEQUIP;
                    row.NRVOO = voo[0].NRVOO;
                    row.NRORG = voo[0].NRORG;
                    row.NRPREVISAOPROD = voo[0].NRPREVISAOPROD;
                    row.IDPREVISAOPROD = voo[0].IDPREVISAOPROD;
                    row.REVISAR = voo[0].REVISAR;
                    row.IDPADRAOCALC = voo[0].IDPADRAOCALC;
                    row.SGCIA = voo[0].SGCIA;
                    row.VOOCIA = voo[0].VOOCIA;
                    row.CDTRECHO = voo[0].CDTRECHO;
                    row.CDCLASSE = voo[0].CDCLASSE;
                    row.SGCLASSE = voo[0].SGCLASSE;
                    for (var i = 0; i < voo.length; i++) {
                        if (voo[i].CDCLASSE && voo[i].SGCLASSE) {
                            row[voo[i].CDCLASSE + ' - ' + voo[i].SGCLASSE] = voo[i].QTPAXPREVINF;
                        } else {
                            validateRows = true;
                        }
                    }
                    if (validateRows || maxlength > voo.length) {
                        createCompletedRow(row);
                    }
                    data.push(row);
                });
            }
            deferred.resolve(data);
        }, function(error) {
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
            deferred.reject(errorMessage);
        });
        return deferred.promise;
    };

    this.goBack = function(args) {
        ScreenService.goBack();
        args.owner.widget.container.getWidget('previsao_passageiros_tabbed_companhias').activate();
    };

    function _setFilterByDefaultWidget(widget, level) {
        let _originWidget = widget.parent.parent;
        widget.dataSourceFilter.forEach(function(filter) {
            let _originField = _originWidget.getField(filter.name) || _originWidget.getField(filterNameConnectByRow[filter.name]);
            filter.value = _originWidget.currentRow[filter.name] || 'T';
            if (filter.name === 'DTPREVISAOPROD') {
                filter.value = _originWidget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
            } else if (filter.name === 'NRORG') {
                filter.value = getLocalVar('INFLIGHTOR_NRORGTRAB');
            }
            if (_originField) {
                if (_originField.template && ~_originField.template.indexOf('calendar')) {
                    filter.value = filter.value.slice(0, 10);
                }
            }
        });
    }

    this.navegationTabbeds = function(widget, level, parentWidget) {
        if (validateInlineEditable(widget)) {
            if (ScreenService.isSwipeOpen(level)) {
                let currentWidget = parentWidget.currentWidget;
                if (currentWidget) {
                    if (currentWidget.currentWidget) {
                        if (currentWidget.currentWidget.name === 'previsao_passageiros_crosstab') {
                            self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), false);
                        }
                        ScreenService.closeSwipe(level + 1).then(function() {
                            let padraocalc = widget.container.getWidget('previsao_passageiros_crosstab').currentRow.IDPADRAOCALC;
                            if (padraocalc == '5') {
                                currentWidget.container.getWidget('previsao_passageiros_itensfaturamento').enabled = true;
                            } else {
                                if (currentWidget.currentWidget.name == 'previsao_passageiros_itensfaturamento') {
                                    currentWidget.currentWidget.enabled = false;
                                    currentWidget.currentWidget = widget.container.getWidget('previsao_passageiros_log');
                                }
                            }
                            if (currentWidget.currentWidget.name == 'previsao_passageiros_log') {
                                _setFilterLogWidget(currentWidget.currentWidget, widget.container.getWidget('previsao_passageiros_crosstab'));
                            } else {
                                _setFilterByDefaultWidget(currentWidget.currentWidget, 1);
                            }
                            currentWidget.currentWidget.reload().then(function() {
                                if (currentWidget.currentWidget.showCheckbox) {
                                    currentWidget.currentWidget.dataSource.unSelectAll();
                                }
                                currentWidget.currentWidget.activate();
                            });
                        });
                    }
                }
            } else {
                //Default tabbed
                switch (level) {
                    case 0:
                        self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), false);
                        break;
                    case 1:
                    case 2:
                        self.afterMoveRowCrosstab(widget);
                        break;
                }
            }
        }
    };

    this.openSwipeTabbedCompanhia = function(widget) {
        var widgetTabbed = widget.container.getWidget('previsao_passageiros_tabbed');
        var widgetCompanhia = widget.container.getWidget('previsao_passageiros_tabbed_companhias');
        ScreenService.openSwipe(widgetTabbed).then(function(success) {
            widgetCompanhia.dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.appliedFilter.FILTER_ROW.CDFILIAL
          },
                {
                    "name": "NRPREVISAOPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRPREVISAOPROD
          }
        ];
            widgetCompanhia.reload().then(function() {
                widgetCompanhia.activate();
                widgetCompanhia.dataSource.unSelectAll();
            });
        });
    };
    this.openSwipeTabbedVoo = function(widget) {
        var widgetTabbed = widget.container.getWidget('previsao_passageiros_tabbed_companhias_voos');
        var widgetVoo = widget.container.getWidget('previsao_passageiros_tabbed_voos');
        ScreenService.openSwipe(widgetTabbed).then(function(success) {
            widgetVoo.dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
          },
                {
                    "name": "NRPREVISAOPROD",
                    "operator": "=",
                    "value": widget.currentRow.NRPREVISAOPROD
          },
                {
                    "name": "CDCIA",
                    "operator": "=",
                    "value": widget.currentRow.CDCIA
          }
        ];
            widgetVoo.reload().then(function() {
                widgetVoo.activate();
                widgetVoo.dataSource.unSelectAll();
                widgetVoo.dataSource.data.forEach(function(voo) {});
            });
        });
    };

    this.openPopupCalcularPax = function(widget) {
        var widgetPopup = widget.container.getWidget('popupCalcularQuantidadePax');
        widgetPopup.currentRow.CDFILIAL = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').appliedFilter.FILTER_ROW.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').appliedFilter.FILTER_ROW.NMFILIAL;
        widgetPopup.currentRow.DTPREVISAOVOO = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
        ScreenService.openPopup(widgetPopup).then(function() {
            widgetPopup.getField('IDPADRAOCALC').applyDefaultValue();
            widgetPopup.getField('DSPADRAOGERPREVPAX').readOnly = true;
            widgetPopup.getField('DSPADRAOGERPREVPAX').validations = {};
            widgetPopup.getField('DSPADRAOGERPREVPAX').applyDefaultValue();
        });
    };

    this.changedTipoCalculo = function(field) {
        if (field.name == 'IDPADRAOCALC') {
            let fieldPadrao = field.widget.getField('DSPADRAOGERPREVPAX');
            if (field.value() == '4') {
                fieldPadrao.readOnly = false;
                fieldPadrao.validations = {
                    "required": true
                };
            } else {
                fieldPadrao.applyDefaultValue();
                fieldPadrao.validations = {};
                fieldPadrao.readOnly = true;
            }
        }
    };

    this.openPopupGerarPrevisao = function(widget) {
        var widgetPopup = widget.container.getWidget('gerarPrevisaoPassageiros');
        var widgetCompanhia = widget.container.getWidget('previsao_passageiros_tabbed_companhias');
        widgetPopup.currentRow.CDFILIAL = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').appliedFilter.FILTER_ROW.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').appliedFilter.FILTER_ROW.NMFILIAL;
        widgetPopup.currentRow.DTPREVISAOVOO = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
        widgetPopup.currentRow.CDCIA = widgetCompanhia.currentRow.CDCIA;
        widgetPopup.currentRow.NMCIA = widgetCompanhia.currentRow.NMCIA;
        widgetPopup.origin = widget.name;
        widgetPopup.getField('NMCIA').dataSourceFilter = angular.copy(widgetCompanhia.dataSourceFilter);
        if (widgetPopup.origin == 'previsao_passageiros_tabbed_companhias') {
            widgetPopup.getField('NMCIA').class = 12;
            widgetPopup.getField('TCIA').isVisible = false;
            widgetPopup.getField('TCIA').setValue("N");
            widgetPopup.getField('NRVOO').isVisible = false;
            widgetPopup.getField('NRVOO').validations = {};
            var checked_rows = widget.dataSource.getCheckedRows().filter(function(row) {
                return row.IDPREVISAOPROD == 'P';
            }).map(function(row) {
                return row.CDCIA;
            });
            ScreenService.openPopup(widgetPopup).then(function() {
                widgetPopup.getField('NMCIA').reload().then(function() {
                    setValueInSelectMultiple(widgetPopup.getField('NMCIA'), checked_rows);
                    widgetPopup.getField('NMCIA').readOnly = true;
                });
            });
        }
    };
    this.cancelarGerarPrevisao = function(widget) {
        var originWidget = widget.container.getWidget(widget.origin);
        ScreenService.closePopup(true);
        if (widget.getField('NMCIA').dataSource.getCheckedRows().length)
            widget.getField('NMCIA').dataSource.unSelectAll();
        widget.getField('NMCIA').setValue('');
        if (widget.getField('NRVOO').isVisible) {
            if (widget.getField('NRVOO').dataSource.getCheckedRows().length)
                widget.getField('NRVOO').dataSource.unSelectAll();
            widget.getField('NRVOO').setValue('');
        }
        originWidget.activate();
    };

    function createDataRegenerate(checkedRows, widget) {
        var data = [];
        checkedRows.forEach(function(row) {
            data.push({
                "P_CDFILIAL": row.CDFILIAL,
                "P_CDFILIPROC": getLocalVar('INFLIGHTOR_CDFILIAL'),
                "P_NRPREVISAOPROD": row.NRPREVISAOPROD,
                "P_CDCIA": row.CDCIA,
                "P_NRVOO": row.NRVOO,
                "P_CDTRECHO": row.CDTRECHO,
                "P_CDEQUIP": row.CDEQUIP,
                "P_IDPADRAOCALC": widget.currentRow.IDPADRAOCALC,
                "P_NRPADRAOGERPREVPAX": widget.currentRow.NRPADRAOGERPREVPAX ? widget.currentRow.NRPADRAOGERPREVPAX : null
            });
        });
        return data;
    }

    this.calcularPassageiros = function(widget) {
        if (widget.isValid()) {
            var promises = [];
            var widgetPrevisao = widget.container.getWidget('previsao_passageiros_crosstab');
            MessageService.confirmMessageByCode("PCP00013_CONF_SOBRESC_PAX").then(function(success) {
                var dataTosave = createDataRegenerate(widgetPrevisao.getCheckedRows(), widget);
                promises.push(PrevisaoPassageirosProducaoService.callProcedurePax(dataTosave));
                callProcedureGeraPax(widget, promises);
            });
        }
    };

    function callProcedureGeraPax(widget, promises) {
        var widgetPrevisao = widget.container.getWidget('previsao_passageiros_crosstab');
        ZHPromise.all(promises).then(function(callback) {
            ScreenService.closePopup(true);
            widgetPrevisao.unSelectAll();
            self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), true);
            widgetPrevisao.activate();
            if (callback.length && callback[0].dataset && callback[0].dataset.ListaErros && callback[0].dataset.ListaErros.length && callback[0].dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.mensagensErroProcedure(callback[0].dataset.ListaErros);
            } else {
                MessageService.showMessageByCode("PCP00013_CALC_SUCES");
                //success
            }
        }, function(error) {
            //error
            var errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
        });
    }

    this.aprovarPrevisao = function(widget, status) {
        if (widget.name == 'previsao_passageiros_tabbed_companhias') {
            self.changeStatus(widget, status);
        } else if (widget.name == 'previsao_passageiros_crosstab') {
            self.aprovarPrevisaoVoos(widget, status);
        }
    };

    function _getLabelByStatus(status) {
        let labelStatus = {};
        switch (status) {
            case 'P':
                labelStatus.LABELSTATUSERROR = mensagensGerais.nPendente;
                labelStatus.LABELSCS = mensagensGerais.pendente;
                labelStatus.STATUSREQUIRED = false;
                labelStatus.STATUSDIFF = 'P';
                break;
            case 'A':
                labelStatus.LABELSTATUSERROR = mensagensGerais.pendentes;
                labelStatus.LABELSCS = mensagensGerais.aprovada;
                labelStatus.STATUSREQUIRED = 'P';
                break;
            case 'C':
                labelStatus.LABELSTATUSERROR = mensagensGerais.nCancelada;
                labelStatus.LABELSCS = mensagensGerais.cancelada;
                labelStatus.STATUSREQUIRED = false;
                labelStatus.STATUSDIFF = 'C';
                break;
        }
        return labelStatus;
    }

    function calculoEfetuado(voos) {
        return voos.map(function(row) {
            return row.IDPADRAOCALC;
        }).join('_').indexOf('1') == -1;
    }

    this.aprovarPrevisaoVoos = function(widget, status) {
        var labelStatus = _getLabelByStatus(status);
        var voos = widget.getCheckedRows().filter(function(voo) {
            if (labelStatus['STATUSREQUIRED']) {
                return voo.IDPREVISAOPROD == labelStatus['STATUSREQUIRED'];
            } else {
                return voo.IDPREVISAOPROD != labelStatus['STATUSDIFF'];
            }
        });
        if (voos.length) {
            if (calculoEfetuado(voos) || status !== 'A') {
                voos[0].DTPREVISAOPROD = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
                voos[0].STATUS = status;
                InflightorGeneral.callRestEngine('pcp#/prevpaxciaprodm/aprovar_voo', 'DataSet', voos).then(function(success) {
                    ScreenService.closePopup(true);
                    widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').reload().then(function() {
                        self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), true);
                        widget.list();
                        widget.dataSource.unSelectAll();
                        widget.commitInlineEdit();
                        widget.activate();
                        if (success.dataset && success.dataset.approvedPrevisao.msgerror && success.dataset.approvedPrevisao.msgerror.length) {
                            MessageService.getMessagesByCode(['PCP00013_PREV_SEM_CLASSE', 'PCP00013_PREV_SEM_CLASSE_VOO',
                                                              'PCP00013_PREV_CHECKLIST_GER', 'PCP00013_PREV_CHECKLIST_GER_VOO']).then(function(mensagens) {
                                for (let i = 0; i < success.dataset.approvedPrevisao.msgerror.length; i++) {
                                    let code_params = success.dataset.approvedPrevisao.msgerror[i].split("#_#");
                                    if (code_params[0].indexOf("PCP00013_PREV_SEM_CLASSE_VOO") >= 0) {
                                        success.dataset.approvedPrevisao.msgerror[i] = ScreenService.i18n(mensagens[1], {
                                            "0": code_params[1],
                                            "1": code_params[2],
                                            "2": code_params[2]
                                        });
                                    } else if (code_params[0].indexOf("PCP00013_PREV_SEM_CLASSE") >= 0) {
                                        success.dataset.approvedPrevisao.msgerror[i] = ScreenService.i18n(mensagens[0]);
                                    }

                                    if (code_params[0].indexOf("PCP00013_PREV_CHECKLIST_GER_VOO") >= 0) {
                                        success.dataset.approvedPrevisao.msgerror[i] = ScreenService.i18n(mensagens[3], {
                                            "0": code_params[1],
                                            "1": code_params[2],
                                            "2": code_params[2]
                                        });
                                    } else if (code_params[0].indexOf("PCP00013_PREV_CHECKLIST_GER") >= 0) {
                                        success.dataset.approvedPrevisao.msgerror[i] = ScreenService.i18n(mensagens[2]);
                                    }
                                }
                                InflightorGeneral.showErrorMessages(widget, success.dataset.approvedPrevisao.msgerror);
                            });
                        } else if (success.dataset.approvedPrevisao.approved_day) {
                            MessageService.showMessageByCode("PCP00013_PREV_ALT_SUCES_APROV", {
                                "0": voos[0].DTPREVISAOPROD.slice(0, 10),
                                "1": labelStatus['LABELSCS']
                            });
                        } else
                            MessageService.showMessageByCode("PCP00013_PREV_ALT_SUCES");
                    });
                }, function(error) {
                    var errorMessage = 'Timeout';
                    if (error.data)
                        trataMensagem(error.data.error);
                    else
                        console.log(error);
                });
            } else {
                MessageService.showMessageByCode("PCP00013_SELE_PREV_COM_QTPAX_CALC");
            }
        } else {
            MessageService.showMessageByCode("PCP00013_SELE_PREV_COM_STATUS", {
                "0": labelStatus['LABELSTATUSERROR']
            });
        }
    };

    this.changeStatus = function(widget, status) {
        var labelStatusError = '';
        var labelStatus = '';
        var statusRequired = '';
        if (status == 'P') {
            labelStatusError = mensagensGerais.aprovadas;
            labelStatus = mensagensGerais.pendentes;
            statusRequired = 'A';
        } else {
            labelStatusError = mensagensGerais.pendentes;
            labelStatus = mensagensGerais.aprovadas;
            statusRequired = 'P';
        }
        var companhias = widget.getCheckedRows().filter(function(cia) {
            return cia.IDPREVISAOPROD == statusRequired;
        });
        if (companhias.length) {
            companhias[0].DTPREVISAOPROD = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
            companhias[0].STATUS = status;
            InflightorGeneral.callRestEngine('pcp#/prevpaxciaprodm/aprovar_cia', 'DataSet', companhias).then(function(callback) {
                ScreenService.closePopup();
                widget.dataSource.unSelectAll();
                widget.reload().then(function() {
                    if (callback && callback.dataset && callback.dataset.errorMsgs && callback.dataset.errorMsgs.length) {
                        InflightorGeneral.showErrorMessages(widget, callback.dataset.errorMsgs);
                    } else {
                        if (callback && callback.dataset && callback.dataset.approvedPrevisao.approved_cia) {
                            widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').reload();
                            MessageService.showMessageByCode("PCP00013_CIA_ALT_SUCES_STATUS", {
                                "0": companhias[0].DTPREVISAOPROD.slice(0, 10),
                                "1": labelStatus
                            });
                        } else
                            MessageService.showMessageByCode("PCP00013_CIA_ALT_SUCES");
                    }
                    widget.activate();
                });
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data)
                    trataMensagem(error.data.error);
                else
                    console.log(error);
            });
        } else {
            MessageService.showMessageByCode("PCP00013_CIA_SEL_STATUS", {
                "0": labelStatusError
            });
        }
    };

    function _setFilterLogWidget(logWidget, widget) {
        logWidget.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL ? widget.currentRow.CDFILIAL : widget.container.getWidget('previsao_passageiros_crosstab').currentRow.CDFILIAL
          },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": widget.currentRow.CDCIA ? widget.currentRow.CDCIA : widget.container.getWidget('previsao_passageiros_crosstab').currentRow.CDCIA
          },
            {
                "name": "NRPREVISAOPROD",
                "operator": "=",
                "value": widget.currentRow.NRPREVISAOPROD ? widget.currentRow.NRPREVISAOPROD : widget.container.getWidget('previsao_passageiros_crosstab').currentRow.NRPREVISAOPROD
          },
            {
                "name": "NRVOO",
                "operator": "=",
                "value": widget.currentRow.NRVOO
          },
            {
                "name": "CDTRECHO",
                "operator": "=",
                "value": widget.currentRow.CDTRECHO
          },
            {
                "name": "NRORG",
                "operator": "=",
                "value": parseInt(getLocalVar('INFLIGHTOR_NRORGTRAB'))
          }
        ];
    }

    this.afterMoveRowCrosstab = function(widget) {
        if (validateInlineEditable(widget)) {
            if (widget && !widget.editing) {
                let activateWidget = widget.container.getActiveWidget();
                let logTabbed = widget.container.getWidget('previsao_passageiros_tabbed_log');
                let logWidget;
                if (widget.currentRow && widget.currentRow.IDPADRAOCALC !== '5') {
                    widget.container.getWidget('previsao_passageiros_itensfaturamento').enabled = false;
                    logWidget = widget.container.getWidget('previsao_passageiros_log');
                } else {
                    logWidget = widget.container.getWidget('previsao_passageiros_itensfaturamento');
                    logWidget.enabled = true;
                }
                if (ScreenService.isSwipeOpen(1)) {
                    _setFilterLogWidget(logWidget, widget);
                    logWidget.reload().then(function() {
                        logWidget.activate();
                    });
                } else {
                    ScreenService.openSwipe(logTabbed).then(function() {
                        _setFilterLogWidget(logWidget, widget);
                        logWidget.reload().then(function() {
                            logWidget.activate();
                        });
                    });
                }
            }
        }
    };

    this.goBackPrevisao = function(widget, backWidget) {
        backWidget.currentWidget = null;
        ScreenService.closeSwipe().then(function() {
            backWidget.activate();
        });
    };

    this.afterLoadDataSource = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            row.DTGERARCOMPL = row.DTGERARCOMPL.slice(0, 10);
        });
    };
    this.afterLoadDataSourceChecklist = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            row.DTPRODUCAOINF = row.DTPRODUCAOINF.slice(0, 10);
        });
    };
    this.beforeSelectRow = function(args, type) {
        if (args.owner.editing) {
            MessageService.showMessageByCode("PCP00013_CLASSE_EM_PROC_EDIT");
            args.e.preventDefault();
        }
    };
    this.beforeInlineEdit = function(args) {
        let editing = args.owner.dataSource.getCheckedRows().length;
        if (editing) {
            MessageService.showMessageByCode("PCP00013_EXISTE_PREV_SEL");
            args.e.preventDefault();
            return false;
        }
        return true;
    };

    this.setFilterPadrao = function(field) {
        field.dataSourceFilter[0].value = field.widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.CDFILIAL;
    };

    this.saveItensPedido = function(args) {
        let widget = args.owner;
        let statusCia = widget.container.getWidget('previsao_passageiros_crosstab').currentRow.IDPREVISAOPROD;
        let dataTosave = widget.dataSource.data.filter(function(row) {
            return row.changed;
        });
        dataTosave.forEach(function(row) {
            row['QTSERVICO'] = row['QTSERVICOINF'];
        });
        if (dataTosave.length) {
            dataTosave[0].DTPREVISAOPROD = widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid').currentRow.DTPREVISAOPROD;
        }
        if (statusCia == 'P') {
            dataTosave.push({
                "VALIDAR_CHECKLIST": false
            });
            _savePedidos(widget, dataTosave);
        } else {
            dataTosave.push({
                "VALIDAR_CHECKLIST": true,
                "CLASSES_MODIFICADAS": joinChangedClasses(dataTosave)
            });
            _savePedidos(widget, dataTosave);
        }
    };

    function _savePedidos(widget, dataTosave) {
        InflightorGeneral.callRestEngine('pcp#/prevpaxclaprodm/saveEditPedidos', 'DataSet', dataTosave).then(function(success) {
            // reload
            // self.openSwipePrevisao(widget.container.getWidget('pcp00013_previsao_passageiros_producao_grid'), true);
            widget.list();
            widget.commitInlineEdit();
            widget.activate();
            if (success.dataset && success.dataset.checklistsafetados && success.dataset.checklistsafetados.length) {
                MessageService.getMessagesByCode(['PCP00013_NECE_CKL_COMPL', 'PCP00013_NECE_CKL_COMPL_DATA']).then(function(mensagens) {
                    for (let i = 0; i < success.dataset.checklistsafetados.length; i++) {
                        let code_params = success.dataset.checklistsafetados[i].split("#_#");
                        if (code_params[0].indexOf("PCP00013_NECE_CKL_COMPL_DATA") >= 0) {
                            success.dataset.checklistsafetados[i] = ScreenService.i18n(mensagens[1], {
                                "0": code_params[1]
                            });
                        } else if (code_params[0].indexOf("PCP00013_NECE_CKL_COMPL") >= 0) {
                            success.dataset.checklistsafetados[i] = ScreenService.i18n(mensagens[0]);
                        }
                    }
                    InflightorGeneral.showErrorMessages(widget, success.dataset.checklistsafetados);
                });
            } else {
                ScreenService.closePopup();
            }
        }, function(error) {
            let errorMessage = 'Timeout';
            if (error.data) {
                trataMensagem(error.data.error);
            } else
                console.log(error);
        });
    }

    this.carregarHistorico = function(widget) {
        if (event && event.target && event.target.className) {
            if (event.target.className.indexOf('inline-editable') === -1 && event.target.className.indexOf('editable-content') === -1) {
                var widgetHistoricoTabbed = widget.container.getWidget('previsao_passageiros_historico_tabbed');
                var widgetHistorico = widget.container.getWidget('previsao_passageiros_historico');
                if (ScreenService.isSwipeOpen(2)) {
                    this.filtraHistoricoPedido(widgetHistorico);
                } else {
                    ScreenService.openSwipe(widgetHistoricoTabbed).then(function(success) {
                        widgetHistorico.activate();
                    });
                }
            }
        }
    };

    this.filtraHistoricoPedido = function(widget) {
        var currentRow = widget.container.getWidget('previsao_passageiros_itensfaturamento').currentRow;
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
        },
            {
                "name": "NRPREVPAXSERVPROD",
                "operator": "=",
                "value": currentRow.NRPREVPAXSERVPROD
        },
            {
                "name": "NRPREVISAOPROD",
                "operator": "=",
                "value": currentRow.NRPREVISAOPROD
        }
      ];
        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.getMensagens = function() {
        MessageService.getMessagesByCode([
            'PCP00013_N_PEND', 'PCP00013_PEND',
            'PCP00013_PENDS', 'PCP00013_APROV',
            'PCP00013_APROVS', 'PCP00013_N_CANCEL',
            'PCP00013_CANCEL'
        ]).then(function(mensagens) {
            mensagensGerais.nPendente = ScreenService.i18n(mensagens[0]);
            mensagensGerais.pendente = ScreenService.i18n(mensagens[1]);
            mensagensGerais.pendentes = ScreenService.i18n(mensagens[2]);
            mensagensGerais.aprovada = ScreenService.i18n(mensagens[3]);
            mensagensGerais.aprovadas = ScreenService.i18n(mensagens[4]);
            mensagensGerais.nCancelada = ScreenService.i18n(mensagens[5]);
            mensagensGerais.cancelada = ScreenService.i18n(mensagens[6]);
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('PrevisaoPassageirosProducao', PrevisaoPassageirosProducao);
});

// FILE: ./assets/js/controllers/ProcessoChecklist.js
var ProcessoChecklist = function(ScreenService, InflightorGeneral, MessageService) {

    var self = this;

    function isFloat(value) {
        return Number(value) === value && value % 1 !== 0;
    }

    this.calculaChecklist = function(widget) {
        if (!widget.isValid())
            return;
        //Rotina pra calcular o Checklist
        ScreenService.confirmMessage('Checklist calculado com Sucesso! Deseja ir para a tela de Emissão do Checklist?', 'question', function(success) {
            openWindow('pcp#/emissao_do_checklist');
        });
    };
    this.openPopupCalcularChecklist = function(widget) {
        var widgetPopup = widget.container.getWidget('calcularChecklist');
        widgetPopup.newRow();
        widgetPopup.currentRow.CDFILIAL = widget.container.getWidget('pcp00012_geracao_checklist').__filterPopup.currentRow.CDFILIAL;
        widgetPopup.currentRow.NMFILIAL = widget.container.getWidget('pcp00012_geracao_checklist').__filterPopup.currentRow.NMFILIAL;
        ScreenService.openPopup(widgetPopup).then(function() {
            widgetPopup.activate();
        });
    };

    this.afterMoveRowChecklist = function(widget, name) {
        if (!widget.currentRow.__is_new && (widget.currentWidget && widget.currentWidget.name == name)) {
            var descChecklist = widget.container.getWidget(name);
            //aplica novo filtro com os valores da currentRow
            var filter = applyNewFilter(widget);
            descChecklist.dataSourceFilter = filter;
            descChecklist.reload().then(function() {
                descChecklist.activate();
            });

        }
    };

    this.afterMoveRowSetor = function(widget) {
        if (!widget.currentRow.__is_new) {
            var receitaChecklist = widget.container.getWidget('pcp00012_listagem_receitas');
            var tabbedChecklist = widget.container.getWidget('pcp00012_receitas');
            var filter = applyNewFilter(widget);
            receitaChecklist.dataSourceFilter = filter;
            ScreenService.openSwipe(tabbedChecklist).then(function() {
                receitaChecklist.reload().then(function() {
                    receitaChecklist.activate();
                });
            });
        }
    };

    function applyNewFilter(widget) {
        return [];
    }

    this.cancelEditReceita = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    this.saveEditReceitas = function(widget) {
        //Receitas alteradas
        var changedReceitas = widget.dataSource.data.filter(function(row) {
            return row.changed;
        });
        //Verifica as alterações feitas
        if (validReceitas(changedReceitas)) {
            //Salva as alterações

        }
    };

    function validReceitas(receitas) {
        var valid = true;
        receitas.forEach(function(row) {
            if (row.SGUNIDADE == 'UN' && isFloat(row.QTRECEITA))
                valid = false;
        });
        if (!valid)
            MessageService.showMessageByCode("RECIPE_QTD_FRACTIONAL_ERROR", 'error');
        return valid;
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ProcessoChecklist', ProcessoChecklist);
});

// FILE: ./assets/js/controllers/ReceitaProduzirController.js
var ReceitaProduzirController = function(ScreenService, InflightorGeneral, ZhFilterPreferences) {

    var self = this;

    this.filterOnEnter = function(args, tab) {
        var widget = args.owner.container.getWidget('widgetReceitaProduzir');
        InflightorGeneral.openFilterOnEnter(widget, ["CDFILIAL", "NMFILIAL", "DTINIVIGENCIA"]);
    };

    this.validateFilter = function(args, widget) {

        var rendimento;
        var quantidade;
        var fator;
        var filterComposicao;
        var composicao;

        retornaDadosReceita(args, widget).then(function(composicao) {

            if (composicao !== undefined) {

                widget.currentRow.NRVARIACAOREC = composicao.NRVARIACAOREC;
                widget.currentRow.FATOR = composicao.QTRENDIMENTOREC;
                widget.currentRow.IDRECFLEXIVEL = composicao.IDRECFLEXIVEL;

                rendimento = parseFloat(widget.currentRow.FATOR);
                quantidade = parseFloat(args.e.preference.FILTER_ROW.QUANTIDADE.replace(",", "."));
                fator = parseFloat((quantidade) / (rendimento));

                /*stripeFields.RENDIMENTO     = rendimento;
                stripeFields.QUANTIDADE     = quantidade;
                
                stripeFields.OPCOES         = composicao.NROPCOES;
                stripeFields.IDRECFLEXIVEL  = composicao.IDRECFLEXIVEL;*/

                retornaSequencial(args, widget).then(function(sequencial) {
                    if (sequencial.length) {
                        widget.currentRow.NRSEQUTILRECFLEX = sequencial[0].NRSEQUTILRECFLEX;
                    } else {
                        widget.currentRow.NRSEQUTILRECFLEX = null;
                    }
                    filterComposicao = [
                        {
                            "name": "CDFILIAL",
                            "operator": "=",
                            "value": args.e.preference.FILTER_ROW.CDFILIAL
                        },
                        {
                            "name": "CDRECEITA",
                            "operator": "=",
                            "value": args.e.preference.FILTER_ROW.CDRECEITA
                        },
                        {
                            "name": "FATOR",
                            "operator": "=",
                            "value": fator
                        },
                        {
                            "name": "NRVARIACAOREC",
                            "operator": "=",
                            "value": widget.currentRow.NRVARIACAOREC
                        },
                        {
                            "name": "NRSEQUTILRECFLEX",
                            "operator": "=",
                            "value": widget.currentRow.NRSEQUTILRECFLEX
                        },
                        {
                            "name": "IDRECFLEXIVEL",
                            "operator": "=",
                            "value": widget.currentRow.IDRECFLEXIVEL
                        },
                        {
                            name: "NRORG",
                            operator: "=",
                            value: getLocalVar('INFLIGHTOR_NRORGTRAB')
                        }
                    ];

                    /*InflightorGeneral.callRestEngine("pcp#/crud/composicao_pcp", 'FilterData', filterComposicao).then(function(callback) {
                        composicao = callback.dataset.composicao_pcp;
                        if(composicao.length > 0) {
                            widget.dataSourceFilter = [];
                            widget.dataSource.data = composicao;
                            widget.reload();
                            ScreenService.closePopup();
                            widget.activate();
                        } else {
                            ScreenService.showMessage('A Receita selecionada não possui composição!');
                            ScreenService.openPopup(widget.__filterPopup);
                        }
                    });*/

                    widget.dataSourceFilter = filterComposicao;
                    widget.reload();

                });
            }
        });
    };

    this.filtraReceita = function(field) {
        field.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": field.widget.currentRow.CDFILIAL
            }
        ];

    };

    var retornaSequencial = function(args, widget) {
        var filtroSeq = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.CDRECEITA
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.CDFILIAL
            },
            {
                "name": "DTPRODUCAO",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.DTINIVIGENCIA
            }
        ];

        return InflightorGeneral.callRestEngine("pcp#/crud/seq_composicao", 'FilterData', filtroSeq).then(function(callback) {
            return callback.dataset.seq_composicao;
        });
    };

    var retornaDadosReceita = function(args, widget) {
        var filtro = [
            {
                "name": "CDRECEITA",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.CDRECEITA
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.CDFILIAL
            }
        ];

        return InflightorGeneral.callRestEngine("pcp#/crud/verifica_receita", 'FilterData', filtro).then(function(callback) {
            return callback.dataset.verifica_receita[0];
        });

    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ReceitaProduzirController', ReceitaProduzirController);
});

// FILE: ./assets/js/controllers/RequisicaoPorSetor.js
/*global angular moment getLocalVar Configuration*/
var RequisicaoPorSetor = function(ScreenService, InflightorGeneral, templateManager, ZhFilterPreferences, restEngine, MessageService) {

    var self = this;

    var CDFILIAL;
    var NRREQUISICAOINF;
    var SETOR;
    var TURNO;

    // Codigo de correção para nao mostra o "Todos: Sim" no stripe 
    this.fixFilterInfo = function(widget) {
        widget._filterInfo = widget._filterInfo.filter(fieldInfo => fieldInfo.label !== 'Setor' && fieldInfo.label !== 'Todos');
    };

    this.startFilter = function(widget) {
        var date = new Date();
        date = moment(date).format("DD/MM/YYYY");

        InflightorGeneral.openFilterOnEnter(widget, ["CDFILIAL", "NMFILIAL", "DTREQUISICAOINF", "TSETOR", "TTURNO"]);
        widget.getField('NMFILIAL').setValue(getLocalVar('INFLIGHTOR_NMFILIAL'));
        widget.getField('CDFILIAL').setValue(getLocalVar('INFLIGHTOR_CDFILIAL'));
        widget.getField('DTREQUISICAOINF').setValue(date);
        // widget.__filterPopup.currentRow.CDFILIAL = getLocalVar('INFLIGHTOR_CDFILIAL');
        // widget.__filterPopup.currentRow.NMFILIAL = getLocalVar('INFLIGHTOR_NMFILIAL');
        // widget.__filterPopup.currentRow.DTREQUISICAOINF = date;
        templateManager.updateTemplate();

    };

    this.saveRequisicao = function(widget) {
        var currentRow = widget.currentRow;
        currentRow.IDORIGEM = "MANUAL";

        if (InflightorGeneral.validateTimeField(widget.getField('HRPREFERENCIAL'), 'HH:mm')) {
            if (currentRow.HRPREFERENCIAL) {
                currentRow.HRPREFERENCIAL = moment(currentRow.HRPREFERENCIAL, 'HH:mm').format('HHmm');
            }
            InflightorGeneral.widgetSave(widget).then(function(success) {
                self.changeActionsVisibility(widget);
            });
        }
    };

    var getReloadedRow = function(widget) {
        return widget.reload([], false).then(function(success) {
            var filter = function(row) {
                return row.NRREQUISICAOINF == widget.currentRow.NRREQUISICAOINF;
            };
            var reloadedRow = widget.dataSource.data.filter(filter)[0];
            widget.currentRow = reloadedRow;
        });
    };

    this.updateRequisicao = function(widget) {
        if (widget.currentRow.IDSITUACAO === 'PENDENTE') {
            var rowToSave = angular.copy(widget.currentRow);
            rowToSave['IDSITUACAO'] = "APROVADA";
            InflightorGeneral.callRestEngine('pcp#/crud/pcp00032_requisicao_por_setor/save', 'Row', rowToSave).then(function(success) {
                ScreenService.closePopup();
                getReloadedRow(widget).then(function() {
                    self.changeActionsVisibility(widget);
                });
                MessageService.showMessageByCode("REQUEST_APPROVED");
            }, InflightorGeneral.treatAndShowError);
        } else {
            MessageService.showMessageByCode("REQUEST_PEND_ERROR");
        }
    };

    this.cancelRequisicao = function(widget) {
        if (widget.currentRow.IDSITUACAO === 'APROVADA' && !widget.currentRow.NRRETIRADAINF) {
            var rowToSave = angular.copy(widget.currentRow);
            rowToSave['IDSITUACAO'] = "PENDENTE";
            InflightorGeneral.callRestEngine('pcp#/crud/pcp00032_requisicao_por_setor/save', 'Row', rowToSave).then(function(success) {
                ScreenService.closePopup();
                getReloadedRow(widget).then(function() {
                    self.changeActionsVisibility(widget);
                });
                MessageService.showMessageByCode("CANCEL_APPROVED");
            }, InflightorGeneral.treatAndShowError);
        } else {
            MessageService.showMessageByCode("CANCEL_REQUEST_ERROR");
        }
    };

    this.saveNewProduto = function(widget) {
        widget.currentRow.NRREQUISICAOINF = NRREQUISICAOINF;
        widget.currentRow.CDFILIAL = CDFILIAL;
        widget.currentRow.QTPRODUTOINF = parseFloat(widget.currentRow.QTPRODUTOINF);
        if (widget.getParent().currentRow.IDORIGEM !== "MANUAL" && !widget.currentRow.DSJUSTIFICATIVA) {
            MessageService.showMessageByCode("JUSTIFICATION_NULL_ERROR");
        } else {
            if (widget.currentRow.QTPRODUTOINF >= 0) {
                InflightorGeneral.widgetSave(widget);
            } else {
                MessageService.showMessageByCode("QUANTITY_ERROR");
            }
        }
    };

    this.createNewProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            widget.edit();
            widget.activate();
            widget.newRow();
            ScreenService.openSwipe(widget);
        } else {
            MessageService.showMessageByCode("ADD_PRODUCT_APPROVED");
        }
    };

    this.editProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            InflightorGeneral.edit(widget);
            ScreenService.openSwipe(widget);
        } else {
            MessageService.showMessageByCode("EDIT_PRODUCT_APRROVED");
        }
    };

    this.deleteProduct = function(widget) {
        if (widget.getParent().currentRow.IDSITUACAO !== 'APROVADA') {
            InflightorGeneral.deleteCrud(widget);
        } else {
            MessageService.showMessageByCode("DELETE_PRODUCT_APPROVED");
        }
    };

    this.setDefaultValue = function(widget, field) {
        widget.dataSourceFilter = widget.dataSourceFilter.filter(function(newFilter) {
            if (newFilter.name === field && newFilter.value !== 'T') {
                newFilter.value = 'T';
            }
        });
    };

    this.setFilterRequisicao = function(args, widget) {

        if (!(Util.isEmptyOrBlank(args.e.preference.FILTER_ROW))) {

            var setor = args.e.preference.FILTER_ROW.TSETOR;
            var turno = args.e.preference.FILTER_ROW.TTURNO;
            var arrayFiltro = [];

            if (turno == 'S') {
                turno = 'T';
            } else {
                turno = args.e.preference.FILTER_ROW.NRTURNO.join('_');
            }

            if (setor == 'S') {
                setor = 'T';
            } else {
                setor = args.e.preference.FILTER_ROW.NMSETOR.join('_');
            }

            arrayFiltro.push({
                "name": "CDSETOR",
                "operator": "=",
                "value": setor
            }, {
                "name": "NRTURNO",
                "operator": "=",
                "value": turno
            }, {
                "name": "CDFILIAL",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.CDFILIAL
            }, {
                "name": "DTREQUISICAOINF",
                "operator": "=",
                "value": args.e.preference.FILTER_ROW.DTREQUISICAOINF
            });

            widget.dataSourceFilter = arrayFiltro;

        }

    };

    this.clearValue = function(field) {
        field.clearValue();
    };

    this.setProdutosFilter = function(widget) {
        NRREQUISICAOINF = widget.currentRow.NRREQUISICAOINF;
    };

    this.getProdutosFilter = function() {
        return NRREQUISICAOINF;
    };

    this.toggleTabVisibility = function(widget) {
        widget.map(function(wid) {
            if (wid.isVisible === true) {
                wid.isVisible = false;
            } else {
                wid.isVisible = true;
            }
        });
        templateManager.updateTemplate();
    };

    this.clearFields = function(fields, position) {
        fields.map(function(field) {
            if (field.sequenceInList > position && field.dataSource) {
                for (var key in field.outData) {
                    fields.find(function(value) {
                        if (value.name === field.outData[key]) {
                            value.clearValue();
                            fields[field.sequenceInList].setValue('S');
                            InflightorGeneral.toggleReadOnlySelect(fields[field.sequenceInList], fields[(field.sequenceInList - 1)]);
                            fields[field.sequenceInList].checked = true;
                            templateManager.updateTemplate();
                        }
                    });
                }
            }
        });
    };

    this.setSetorFilter = function(value) {
        SETOR = value;
    };

    this.setTurnoFilter = function(value) {
        var add_ToTurnoFilter = '';
        value.forEach(function(filter) {
            add_ToTurnoFilter = filter + '_' + add_ToTurnoFilter;
        });
        TURNO = add_ToTurnoFilter;
    };

    this.getCdfilialFilter = function() {
        var widget = templateManager.container.getWidget('widget_requisicao_por_setor');
        return widget.currentRow.CDFILIAL;
    };

    this.getSetorFilter = function() {
        return SETOR || getLocalVar('CDFILIAL');
    };

    this.getTurnoFilter = function() {
        var widget = templateManager.container.getActiveWidget();
        if (typeof(widget.container.getWidget('widget_requisicao_por_setor').appliedFilter) == 'undefined') {
            return "T";
        } else {
            var wid = widget.container.getWidget('widget_requisicao_por_setor').appliedFilter.FILTER_ROW;
            if (wid.NMSETOR.length) {
                return TURNO;
            } else {
                return "T";
            }
        }
    };

    this.getAddTurnoFilter = function() {
        var wid = templateManager.container.getWidget('widget_requisicao_por_setor');
        return wid.currentRow.CDSETOR || undefined;
    };

    this.confirmDelete = function(widget) {
        if (widget.currentRow.IDSITUACAO === 'APROVADA') {
            MessageService.showMessageByCode("OPERATION_ERROR");
        } else {
            var datasourceToDelete = [];
            var wid = widget.container.getWidget('widget_produto_requisicao_por_setor');
            wid.dataSource.data.map(function(row) {
                datasourceToDelete.push(row);
            });
            if (datasourceToDelete.length > 0) {
                InflightorGeneral.callRestEngine(wid.dataSource.name + "/remove", "DataSet", datasourceToDelete).then(function() {
                    InflightorGeneral.deleteCrud(widget);
                }, function(error) {
                    MessageService.showMessageByCode("DELETE_PRODUCT_REQUEST");
                });
            } else {
                InflightorGeneral.deleteCrud(widget);
            }
        }
    };

    this.gerarAtendimento = function(widget) {
        var row = {};
        row.P_CDFILIAL = widget.currentRow.CDFILIAL;
        row.P_NRREQUISICAOINF = widget.currentRow.NRREQUISICAOINF;
        callProcedureGeraAtendimento(widget, row, 'gerar').then(function() {
            getReloadedRow(widget).then(function() {
                self.changeActionsVisibility(widget);
            });
        });
    };

    this.cancelarAtendimento = function(widget) {
        var row = {};
        row.P_CDFILIAL = widget.currentRow.CDFILIAL;
        row.P_NRREQUISICAOINF = widget.currentRow.NRREQUISICAOINF;
        callProcedureGeraAtendimento(widget, row, 'cancelar').then(function() {
            getReloadedRow(widget).then(function() {
                self.changeActionsVisibility(widget);
            });
        });
    };

    function callProcedure(row, procedure) {
        if (procedure === 'gerar') {
            return restEngine.request("pcp#/aprovaRequisicao", {
                "requestType": "Row",
                "row": row
            }, null, null, null, null, 300000);
        } else {
            return restEngine.request("pcp#/cancelaRequisicao", {
                "requestType": "Row",
                "row": row
            }, null, null, null, null, 300000);
        }

    }

    function callProcedureGeraAtendimento(widget, row, procedure) {
        return callProcedure(row, procedure).then(function(callback) {
            if (callback.dataset && callback.dataset.ListaErros && callback.dataset.ListaErros.length) {
                //Lista de erros gerados pela Procedure
                InflightorGeneral.showErrorMessages(widget, callback.dataset.ListaErros, 'Lista de Erros');
            } else {
                //success
                ScreenService.closePopup();
                var widgetSetor = widget.container.getWidget('widget_requisicao_por_setor');
                widgetSetor.reload().then(function() {
                    var successMsg = procedure == 'gerar' ? 'Atendimentos gerados com sucesso!' : 'Atendimentos cancelados com sucesso!';
                    widgetSetor.currentRow['NRRETIRADAINF'] = widgetSetor.dataSource.data.filter(function(row) {
                        return row['NRREQUISICAOINF'] === widgetSetor.currentRow['NRREQUISICAOINF'];
                    })[0]['NRRETIRADAINF'];
                    ScreenService.showMessage(ScreenService.i18n(successMsg), 'SUCCESS');

                    var filter = function(row) {
                        return row.NRREQUISICAOINF == widget.currentRow.NRREQUISICAOINF;
                    };
                    var reloadedRow = widget.dataSource.data.filter(filter)[0];
                    widget.currentRow = reloadedRow;

                    widgetSetor.activate();
                });
            }
        }, function(error) {
            //error
            var errorMessage = 'Timeout';
            if (error.data) {
                errorMessage = error.data.error;
            }
            ScreenService.showMessage(errorMessage);
        });
    }

    this.requisicaoAfterMoveRow = function(widgetRequisicao) {
        var widgetProduto = widgetRequisicao.container.getWidget('widget_produto_requisicao_por_setor');
        widgetProduto.unSelectAll();
        widgetProduto.dataSource.clearCheckedRows();
        self.setProdutosFilter(widgetRequisicao);
        self.changeActionsVisibility(widgetRequisicao);
    };

    this.addRequisicao = function(widget) {
        InflightorGeneral.widgetNewRow(widget);
        widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
    };

    var isRequisicaoPendente = function(widgetRequisicao) {
        return (widgetRequisicao.currentRow.IDSITUACAO == "PENDENTE");
    };

    this.changeActionsVisibility = function(widgetRequisicao) {};

    this.cancelInlineEdit = function(widget) {
        widget.rollbackInlineEdit();
        widget.list();
        widget.activate();
    };

    this.openPopupAdicionar = function(widget) {
        if (isRequisicaoPendente(widget.container.getWidget('widget_requisicao_por_setor'))) {
            InflightorGeneral.openLinkPopup(widget);
        } else {
            ScreenService.showMessage(ScreenService.i18n("Não é possível adicionar produtos em requisições já aprovadas."), 'error');
        }
    };

    this.saveLineEdit = function(widgetProduto) {
        if (isRequisicaoPendente(widgetProduto.getParent())) {
            let valueInvalid = false;
            widgetProduto.dataSource.data.forEach(function(row) {
                row.QTPRODUTOINF = (row.QTPRODUTOINF === null) ? 0 : row.QTPRODUTOINF;
                if (InflightorGeneral.valueToFloat(row.QTPRODUTOINF) < 0) {
                    valueInvalid = true;
                }
                InflightorGeneral.setFloatAttrOnCurrentRow(row, 'QTPRODUTOINF');
            });
            if (valueInvalid) {
                let msgError = ScreenService.i18n("Quantidade inválida.");
                ScreenService.showMessage(msgError);
            } else {
                if (widgetProduto.currentRow.QTPRODUTOCALC) {
                    if (!widgetProduto.currentRow.DSJUSTIFICATIVA) {
                        let msgJError = ScreenService.i18n("Justificativa deve ser informada!");
                        ScreenService.showMessage(msgJError);
                    } else {
                        InflightorGeneral.widgetSave(widgetProduto).then(function() {
                            widgetProduto.commitInlineEdit();
                            widgetProduto.list();
                            widgetProduto.activate();
                            InflightorGeneral.widgetReload(widgetProduto);
                        });
                    }
                } else {
                    InflightorGeneral.widgetSave(widgetProduto).then(function() {
                        widgetProduto.commitInlineEdit();
                        widgetProduto.list();
                        widgetProduto.activate();
                        InflightorGeneral.widgetReload(widgetProduto);
                    });
                }
            }
        } else {
            var msg = ScreenService.i18n("Não é possível alterar produtos em requisições já aprovadas.");
            ScreenService.showMessage(msg);
            widgetProduto.rollbackInlineEdit();
            widgetProduto.list();
        }
    };

    this.validValue = function(field) {
        if (InflightorGeneral.valueToFloat(field.value()) < 0) {
            let msgError = ScreenService.i18n("Quantidade inválida.");
            ScreenService.showMessage(msgError);
            field.setValue("");
        } else {
            field.setValue(InflightorGeneral.valueToFloat(field.value()));
        }
    };

    this.deletaProdutos = function(widgetProduto) {
        try {
            if (isRequisicaoPendente(widgetProduto.container.getWidget('widget_requisicao_por_setor'))) {
                var rows = widgetProduto.getCheckedRows();
                rows.forEach(function(row) {
                    if (row.QTPRODUTOCALC) {
                        throw "Não é possível excluir produtos originais da requisição!";
                    }
                });
                InflightorGeneral.deleteCrudCheckedRows(widgetProduto);
            } else {
                ScreenService.showMessage(ScreenService.i18n("Não é possível deletar produtos em requisições já aprovadas."));
            }
        } catch (e) {
            var errorMessage = ScreenService.i18n(e);
            ScreenService.showMessage(errorMessage, 'error');
            widgetProduto.list();
        }
    };

    this.cancelOptionsReport = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget('widget_requisicao_por_setor').activate();
    };

    this.openOptionsReport = function(widget) {
        let popupOptions = widget.container.getWidget('popup_relatorios');
        ScreenService.openPopup(popupOptions).then(function() {
            popupOptions.activate();
        });
    };

    this.openWindowReport = function(widget) {
        InflightorGeneral.openReport("pcp#/relatorioProducaoReceita", "PCP00332", getRowToReport(widget));
    };

    function getRowToReport(widget) {
        widgetRequisicao = widget.container.getWidget('widget_requisicao_por_setor');
        return {
            CDFILIAL: widgetRequisicao.currentRow.CDFILIAL,
            T_SETOR: "T",
            T_TURNO: "T",
            T_REQUISICAO: widgetRequisicao.currentRow.NRREQUISICAOINF,
            T_PRODUTO: "T",
            DTREFINI: moment(widgetRequisicao.currentRow.DTREQUISICAOINF, 'DD/MM/YYYY').format("DD/MM/YYYY"),
            DTREFFIM: moment(widgetRequisicao.currentRow.DTREQUISICAOINF, 'DD/MM/YYYY').format("DD/MM/YYYY"),
            NRORG: getLocalVar("INFLIGHTOR_NRORGTRAB"),
            REPORTNAME: "PCP00332",
            REPORTFORMAT: widget.currentRow.REPORTFORMAT
        };
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('RequisicaoPorSetor', RequisicaoPorSetor);
});

// FILE: ./assets/js/controllers/TipoProducaoClasseController.js
var TipoProducaoClasseController = function(ScreenService, InflightorGeneral) {
    var self = this;

    this.openPopupVinculo = function(popupVinculo) {
        ScreenService.openPopup(popupVinculo);
        popupVinculo.newRow();
        popupVinculo.activate();
    };

    this.closePopupVinculo = function(popupVinculo) {
        var parentWidget = popupVinculo.parent;
        ScreenService.closePopup();
        parentWidget.activate();
    };

    this.salvaVinculo = function(popupVinculo) {
        if (popupVinculo.isValid() && popupVinculo.currentRow.NRTIPOPROD) {
            var parentWidget = popupVinculo.parent;
            var checkedRows = parentWidget.getCheckedRows();

            var nrtipoprod = popupVinculo.currentRow.NRTIPOPROD;

            var rowsToSave = [];

            checkedRows.forEach(function(checkedRow) {
                var rowToSave = angular.copy(checkedRow);

                rowToSave.__is_new = true;
                rowToSave.NRTIPOPROD = nrtipoprod;

                rowsToSave.push(rowToSave);
            });

            InflightorGeneral.callRestEngine(parentWidget.dataSource.name + "/remove", "DataSet", checkedRows).then(function() {
                if (rowsToSave.length > 0) {
                    InflightorGeneral.callRestEngine(parentWidget.dataSource.name + "/save", "DataSet", rowsToSave).then(function() {
                        self.closePopupVinculo(popupVinculo);
                        parentWidget.unSelectAll();
                        parentWidget.reload();

                        var successMessage = 'Vínculo realizado com sucesso';
                        ScreenService.showMessage(ScreenService.i18n(successMessage));
                    }, function(erro) {
                        if (erro.data && erro.data.error) {
                            ScreenService.showMessage(erro.data.error);
                        } else {
                            ScreenService.showMessage("Erro ao salvar.");
                        }
                    });
                }
            });
        }
    };

    this.openSwipeVinculoCia = function(mainWidget) {
        var currentRow = mainWidget.currentRow;

        var cdfilial = currentRow.CDFILIAL;
        var cdclasse = currentRow.CDCLASSE;
        var nrorg = getLocalVar('INFLIGHTOR_NRORGTRAB');

        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": cdclasse
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": nrorg
            }
        ];

        var vinculoCiaWidget = mainWidget.container.getWidget('vinculoCia');
        vinculoCiaWidget.dataSourceFilter = dataSourceFilter;
        ScreenService.openSwipe(vinculoCiaWidget.parent);
        vinculoCiaWidget.reload();
    };

    this.openSwipeVinculoSubgrupo = function(menuGridSwipe) {
        var mainWidget = menuGridSwipe.container.getWidget('mainWidget');
        var subgrupoSwipe = mainWidget.container.getWidget('subgrupoSwipe');

        var currentRow = menuGridSwipe.currentRow;
        var dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRow.CDFILIAL
            },
            {
                "name": "CDCIA",
                "operator": "=",
                "value": currentRow.CDCIA
            },
            {
                "name": "CDMANCIA",
                "operator": "=",
                "value": currentRow.CDMANCIA
            },
            {
                "name": "CDCLASSE",
                "operator": "=",
                "value": mainWidget.currentRow.CDCLASSE
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('INFLIGHTOR_NRORGTRAB')
            }
        ];

        subgrupoSwipe.dataSourceFilter = dataSourceFilter;
        ScreenService.openSwipe(subgrupoSwipe.parent);
        subgrupoSwipe.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('TipoProducaoClasseController', TipoProducaoClasseController);
});

// FILE: ./assets/js/controllers/TurnoDeTrabalhoController.js
var TurnoDeTrabalhoController = function(ScreenService, InflightorGeneral) {

    this.validaParaSalvar = function(widgetTurno) {
        var somaValidacao = 0;
        var horaFaltando;

        if (widgetTurno.currentRow.HRINITURNO) {
            somaValidacao++;
            horaFaltando = 'fim';
        }
        if (widgetTurno.currentRow.HRFIMTURNO) {
            somaValidacao++;
            horaFaltando = 'inicio';
        }
        if (somaValidacao == 1) {
            ScreenService.showMessage('Informar hora ' + horaFaltando + '!', 'alert');
        } else {
            widgetTurno.currentRow.HRINITURNO = widgetTurno.currentRow.HRINITURNO.replace(":", "");
            widgetTurno.currentRow.HRFIMTURNO = widgetTurno.currentRow.HRFIMTURNO.replace(":", "");
            widgetTurno.getField('DTULTATUTURN').applyDefaultValue();
            InflightorGeneral.saveCrud(widgetTurno);
        }
    };

    this.recebeDadosDaTabela = function(widgetTurno) {
        if (!widgetTurno.dataSource.data) widgetTurno.dataSource.data = [];
        widgetTurno.dataSource.data.forEach(function(row) {
            converteFieldTime(row, ["HRINITURNO", "HRFIMTURNO"], "hh:mm");
        });
    };

    var converteFieldTime = function(row, fields, formato) {
        fields.forEach(function(field) {
            let valor = row[field];
            if (valor) {
                switch (formato) {
                    case 'hh:mm':
                        if (valor.length == 4) row[field] = valor[0] + valor[1] + ':' + valor[2] + valor[3];
                        break;
                    case 'hhmm':
                        if (valor.length == 5) row[field] = valor[0] + valor[1] + valor[3] + valor[4];
                        break;
                }
            }
        });
    };

    this.formatTime = function(widget) {

        widget.dataSourceFilter.map(function(obj) {
            if ((obj.name === "HRINITURNO") && (typeof(widget.appliedFilter.FILTER_ROW.HRINITURNO) != "undefined")) {
                obj.value = widget.appliedFilter.FILTER_ROW.HRINITURNO.split(":").join("");
            }
            if ((obj.name === "HRFIMTURNO") && (typeof(widget.appliedFilter.FILTER_ROW.HRFIMTURNO) != "undefined")) {
                obj.value = widget.appliedFilter.FILTER_ROW.HRFIMTURNO.split(":").join("");
            }
        });
        widget.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('TurnoDeTrabalhoController', TurnoDeTrabalhoController);
});

// FILE: ./assets/js/controllers/UtilizacaoInsumosController.js
var UtilizacaoInsumosController = function(ZHPromise, ScreenService, templateManager, InflightorGeneral, $rootScope, Colors, General) {
    var _self = this;
    var filterRow;

    this.widgetOnEnter = function(widget) {

        if (Boolean(widget.appliedFilter)) {
            filterRow = widget.appliedFilter.FILTER_ROW;
        }

    };

    function _isVisibleTab(widgetException, widget, booVisible) {
        widget.widgets.forEach(function(widget) {
            if (widget.widgets && widget.widgets.length) {
                _isVisibleTab(widgetException, widget, booVisible);
            }
            if (widget.name != widgetException.name) {
                widget.isVisible = booVisible;
            }
        });
    }

    function _booValidaData(widget) {
        let booDate = true;
        if (widget.currentRow.DTFIMVIGENCIA && widget.currentRow.DTINIVIGENCIA)
            booDate = moment(widget.currentRow.DTINIVIGENCIA, 'DD/MM/YYYY').unix() > moment(widget.currentRow.DTFIMVIGENCIA, 'DD/MM/YYYY').unix() ? false : true;
        if (!widget.isValid()) {
            return false;
        } else if (!booDate) {
            ScreenService.showMessage(ScreenService.i18n("Período de vigência inválido."), "error");
            return false;
        }
        return true;
    }

    this.newReceita = function(widget) {

        widget.newRow();

        _isVisibleTab(widget, widget, false);
        widget.activate();
        widget.fields.forEach(function(field) {
            if (field.fieldGroup && field.fieldGroup == "VIGENCIA") {
                field.isVisible = true;
            } else if (field.name == 'DSRECEITA') {
                _self.createFilterReceita(field, widget);
                //field.dataSourceFilter[0].value = widget.appliedFilter.FILTER_ROW.CDFILIAL;
                field.reload();
            }
        });
    };

    this.createFilterReceita = function(field, widget) {
        field.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.appliedFilter.FILTER_ROW.CDFILIAL
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": "@getLocalVar('INFLIGHTOR_NRORGTRAB')@"
            }
        ];
        widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
    };

    this.newVigencia = function(widget) {
        widget.newRow();
        _isVisibleTab(widget, widget, false);
        widget.activate();
        widget.fields.forEach(function(field) {
            field.setValue("");
        });
    };

    this.saveReceita = function(widget) {
        if (_booValidaData(widget)) {
            if (widget.name == "widgetVigencia" && widget.currentRow.__is_new) {
                let oldRow = widget.currentRow;
                Object.assign(widget.currentRow, widget.parent.parent.currentRow, oldRow);
                widget.currentRow.__is_new = true;
            }
            let paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
                widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk + widget.currentRow.CDFILIAL;
                widget.currentRow.__table_pk = paramsNewCode.__table_pk;
                widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
                widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
            }
            widget.currentRow.__columns_to_return = ["NRSEQUTILRECFLEX"];
            InflightorGeneral.callRestEngine('pcp#/pcp00020_nova_receita', 'Row', widget.currentRow).then(function(success) {
                widget.activate();
                if (widget.currentRow.__is_new) {
                    widget.list(true);
                    ScreenService.goBack();
                    widget.reload().then(function() {});
                } else {
                    widget.computeDescriptionOrder();
                    widget.view();
                }
                _isVisibleTab(widget, widget, true);
            }, function(error) {
                InflightorGeneral.callbackError(error);
            });
        }
    };

    this.cancelReceita = function(widget, args) {
        if (widget.currentRow && widget.currentRow.__is_new) {
            General.backOrCancel(args);
            _isVisibleTab(widget, widget, true);
        } else {
            _isVisibleTab(widget, widget, true);
            widget.view();
            widget.activate();
        }
    };

    this.excluirVigencia = function(widget) {
        InflightorGeneral.callRestEngine('pcp#/pcp00020_deleta_vigencia', 'Row', widget.currentRow).then(function(success) {
            ScreenService.closeSwipe().then(function() {
                if (success.messages || success.notifications) {
                    success.messages = success.messages || success.notifications;
                    ScreenService.showMessage(ScreenService.i18n(success.messages[0].message), 'success').then(function() {
                        widget.reload().then(function() {
                            if (widget.dataSource.data && !widget.dataSource.data.length) {
                                let widgetReceita = widget.container.getWidget('widgetReceita');
                                widgetReceita.reload().then(function() {
                                    widgetReceita.activate();
                                    widgetReceita.currentWidget = null;
                                    widgetReceita.currentRow.__is_new = false;
                                });
                            }
                        });
                    });
                }
            });
        }, function(error) {
            InflightorGeneral.callBackError(error);
        });
    };

    this.backView = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.list();
            widget.activate();
        });
    };

    this.backList = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.parent.parent.currentWidget = null;
            widget.parent.parent.activate();
        });
    };

    this.backListComposicao = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.getParent().currentWidget = null;
            widget.getParent().activate();
        });
    };

    this.widgetOnEnterComposicao = function(widget) {
        _setFilterWidget(widget, widget.parent.currentRow);
        widget.reload().then(function() {
            widget.activate();
        });
    };

    this.changedDataFim = function(field) {
        let _valueIni = field.widget.getField('DTINIVIGENCIA').value();
        let _valueFim = field.value();
        if (_valueIni && moment(_valueIni, 'DD/MM/YYYY').unix() > moment(_valueFim, 'DD/MM/YYYY').unix()) {
            ScreenService.showMessage(ScreenService.i18n("Período de vigência inválido."), "error");
            field.setValue("");
        }
    };

    this.changedDataInicio = function(field) {
        let fieldFim = field.widget.getField('DTFIMVIGENCIA');
        if (field.value()) {
            if (fieldFim.value() && moment(fieldFim.value(), 'DD/MM/YYYY').unix() < moment(field.value(), 'DD/MM/YYYY').unix()) {
                fieldFim.setValue("");
            }
            fieldFim.readOnly = false;
        } else {
            fieldFim.readOnly = true;
            fieldFim.setValue("");
        }
    };

    function _setFilterWidget(widget, currentRow) {
        widget.dataSourceFilter.forEach(function(filter) {
            filter.value = currentRow[filter.name];
        });
    }

    this.afterMoveRow = function(widget) {
        let _widgetOpen;
        if (widget.mode != 'edit' && !widget.currentRow.__is_new) {
            if (widget.currentWidget) {
                _setFilterWidget(widget.currentWidget.widgets[0], widget.currentRow);
                widget.currentWidget.widgets[0].reload().then(function() {
                    widget.currentWidget.widgets[0].activate();
                });
            } else {
                _widgetOpen = widget.widgets[0];
                _setFilterWidget(_widgetOpen.widgets[0], widget.currentRow);
                ScreenService.openSwipe(_widgetOpen).then(function() {
                    _widgetOpen.widgets[0].reload().then(function() {
                        _widgetOpen.widgets[0].activate();
                    });
                });
            }
        }
    };

    this.setStripeFooterValues = function(widgetComposicao) {
        let stripeFooter = widgetComposicao.container.getWidget('stripefooter_composicao');
        stripeFooter.dataSourceFilter = angular.copy(widgetComposicao.dataSourceFilter.filter(function(filter) {
            return filter.name !== "NRSEQUTILRECFLEX";
        }));
        _setValuesToStripeFooter(stripeFooter, stripeFooter.dataSourceFilter);
    };

    function _setValuesToStripeFooter(stripeWidget, stripeFilter) {
        InflightorGeneral.callRestEngine('pcp#/crud/pcp00020_get_rendimentoBruto', 'FilterData', stripeFilter).then(
            function(success) {
                if (success.dataset && success.dataset.pcp00020_get_rendimentoBruto.length) {
                    let callbackVariacao = success.dataset.pcp00020_get_rendimentoBruto;
                    callbackVariacao[0].QTD = callbackVariacao[0].QTD || 0;
                    callbackVariacao[0].OPCOES = callbackVariacao[0].OPCOES || 0;
                    callbackVariacao[0].QTRENDIMENTOREC = callbackVariacao[0].QTRENDIMENTOREC || 0;
                    let rendLiquido = InflightorGeneral.valueToFloat(success.dataset.pcp00020_get_rendimentoBruto[0].QTRENDIMENTOREC);
                    stripeWidget.getField('QTPRODTCOMP').setValue(String(rendLiquido.toFixed(3)).replace(".", ","));
                    let rendBruto = InflightorGeneral.valueToFloat(success.dataset.pcp00020_get_rendimentoBruto[0].QTD);
                    stripeWidget.getField('QTRENDBRUTO').setValue(String(rendBruto.toFixed(3)).replace(".", ","));
                    let percentLiq;
                    if (rendLiquido && rendBruto) {
                        percentLiq = ((rendLiquido * 100) / rendBruto).toFixed(2);
                    } else {
                        percentLiq = 0;
                    }
                    stripeWidget.getField('QTPERCENTAPRO').setValue(percentLiq);
                    stripeWidget.getField('QTOPCOES').setValue(success.dataset.pcp00020_get_rendimentoBruto[0].OPCOES);
                }
            }
        );
    }

    this.changedComposicao = function(widget) {
        let stripeWidget = widget.container.getWidget('stripefooter_composicao');
        let qtopcoes = stripeWidget.currentRow.QTOPCOES ? stripeWidget.currentRow.QTOPCOES : stripeWidget.getField('QTOPCOES').value();
        if (qtopcoes) {
            let rowsChecked = widget.dataSource.getCheckedRows();
            if (rowsChecked.length >= qtopcoes) {
                _changedComposicao(widget, rowsChecked);
            } else {
                ScreenService.showMessage(ScreenService.i18n("Quantidade de opções inválida. (Minímo: " + qtopcoes + ")"), 'error');
            }
        } else {
            ScreenService.showMessage(ScreenService.i18n("Quantidade de opções não definida."), 'error');
        }
    };

    function _changedComposicao(widget, rows) {
        let dataSave = _dataSetComposicao(widget, rows);
        InflightorGeneral.callRestEngine('pcp#/pcp00020_altera_composicao', 'DataSet', dataSave).then(function(success) {
            widget.dataSource.unSelectAll();
            widget.reload().then(function() {
                widget.activate();
                widget.list();
            });
        }, InflightorGeneral.callBackError);
    }

    function _dataSetComposicao(widget, rows) {
        let data = [];
        let widgetVigencia = widget.container.getWidget('widgetVigencia');
        rows.forEach(function(row) {
            row.CDFILIAL = widgetVigencia.currentRow.CDFILIAL;
            row.CDRECEITA = widgetVigencia.currentRow.CDRECEITA;
            row.NRSEQUTILRECFLEX = widgetVigencia.currentRow.NRSEQUTILRECFLEX;
            if (row.__leftActions) {
                delete row.__leftActions;
            }
            if (row.__rightActions) {
                delete row.__rightActions;
            }
            data.push(row);
        });
        return data;
    }

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UtilizacaoInsumosController', UtilizacaoInsumosController);
});
